/* 
 * $RCSfile: gj_agents.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/05/31 12:17:55 $
 * 
 * $Revision: 1.4 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <glib.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include "gj_agents.h"
#include "gj_parser.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_iq_requests.h"

/* agent item */
void agentDestroy(agent ag);

/* agent list */
gboolean agentListAdd(agent ag);
gboolean agentListDel(agent ag);

void agentListResponse(guint requestID);
GList *agentListParse(xmlNodePtr parent);

/* call backs */
gboolean agentSetResponseProc(agentResponseProc proc);
agentResponseProc agentGetResponseProc();

struct t_agent
{
  gchar *jid;
  gchar *name;
  gchar *description;
 
  gchar *transport;
  gchar *groupchat;
  gchar *service;
  gchar *registers;
  gchar *search;
};

GList *agentList;
agentResponseProc agentProc;

gboolean agentInit()
{
  if(agentList != NULL)
    agentTerm();

  /* free agent response proc */
  agentProc = NULL;

  /* create mem  - nothing to do, created on g_list_append */
  return TRUE;
}

gboolean agentTerm()
{
  gint index = 0;

  /* list */
  if(agentList == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(agentList);index++)
    {
      agent ag = NULL;
      
      if((ag = (agent) g_list_nth_data(agentList,index)) != NULL)
	agentFree(ag);
    }

  g_list_free(agentList);
  agentList = NULL;

  /* free agent response proc */
  agentProc = NULL;

  return TRUE;
}

gboolean agentListAdd(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentListAdd: failed to add item to list, it was NULL");
      return FALSE;
    }

  agentList = g_list_append(agentList,ag);

  return TRUE;
}

gboolean agentListDel(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentListDel: failed to del item from list, it was NULL");
      return FALSE;
    }

  agentList = g_list_remove(agentList,ag);
  return TRUE;
}

GList *agentListGet()
{
  return agentList;
}

/* 
 * item functions
 */
agent agentNew(gchar *jid)
{
  agent ag;

  if(jid == NULL)
    {
      g_warning("agentNew: failed to create new agent item, jid was NULL");
    }
  
  if((ag = g_new(struct t_agent,1)))
    {
      ag->jid = g_strdup(jid);
      ag->name = NULL;
      ag->description = NULL;

      ag->transport = NULL;
      ag->groupchat = NULL;
      ag->service = NULL;
      ag->registers = NULL;
      ag->search = NULL;

      agentListAdd(ag);
    }
  
  return ag;  
}

void agentFree(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentFree: failed to free, agent was NULL");
      return;
    }

  /* free mem */
  agentListDel(ag);  

  /* destroy members */
  agentDestroy(ag);

  /* free structure */
  g_free(ag);
}

void agentDestroy(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentDestroy: failed to destroy, agent was NULL");
      return;
    }
  
  if(ag->jid != NULL)
    {
      g_free(ag->jid);
      ag->jid = NULL;
    }

  if(ag->name != NULL)
    {
      g_free(ag->name);
      ag->name = NULL;
    }

  if(ag->description != NULL)
    {
      g_free(ag->description);
      ag->description = NULL;
    }

  if(ag->transport != NULL)
    {
      g_free(ag->transport);
      ag->transport = NULL;
    }

  if(ag->groupchat != NULL)
    {
      g_free(ag->groupchat);
      ag->groupchat = NULL;
    }

  if(ag->service != NULL)
    {
      g_free(ag->service);
      ag->service = NULL;
    }

  if(ag->registers != NULL)
    {
      g_free(ag->registers);
      ag->registers = NULL;
    }

  if(ag->search != NULL)
    {
      g_free(ag->search);
      ag->search = NULL;
    }
}

/*
 * agent handling
 *
 */
gboolean agentListRequest(gchar *server, agentResponseProc proc)
{
  if(server == NULL)
    {
      g_warning("agentRequestList: server was NULL");
      return FALSE;
    }

  if(proc == NULL)
    {
      g_warning("agentRequestList: proc was NULL");
      return FALSE;
    }

  /* set call back proc */
  agentSetResponseProc(proc);

  /* do request */
  infoqueryRequestAgents(server, agentListResponse);

  return TRUE;
}

void agentListResponse(guint eventID)
{
  infoquery iq = NULL;
  infoquery iq_related = NULL;

  guint eventIDRelated = 0;
  gint error_code = 0;
  gchar *error_message = NULL;
  agentResponseProc proc = NULL;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  if((iq = infoqueryFindByEventID(eventID)) == NULL)
    {
      g_warning("agentListResponse: could not get request from request id:%d", eventIDRelated);
      return;
    }

  eventIDRelated = infoqueryGetEventIDRelated(iq);
  if((iq_related = infoqueryFindByEventID(eventIDRelated)) == NULL)
    {
      g_warning("agentListResponse: could not get request from response id:%d", eventIDRelated);
      return;
    }

  node = infoqueryGetXML(iq_related);
  type = parserFindByAttrAndReturn(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      xmlChar *errorCode = NULL;
      xmlChar *errorMessage = NULL;

      if((node = parserFindByNode(node, (guchar*)"error")) != NULL)
	{
	  if((errorCode = parserFindByAttrAndReturn(node, (guchar*)"code")) != NULL)
	    error_code = atoi(errorCode);
	  
	  if((errorMessage = parserFindByNodeAndReturn(node, (guchar*)"error")) != NULL) 
	    error_message = g_strdup(errorMessage);
	}
      else
	g_warning("agentListResponse: failed to find node 'error'");
    }

  if((proc = agentGetResponseProc()) != NULL)
    {
      if(agentList != NULL)
	{
	  /* free list */
	}
      
      /* parse list */
      agentList = agentListParse(node);
      
      /* return list */
      (proc)(agentList, error_code, error_message);
    }
  else
    g_warning("agentListResponse: response proc is NULL");
  
  /* clean up */
  infoqueryFreeByEventID(eventID);
  infoqueryFreeByEventID(eventIDRelated);

  if(error_message != NULL) g_free(error_message);
}

GList *agentListParse(xmlNodePtr parent)
{
  gint count = 0;
  xmlNodePtr node = NULL;

  if(parent == NULL)
    {
      g_warning("agentListParse: failed to get agent because query (xml) was NULL");
      return NULL;
    }

  /* create list */
  agentInit();

  /* get item node */
  node = parserFindByNode(parent, (guchar*)"agent");

  /* get items in agent */
  while(node)
    {
      xmlChar *tmp = NULL;

      count++;
      
      /* get data */
      if((tmp = parserFindByAttrAndReturn(node, (guchar*)"jid")) != NULL)
	{
	  gchar *jid = NULL;
	  gchar *resource = NULL;
	  gchar **vjid = NULL;

	  agent ag = NULL;

	  vjid = g_strsplit((gchar*)tmp,"/",2);

	  if(vjid != NULL)
	    {
	      jid = g_strdup(vjid[0]);
	      
	      if(vjid[1] != NULL)
		resource = g_strdup(vjid[1]);
	      
	      g_strfreev(vjid);
	    }
	  
	  /* create new agent item */
	  ag = agentNew(jid);
      
	  /* check it... */
	  if(ag == NULL)
	    {
	      g_warning("agentListParse: failed to create new agent item, continuing to next item...");
	      continue;
	    }
	  
#if 0
	  /* resource */
	  if(resource != NULL)
	    ag->resource = resource;
#endif

	  /* find other details */
	  if((tmp = parserFindByNodeAndReturn(node, (guchar*)"name")) != NULL)
	    agentSetName(ag, (gchar*)tmp);
	  if((tmp = parserFindByNodeAndReturn(node, (guchar*)"description")) != NULL)
	    agentSetDescription(ag, (gchar*)tmp);
	  if((tmp = parserFindByNodeAndReturn(node, (guchar*)"transport")) != NULL)
	    agentSetTransport(ag, (gchar*)tmp);
	  if((tmp = parserFindByNodeAndReturn(node, (guchar*)"groupchat")) != NULL)
	    agentSetGroupchat(ag, (gchar*)tmp);
	  if((tmp = parserFindByNodeAndReturn(node, (guchar*)"service")) != NULL)
	    agentSetService(ag, (gchar*)tmp);
	  if((tmp = parserFindByNodeAndReturn(node, (guchar*)"register")) != NULL)
	    agentSetRegisters(ag, (gchar*)tmp);
	  if((tmp = parserFindByNodeAndReturn(node, (guchar*)"search")) != NULL)
	    agentSetSearch(ag, (gchar*)tmp);
	}

      node = node->next;
    }

  g_message("agentListParse: items:%d",count);

  return agentList;
}

gboolean agentSetResponseProc(agentResponseProc proc)
{
  if(proc == NULL)
    {
      g_warning("agentSetResponseProc: the response proc was NULL");
      return FALSE;
    }

  agentProc = proc;
  
  return TRUE;
}

agentResponseProc agentGetResponseProc()
{
  return agentProc;
}

/*
 * agent functions 
 *
 */

agent agentFindByName(gchar *name)
{
  gint index = 0;

  if(name == NULL)
    {
      g_warning("agentFindByName: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(agentList);index++)
    {
      agent ag  = g_list_nth_data(agentList,
				       index);

      if(ag == NULL)
	{
	  g_warning("agentFindByName: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(ag->name,name) == 0)
	return ag;
    }

  return NULL;
}

agent agentFindByJID(gchar *jid)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("agentFindByJID: JID was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(agentList);index++)
    {
      agent ag  = g_list_nth_data(agentList,
				       index);

      if(ag == NULL)
	{
	  g_warning("agentFindByJID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(ag->jid,jid) == 0)
	return ag;
    }

  return NULL;
}

/*
 * SET functions 
 *
 */
gboolean agentSetJID(agent ag, gchar *jid)
{
  if(ag == NULL)
    {
      g_warning("agentSetJID: agent was NULL");
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("agentSetJID: NULL passed");
      return FALSE;
    }

  if(ag->jid != NULL)
    g_free(ag->jid);
  
  ag->jid = g_strdup(jid);

  return TRUE;
}

gboolean agentSetName(agent ag, gchar *name)
{
  if(ag == NULL)
    {
      g_warning("agentSetName: agent was NULL");
      return FALSE;
    }

  if(name == NULL)
    {
      g_warning("agentSetName: name was NULL");
      return FALSE;
    }

  if(ag->name != NULL)
    g_free(ag->name);
  
  ag->name = g_strdup(name);

  return TRUE;
}

gboolean agentSetDescription(agent ag, gchar *description)
{
  if(ag == NULL)
    {
      g_warning("agentSetDescription: agent was NULL");
      return FALSE;
    }

  if(description == NULL)
    {
      g_warning("agentSetDescription: description was NULL");
      return FALSE;
    }

  if(ag->description != NULL)
    g_free(ag->description);
  
  ag->description = g_strdup(description);

  return TRUE;
}

gboolean agentSetTransport(agent ag, gchar *transport)
{
  if(ag == NULL)
    {
      g_warning("agentSetTransport: agent was NULL");
      return FALSE;
    }

  if(transport == NULL)
    {
      g_warning("agentSetTransport: transport was NULL");
      return FALSE;
    }

  if(ag->transport != NULL)
    g_free(ag->transport);
  
  ag->transport = g_strdup(transport);

  return TRUE;
}

gboolean agentSetGroupchat(agent ag, gchar *groupchat)
{
  if(ag == NULL)
    {
      g_warning("agentSetGroupchat: agent was NULL");
      return FALSE;
    }

  if(groupchat == NULL)
    {
      g_warning("agentSetGroupchat: groupchat was NULL");
      return FALSE;
    }

  if(ag->groupchat != NULL)
    g_free(ag->groupchat);
  
  ag->groupchat = g_strdup(groupchat);

  return TRUE;
}

gboolean agentSetService(agent ag, gchar *service)
{
  if(ag == NULL)
    {
      g_warning("agentSetService: agent was NULL");
      return FALSE;
    }

  if(service == NULL)
    {
      g_warning("agentSetService: service was NULL");
      return FALSE;
    }

  if(ag->service != NULL)
    g_free(ag->service);
  
  ag->service = g_strdup(service);

  return TRUE;
}

gboolean agentSetRegisters(agent ag, gchar *registers)
{
  if(ag == NULL)
    {
      g_warning("agentSetRegisters: agent was NULL");
      return FALSE;
    }

  if(registers == NULL)
    {
      g_warning("agentSetRegisters: registers was NULL");
      return FALSE;
    }

  if(ag->registers != NULL)
    g_free(ag->registers);
  
  ag->registers = g_strdup(registers);

  return TRUE;
}

gboolean agentSetSearch(agent ag, gchar *search)
{
  if(ag == NULL)
    {
      g_warning("agentSetSearch: agent was NULL");
      return FALSE;
    }

  if(search == NULL)
    {
      g_warning("agentSetSearch: search was NULL");
      return FALSE;
    }

  if(ag->search != NULL)
    g_free(ag->search);
  
  ag->search = g_strdup(search);

  return TRUE;
}


/*
 * GET functions 
 *
 */
gchar *agentGetJID(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentGetJID: agent was NULL");
      return NULL;
    }

  return ag->jid;
}

gchar *agentGetName(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentGetName: agent was NULL");
      return NULL;
    }

  return ag->name;
}

gchar *agentGetDescription(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentGetDescription: agent was NULL");
      return NULL;
    }

  return ag->description;
}

gchar *agentGetTransport(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentGetTransport: agent was NULL");
      return NULL;
    }

  return ag->transport;
}

gchar *agentGetGroupchat(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentGetGroupchat: agent was NULL");
      return NULL;
    }

  return ag->groupchat;
}

gchar *agentGetService(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentGetService: agent was NULL");
      return NULL;
    }

  return ag->service;
}

gchar *agentGetRegisters(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentGetRegisters: agent was NULL");
      return NULL;
    }

  return ag->registers;
}

gchar *agentGetSearch(agent ag)
{
  if(ag == NULL)
    {
      g_warning("agentGetSearch: agent was NULL");
      return NULL;
    }

  return ag->search;
}

/* debugging */
void agentPrintAll()
{
  agent ag = NULL;
  gint index = 0;

  g_message("\n");
  g_message("++++++++++++++++++++++ Agent - Print All, Begin. +++++++++++++++++++++");

  for(index=0;index<g_list_length(agentList);index++)
    {
      ag = (agent) g_list_nth_data(agentList,index);
      agentPrint(ag);
    }

  g_message("++++++++++++++++++++++ Agent - Print All, End.   +++++++++++++++++++++");
  
  return;  
}

void agentPrint(agent ag)
{
  if(ag != NULL)
    {
      g_message("+++++++++++++++++++++++++++ Agent +++++++++++++++++++++++++");
      
      g_message("++++");
      g_message("++++ jid:'%s'",ag->jid);
      g_message("++++ name:'%s'",ag->name);
      g_message("++++ description:'%s'",ag->description);
      g_message("++++");
      g_message("++++ transport:'%s'",ag->transport);
      g_message("++++ groupchat:'%s'",ag->groupchat);
      g_message("++++ service:'%s'",ag->service);
      g_message("++++ registers:'%s'",ag->registers);
      g_message("++++ search:'%s'",ag->search);
      g_message("++++");

      g_message("+++++++++++++++++++++++++++ Agent +++++++++++++++++++++++++");
    }
  else 
    g_warning("agentPrint: failed to print, NULL Agent.");
}

#ifdef __cplusplus
}
#endif
