/* 
 * $RCSfile: gj_connection.h,v $
 *
 * $Author: ginxd $
 * $Date: 2003/05/11 12:19:36 $
 * 
 * $Revision: 1.4 $
 *
 */

#ifndef __gj_connection_h
#define __gj_connection_h

#include <gnet.h>

typedef struct t_connection *connection;

typedef void (*socket_cb_in)(connection c, gchar *text);
typedef void (*socket_cb_hup)(connection c);
typedef void (*socket_cb_connect)(connection c);
typedef void (*socket_cb_disconnect)(connection c);
typedef void (*socket_cb_error)(connection c, GTcpSocketConnectAsyncStatus status);

gboolean gj_connection_init();
gboolean gj_connection_term();

/* 
 * member functions 
 */

/* gets */
const guint gj_connection_get_id(connection c);
const gchar *gj_connection_get_host(connection c);
const guint gj_connection_get_port(connection c);

GTcpSocket *gj_connection_get_tcpsocket(connection c);
GTcpSocketConnectAsyncID gj_connection_get_connect_id(connection c);

gboolean gj_connection_get_connected(connection c);

socket_cb_in gj_connection_get_cb_in(connection c);
socket_cb_hup gj_connection_get_cb_hup(connection c);
socket_cb_connect gj_connection_get_cb_connect(connection c);
socket_cb_disconnect gj_connection_get_cb_disconnect(connection c);

/* sets */
gboolean gj_connection_set_host(connection c, const gchar *host);
gboolean gj_connection_set_port(connection c, const guint port);

gboolean gj_connection_set_tcpsocket(connection c, GTcpSocket *ts);
gboolean gj_connection_set_connect_id(connection c, GTcpSocketConnectAsyncID connect_id);

gboolean gj_connection_set_connected(connection c, gboolean connected);

gboolean gj_connection_set_cb_in(connection c, socket_cb_in cb);
gboolean gj_connection_set_cb_hup(connection c, socket_cb_hup cb);
gboolean gj_connection_set_cb_connect(connection c, socket_cb_connect cb);
gboolean gj_connection_set_cb_disconnect(connection c, socket_cb_disconnect cb);
gboolean gj_connection_set_cb_error(connection c, socket_cb_error cb);

/* find */
connection gj_connection_find_by_host_and_port(const gchar *host, const guint port);
connection gj_connection_find_by_id(const guint id);


/*
 * connect / disconnect / etc
 */
connection gj_connection_create(const gchar *host, const guint port);
gboolean gj_connection_free(connection c);

gboolean gj_connection_connect(connection c);

gboolean gj_connection_disconnect(connection c);
gboolean gj_connection_disconnect_by_host_and_port(const gchar *host, const guint port);

void gj_connection_connect_cb(GTcpSocket *socket, GTcpSocketConnectAsyncStatus status, gpointer data);

gboolean gj_connection_write(connection c, const gchar *text);

/* callbacks */
gboolean gj_connection_watch_cb(GIOChannel *source, GIOCondition condition, gpointer data);

/* translations */
const gchar *gj_connection_connect_translate_status(GTcpSocketConnectAsyncStatus status);
const gchar *gj_connection_io_status_translate(GIOStatus status);
const gchar *gj_connection_io_error_translate(gint error);
const gchar *gj_connection_io_condition_translate(gint condition);

#endif
