/* 
 * $RCSfile: gj_event.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/26 11:09:57 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>

#include <time.h> 

#include <glib.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include "gj_event.h"
#include "gj_roster.h"
#include "gj_parser.h"
#include "gj_presence.h"
#include "gj_message.h"

/* structs */
struct t_event
{
  guint id;
  gchar *jid;

  gboolean inEvent; /* event has been looked at but not free'd, eg. in chat window already */
    
  eventType type;
  eventProc proc;
    
  rosterItem ri;
    
  time_t time;
    
  void *data;
};

gboolean eventListAdd(event ev);
gboolean eventListDel(event ev);

void eventDestroy(event ev);

GList *eventList = NULL;
eventProc eventProcGenericResponse = NULL;

/*
 * code
 */
gboolean eventInit()
{
  return TRUE;
}

gboolean eventTerm()
{
  gint index = 0;
    
  /* list */
  if(eventList == NULL)
    return TRUE;
    
  /* free all items on the list */
  for(index=0;index<g_list_length(eventList);index++)
    {
      event ev = NULL;
	
      if((ev = (event) g_list_nth_data(eventList,index)) != NULL)
	eventFree(ev);
    }
    
  g_list_free(eventList);
  eventList = NULL;
    
  return TRUE;
}

gboolean eventListAdd(event ev)
{
  if(ev == NULL)
    {
      g_warning("eventListAdd: failed to add item to list, it was NULL");
      return FALSE;
    }

  eventList = g_list_append(eventList,ev);

  return TRUE;
}

gboolean eventListDel(event ev)
{
  if(ev == NULL)
    {
      g_warning("eventListDel: failed to del item from list, it was NULL");
      return FALSE;
    }

  eventList = g_list_remove(eventList,ev);
  return TRUE;
}

event eventNew(gchar *jid, eventType type, void *data)
{
  event ev = NULL;
  rosterItem ri = NULL;

  if(jid == NULL) 
    {
      g_warning("eventNew: jid was NULL");
      return NULL;
    }

  if((ri = rosterItemFindByJID(jid)) == NULL)
    {
      g_message("eventNew: roster item not found by jid:'%s', trying WITHOUT the resource...", jid);
    
      if((ri = rosterItemFindByJIDWithoutResource(jid)) == NULL)
	g_message("eventNew: roster item not found by jid:'%s', perhaps this is a temporary contact?", jid);
    }

  if(type < 1 || type >= eventTypeEnd)
    {
      g_warning("eventNew: event type was out of range");
      return NULL;
    }

  if((ev = g_new(struct t_event,1)))
    {
      ev->id = supportGetUniqueID();

      ev->jid = g_strdup(jid);

      ev->type = type;
      ev->data = data;

      ev->inEvent = FALSE;
      ev->proc = NULL;
      ev->ri = ri;

      ev->time = time(0);
    }

  eventList = g_list_append(eventList, ev);

  return ev;
}

gboolean eventNewByPostFromJID(gchar *jid, eventType type, void *data)
{
  event ev = NULL;
  eventProc proc = NULL;

  if(jid == NULL)
    {
      g_warning("eventNewByPostFromJID: jid was NULL");
      return FALSE;
    }

  if((proc = eventGetResponseProc()) == NULL)
    {
      g_warning("eventNewByPostFromJID: no response callback to post new event to");
      return FALSE;
    }

  if((ev = eventNew(jid, type, data)) == NULL)
    {
      g_warning("eventNewByPostFromJID: could not create new event");
      return FALSE;
    }

  /* call callback to post new event */
  (proc)(ev->id);
  return TRUE;
}

gboolean eventNewByPostFromRosterItem(rosterItem ri, eventType type, void *data)
{
  if(ri == NULL)
    {
      g_warning("eventNewByPostFromRosterItem: roster item was NULL");
      return FALSE;
    }

  return eventNewByPostFromJID(rosterItemGetJID(ri), type, data);
}

gboolean eventFree(event ev)
{
  if(ev == NULL)
    {
      g_warning("eventFree: failed to free, event was NULL");
      return FALSE;
    }

  /* free mem */
  eventListDel(ev);  

  /* destroy members */
  eventDestroy(ev);

  /* free structure */
  g_free(ev);

  return TRUE;
}

gboolean eventFreeByID(gint eventID)
{
  gint index = 0;

  if(eventID < 0)
    {
      g_warning("eventFreeByID: eventID:%d was < 0", eventID);
      return FALSE;
    }

 
  /* loop through all events on contacts event list to look for unique eventID */
  for(index=0;index<g_list_length(eventList);index++)
    {
      event ev = g_list_nth_data(eventList, index);
      
      if(ev == NULL)
	{
	  g_warning("eventFreeByID: getting nth item(%d) in roster item event list was NULL",index);
	  continue;
	}
      
      if(ev->id == eventID)
	return eventFree(ev);
    }   

  g_warning("eventFreeByID: could not find event by id:%d", eventID);
  return FALSE;
}

void eventDestroy(event ev)
{
  if(ev == NULL)
    {
      g_warning("eventDestroy: failed to destroy, event was NULL");
      return;
    }

  ev->id = 0;

  if(ev->jid != NULL)
    {
      g_free(ev->jid);
      ev->jid = NULL;
    }

  ev->type = eventTypeEnd;
  ev->proc = NULL;

  ev->ri = NULL;

  /*
   * should we be freeing these types here or should the module clear them
   * up when it is terminated, e.g. infoqueryTerm();
   */
  if(ev->data != NULL)
    {
      switch(ev->type)
	{
	case eventTypeMessage:
	  {
	    message msg = (message) ev->data;
	    messageFree(msg);
	  }
	  break;

	case eventTypePresence:
	  {
	    presence pres = (presence) ev->data;
	    presenceFree(pres);
	  }
	  break;
	
	default:
	  break;
	}
    }
}

/*
 * response proc for inbound events 
 */
gboolean eventSetResponseProc(eventProc proc)
{
  if(proc == NULL)
    {
      g_warning("eventSetResponseProc: the proc was NULL");
      return FALSE;
    }

  eventProcGenericResponse = proc;

  return TRUE;
}

eventProc eventGetResponseProc()
{
  return eventProcGenericResponse;
}


/*
 * utils
 */
event eventNth(gint index)
{
  event ev = NULL;

  if(eventList == NULL)
    {
      g_warning("eventNth: event list was NULL");
      return NULL;
    }

  if(index < 0 || index > g_list_length(eventList))
    {
      g_warning("eventNth: index:%d was < 0 || > %d", index, g_list_length(eventList));
      return NULL;
    }

  /* get first item and return eventID */
  ev = (event)g_list_nth_data(eventList, index);
  
  return ev;
}

event eventNthByRosterItem(rosterItem ri, gint index)
{
  guint count = 0;

  if(ri == NULL)
    {
      g_warning("eventNthByRosterItem: roster item was NULL");
      return NULL;
    }

  /* do not warning this - this is perfectly normal */
  if(eventList == NULL || g_list_length(eventList) < 1)
    return NULL;

  if(index < 0 || index > g_list_length(eventList))
    {
      g_warning("eventNthByRosterItem: index:%d was < 0 || > %d", index, g_list_length(eventList));
      return NULL;
    }

  for(index=0,count=0;index<g_list_length(eventList);index++)
    {
      event ev = g_list_nth_data(eventList, index);
      
      if(ev == NULL)
	{
	  g_warning("eventNthByRosterItem: getting nth item(%d) in event list was NULL",index);
	  continue;
	}
      
      if(ev->ri == ri && ++count == index)
	return ev;
    }

  return NULL;
}

gint eventIDNext(rosterItem ri)
{
  gint index = 0;

  if(ri == NULL)
    {
      g_warning("eventIDNext: roster item was NULL");
      return -1;
    }

  /* do not warning this - this is perfectly normal */
  if(eventList == NULL || g_list_length(eventList) < 1)
    return -1;

  /* loop through all eventList on contacts event list to look for unique eventID */
  for(index=0;index<g_list_length(eventList);index++)
    {
      event ev = g_list_nth_data(eventList, index);
      
      if(ev == NULL)
	{
	  g_warning("eventIDNext: getting nth item(%d) in event list was NULL",index);
	  continue;
	}
      
      if(ev->ri == ri)
	{
	  return ev->id;
	}
    }
  
  return -1;
}

gint eventCount(rosterItem ri)
{
  gint index = 0;
  guint count = 0;

  if(ri == NULL)
    {
      g_warning("eventCount: roster item was NULL");
      return -1;
    }

  /* do not warning this - this is perfectly normal */
  if(eventList == NULL)
    return 0;

  /* loop through all eventList on contacts event list to look for unique eventID */
  for(index=0,count=0;index<g_list_length(eventList);index++)
    {
      event ev = g_list_nth_data(eventList, index);
      
      if(ev == NULL)
	{
	  g_warning("eventCount: getting nth item(%d) in event list was NULL",index);
	  continue;
	}
      
      if(ev->ri == ri)
	count++;
    }

  return count;
}

event eventFindByEventID(gint eventID)
{
  gint index = 0;

  if(eventID < 0)
    {
      g_warning("eventFindByEventID: eventID was < 0");
      return NULL;
    }

  /* loop through all eventList on contacts event list to look for unique eventID */
  for(index=0;index<g_list_length(eventList);index++)
    {
      event ev = g_list_nth_data(eventList, index);
      
      if(ev == NULL)
	{
	  g_warning("eventFindByEventID: getting nth item(%d) in event list was NULL",index);
	  continue;
	}
      
      if(ev->id == eventID)
	{
	  return ev;
	}
    }
 
  return NULL;
}

event eventFindNextByType(rosterItem ri, eventType type)
{
  gint index = 0;

  if(ri == NULL)
    {
      g_warning("eventFindNextByType: roster item was NULL");
      return NULL;
    }

  if(type < 0)
    {
      g_warning("eventFindNextByType: event type:%d was < 0",type);
      return NULL;
    }

  /* do not warning this - this is perfectly normal */
  if(eventList == NULL || g_list_length(eventList) < 1)
    return NULL;

  /* loop through all events on contacts event list to look for unique eventID */
  for(index=0;index<g_list_length(eventList);index++)
    {
      event ev = g_list_nth_data(eventList, index);
      
      if(ev == NULL)
	{
	  g_warning("eventFindNextByType: getting nth item(%d) in roster item event list was NULL",index);
	  continue;
	}
      
      if(ev->ri == ri && ev->type == type)
	return ev;
    }

  return NULL;
}

event eventFindNextByTypeAndMessageType(rosterItem ri, eventType type, messageType messageType)
{
  gint index = 0;

  if(ri == NULL)
    {
      g_warning("eventFindNextByTypeAndMessageType: roster item was NULL");
      return NULL;
    }

  if(type < 0)
    {
      g_warning("eventFindNextByTypeAndMessageType: event type:%d was < 0",type);
      return NULL;
    }

  /* do not warning this - this is perfectly normal */
  if(eventList == NULL || g_list_length(eventList) < 1)
    return NULL;

  /* loop through all events on contacts event list to look for unique eventID */
  for(index=0;index<g_list_length(eventList);index++)
    {
      event ev = g_list_nth_data(eventList, index);
      
      if(ev == NULL)
	{
	  g_warning("eventFindNextByTypeAndMessageType: getting nth item(%d) in roster item event list was NULL",index);
	  continue;
	}
      
      if(ev->ri == ri && ev->type == type)
	{
	  message msg = (message) ev->data;
	    
	  if(msg == NULL)
	    continue;

	  if(messageGetType(msg) == messageType)
	    return ev;
	}
    }

  return NULL;
}

gint eventCountByType(rosterItem ri, eventType type)
{
  gint index = 0;
  gint count = 0;

  if(ri == NULL)
    {
      g_warning("eventCountByType: roster item was NULL");
      return -1;
    }

  if(type < 0)
    {
      g_warning("eventCountByType: roster item type:%d was < 0",type);
      return -1;
    }

  /* loop through all events on contacts event list to look for unique eventID */
  for(index=0,count=0;index<g_list_length(eventList);index++)
    {
      event ev = g_list_nth_data(eventList, index);
      
      if(ev == NULL)
	{
	  g_warning("eventCountByType: getting nth item(%d) in roster item event list was NULL",index);
	  continue;
	}
      
      if(ev->type == type)
	count++;
    }

  return count;
}

/*
 * member data
 */ 
gboolean eventSetRosterItem(event ev, rosterItem ri)
{
  if(ev == NULL)
    {
      g_warning("eventSetRosterItem: event was NULL");
      return FALSE;
    }

  if(ri == NULL)
    {
      g_warning("eventSetRosterItem: roster item was NULL");
      return FALSE;
    }

  ev->ri = ri;

  return TRUE;
}


gint eventGetID(event ev)
{
  if(ev == NULL)
    {
      g_warning("eventGetID: event was NULL");
      return -1;
    }

  return ev->id;
}

gchar *eventGetJID(event ev)
{
  if(ev == NULL)
    {
      g_warning("eventGetJID: event was NULL");
      return NULL;
    }

  return ev->jid;
}

eventType eventGetType(event ev)
{
  if(ev == NULL)
    {
      g_warning("eventGetType: event was NULL");
      return -1;
    }

  return ev->type;
}

void *eventGetData(event ev)
{
  if(ev == NULL)
    {
      g_warning("eventGetData: event was NULL");
      return NULL;
    }

  return ev->data;
}

time_t eventGetTime(event ev)
{
  if(ev == NULL)
    {
      g_warning("eventGetTime: event was NULL");
      return -1;
    }

  return ev->time;
}

rosterItem eventGetRosterItem(event ev)
{
  if(ev == NULL)
    {
      g_warning("eventGetRosterItem: event was NULL");
      return NULL;
    } 

  return ev->ri;
}

/*
 * debugging 
 */
void eventPrintAll(rosterItem ri)
{
  gint index = 0;

  if(ri == NULL)
    {
      g_warning("eventPrintAll: roster item was NULL");
      return;
    }

  g_message("******* Roster Item Event List Start ********");  

  for(index=0;index<eventCount(ri);index++)
    {
      event ev = NULL;

      ev = g_list_nth_data(eventList,index);
      if(ev == NULL)
	{
	  g_warning("eventPrintAll: roster item event was NULL");
	  return;
	}
      
      eventPrint(ev);
    }

  g_message("******* Roster Item Event List End **********");  

}

void eventPrint(event ev)
{
  if(ev == NULL)
    return;

  g_message("******* Roster Item Event ********");
  g_message("*******");
  g_message("******* id:%d",ev->id);
  g_message("******* jid:'%s'",ev->jid);
  g_message("******* ri:0x%.8x",(gint)ev->ri);
  g_message("*******"); 
  g_message("******* inEvent:%s",ev->inEvent?"yes":"no");
  g_message("*******"); 
  g_message("******* type:%d",ev->type);
  g_message("******* data:0x%.8x",(gint)ev->data);
  g_message("******* proc:0x%.8x",(gint)ev->proc);
  g_message("*******"); 
  g_message("******* Roster Item Event ********");
}

#ifdef __cplusplus
}
#endif
