/* 
 * $RCSfile: gj_gatherdata.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/26 11:09:57 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <glib.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include "gj_gatherdata.h"
#include "gj_parser.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_iq_requests.h"

/* gatherdata item */
void gatherdataDestroy(gatherdata gd);

/* gatherdata list */
gboolean gatherdataListAdd(gatherdata gd);
gboolean gatherdataListDel(gatherdata gd);

/* gatherdata field item */
void gatherdataFieldDestroy(gatherdata_field gdf);

/* gatherdata field list */
gboolean gatherdataFieldListAdd(gatherdata gd, gatherdata_field gdf);
gboolean gatherdataFieldListDel(gatherdata gd, gatherdata_field gdf);

gboolean gatherdataFieldListFree(gatherdata gd);

/* gatherdata field option item */
void gatherdataFieldOptionDestroy(gatherdata_field_option gdfo);

/* gatherdata field option list */
gboolean gatherdataFieldOptionListAdd(gatherdata_field gdf, gatherdata_field_option gdfo);
gboolean gatherdataFieldOptionListDel(gatherdata_field gdf, gatherdata_field_option gdfo);

gboolean gatherdataFieldOptionListFree(gatherdata_field gdf);

/* parsing functions */
void gatherdataResponse(guint eventID);

/* parsing functions */
/* GList *gatherdataListParse(xmlNodePtr parent); */
GList *gatherdataResponseParse(xmlNodePtr parent);

/* call backs */
/* gboolean gatherdataSetResponseProc(gatherdataResponseProc proc); */
/* gatherdataResponseProc gatherdataGetResponseProc(); */

struct t_gatherdata_field_option
{
  gchar *label;
  gchar *value;
};

struct t_gatherdata_field
{
  gatherdataFieldType type;

  gchar *label;
  gchar *var;
  gchar *value;

  gboolean required;

  GList *options;
};

struct t_gatherdata
{
  gchar *id;
  gchar *to;
  gchar *from;

  gatherdataFormType type; 
 
  gchar *instructions;
  GList *fields;

  gatherdataCallback cb; 

  gchar *related_id;
};

GList *gatherdataList = NULL;
gatherdataRequestCallback gatherdataCB = NULL;

gboolean gatherdataInit()
{
  if(gatherdataList != NULL)
    gatherdataTerm();

  /* create mem  - nothing to do, created on g_list_append */
  return TRUE;
}

gboolean gatherdataTerm()
{
  gint index = 0;

  /* list */
  if(gatherdataList == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(gatherdataList);index++)
    {
      gatherdata gd = NULL;
      
      if((gd = (gatherdata) g_list_nth_data(gatherdataList,index)) != NULL)
	gatherdataFree(gd);
    }

  g_list_free(gatherdataList);
  gatherdataList = NULL;

  return TRUE;
}

gboolean gatherdataListAdd(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataListAdd: gd was NULL");
      return FALSE;
    }

  gatherdataList = g_list_append(gatherdataList, gd);

  return TRUE;
}

gboolean gatherdataListDel(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataListDel: gd was NULL");
      return FALSE;
    }

  gatherdataList = g_list_remove(gatherdataList, gd);
  return TRUE;
}

GList *gatherdataListGet()
{
  return gatherdataList;
}

/* 
 * item functions
 */
gatherdata gatherdataNew()
{
  gatherdata gd;

  if((gd = g_new(struct t_gatherdata,1)))
    {
      gd->id = NULL;
      gd->to = NULL;
      gd->from = NULL;
      
      gd->instructions = NULL;
      gd->fields = NULL;
      
      gd->cb = NULL;

      gd->related_id = NULL;

      gatherdataListAdd(gd);
    }
  
  return gd;  
}

void gatherdataFree(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataFree: gd was NULL");
      return;
    }

  /* free mem */
  gatherdataListDel(gd);  

  /* destroy members */
  gatherdataDestroy(gd);

  /* free structure */
  g_free(gd);
}

void gatherdataDestroy(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataDestroy: gd was NULL");
      return;
    }
  
  if(gd->id != NULL)
    {
      g_free(gd->id);
      gd->id = NULL;
    }

  if(gd->to != NULL)
    {
      g_free(gd->to);
      gd->to = NULL;
    }

  if(gd->from != NULL)
    {
      g_free(gd->from);
      gd->from = NULL;
    }

  if(gd->instructions != NULL)
    {
      g_free(gd->instructions);
      gd->instructions = NULL;
    }

  gatherdataFieldListFree(gd);
  gd->fields = NULL;

  gd->cb = NULL;

  if(gd->related_id != NULL)
    {
      g_free(gd->related_id);
      gd->related_id = NULL;
    }
}

/*
 * find 
 */
gatherdata gatherdataFindByID(gchar *id)
{
  gint index = 0;

  if(id == NULL)
    {
      g_warning("gatherdataFindByID: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(gatherdataList);index++)
    {
      gatherdata gd  = g_list_nth_data(gatherdataList,
				       index);

      if(gd == NULL)
	{
	  g_warning("gatherdataFindByID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(gd->id, id) == 0)
	return gd;
    }

  return NULL;
}

gatherdata gatherdataFindByTo(gchar *to)
{
  gint index = 0;

  if(to == NULL)
    {
      g_warning("gatherdataFindByTo: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(gatherdataList);index++)
    {
      gatherdata gd  = g_list_nth_data(gatherdataList,
				       index);

      if(gd == NULL)
	{
	  g_warning("gatherdataFindByTo: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(gd->to, to) == 0)
	return gd;
    }

  return NULL;
}

gatherdata gatherdataFindByFrom(gchar *from)
{
  gint index = 0;

  if(from == NULL)
    {
      g_warning("gatherdataFindByFrom: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(gatherdataList);index++)
    {
      gatherdata gd  = g_list_nth_data(gatherdataList,
				       index);

      if(gd == NULL)
	{
	  g_warning("gatherdataFindByFrom: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(gd->from, from) == 0)
	return gd;
    }

  return NULL;
}

gatherdata gatherdataFindByRelatedID(gchar *id)
{
  gint index = 0;

  if(id == NULL)
    {
      g_warning("gatherdataFindByRelatedID: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(gatherdataList);index++)
    {
      gatherdata gd  = g_list_nth_data(gatherdataList,
				       index);

      if(gd == NULL)
	{
	  g_warning("gatherdataFindByRelatedID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(gd->related_id, id) == 0)
	return gd;
    }

  return NULL;
}

/*
 * member functions  
 */

/* sets */
gboolean gatherdataSetID(gatherdata gd, gchar *id)
{
  if(gd == NULL)
    {
      g_warning("gatherdataSetID: gd was NULL");
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("gatherdataSetID: id was NULL");
      return FALSE;
    }

  if(gd->id != NULL)
    g_free(gd->id);
  
  gd->id = g_strdup(id);

  return TRUE;
}

gboolean gatherdataSetTo(gatherdata gd, gchar *to)
{
  if(gd == NULL)
    {
      g_warning("gatherdataSetTo: gd was NULL");
      return FALSE;
    }

  if(to == NULL)
    {
      g_warning("gatherdataSetTo: to was NULL");
      return FALSE;
    }

  if(gd->to != NULL)
    g_free(gd->to);
  
  gd->to = g_strdup(to);

  return TRUE;
}

gboolean gatherdataSetFrom(gatherdata gd, gchar *from)
{
  if(gd == NULL)
    {
      g_warning("gatherdataSetFrom: gd was NULL");
      return FALSE;
    }

  if(from == NULL)
    {
      g_warning("gatherdataSetFrom: from was NULL");
      return FALSE;
    }

  if(gd->from != NULL)
    g_free(gd->from);
  
  gd->from = g_strdup(from);

  return TRUE;
}

gboolean gatherdataSetType(gatherdata gd, gatherdataFormType type)
{
  if(gd == NULL)
    {
      g_warning("gatherdataSetType: gd was NULL");
      return FALSE;
    }
  
  if(type < gatherdataFormTypeForm || type > gatherdataFormTypeEnd)
    {
      g_warning("gatherdataSetType: type:%d was out of valid range", type);
      return FALSE;
    }

  gd->type = type;
  return TRUE;
}

gboolean gatherdataSetInstructions(gatherdata gd, gchar *instructions)
{
  if(gd == NULL)
    {
      g_warning("gatherdataSetInstructions: gd was NULL");
      return FALSE;
    }

  if(instructions == NULL)
    {
      g_warning("gatherdataSetInstructions: id was NULL");
      return FALSE;
    }

  if(gd->instructions != NULL)
    g_free(gd->instructions);
  
  gd->instructions = g_strdup(instructions);

  return TRUE;
}

gboolean gatherdataSetFields(gatherdata gd, GList *fields)
{
  if(gd == NULL)
    {
      g_warning("gatherdataSetFields: gd was NULL");
      return FALSE;
    }
  
  if(fields == NULL)
    {
      g_warning("gatherdataSetFields: fields list was NULL");
      return FALSE;
    }

  gd->fields = fields;
  return TRUE;
}

gboolean gatherdataSetCallback(gatherdata gd, gatherdataCallback cb)
{
  if(gd == NULL)
    {
      g_warning("gatherdataSetCallback: gd was NULL");
      return FALSE;
    }
  
  if(cb == NULL)
    {
      g_warning("gatherdataSetFields: callback was NULL");
      return FALSE;
    }

  gd->cb = cb;
  return TRUE;
}

gboolean gatherdataSetRelatedID(gatherdata gd, gchar *id)
{
  if(gd == NULL)
    {
      g_warning("gatherdataSetRelatedID: gd was NULL");
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("gatherdataSetRelatedID: id was NULL");
      return FALSE;
    }

  if(gd->related_id != NULL)
    g_free(gd->related_id);
  
  gd->related_id = g_strdup(id);

  return TRUE;
}

/* gets */
gchar *gatherdataGetID(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataGetID: gd was NULL");
      return NULL;
    }

  return gd->id;
}

gchar *gatherdataGetTo(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataGetTo: gd was NULL");
      return NULL;
    }

  return gd->to;
}

gchar *gatherdataGetFrom(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataGetFrom: gd was NULL");
      return NULL;
    }

  return gd->from;
}

gatherdataFormType gatherdataGetType(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataGetType: gd was NULL");
      return gatherdataFormTypeEnd;
    }

  return gd->type;
}

gchar *gatherdataGetInstructions(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataGetInstructions: gd was NULL");
      return NULL;
    }

  return gd->instructions;
}

GList *gatherdataGetFields(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataGetFields: gd was NULL");
      return NULL;
    }

  return gd->fields;
}

gatherdataCallback gatherdataGetCallback(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataGetCallback: gd was NULL");
      return NULL;
    }

  return gd->cb;
}

gchar *gatherdataGetRelatedID(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataGetRelatedID: gd was NULL");
      return NULL;
    }

  return gd->related_id;
}


/* debugging */
void gatherdataPrintAll()
{
  gatherdata gd = NULL;
  gint index = 0;

  g_message("\n");
  g_message("++++++++++++++++++++++ Gatherdata - Print All, Begin. +++++++++++++++++++++");

  for(index=0;index<g_list_length(gatherdataList);index++)
    {
      gd = (gatherdata) g_list_nth_data(gatherdataList,index);
      gatherdataPrint(gd);
    }

  g_message("++++++++++++++++++++++ Gatherdata - Print All, End.   +++++++++++++++++++++");
  
  return;  
}

void gatherdataPrint(gatherdata gd)
{
  if(gd != NULL)
    {
      g_message("+++++++++++++++++++++++++++ Gatherdata +++++++++++++++++++++++++");
      
      g_message("++++");
      g_message("++++ id:'%s'",gd->id);
      g_message("++++ to:'%s'",gd->to);
      g_message("++++ from:'%s'",gd->from);
      g_message("++++");
      g_message("++++ instructions:'%s'",gd->instructions);
      g_message("++++");
      g_message("++++ related id:'%s'",gd->related_id);
      g_message("++++");

      g_message("+++++++++++++++++++++++++++ Gatherdata +++++++++++++++++++++++++");
    }
  else 
    g_warning("gatherdataPrint: failed to print, NULL Gatherdata.");
}

/*
 *
 * GATHERDATA FIELD functions
 *
 */
gboolean gatherdataFieldListAdd(gatherdata gd, gatherdata_field gdf)
{
  if(gd == NULL)
    {
      g_warning("gatherdataFieldListAdd: gd was NULL");
      return FALSE;
    }

  if(gdf == NULL)
    {
      g_warning("gatherdataFieldListAdd: gdf was NULL");
      return FALSE;
    }

  gd->fields = g_list_append(gd->fields, gd);

  return TRUE;
}

gboolean gatherdataFieldListDel(gatherdata gd, gatherdata_field gdf)
{
  if(gd == NULL)
    {
      g_warning("gatherdataFieldListDel: gd was NULL");
      return FALSE;
    }

  if(gdf == NULL)
    {
      g_warning("gatherdataFieldListDel: gdf was NULL");
      return FALSE;
    }

  gd->fields = g_list_remove(gd->fields, gdf);
  return TRUE;
}

gboolean gatherdataFieldListFree(gatherdata gd)
{
  gint index = 0;

  if(gd == NULL)
    {
      g_warning("gatherdataFieldListFree: gd was NULL");
      return FALSE;
    }
  
  if(gatherdataGetFields(gd) == NULL)
    return TRUE;

  for(index=0;index<g_list_length(gatherdataGetFields(gd));index++)
    {
      gatherdata_field gdf = g_list_nth_data(gatherdataGetFields(gd), index);
      
      if(gdf == NULL)
	{
	  g_warning("gatherdataFieldListFree: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      gatherdataFieldFree(gd, gdf);
    }

  return TRUE;
}

GList *gatherdataFieldListGet(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gatherdataFieldListGet: gd was NULL");
      return NULL;
    }

  return gd->fields;
}

/* 
 * item functions
 */
gatherdata_field gatherdataFieldNew(gatherdata gd, gatherdataFieldType type)
{
  gatherdata_field gdf = NULL;

  if(gd == NULL)
    {
      g_warning("gatherdataFieldNew: gd was NULL");
      return NULL;
    }

  if(type < 1 || type > gatherdataFieldTypeEnd)
    {
      g_warning("gatherdataFieldNew: type:%d was out of bounds", type);
      return NULL;
    }

  if((gdf = g_new(struct t_gatherdata_field,1)))
    {
      gdf->type = type;

      gdf->label = NULL;
      gdf->var = NULL;
      gdf->value = NULL;
      
      gdf->required = FALSE;

      gdf->options = NULL;

      gatherdataFieldListAdd(gd, gdf);
    }
  
  return gdf;  
}

void gatherdataFieldFree(gatherdata gd, gatherdata_field gdf)
{
  if(gd == NULL)
    {
      g_warning("gatherdataFieldFree: gd was NULL");
      return;
    }  

  if(gdf == NULL)
    {
      g_warning("gatherdataFieldFree: gdf was NULL");
      return;
    }

  /* free mem */
  gatherdataFieldListDel(gd, gdf);  

  /* destroy members */
  gatherdataFieldDestroy(gdf);

  /* free structure */
  g_free(gdf);
}

void gatherdataFieldDestroy(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldDestroy: gdf was NULL");
      return;
    }

  gdf->type = gatherdataFieldTypeEnd;
  
  if(gdf->label != NULL)
    {
      g_free(gdf->label);
      gdf->label = NULL;
    }

  if(gdf->var != NULL)
    {
      g_free(gdf->var);
      gdf->var = NULL;
    }

  if(gdf->value != NULL)
    {
      g_free(gdf->value);
      gdf->value = NULL;
    }

  gdf->required = FALSE;

  gatherdataFieldOptionListFree(gdf);
  gdf->options = NULL;  
}

/*
 * member functions  
 */

/* sets */
gboolean gatherdataFieldSetType(gatherdata_field gdf, gatherdataFieldType type)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldSetType: gdf was NULL");
      return FALSE;
    }

  if(type < 1 || type > gatherdataFieldTypeEnd)
    {
      g_warning("gatherdataFieldSetType: type:%d was out of bounds", type);
      return FALSE;
    }

  gdf->type = type;

  return TRUE;
}

gboolean gatherdataFieldSetLabel(gatherdata_field gdf, gchar *label)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldSetLabel: gdf was NULL");
      return FALSE;
    }

  if(label == NULL)
    {
      g_warning("gatherdataFieldSetLabel: label was NULL");
      return FALSE;
    }

  if(gdf->label != NULL)
    g_free(gdf->label);
  
  gdf->label = g_strdup(label);

  return TRUE;
}

gboolean gatherdataFieldSetVar(gatherdata_field gdf, gchar *var)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldSetVar: gdf was NULL");
      return FALSE;
    }

  if(var == NULL)
    {
      g_warning("gatherdataFieldSetVar: var was NULL");
      return FALSE;
    }

  if(gdf->var != NULL)
    g_free(gdf->var);
  
  gdf->var = g_strdup(var);

  return TRUE;
}

gboolean gatherdataFieldSetValue(gatherdata_field gdf, gchar *value)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldSetValue: gdf was NULL");
      return FALSE;
    }

  if(value == NULL)
    {
      g_warning("gatherdataFieldSetValue: value was NULL");
      return FALSE;
    }

  if(gdf->value != NULL)
    g_free(gdf->value);
  
  gdf->value = g_strdup(value);

  return TRUE;
}

gboolean gatherdataFieldSetRequired(gatherdata_field gdf, gboolean required)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldSetRequired: gdf was NULL");
      return FALSE;
    }

  gdf->required = required;

  return TRUE;
}

gboolean gatherdataFieldSetOptions(gatherdata_field gdf, GList *options)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldSetOptions: gdf was NULL");
      return FALSE;
    }

  /* warning ... what happens to previous list? */
  gdf->options = options;

  return TRUE;
}

/* gets */
gatherdataFieldType gatherdataFieldGetType(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldGetType: gdf was NULL");
      return gatherdataFieldTypeEnd;
    }

  return gdf->type;
}


gchar *gatherdataFieldGetLabel(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldGetLabel: gdf was NULL");
      return NULL;
    }

  return gdf->label;
}


gchar *gatherdataFieldGetVar(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldGetVar: gdf was NULL");
      return NULL;
    }

  return gdf->var;
}


gchar *gatherdataFieldGetValue(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldGetValue: gdf was NULL");
      return NULL;
    }

  return gdf->value;
}


gboolean gatherdataFieldGetRequired(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldGetRequired: gdf was NULL");
      return FALSE;
    }

  return gdf->required;
}


GList *gatherdataFieldGetOptions(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldGetOptions: gdf was NULL");
      return NULL;
    }

  return gdf->options;
}

/*
 *
 * GATHERDATA FIELD OPTION functions
 *
 */
gboolean gatherdataFieldOptionListAdd(gatherdata_field gdf, gatherdata_field_option gdfo)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldOptionListAdd: gdf was NULL");
      return FALSE;
    }

  if(gdfo == NULL)
    {
      g_warning("gatherdataFieldOptionListAdd: gdfo was NULL");
      return FALSE;
    }

  gdf->options = g_list_append(gdf->options, gdfo);

  return TRUE;
}

gboolean gatherdataFieldOptionListDel(gatherdata_field gdf, gatherdata_field_option gdfo)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldOptionListDel: gdf was NULL");
      return FALSE;
    }

  if(gdfo == NULL)
    {
      g_warning("gatherdataFieldOptionListDel: gdfo was NULL");
      return FALSE;
    }

  gdf->options = g_list_remove(gdf->options, gdfo);
  return TRUE;
}

gboolean gatherdataFieldOptionListFree(gatherdata_field gdf)
{
  gint index = 0;

  if(gdf == NULL)
    {
      g_warning("gatherdataFieldOptionListFree: gdf was NULL");
      return FALSE;
    }
  
  if(gatherdataFieldGetOptions(gdf) == NULL)
    return TRUE;

  for(index=0;index<g_list_length(gatherdataFieldGetOptions(gdf));index++)
    {
      gatherdata_field_option gdfo = g_list_nth_data(gatherdataFieldGetOptions(gdf), index);
      
      if(gdfo == NULL)
	{
	  g_warning("gatherdataFieldOptionListFree: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      gatherdataFieldOptionFree(gdf, gdfo);
    }

  return TRUE;
}

GList *gatherdataFieldOptionListGet(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldOptionListGet: gd was NULL");
      return NULL;
    }

  return gdf->options;
}

/* 
 * item functions
 */
gatherdata_field_option gatherdataFieldOptionNew(gatherdata_field gdf, gchar *label)
{
  gatherdata_field_option gdfo = NULL;

  if(gdf == NULL)
    {
      g_warning("gatherdataFieldOptionNew: gdf was NULL");
      return NULL;
    }

  if(label == NULL)
    {
      g_warning("gatherdataFieldOptionNew: label was NULL");
      return NULL;
    }

  if((gdfo = g_new(struct t_gatherdata_field_option,1)))
    {
      gdf->label = g_strdup(label);
      gdf->value = NULL;

      gatherdataFieldOptionListAdd(gdf, gdfo);
    }
  
  return gdfo;  
}

void gatherdataFieldOptionFree(gatherdata_field gdf, gatherdata_field_option gdfo)
{
  if(gdf == NULL)
    {
      g_warning("gatherdataFieldOptionFree: gdf was NULL");
      return;
    }  

  if(gdfo == NULL)
    {
      g_warning("gatherdataFieldOptionFree: gdfo was NULL");
      return;
    }

  /* free mem */
  gatherdataFieldOptionListDel(gdf, gdfo);  

  /* destroy members */
  gatherdataFieldOptionDestroy(gdfo);

  /* free structure */
  g_free(gdfo);
}

void gatherdataFieldOptionDestroy(gatherdata_field_option gdfo)
{
  if(gdfo == NULL)
    {
      g_warning("gatherdataFieldOptionDestroy: gdfo was NULL");
      return;
    }

  if(gdfo->label != NULL)
    {
      g_free(gdfo->label);
      gdfo->label = NULL;
    }

  if(gdfo->value != NULL)
    {
      g_free(gdfo->value);
      gdfo->value = NULL;
    }
}

/*
 * member functions  
 */

/* sets */
gboolean gatherdataFieldOptionSetLabel(gatherdata_field_option gdfo, gchar *label)
{
  if(gdfo == NULL)
    {
      g_warning("gatherdataFieldOptionSetLabel: gdfo was NULL");
      return FALSE;
    }

  if(label == NULL)
    {
      g_warning("gatherdataFieldOptionSetLabel: label was NULL");
      return FALSE;
    }

  if(gdfo->label != NULL)
    g_free(gdfo->label);
  
  gdfo->label = g_strdup(label);
  return TRUE;
}

gboolean gatherdataFieldOptionSetValue(gatherdata_field_option gdfo, gchar *value)
{
  if(gdfo == NULL)
    {
      g_warning("gatherdataFieldOptionSetValue: gdfo was NULL");
      return FALSE;
    }

  if(value == NULL)
    {
      g_warning("gatherdataFieldOptionSetValue: value was NULL");
      return FALSE;
    }

  if(gdfo->value != NULL)
    g_free(gdfo->value);
  
  gdfo->value = g_strdup(value);
  return TRUE;
}

/* gets */
gchar *gatherdataFieldOptionGetLabel(gatherdata_field_option gdfo)
{
  if(gdfo == NULL)
    {
      g_warning("gatherdataFieldOptionGetLabel: gdfo was NULL");
      return NULL;
    }

  return gdfo->label;
}

gchar *gatherdataFieldOptionGetValue(gatherdata_field_option gdfo)
{
  if(gdfo == NULL)
    {
      g_warning("gatherdataFieldOptionGetValue: gdfo was NULL");
      return NULL;
    }

  return gdfo->value;
}





/*
 *
 * SERVICES
 *
 */
gboolean gatherdataRequest(gchar *server, gatherdataRequestCallback cb)
{
  if(server == NULL)
    {
      g_warning("gatherdataRequest: server was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("gatherdataRequest: cb was NULL");
      return FALSE;
    }

  gatherdataCB = cb;

  /* do request */
  infoqueryRequestRegister(server, gatherdataResponse);

  return TRUE;
}

void gatherdataResponse(guint eventID)
{
  infoquery iq = NULL;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  GList *list = NULL;

  if(gatherdataCB == NULL)
    {
      g_warning("gatherdataResponse: callback was NULL");
      return;
    }

  if((iq = infoqueryFindByEventID(eventID)) == NULL)
    {
      g_warning("gatherdataResponse: could not get infoquery event id:%d", eventID);
      return;
    }

  node = infoqueryGetXML(iq);
  type = parserFindByAttrAndReturn(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      gchar *detail = NULL;

      xmlChar *errorCode = NULL;
      xmlChar *errorMessage = NULL;

      if((node = parserFindByNode(node, (guchar*)"error")) == NULL)
	{
	  g_warning("gatherdataResponse: failed to find node 'error'");
	  infoqueryFreeByEventID(eventID);
	  return;
	}

      if((errorCode = parserFindByAttrAndReturn(node, (guchar*)"code")) != NULL)
	{
	  detail = g_strdup_printf("Error:%s", errorCode);
	}

      if((errorMessage = parserFindByNodeAndReturn(node, (guchar*)"error")) != NULL) 
	{
	  if(detail != NULL)
	    detail = g_strdup_printf("%s - %s", detail, errorMessage);
	}

       /* clean up */
      g_free(detail);
      infoqueryFreeByEventID(eventID);

      return;
    }

  if(gatherdataList != NULL)
    {
      /* free list */

    }
  
  /* parse list */
  if((list = gatherdataResponseParse(node)) == NULL)
    {
      g_warning("gatherdataResponse: did not complete the parse from xml -> list");
      return;
    }

  /* set */
  gatherdataList = list;

  /* return list */
  (gatherdataCB)(gatherdataList);

  /* clean up */
  infoqueryFreeByEventID(eventID);
}

GList *gatherdataResponseParse(xmlNodePtr parent)
{
  if(parent == NULL)
    {
      g_warning("gatherdataResponseParse: failed to get gatherdata because query (xml) was NULL");
      return NULL;
    }

  return NULL;
}

#ifdef __cplusplus
}
#endif
