/* 
 * $RCSfile: gj_gatherdata.h,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/22 10:54:29 $
 * 
 * $Revision: 1.2 $ 
 *
 */

#ifndef __gj_gatherdata_h
#define __gj_gatherdata_h

#include <glib.h>

typedef struct t_gatherdata *gatherdata;
typedef struct t_gatherdata_field *gatherdata_field;
typedef struct t_gatherdata_field_option *gatherdata_field_option;

typedef void (*gatherdataCallback)(gatherdata gd);
typedef void (*gatherdataRequestCallback)(GList *list);

typedef enum gatherdataFormType {
  gatherdataFormTypeForm = 1,        /* This packet contains a form to fill out. Display it to the user */    
  gatherdataFormTypeSubmit,          /* The form is filled out, and this is the data that is being returned from the form. */
  gatherdataFormTypeCancel,          /* The form was cancelled. Tell the asker that piece of information. */
  gatherdataFormTypeResult,          /* Data results being returned from a search, or some other query. */

  gatherdataFormTypeEnd

} gatherdataFormType;

typedef enum gatherdataFieldType {
  
  gatherdataFieldTypeTextSingle = 1, /* single line or word of text */
  gatherdataFieldTypeTextPrivate,    /* instead of showing the user what they typed, you show ***** to protect it */
  gatherdataFieldTypeTextMulti,      /* multiple lines of text entry */
  gatherdataFieldTypeListSingle,     /* given a list of choices, pick one */
  gatherdataFieldTypeListMulti,      /* given a list of choices, pick one or more */
  gatherdataFieldTypeBoolean,        /* 0 or 1, true or false, yes or no. Default value is 0 */
  gatherdataFieldTypeFixed,          /* fixed for putting in text to show sections, 
					or just advertise your web site in the middle of the form */
  gatherdataFieldTypeHidden,         /* is not given to the user at all, but returned with the questionnaire
					(e.g., a unique number at the top of a sheet of paper to identify the form) */
  gatherdataFieldTypeJIDSingle,      /* Jabber ID - choosing a JID from your roster, and entering one based on the rules for a JID. */
  gatherdataFieldTypeJIDMulti,       /* multiple entries for JIDs */

  gatherdataFieldTypeEnd
  
} gatherdataFieldType;

/* list functions */
gboolean gatherdataInit();
gboolean gatherdataTerm();

/* list functions */
GList *gatherdataListGet();

/* item functions */
gatherdata gatherdataNew();
void gatherdataFree(gatherdata gd);

/* find */
gatherdata gatherdataFindByID(gchar *id);
gatherdata gatherdataFindByTo(gchar *to);
gatherdata gatherdataFindByFrom(gchar *from);
gatherdata gatherdataFindByRelatedID(gchar *id);

/* sets */
gboolean gatherdataSetID(gatherdata gd, gchar *id);
gboolean gatherdataSetTo(gatherdata gd, gchar *to);
gboolean gatherdataSetFrom(gatherdata gd, gchar *to);
gboolean gatherdataSetType(gatherdata gd, gatherdataFormType type);
gboolean gatherdataSetInstructions(gatherdata gd, gchar *instructions);
gboolean gatherdataSetFields(gatherdata gd, GList *fields);
gboolean gatherdataSetCallback(gatherdata gd, gatherdataCallback cb);
gboolean gatherdataSetRelatedID(gatherdata gd, gchar *id);

/* gets */
gchar *gatherdataGetID(gatherdata gd);
gchar *gatherdataGetTo(gatherdata gd);
gchar *gatherdataGetFrom(gatherdata gd);
gatherdataFormType gatherdataGetType(gatherdata gd);
gchar *gatherdataGetInstructions(gatherdata gd);
GList *gatherdataGetFields(gatherdata gd);
gatherdataCallback gatherdataGetCallback(gatherdata gd);
gchar *gatherdataGetRelatedID(gatherdata gd);

/* callbacks */
/* gboolean gatherdataListRequest(gchar *server, gatherdataResponseProc proc); */

/* debugging */
void gatherdataPrint(gatherdata gd);
void gatherdataPrintAll();

/* 
 * gatherdata field
 */

/* list functions */
GList *gatherdataFieldListGet(gatherdata gd);

/* item functions */
gatherdata_field gatherdataFieldNew(gatherdata gd, gatherdataFieldType type);
void gatherdataFieldFree(gatherdata gd, gatherdata_field gdf);

/* sets */
gboolean gatherdataFieldSetType(gatherdata_field gdf, gatherdataFieldType type);
gboolean gatherdataFieldSetLabel(gatherdata_field gdf, gchar *label);
gboolean gatherdataFieldSetVar(gatherdata_field gdf, gchar *var);
gboolean gatherdataFieldSetValue(gatherdata_field gdf, gchar *value);
gboolean gatherdataFieldSetRequired(gatherdata_field gdf, gboolean required);
gboolean gatherdataFieldSetOptions(gatherdata_field gdf, GList *options);

/* gets */
gatherdataFieldType gatherdataFieldGetType(gatherdata_field gdf);
gchar *gatherdataFieldGetLabel(gatherdata_field gdf);
gchar *gatherdataFieldGetVar(gatherdata_field gdf);
gchar *gatherdataFieldGetValue(gatherdata_field gdf);
gboolean gatherdataFieldGetRequired(gatherdata_field gdf);
GList *gatherdataFieldGetOptions(gatherdata_field gdf);

/* 
 * gatherdata field option
 */

/* list functions */
GList *gatherdataFieldOptionListGet(gatherdata_field gdf);

/* item functions */
gatherdata_field_option gatherdataFieldOptionNew(gatherdata_field gdf, gchar *label);
void gatherdataFieldOptionFree(gatherdata_field gdf, gatherdata_field_option gdfo);

/* sets */
gboolean gatherdataFieldOptionSetLabel(gatherdata_field_option gdfo, gchar *label);
gboolean gatherdataFieldOptionSetValue(gatherdata_field_option gdfo, gchar *value);

/* gets */
gchar *gatherdataFieldOptionGetLabel(gatherdata_field_option gdfo);
gchar *gatherdataFieldOptionGetValue(gatherdata_field_option gdfo);

/*
 * parsing functions  
 */
gboolean gatherdataRequest(gchar *server, gatherdataRequestCallback cb);

#endif

