/* 
 * $RCSfile: gj_gtk_chat.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/24 08:21:54 $
 * 
 * $Revision: 1.12 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glade/glade.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_GNOME
#include <gnome.h>
#endif

#include "common.h"

#include "gj_main.h"
#include "gj_roster.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_translate.h"
#include "gj_history.h"
#include "gj_support.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_chat.h"
#include "gj_gtk_connection_settings.h"
#include "gj_gtk_my_preferences.h"
#include "gj_gtk_user_information.h"
#include "gj_gtk_support.h"
#include "gj_gtk_add_contact.h"

struct t_controls
{
  GtkWidget *window;

  GtkWidget *textview_send;
  GtkWidget *textview_receive;

  GtkWidget *image_composing;
  GtkWidget *image_status;
  GtkWidget *label_status;

  GtkWidget *button_send;
  GtkWidget *button_information;
  GtkWidget *button_add;
  GtkWidget *button_close;
  GtkWidget *button_emote;
  GtkWidget *button_history;

};

struct t_chatWindow
{
  struct t_controls controls;

  gchar *jid;
  gchar *nickname;
  gchar *groups;

  gchar *thread;

  gboolean reverse_text_direction;

  gint last_recv_xevent_type;
  gchar *last_recv_xevent_id;
  
  gint last_sent_xevent_type;
  gchar *last_sent_xevent_id;
};

GList *chatWindowList;

gboolean gj_gtk_cw_load(chatWindow cw);
gboolean gj_gtk_cw_unload_by_widget(GtkWidget *widget);

void gj_gtk_cw_raise();

/* list functions */
gboolean gj_gtk_cw_list_add(chatWindow cw);
gboolean gj_gtk_cw_list_del(chatWindow cw);
GList *gj_gtk_cw_list_get();

/* window functions */
chatWindow gj_gtk_cw_new(gchar *jid);
gboolean gj_gtk_cw_free(chatWindow cw);
void gj_gtk_cw_destroy(chatWindow cw);

/* utilities */
gboolean gj_gtk_cw_setup(chatWindow cw);
gboolean gj_gtk_cw_send_set(chatWindow cw, gchar *text);
gboolean gj_gtk_cw_receive_set(chatWindow cw, message msg);

void gj_gtk_cw_emote_icon_create_popup_menu(GtkWidget *widget);
void gj_gtk_cw_emote_icon_get_as_text(gchar *emote_icon, gpointer user_data);
GdkPixbuf *gj_gtk_cw_emote_icon_get_as_pixbuf(msgEmote icon);

/* utils */
gboolean gj_gtk_cw_check_for_url(GtkTextBuffer *buffer, gchar *text, guint from_pos);

/* gui callbacks */
void on_textbuffer_cw_receive_changed(GtkTextBuffer *textbuffer, gpointer user_data);
gboolean on_textview_cw_receive_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data);
gboolean on_textview_cw_receive_find_space(gunichar ch, gpointer user_data);
gboolean on_textview_cw_receive_find_alpha_numeric(gunichar ch, gpointer user_data);
gboolean on_textview_cw_receive_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
gboolean on_textview_cw_send_key_release_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
gboolean on_textview_cw_send_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
gboolean on_textview_cw_send_focus_in_event(GtkWidget *widget, GdkEventFocus *event, gpointer user_data);
gboolean on_textview_cw_send_focus_out_event(GtkWidget *widget, GdkEventFocus *event, gpointer user_data);

void on_button_cw_close_clicked(GtkButton *button, gpointer user_data);
void on_button_cw_send_clicked(GtkButton *button, gpointer user_data);
void on_button_cw_information_clicked(GtkButton *button, gpointer user_data);
void on_button_cw_history_clicked(GtkButton *button, gpointer user_data);
void on_button_cw_add_clicked(GtkButton *button, gpointer user_data);
void on_button_cw_emote_clicked(GtkButton *button, gpointer user_data);

void on_mnu_emoteicon_big_grin_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_emoteicon_confused_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_emoteicon_cool_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_emoteicon_eek_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_emoteicon_frown_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_emoteicon_mad_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_emoteicon_red_face_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_emoteicon_roll_eyes_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_emoteicon_smile_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_emoteicon_tongue_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_emoteicon_wink_activate(GtkMenuItem *menuitem, gpointer user_data);

gboolean on_chat_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * code
 */

/* list handling */
gboolean gj_gtk_cw_list_add(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_list_add: cw was NULL");
      return FALSE;
    }

  chatWindowList = g_list_append(chatWindowList,cw);
  return TRUE;
}

gboolean gj_gtk_cw_list_del(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_list_del: cw was NULL");
      return FALSE;
    }

  chatWindowList = g_list_remove(chatWindowList,cw);
  return TRUE;
}

GList *gj_gtk_cw_list_get()
{
  return chatWindowList;
}

chatWindow gj_gtk_cw_new(gchar *jid)
{
  chatWindow cw = NULL;
  rosterItem ri = NULL;

  if(jid == NULL)
    {
      g_warning("gj_gtk_cw_new: remote JID was NULL");
      return NULL;
    }

  if((ri = rosterItemFindByJID(jid)) == NULL)
    {
      g_message("gj_gtk_cw_new: no chat window based on JID:'%s', trying without resource...",jid);

      if((ri = rosterItemFindByJIDWithoutResource(jid)) == NULL)
	g_warning("gj_gtk_cw_new: no chat window based on JID:'%s', creating new window...",jid);
    }

  if((cw = (chatWindow) g_new(struct t_chatWindow,1)) != NULL)
    {
      cw->jid = g_strdup(jid);
      cw->nickname = g_strdup(rosterItemGetName(ri));
      cw->groups = rosterItemGetGroupsAsString(ri);

      cw->thread = NULL;

      cw->last_recv_xevent_type = 0;
      cw->last_recv_xevent_id = NULL;
      
      cw->last_sent_xevent_type = 0;
      cw->last_sent_xevent_id = NULL;

      gj_gtk_cw_list_add(cw);
    }

  return cw;
}

gboolean gj_gtk_cw_free(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_free: cw was NULL");
      return FALSE;
    }

  /* free from list */
  gj_gtk_cw_list_del(cw);
  
  /* destroy members */
  gj_gtk_cw_destroy(cw);

  /* free struct */
  g_free(cw);
  
  return TRUE;
}

void gj_gtk_cw_destroy(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_destroy: cw was NULL");
      return;
    }

  if(cw->jid != NULL)
    {
      g_free(cw->jid);
      cw->jid = NULL;
    }

  if(cw->nickname != NULL)
    {
      g_free(cw->nickname);
      cw->nickname = NULL;
    }

  if(cw->groups != NULL)
    {
      g_free(cw->groups);
      cw->groups = NULL;
    }

  if(cw->thread != NULL)
    {
      g_free(cw->thread);
      cw->thread = NULL;
    }

  cw->last_recv_xevent_type = 0;

  if(cw->last_recv_xevent_id != NULL)
  {
      g_free(cw->last_recv_xevent_id);
      cw->last_recv_xevent_id = NULL;
  }

  cw->last_sent_xevent_type = 0;

  if(cw->last_sent_xevent_id != NULL)
  {
      g_free(cw->last_sent_xevent_id);
      cw->last_sent_xevent_id = NULL;
  }
}

chatWindow gj_gtk_cw_find_by_jid(gchar *jid)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("gj_gtk_cw_find_by_jid: JID was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(chatWindowList);index++)
    {
      chatWindow cw = g_list_nth_data(chatWindowList, index);

      if(cw == NULL)
	{
	  g_warning("gj_gtk_cw_find_by_jid: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(strcmp(gj_gtk_cw_get_jid(cw),jid) == 0)
	  return cw;
    }

  return NULL;  
}

chatWindow gj_gtk_cw_find_by_widget(GtkWidget *widget)
{
  gint index = 0;

  GtkWidget *toplevel = NULL;

  if(widget == NULL)
    {
      g_warning("gj_gtk_cw_find_by_widget: widget was NULL");
      return NULL;
    }

  /* get top level widget */
  toplevel = gtk_widget_get_toplevel(widget);

  if(toplevel == NULL)
    {
      g_warning("gj_gtk_cw_find_by_widget: top level widget was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(chatWindowList);index++)
    {
      chatWindow cw = g_list_nth_data(chatWindowList, index);

      if(cw == NULL)
	{
	  g_warning("gj_gtk_cw_find_by_widget: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(cw->controls.window == toplevel)
	  return cw;
    }

  return NULL;  
}

chatWindow gj_gtk_cw_find_by_last_recv_xevent_id(gchar *message_id)
{
  gint index = 0;

  if(message_id == NULL)
    {
      g_warning("gj_gtk_cw_find_by_last_recv_xevent_id: message id was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(chatWindowList);index++)
    {
      chatWindow cw = g_list_nth_data(chatWindowList, index);

      if(cw == NULL)
	{
	  g_warning("gj_gtk_cw_find_by_last_recv_xevent_id: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(gj_gtk_cw_get_last_recv_xevent_id(cw) != NULL && 
	 strcmp(gj_gtk_cw_get_last_recv_xevent_id(cw), message_id) == 0)
	  return cw;
    }

  return NULL;  
}

chatWindow gj_gtk_cw_find_by_last_sent_xevent_id(gchar *message_id)
{
  gint index = 0;

  if(message_id == NULL)
    {
      g_warning("gj_gtk_cw_find_by_last_sent_xevent_id: message id was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(chatWindowList);index++)
    {
      chatWindow cw = g_list_nth_data(chatWindowList, index);

      if(cw == NULL)
	{
	  g_warning("gj_gtk_cw_find_by_last_sent_xevent_id: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(gj_gtk_cw_get_last_sent_xevent_id(cw) != NULL && 
	 strcmp(gj_gtk_cw_get_last_sent_xevent_id(cw), message_id) == 0)
	  return cw;
    }

  return NULL;  
}

chatWindow gj_gtk_cw_find_by_thread(gchar *thread)
{
  gint index = 0;

  if(thread == NULL)
    {
      g_warning("gj_gtk_cw_find_by_thread: thread was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(chatWindowList);index++)
    {
      chatWindow cw = g_list_nth_data(chatWindowList, index);

      if(cw == NULL)
	{
	  g_warning("gj_gtk_cw_find_by_thread: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(strcmp(gj_gtk_cw_get_thread(cw), thread) == 0)
	  return cw;
    }

  return NULL;  
}

/* 
 * member functions 
 */

/* gets */ 
gchar *gj_gtk_cw_get_jid(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_get_jid: chatWindow was NULL");
      return NULL;
    }

  return cw->jid;
}

gchar *gj_gtk_cw_get_nickname(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_get_nickname: chatWindow was NULL");
      return NULL;
    }

  return cw->nickname;
}

gchar *gj_gtk_cw_get_groups(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_get_groups: chatWindow was NULL");
      return NULL;
    }

  return cw->groups;
}

gchar *gj_gtk_cw_get_thread(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_get_thread: chatWindow was NULL");
      return NULL;
    }

  return cw->thread;
}

gboolean gj_gtk_cw_get_reverse_text_direction(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_get_reverse_text_direction: chatWindow was NULL");
      return FALSE;
    }

  return cw->reverse_text_direction;
}

gint gj_gtk_cw_get_last_recv_xevent_type(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_get_last_recv_xevent_type: chatWindow was NULL");
      return 0;
    }

  return cw->last_recv_xevent_type;
}

gchar *gj_gtk_cw_get_last_recv_xevent_id(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_get_last_recv_xevent_id: chatWindow was NULL");
      return NULL;
    }

  return cw->last_recv_xevent_id;
}

gint gj_gtk_cw_get_last_sent_xevent_type(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_get_last_sent_xevent_type: chatWindow was NULL");
      return 0;
    }

  return cw->last_sent_xevent_type;
}

gchar *gj_gtk_cw_get_last_sent_xevent_id(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_get_last_sent_xevent_id: chatWindow was NULL");
      return NULL;
    }

  return cw->last_sent_xevent_id;
}


/* sets */
gboolean gj_gtk_cw_set_jid(chatWindow cw, gchar *jid)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_set_jid: chatWindow was NULL");
      return FALSE;
    }  
  
  if(jid == NULL)
    {
      g_warning("gj_gtk_cw_set_jid: jid was NULL");
      return FALSE;
    }    

  if(cw->jid != NULL)
    {
      g_free(cw->jid);
      cw->jid = NULL;
    }

  cw->jid = g_strdup(jid);
  return TRUE;
}

gboolean gj_gtk_cw_set_nickname(chatWindow cw, gchar *nickname)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_set_nickname: chatWindow was NULL");
      return FALSE;
    }  
  
  if(nickname == NULL)
    {
      g_warning("gj_gtk_cw_set_nickname: nickname was NULL");
      return FALSE;
    }    

  if(cw->nickname != NULL)
    {
      g_free(cw->nickname);
      cw->nickname = NULL;
    }

  cw->nickname = g_strdup(nickname);
  return TRUE;
}

gboolean gj_gtk_cw_set_groups(chatWindow cw, gchar *groups)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_set_groups: chatWindow was NULL");
      return FALSE;
    }  
  
  if(groups == NULL)
    {
      g_warning("gj_gtk_cw_set_groups: groups was NULL");
      return FALSE;
    }    

  if(cw->groups != NULL)
    {
      g_free(cw->groups);
      cw->groups = NULL;
    }

  cw->groups = g_strdup(groups);
  return TRUE;
}

gboolean gj_gtk_cw_set_thread(chatWindow cw, gchar *thread)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_set_thread: chatWindow was NULL");
      return FALSE;
    }  
  
  if(thread == NULL)
    {
      g_warning("gj_gtk_cw_set_thread: thread was NULL");
      return FALSE;
    }    

  if(cw->thread != NULL)
    {
      g_free(cw->thread);
      cw->thread = NULL;
    }

  cw->thread = g_strdup(thread);
  return TRUE;
}

gboolean gj_gtk_cw_set_reverse_text_direction(chatWindow cw, gboolean reverse_text_direction)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_set_reverse_text_direction: chatWindow was NULL");
      return FALSE;
    }  
  
  cw->reverse_text_direction = reverse_text_direction;
  return TRUE;
}

gboolean gj_gtk_cw_set_last_recv_xevent_type(chatWindow cw, gint type)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_set_last_recv_xevent_type: chatWindow was NULL");
      return FALSE;
    }  
  
  cw->last_recv_xevent_type = type;
  return TRUE;
}

gboolean gj_gtk_cw_set_last_recv_xevent_id(chatWindow cw, gchar *id)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_set_last_recv_xevent_id: chatWindow was NULL");
      return FALSE;
    }  
  
  if(id == NULL)
    {
      g_warning("gj_gtk_cw_set_last_recv_xevent_id: id was NULL");
      return FALSE;
    }    

  if(cw->last_recv_xevent_id != NULL)
    {
      g_free(cw->last_recv_xevent_id);
      cw->last_recv_xevent_id = NULL;
    }

  g_message("gj_gtk_cw_set_last_recv_xevent_id: cw:%#x, id:'%s'", cw, id);
  cw->last_recv_xevent_id = g_strdup(id);
  return TRUE;
}

gboolean gj_gtk_cw_set_last_sent_xevent_type(chatWindow cw, gint type)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_set_last_sent_xevent_type: chatWindow was NULL");
      return FALSE;
    }  
  
  cw->last_sent_xevent_type = type;
  return TRUE;
}

gboolean gj_gtk_cw_set_last_sent_xevent_id(chatWindow cw, gchar *id)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_set_last_sent_xevent_id: chatWindow was NULL");
      return FALSE;
    }  
  
  if(id == NULL)
    {
      g_warning("gj_gtk_cw_set_last_sent_xevent_id: id was NULL");
      return FALSE;
    }    

  if(cw->last_sent_xevent_id != NULL)
    {
      g_free(cw->last_sent_xevent_id);
      cw->last_sent_xevent_id = NULL;
    }

  g_message("gj_gtk_cw_set_last_sent_xevent_id: cw:%#x id:'%s'", cw, id);
  cw->last_sent_xevent_id = g_strdup(id);
  return TRUE;
}

void gj_gtk_cw_print_all()
{
  gint index = 0;

  /* loop through all contacts on roster */
  for(index=0;index<g_list_length(chatWindowList);index++)
    {
      chatWindow cw = g_list_nth_data(chatWindowList, index);

      if(cw == NULL)
	{
	  g_warning("gj_gtk_cw_print_all: getting nth item(%d) in roster list was NULL",index);
	  continue;
	}

      gj_gtk_cw_print(cw);
    }  
}

void gj_gtk_cw_print(chatWindow cw)
{
  if(cw == NULL)
    return;

  g_message("++++++++++++++++++++++++++++++++++++++++++++++++");
  g_message("++++");
  g_message("++++ jid:'%s'", cw->jid);
  g_message("++++ nickname:'%s'", cw->nickname);
  g_message("++++ groups:'%s'", cw->groups);
  g_message("++++ thread:'%s'", cw->thread);
  g_message("++++");
  g_message("++++ reverse_text_direction:%s", cw->reverse_text_direction?"true":"false");
  g_message("++++ last_recv_xevent_type:%d", cw->last_recv_xevent_type);
  g_message("++++ last_recv_xevent_id:%s", cw->last_recv_xevent_id);
  g_message("++++ last_sent_xevent_type:%d", cw->last_sent_xevent_type);
  g_message("++++ last_sent_xevent_id:%s", cw->last_sent_xevent_id);
  g_message("++++");
  g_message("++++++++++++++++++++++++++++++++++++++++++++++++");
}




/*
 * window functions 
 */
gboolean gj_gtk_cw_show(chatWindow cw, gboolean show)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_show: cw was NULL");
      return FALSE;
    }
  
  if(show == TRUE)
      gtk_widget_show(cw->controls.window);
  else
    gtk_widget_hide(cw->controls.window);

  return TRUE;
}

void gj_gtk_cw_raise(chatWindow cw)
{
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_raise: cw was NULL");
      return;
    }

  if(cw->controls.window == NULL || cw->controls.window->window == NULL)
    {
      g_warning("gj_gtk_cw_raise: window was NULL or GDK window was NULL");
      return;
    }

  gdk_window_raise(GDK_WINDOW(cw->controls.window->window));
}

gboolean gj_gtk_cw_load(chatWindow cw)
{
  GladeXML *xml = NULL;
  GdkPixbuf *pb = NULL;

  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_load: cw was NULL");
      return FALSE;
    }

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "chat",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_cw_load: couldn't open glade file at root 'chat'");
      return FALSE;
    }
  
  cw->controls.window = glade_xml_get_widget(xml,"chat");

  cw->controls.textview_send = glade_xml_get_widget(xml, "textview_cw_send");
  cw->controls.textview_receive = glade_xml_get_widget(xml, "textview_cw_receive");

  cw->controls.image_composing = glade_xml_get_widget(xml, "image_cw_xevent_composing");
  cw->controls.image_status = glade_xml_get_widget(xml, "image_cw_status");
  cw->controls.label_status = glade_xml_get_widget(xml, "label_cw_status");

  cw->controls.button_send = glade_xml_get_widget(xml, "button_cw_send");
  cw->controls.button_information = glade_xml_get_widget(xml, "button_cw_information");
  cw->controls.button_history = glade_xml_get_widget(xml, "button_cw_history");
  cw->controls.button_close = glade_xml_get_widget(xml, "button_cw_close");
  cw->controls.button_add = glade_xml_get_widget(xml, "button_cw_add");
  cw->controls.button_emote = glade_xml_get_widget(xml, "button_cw_emote");

#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (cw->controls.window), "delete_event",
                      GTK_SIGNAL_FUNC (on_chat_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.button_close), "clicked",
                      GTK_SIGNAL_FUNC (on_button_cw_close_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.button_send), "clicked",
                      GTK_SIGNAL_FUNC (on_button_cw_send_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.button_information), "clicked",
                      GTK_SIGNAL_FUNC (on_button_cw_information_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.button_history), "clicked",
                      GTK_SIGNAL_FUNC (on_button_cw_history_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.button_add), "clicked",
                      GTK_SIGNAL_FUNC (on_button_cw_add_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.button_emote), "clicked",
                      GTK_SIGNAL_FUNC (on_button_cw_emote_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.textview_receive), "button_press_event",
                      GTK_SIGNAL_FUNC (on_textview_cw_receive_button_press_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.textview_receive), "motion_notify_event",
                      GTK_SIGNAL_FUNC (on_textview_cw_receive_motion_notify_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.textview_send), "focus_in_event",
                      GTK_SIGNAL_FUNC (on_textview_cw_send_focus_in_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.textview_send), "focus_out_event",
                      GTK_SIGNAL_FUNC (on_textview_cw_send_focus_out_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.textview_send), "key_press_event",
                      GTK_SIGNAL_FUNC (on_textview_cw_send_key_press_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cw->controls.textview_send), "key_release_event",
                      GTK_SIGNAL_FUNC (on_textview_cw_send_key_release_event),
                      NULL);
#endif

  /* set icon */
  pb = gj_gtk_support_pixbuf_new_from_file(FILE_MESSAGE_CHAT);
  gtk_window_set_icon(GTK_WINDOW(cw->controls.window), pb);
  g_object_unref(G_OBJECT(pb));

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_cw_unload_by_widget(GtkWidget *widget)
{
  chatWindow cw = NULL;

  if(widget == NULL)
    {
      g_warning("gj_gtk_cw_unload_by_widget: widget was NULL");
      return FALSE;
    }

  if((cw = gj_gtk_cw_find_by_widget(widget)) != NULL)
    {
      /* if no more msgs reset icon */
      gtk_widget_destroy(cw->controls.window);
      gj_gtk_cw_free(cw);
      
      g_message("gj_gtk_cw_unload_by_widget: unloaded...");
    }
  else
    g_warning("gj_gtk_cw_unload_by_widget: could not find cw from widget");
  
  return TRUE;
}

gboolean gj_gtk_cw_setup(chatWindow cw)
{
  gchar *title = NULL;
  
  GtkTextBuffer *buffer;

  rosterItem ri = NULL;
  presence pres = NULL;

  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_setup: cw was NULL");
      return FALSE;
    }

  /* set title */
  title = g_strdup_printf("Chatting with %s %s%s%s",
			  gj_gtk_cw_get_nickname(cw),
			  gj_gtk_cw_get_groups(cw)?"[":"",
			  gj_gtk_cw_get_groups(cw)?gj_gtk_cw_get_groups(cw):"",
			  gj_gtk_cw_get_groups(cw)?"]":"");
  gtk_window_set_title(GTK_WINDOW(cw->controls.window), title);
  if(title != NULL) g_free(title);

  /* set buffer iterator at position 0 */
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(cw->controls.textview_receive));
  
  /* set function callbacks */
  g_signal_connect(G_OBJECT(buffer), "changed", 
		   G_CALLBACK(on_textbuffer_cw_receive_changed), 
		   NULL);
 
  /* set styles */
  gtk_text_buffer_create_tag(buffer, "bold", 
			     "weight", PANGO_WEIGHT_BOLD, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "italic", 
			     "style", PANGO_STYLE_ITALIC, 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "blue_foreground", 
			     "foreground", "darkblue", 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "red_foreground", 
			     "foreground", "darkred", 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "green_foreground", 
			     "foreground", "darkgreen", 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "black_foreground", 
			     "foreground", "black", 
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "url", 
			     "foreground", "blue", 
			     NULL); 

  /* find roster item by JID */
  if((ri = rosterItemFindByJID(gj_gtk_cw_get_jid(cw))) == NULL)
    {
      g_message("gj_gtk_cw_setup: could not find roster item from JID:'%s', trying without resource...",gj_gtk_cw_get_jid(cw)); 

      if((ri = rosterItemFindByJIDWithoutResource(gj_gtk_cw_get_jid(cw))) == NULL)
	{
	  g_warning("gj_gtk_cw_setup: could not find roster item from JID:'%s'.",gj_gtk_cw_get_jid(cw));
	  return FALSE;
	}
    }

  if((pres = rosterItemGetPresence(ri)) == NULL)
    {
      g_warning("gj_gtk_cw_setup: could not get presence from roster item with JID:'%s'",gj_gtk_cw_get_jid(cw)); 
      return FALSE;
    }  

  /* set toolbar */
  if(rosterItemGetIsPermanent(ri) == FALSE)
    gtk_widget_set_sensitive(GTK_WIDGET(cw->controls.button_add), TRUE);
  else
    gtk_widget_set_sensitive(GTK_WIDGET(cw->controls.button_add), FALSE);

  /* set text direction */
  gj_gtk_cw_set_reverse_text_direction(cw, gj_gtk_mpf_get_reverse_text_direction());

  /* set presence */
  gj_gtk_cw_presence_update(cw, pres);

  return TRUE;
}

chatWindow gj_gtk_cw_receive_new(gchar *jid)
{
  chatWindow cw = NULL;

  /* check for current chat window for this JID first */
  if((cw = gj_gtk_cw_find_by_jid(jid)) != NULL)
    {
      g_message("gj_gtk_cw_receive_new: not creating new window, re-using window already open for jid:'%s'", jid);
      gtk_window_present(GTK_WINDOW(cw->controls.window));
      return cw;
    }

  cw = gj_gtk_cw_new(jid);
  gj_gtk_cw_load(cw);

  /* setup ... */
  gj_gtk_cw_setup(cw);

  return cw;
}

chatWindow gj_gtk_cw_send_new(gchar *jid)
{
  chatWindow cw = NULL;

  /* check for current chat window for this JID first */
  if((cw = gj_gtk_cw_find_by_jid(jid)) != NULL)
    {
      g_message("gj_gtk_cw_send_new: not creating new window, re-using window already open for jid:'%s'", jid);
      gtk_window_present(GTK_WINDOW(cw->controls.window));
      return cw;
    }

  cw = gj_gtk_cw_new(jid);
  gj_gtk_cw_load(cw);

  /* setup ... */
  gj_gtk_cw_setup(cw);

  return cw;
}

gboolean gj_gtk_cw_receive_set(chatWindow cw, message msg)
{
  GtkTextView *textview = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkTextMark *mark = NULL;
  GtkTextIter iter;

  GdkPixbuf *pb = NULL;

  gchar *message = NULL;
  gchar *time = NULL;
  guint i = 0;

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(cw->controls.textview_receive));
  time = g_strdup_printf("[%s]", supportGetTimestampFromTimeT(messageGetTime(msg)));
  
  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_receive_set: chatWindow was NULL");
      return FALSE;
    }

  if(msg == NULL)
    {
      g_warning("gj_gtk_cw_receive_set: msg was NULL");
      return FALSE;
    }

  if((textview = GTK_TEXT_VIEW(cw->controls.textview_receive)) == NULL)
    {
      g_warning("gj_gtk_cw_receive_set: textview was NULL");
      return FALSE;
    }

  /* set details */
  message = g_strdup_printf(": %s\n", messageGetBody(msg));

  /* get text direction */
  if(gj_gtk_cw_get_reverse_text_direction(cw) == TRUE)
    gtk_text_buffer_get_start_iter(buffer, &iter);
  else
    gtk_text_buffer_get_end_iter(buffer, &iter);

  if(gj_gtk_mpf_get_hide_time() == FALSE)
    {
      /* insert at iter */
      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
					       time, -1,
					       "blue_foreground",
					       NULL);
      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
					       " ", -1,
					       "blue_foreground",
					       NULL);
    }

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
					   gj_gtk_cw_get_nickname(cw), -1,
					   "bold", 
					   "blue_foreground",
					   NULL);

  /* search for occurences of emote icons */
  for(i=0;i<strlen(message);i++)
    {
      if(strncmp(message+i, EMOTE_BIGGRIN, strlen(EMOTE_BIGGRIN)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_biggrin)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_BIGGRIN)-1);
	}
      else if(strncmp(message+i, EMOTE_CONFUSED, strlen(EMOTE_CONFUSED)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_confused)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_CONFUSED)-1);
	}
      else if(strncmp(message+i, EMOTE_COOL, strlen(EMOTE_COOL)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_cool)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_COOL)-1);
	}
      else if(strncmp(message+i, EMOTE_EEK, strlen(EMOTE_EEK)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_eek)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_EEK)-1);
	}
      else if(strncmp(message+i, EMOTE_FROWN, strlen(EMOTE_FROWN)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_frown)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_FROWN)-1);
	}
      else if(strncmp(message+i, EMOTE_MAD, strlen(EMOTE_MAD)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_mad)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_MAD)-1);
	}
      else if(strncmp(message+i, EMOTE_REDFACE, strlen(EMOTE_REDFACE)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_redface)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_REDFACE)-1);
	}
      else if(strncmp(message+i, EMOTE_ROLLEYES, strlen(EMOTE_ROLLEYES)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_rolleyes)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_ROLLEYES)-1);
	}
      else if(strncmp(message+i, EMOTE_SMILE, strlen(EMOTE_SMILE)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_smile)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_SMILE)-1);
	}
      else if(strncmp(message+i, EMOTE_TONGUE, strlen(EMOTE_TONGUE)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_tongue)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_TONGUE)-1);
	}
      else if(strncmp(message+i, EMOTE_WINK, strlen(EMOTE_WINK)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_wink)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_WINK)-1);
	}
      else
	{
	  /* insert text */
	  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
						   message+i, 1,
						   "black_foreground",
						   NULL);
	}
    }

  /* log to history */
  historyWriteReceivedByJID(gj_gtk_cw_get_jid(cw), (gchar*)supportGetTimestampFull(), NULL, messageGetBody(msg));

  if(gj_gtk_cw_get_reverse_text_direction(cw) == FALSE)
    {
      mark = gtk_text_buffer_create_mark(buffer, NULL, &iter, FALSE);
      gtk_text_view_scroll_mark_onscreen(textview, mark);
      gtk_text_buffer_delete_mark(buffer, mark);
    }

  /* save details */
  gj_gtk_cw_set_last_recv_xevent_type(cw, messageGetXEventType(msg));
  gj_gtk_cw_set_last_recv_xevent_id(cw, messageGetID(msg)?messageGetID(msg):messageGetThread(msg));
  gj_gtk_cw_set_thread(cw, messageGetThread(msg));

  /* clean up */
  if(time != NULL) g_free(time);
  if(message != NULL) g_free(message);

  return TRUE;
}

gboolean gj_gtk_cw_send_set(chatWindow cw, gchar *text)
{
  GtkTextView *textview = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkTextMark *mark = NULL;
  GtkTextIter iter;

  GdkPixbuf *pb = NULL;

  gchar *message = NULL;
  gchar *message_id = NULL;
  gint message_event = 0;
  gchar *time = NULL;
  guint i = 0;

  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_send_set: chatWindow was NULL");
      return FALSE;
    }

  if(text == NULL)
    {
      g_warning("gj_gtk_cw_send_set: can not send NULL");
      return FALSE;
    }

  if((textview = GTK_TEXT_VIEW(cw->controls.textview_receive)) == NULL)
    {
      g_warning("gj_gtk_cw_send_set: textview was NULL");
      return FALSE;
    }

  buffer = gtk_text_view_get_buffer(textview);
  time = g_strdup_printf("[%s]", supportGetTimestamp());
  
  /* set details */
  message = g_strdup_printf(": %s\n", text);

  /* get text direction */
  if(gj_gtk_cw_get_reverse_text_direction(cw) == TRUE)
    gtk_text_buffer_get_start_iter(buffer, &iter);
  else
    gtk_text_buffer_get_end_iter(buffer, &iter);

  /* insert at iter */
  if(gj_gtk_mpf_get_hide_time() == FALSE)
    {
      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
					       time, -1,
					       "green_foreground",
					       NULL);
      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
					       " ", -1,
					       "green_foreground",
					       NULL);
    }

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
					   gj_gtk_cs_get_nickname(), -1,
					   "bold", 
					   "green_foreground",
					   NULL);

  /* search for occurences of emote icons */
  for(i=0;i<strlen(message);i++)
    {
      if(strncmp(message+i, EMOTE_BIGGRIN, strlen(EMOTE_BIGGRIN)) == 0)
	{
	  g_message("use bggrin");
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_biggrin)) != NULL)
	    {
	      g_message("insert bggrin");
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_BIGGRIN)-1);
	}
      else if(strncmp(message+i, EMOTE_CONFUSED, strlen(EMOTE_CONFUSED)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_confused)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_CONFUSED)-1);
	}
      else if(strncmp(message+i, EMOTE_COOL, strlen(EMOTE_COOL)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_cool)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_COOL)-1);
	}
      else if(strncmp(message+i, EMOTE_EEK, strlen(EMOTE_EEK)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_eek)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_EEK)-1);
	}
      else if(strncmp(message+i, EMOTE_FROWN, strlen(EMOTE_FROWN)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_frown)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_FROWN)-1);
	}
      else if(strncmp(message+i, EMOTE_MAD, strlen(EMOTE_MAD)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_mad)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_MAD)-1);
	}
      else if(strncmp(message+i, EMOTE_REDFACE, strlen(EMOTE_REDFACE)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_redface)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_REDFACE)-1);
	}
      else if(strncmp(message+i, EMOTE_ROLLEYES, strlen(EMOTE_ROLLEYES)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_rolleyes)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_ROLLEYES)-1);
	}
      else if(strncmp(message+i, EMOTE_SMILE, strlen(EMOTE_SMILE)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_smile)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_SMILE)-1);
	}
      else if(strncmp(message+i, EMOTE_TONGUE, strlen(EMOTE_TONGUE)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_tongue)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_TONGUE)-1);
	}
      else if(strncmp(message+i, EMOTE_WINK, strlen(EMOTE_WINK)) == 0)
	{
	  if((pb = gj_gtk_cw_emote_icon_get_as_pixbuf(emote_wink)) != NULL)
	    {
	      gtk_text_buffer_insert_pixbuf(buffer, &iter, pb);
	      g_object_unref(G_OBJECT(pb));
	    }
	  
	  i += (strlen(EMOTE_WINK)-1);
	}
      else
	{
	  /* insert text */
	  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
						   message+i, 1,
						   "black_foreground",
						   NULL);
	}
    }

  /* scroll to end if not reversing text direction */
  if(gj_gtk_cw_get_reverse_text_direction(cw) == FALSE)
    {
      mark = gtk_text_buffer_create_mark(buffer, NULL, &iter, FALSE);
      gtk_text_view_scroll_mark_onscreen(textview, mark);
      gtk_text_buffer_delete_mark(buffer, mark);
    }

  /* set message */
  message_event = messageXEventTypeOffline | 
                  messageXEventTypeDelivered | 
                  messageXEventTypeDisplayed | 
                  messageXEventTypeComposing;

  message_id = messageSendWithXEventRequest(messageTypeChat, 
					    gj_gtk_cw_get_jid(cw), 
					    NULL, 
					    text, 
					    gj_gtk_cw_get_thread(cw), 
					    message_event);

  /* log to history */
  historyWriteSentByJID(gj_gtk_cw_get_jid(cw), (gchar*)supportGetTimestampFull(), NULL, text);

  /* save details */
  gj_gtk_cw_set_last_sent_xevent_id(cw, message_id);
  gj_gtk_cw_set_last_sent_xevent_type(cw, message_event);
  /* should we use a thread? */

  /* clean up */
  if(time != NULL) g_free(time);
  if(message != NULL) g_free(message);
  if(message_id != NULL) g_free(message_id);

  return TRUE;
}

chatWindow gj_gtk_cw_notification_new(message msg)
{
  chatWindow cw = NULL;

  if(msg == NULL)
    {
      g_warning("gj_gtk_cw_notification_new: msg was NULL");
      return NULL;
    }

  cw = gj_gtk_cw_find_by_jid(messageGetFromJID(msg));

  if(cw == NULL)
    {
      g_message("gj_gtk_cw_notification_new: no previous window, creating new for jid:'%s'", messageGetFromJID(msg)); 

      /* window doesnt exist - create it */
      cw = gj_gtk_cw_receive_new(messageGetFromJID(msg));

      /* NOTE !!!!! this should perhaps be sent using map/unmap events 
	 so we know WHEN and WHEN NOT the window is displayed!!!! */

      /* if XEvents ask to be notified when the message has 
	 been displayed, this is the time to send that notification */
      if(messageGetXEventType(msg) & messageXEventTypeDisplayed)
	  messageSendXEvent(messageXEventTypeDisplayed, messageGetFromJID(msg), messageGetID(msg));
    }
  else
    {
      g_message("gj_gtk_cw_notification_new: found window for jid:'%s'", messageGetFromJID(msg)); 

      /* should we show it? */
      if(gj_gtk_mpf_get_raise_window_on_new_message() == TRUE)
      {
	  gj_gtk_cw_raise(cw);

	  /* if XEvents ask to be notified when the message has 
	     been displayed, this is the time to send that notification */
	  if(messageGetXEventType(msg) & messageXEventTypeDisplayed)
	      messageSendXEvent(messageXEventTypeDisplayed, messageGetFromJID(msg), messageGetID(msg));
      }
    }

  /* if XEvents ask to be notified when the message has 
     been displayed, this is the time to send that notification */
  if(messageGetXEventType(msg) & messageXEventTypeComposing)
    gj_gtk_cw_notification_of_xevent_new(cw, msg);

  gj_gtk_cw_receive_set(cw,msg);

  return cw;
}

void gj_gtk_cw_notification_of_xevent_new(chatWindow cw, message msg)
{
   if(cw == NULL)
   {
       g_warning("gj_gtk_cw_notification_of_xevent_new: cw was NULL");
       return;
   }

   if(msg == NULL)
   {
       g_warning("gj_gtk_cw_notification_of_xevent_new: msg was NULL");
       return;
   }
   
   switch(messageGetXEventType(msg))
   {
     case messageXEventTypeOffline:
       break;
     case messageXEventTypeDelivered:
       break;
     case messageXEventTypeDisplayed: 
       break;
     case messageXEventTypeComposing:
       gtk_widget_show(GTK_WIDGET(cw->controls.image_composing));
       break;
     case messageXEventTypeCancelled:
       gtk_widget_hide(GTK_WIDGET(cw->controls.image_composing));
       break;
   }
}

void gj_gtk_cw_default_action(gchar *jid)
{
  chatWindow cw = NULL;

  if(jid == NULL)
    {
      g_warning("gj_gtk_cw_default_action: JID was NULL");
      return;
    }
  
  if((cw = gj_gtk_cw_find_by_jid(jid)) == NULL)
    {
      g_message("gj_gtk_cw_default_action: no previous window, creating new for jid:'%s'", jid); 

      /* make conversation */
      cw = gj_gtk_cw_new(jid);

      /* set up window */
      gj_gtk_cw_load(cw);
      gj_gtk_cw_setup(cw);
      gj_gtk_cw_show(cw,TRUE);
    }
  else
    {
      g_message("gj_gtk_cw_default_action: found window, showing window for jid:'%s'", jid); 

      /* receive conversation */
      gj_gtk_cw_show(cw,TRUE);
    }


  /* if XEvents ask to be notified when the message has 
     been displayed, this is the time to send that notification */
  if(gj_gtk_cw_get_last_recv_xevent_type(cw) & messageXEventTypeDisplayed)
      messageSendXEvent(messageXEventTypeDisplayed, gj_gtk_cw_get_jid(cw), gj_gtk_cw_get_last_recv_xevent_id(cw));
}

void gj_gtk_cw_emote_icon_create_popup_menu(GtkWidget *widget)
{
  GtkWidget *emote_icons_menu;
  GtkWidget *image;
  GtkWidget *mnu_emoteicon_big_grin;
  GtkWidget *mnu_emoteicon_confused;
  GtkWidget *mnu_emoteicon_cool;
  GtkWidget *mnu_emoteicon_eek;
  GtkWidget *mnu_emoteicon_frown;
  GtkWidget *mnu_emoteicon_mad;
  GtkWidget *mnu_emoteicon_red_face;
  GtkWidget *mnu_emoteicon_roll_eyes;
  GtkWidget *mnu_emoteicon_smile;
  GtkWidget *mnu_emoteicon_tongue;
  GtkWidget *mnu_emoteicon_wink;

  if(widget == NULL)
  {
      g_warning("gj_gtk_cw_emote_icon_create_popup_menu: widget was NULL, no widget to relate menu to chat window");
      return;
  }

  emote_icons_menu = gtk_menu_new ();
  gtk_widget_set_name (emote_icons_menu, "emote_icons_menu");

  mnu_emoteicon_big_grin = gtk_image_menu_item_new_with_mnemonic ("Big Grin");
  gtk_widget_set_name (mnu_emoteicon_big_grin, "mnu_emoteicon_big_grin");
  gtk_widget_show (mnu_emoteicon_big_grin);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_big_grin);

  image = gtk_image_new_from_file (FILE_EMOTE_BIGGRIN);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_big_grin), image);

  mnu_emoteicon_confused = gtk_image_menu_item_new_with_mnemonic ("Confused");
  gtk_widget_set_name (mnu_emoteicon_confused, "mnu_emoteicon_confused");
  gtk_widget_show (mnu_emoteicon_confused);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_confused);

  image = gtk_image_new_from_file (FILE_EMOTE_CONFUSED);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_confused), image);

  mnu_emoteicon_cool = gtk_image_menu_item_new_with_mnemonic ("Cool");
  gtk_widget_set_name (mnu_emoteicon_cool, "mnu_emoteicon_cool");
  gtk_widget_show (mnu_emoteicon_cool);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_cool);

  image = gtk_image_new_from_file (FILE_EMOTE_COOL);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_cool), image);

  mnu_emoteicon_eek = gtk_image_menu_item_new_with_mnemonic ("EEk");
  gtk_widget_set_name (mnu_emoteicon_eek, "mnu_emoteicon_eek");
  gtk_widget_show (mnu_emoteicon_eek);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_eek);

  image = gtk_image_new_from_file (FILE_EMOTE_EEK);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_eek), image);

  mnu_emoteicon_frown = gtk_image_menu_item_new_with_mnemonic ("Frown");
  gtk_widget_set_name (mnu_emoteicon_frown, "mnu_emoteicon_frown");
  gtk_widget_show (mnu_emoteicon_frown);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_frown);

  image = gtk_image_new_from_file (FILE_EMOTE_FROWN);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_frown), image);

  mnu_emoteicon_mad = gtk_image_menu_item_new_with_mnemonic ("Mad");
  gtk_widget_set_name (mnu_emoteicon_mad, "mnu_emoteicon_mad");
  gtk_widget_show (mnu_emoteicon_mad);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_mad);

  image = gtk_image_new_from_file (FILE_EMOTE_MAD);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_mad), image);

  mnu_emoteicon_red_face = gtk_image_menu_item_new_with_mnemonic ("Red Face");
  gtk_widget_set_name (mnu_emoteicon_red_face, "mnu_emoteicon_red_face");
  gtk_widget_show (mnu_emoteicon_red_face);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_red_face);

  image = gtk_image_new_from_file (FILE_EMOTE_REDFACE);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_red_face), image);

  mnu_emoteicon_roll_eyes = gtk_image_menu_item_new_with_mnemonic ("Roll Eyes");
  gtk_widget_set_name (mnu_emoteicon_roll_eyes, "mnu_emoteicon_roll_eyes");
  gtk_widget_show (mnu_emoteicon_roll_eyes);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_roll_eyes);

  image = gtk_image_new_from_file (FILE_EMOTE_ROLLEYES);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_roll_eyes), image);

  mnu_emoteicon_smile = gtk_image_menu_item_new_with_mnemonic ("Smile");
  gtk_widget_set_name (mnu_emoteicon_smile, "mnu_emoteicon_smile");
  gtk_widget_show (mnu_emoteicon_smile);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_smile);

  image = gtk_image_new_from_file (FILE_EMOTE_SMILE);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_smile), image);

  mnu_emoteicon_tongue = gtk_image_menu_item_new_with_mnemonic ("Tongue");
  gtk_widget_set_name (mnu_emoteicon_tongue, "mnu_emoteicon_tongue");
  gtk_widget_show (mnu_emoteicon_tongue);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_tongue);

  image = gtk_image_new_from_file (FILE_EMOTE_TONGUE);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_tongue), image);

  mnu_emoteicon_wink = gtk_image_menu_item_new_with_mnemonic ("Wink");
  gtk_widget_set_name (mnu_emoteicon_wink, "mnu_emoteicon_wink");
  gtk_widget_show (mnu_emoteicon_wink);
  gtk_container_add (GTK_CONTAINER (emote_icons_menu), mnu_emoteicon_wink);

  image = gtk_image_new_from_file (FILE_EMOTE_WINK);
  gtk_widget_set_name (image, "image");
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mnu_emoteicon_wink), image);

  /* hook up signals */
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_big_grin), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_big_grin_activate),
                      (gpointer)widget);
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_confused), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_confused_activate),
                      (gpointer)widget);
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_cool), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_cool_activate),
                      (gpointer)widget);
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_eek), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_eek_activate),
                      (gpointer)widget);
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_frown), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_frown_activate),
                      (gpointer)widget);
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_mad), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_mad_activate),
                      (gpointer)widget);
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_red_face), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_red_face_activate),
                      (gpointer)widget);
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_roll_eyes), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_roll_eyes_activate),
                      (gpointer)widget);
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_smile), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_smile_activate),
                      (gpointer)widget);
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_tongue), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_tongue_activate),
                      (gpointer)widget);
  gtk_signal_connect (GTK_OBJECT (mnu_emoteicon_wink), "activate",
                      GTK_SIGNAL_FUNC (on_mnu_emoteicon_wink_activate),
                      (gpointer)widget);

  /* show menu */
  gtk_menu_popup(GTK_MENU(emote_icons_menu),
		 NULL,
		 NULL, 
		 NULL, 
		 NULL,
		 1, 
		 gtk_get_current_event_time()); 
}

GdkPixbuf *gj_gtk_cw_emote_icon_get_as_pixbuf(msgEmote icon)
{
    GdkPixbuf *pixbuf = NULL;
    GdkPixbuf *scaled = NULL;
    gchar *filename = NULL;

    switch(icon)
    {
      case emote_biggrin:
	filename = FILE_EMOTE_BIGGRIN;
	break;
      case emote_confused:
	filename = FILE_EMOTE_CONFUSED;
	break;
      case emote_cool:
	filename = FILE_EMOTE_COOL;
	break;
      case emote_eek:
	filename = FILE_EMOTE_EEK;
	break;
      case emote_frown:
	filename = FILE_EMOTE_FROWN;
	break;
      case emote_mad:
	filename = FILE_EMOTE_MAD;
	break;
      case emote_redface:
	filename = FILE_EMOTE_REDFACE;
	break;
      case emote_rolleyes:
	filename = FILE_EMOTE_ROLLEYES;
	break;
      case emote_smile:
	filename = FILE_EMOTE_SMILE;
	break;
      case emote_tongue:
	filename = FILE_EMOTE_TONGUE;
	break;
      case emote_wink:
	filename = FILE_EMOTE_WINK;
	break;
      default:
	g_warning("gj_gtk_cw_emote_icon_get_as_pixbuf: failed to determin which icon to use from %d",icon);
	return NULL;
	break;
    }

    if((pixbuf = gdk_pixbuf_new_from_file(filename, NULL)) == NULL)
    {
	g_warning("gj_gtk_cw_emote_icon_get_as_pixbuf: failed to load image file:'%s'",filename);
	return NULL;
    }

  scaled = gdk_pixbuf_scale_simple(pixbuf, 15, 15, GDK_INTERP_BILINEAR);
  g_object_unref(G_OBJECT(pixbuf));
  pixbuf = scaled;

  return pixbuf;
}

void gj_gtk_cw_emote_icon_get_as_text(gchar *emote_icon, gpointer user_data)
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter;

  chatWindow cw = NULL;

  if(user_data == NULL)
  {
      g_warning("gj_gtk_cw_emote_icon_get_as_text: user data is NULL");
      return;
  }
  
  if((cw = gj_gtk_cw_find_by_widget(GTK_WIDGET(user_data))) == NULL)
  {
      g_warning("gj_gtk_cw_emote_icon_get_as_text: failed to find window based on button widget");
      return;
  }
  
  /* get buffer and end iterator */
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(cw->controls.textview_send));
  gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer), &iter);
  gtk_text_buffer_insert(buffer, &iter,
			 emote_icon, -1);

  /* set focus back to the chat text box */
  gtk_widget_grab_focus(GTK_WIDGET(cw->controls.textview_send));
}

void gj_gtk_cw_presence_update(chatWindow cw, presence pres)
{
  gchar *file = NULL;
  gchar *text = NULL;
  GdkPixbuf *pixbuf = NULL;

  if(cw == NULL)
    {
      g_warning("gj_gtk_cw_presence_update: chat window was NULL");
      return;
    }

  if(pres == NULL)
    {
      g_warning("gj_gtk_cw_presence_update: presence was NULL");
      return;
    }

  switch(presenceGetType(pres))
    {
    case presenceTypeAvailable:
      {
	switch(presenceGetShow(pres))
	  {
	  case presenceShowAway:
	    file = FILE_PRESENCE_AWAY;
	    text = g_strdup_printf("Away");
	    break;
	  case presenceShowChat:
	    file = FILE_PRESENCE_CHAT;
	    text = g_strdup_printf("Chat");
	    break;
	  case presenceShowDND:
	    file = FILE_PRESENCE_DND;
	    text = g_strdup_printf("Do Not Disturb");
	    break;
	  case presenceShowNormal:
	    file = FILE_PRESENCE_NORMAL;
	    text = g_strdup_printf("Normal");
	    break;
	  case presenceShowXA:
	    file = FILE_PRESENCE_XA;
	    text = g_strdup_printf("eXtended Away");
	    break;
	  default:
	    file = FILE_PRESENCE_AVAILABLE;
	    text = g_strdup_printf("Available");
	    g_warning("gj_gtk_cw_presence_update: failed to determin presence 'show' from %d", presenceGetShow(pres));
	    return;
	  }
      }
      break;
      
    default:
    case presenceTypeUnAvailable:
      {
	file = FILE_PRESENCE_UNAVAILABLE;
	text = g_strdup_printf("Unavailable");	
      }
      break;
    }

  /* update image and label */
  pixbuf = gj_gtk_support_pixbuf_new_from_file(file);
  gtk_image_set_from_pixbuf(GTK_IMAGE(cw->controls.image_status), pixbuf);
  gtk_label_set_text(GTK_LABEL(cw->controls.label_status), text);

  /* clean up */
  if(text != NULL) g_free(text);
  if(pixbuf != NULL) g_object_unref(G_OBJECT(pixbuf));
}

/*
 * GUI events
 */
void on_textbuffer_cw_receive_changed(GtkTextBuffer *textbuffer, gpointer user_data)
{
  GtkTextMark *mark = NULL;
  GtkTextIter iter_space;
  GtkTextIter iter_word_start;
  GtkTextIter iter_word_end;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  gint length = 0;

  /* check WHOLE buffer for urls */
  gtk_text_buffer_get_bounds(textbuffer, &iter_start, &iter_end);
  length = gtk_text_buffer_get_char_count(textbuffer);

  if((mark = gtk_text_buffer_get_mark(textbuffer, "url")) != NULL)
  {
      gtk_text_buffer_get_iter_at_mark(textbuffer, &iter_start, mark);
/*       gtk_text_mark_set_visible(mark, TRUE);  */
  }

  /* save last iter */
  iter_word_start = iter_start;
  iter_word_end = iter_start;
  iter_space = iter_start;

  while(TRUE)
  {
      gboolean found = FALSE;
      gchar *buf = NULL;

      /* search for next space */
      found = gtk_text_iter_forward_find_char(&iter_space, on_textview_cw_receive_find_space, NULL, NULL);

      if(found == FALSE)
	  break;
      
      /* set word end to the space */
      iter_word_start = iter_word_end;
      iter_word_end = iter_space;

      /* move word end iter forward 1 */
      gtk_text_iter_forward_char(&iter_word_start);

      /* get */
      buf = gtk_text_iter_get_text(&iter_word_start, &iter_word_end);

/*       if(buf != NULL && strlen(buf) > 0) */
/* 	  g_message("buf:'%s'", buf); */
      
      if(buf != NULL) 
      {
	  if(supportIsURI(buf) != uriTypeUnknown) 
	  {
	      /* highlight as uri */
	      gtk_text_buffer_apply_tag_by_name(textbuffer, "url", &iter_word_start, &iter_word_end);
	  } 
	  else
	  {
	      /* none highlighted */
	      gtk_text_buffer_remove_tag_by_name(textbuffer, "url", &iter_word_start, &iter_word_end);
	  }

	  g_free(buf);
      }

      /* set mark, then we can check next time from 
	 the last mark and it is more efficient */
      if((mark = gtk_text_buffer_get_mark(textbuffer, "url")) == NULL)
      {
	  if((mark = gtk_text_buffer_create_mark(textbuffer, "url", &iter_space, FALSE)) == NULL)
	  {
	      g_warning("on_textbuffer_cw_receive_changed: could not create mark!");
	      return;
	  }

	  /* no need to move it, just continue */
	  continue;
      }

      /* move mark to latest iter */
      gtk_text_buffer_move_mark_by_name(textbuffer, "url", &iter_space);
  }
}

gboolean on_textview_cw_receive_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
{
  GdkModifierType state = 0;
  GdkWindow *window = NULL;

  GtkTextView *textview = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter;
  GtkTextIter iter_start;
  GtkTextIter iter_end;
  GtkTextWindowType text_window_type = 0;

  gboolean found_start = FALSE;
  gboolean found_end = FALSE;

  gint pointer_x = 0;
  gint pointer_y = 0;
  gint x = 0;
  gint y = 0;

  gchar *buf = NULL;

  chatWindow cw = NULL;

  if(event->is_hint)
  {
      window = gdk_window_get_pointer(event->window, &pointer_x, &pointer_y, &state);
  }
  else
  {
      pointer_x = event->x;
      pointer_y = event->y;
      state = event->state;
  }

  if((cw = gj_gtk_cw_find_by_widget(GTK_WIDGET(widget))) == NULL)
  {
      g_warning("on_textview_cw_receive_motion_notify_event: failed to find headlineWindow from widget");
      return FALSE;
  }
  
  if((textview = GTK_TEXT_VIEW(widget)) == NULL)
  {
      g_warning("on_textview_cw_receive_motion_notify_event: view was NULL");
      return FALSE;
  }

  if((buffer = GTK_TEXT_BUFFER(gtk_text_view_get_buffer(textview))) == NULL)
  {
      g_warning("on_textview_cw_receive_motion_notify_event: buffer was NULL");
      return FALSE;
  }

  text_window_type = gtk_text_view_get_window_type(textview, event->window);
  gtk_text_view_window_to_buffer_coords(textview, text_window_type, pointer_x, pointer_y, &x, &y);
  gtk_text_view_get_iter_at_location(textview, &iter, x, y);

  if(g_unichar_isspace(gtk_text_iter_get_char(&iter)) == TRUE)
  {
      gdk_window_set_cursor(event->window, NULL);
      return FALSE;
  }

  iter_start = iter;
  iter_end = iter;

  /* search for next space */
  found_start = gtk_text_iter_backward_find_char(&iter_start, on_textview_cw_receive_find_space, NULL, NULL);
  found_end = gtk_text_iter_forward_find_char(&iter_end, on_textview_cw_receive_find_space, NULL, NULL);

  if(found_start == FALSE && found_end == FALSE)
  {
      gdk_window_set_cursor(event->window, NULL);
      return FALSE;
  }

  if(gtk_text_iter_is_start(&iter_start) == FALSE)
      gtk_text_iter_forward_char(&iter_start);
  
  if((buf = gtk_text_buffer_get_text(buffer, &iter_start, &iter_end, FALSE)) == NULL)
  {
      g_warning("on_textview_cw_receive_motion_notify_event: buf returned was NULL");
      return FALSE;
  }

/*   if(buf != NULL && strlen(buf) > 0) */
/*       g_message("buf:'%s'", buf); */
  
  if(supportIsURI(buf) != uriTypeUnknown)
  {
      /* width wise it works... */
      GdkCursor *cursor = NULL;
      cursor = gdk_cursor_new(GDK_HAND2);
      gdk_window_set_cursor (event->window, cursor);
      
      /* clean up */
      gdk_cursor_unref(cursor);
  }  
  else
  {
      /* return to parent window's cursor */
      gdk_window_set_cursor(event->window, NULL);
  }

      
  /* clean up */
  if(buf != NULL) g_free(buf);

  return FALSE;
}

gboolean on_textview_cw_receive_find_space(gunichar ch, gpointer user_data)
{
    if(g_unichar_isspace(ch) == TRUE)
	return TRUE;

    return FALSE;
}

gboolean on_textview_cw_receive_find_alpha_numeric(gunichar ch, gpointer user_data)
{
    if(g_unichar_isalnum(ch) == TRUE)
	return TRUE;

    return FALSE;
}

gboolean on_textview_cw_receive_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
    GdkModifierType state;
    GtkTextBuffer *buffer = NULL;
    GtkTextIter iter;
    GtkTextIter iter_start;
    GtkTextIter iter_end;
    GtkTextWindowType text_window_type = 0;

    gboolean found_start = FALSE;
    gboolean found_end = FALSE;
    
    gint pointer_x = 0;
    gint pointer_y = 0;
    gint x = 0;
    gint y = 0;

    gchar *buf = NULL;

    /* if not mouse 1 return */
    if(event->button != 1)
	return FALSE;

    gdk_window_get_pointer(event->window, &pointer_x, &pointer_y, &state);
    
    if((buffer = GTK_TEXT_BUFFER(gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget)))) == NULL)
    {
	g_warning("on_textview_cw_receive_button_press_event: could not get buffer.");
	return FALSE;
    }
    
    text_window_type = gtk_text_view_get_window_type(GTK_TEXT_VIEW(widget), event->window);
    gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(widget), text_window_type, pointer_x, pointer_y, &x, &y);
    gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(widget), &iter, x, y);
 
    iter_start = iter;
    iter_end = iter;
    
    /* search for next space */
    found_start = gtk_text_iter_backward_find_char(&iter_start, on_textview_cw_receive_find_space, NULL, NULL);
    found_end = gtk_text_iter_forward_find_char(&iter_end, on_textview_cw_receive_find_space, NULL, NULL);
    
    if(found_start == FALSE && found_end == FALSE)
	return FALSE;
    
    if(gtk_text_iter_is_start(&iter_start) == FALSE)
	gtk_text_iter_forward_char(&iter_start);
    
    if((buf = gtk_text_buffer_get_text(buffer, &iter_start, &iter_end, FALSE)) == NULL)
    {
	g_warning("on_textview_cw_receive_button_press_event: buf returned was NULL");
	return FALSE;
    }
  
    if(supportIsURI(buf) != uriTypeUnknown)
    {
	gj_gtk_support_open_url(buf);
    }  
    
    /* clean up */
    if(buf != NULL) g_free(buf);
   
    return FALSE;
}

gboolean on_textview_cw_send_key_release_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  GtkTextBuffer *buffer = NULL;
  chatWindow cw = NULL;

  if((buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget))) == NULL)
  {
      g_warning("on_textview_cw_send_key_release_event: buffer was NULL");
      return FALSE;
  }

  if((cw = gj_gtk_cw_find_by_widget(GTK_WIDGET(widget))) == NULL)
  {
      g_warning("on_textview_cw_send_key_release_event: failed to find window based on widget:0x%.8x", (gint)widget);
      return FALSE;
  }

  /* if was enter then clear box */
  if(event != NULL && event->keyval == GDK_Return)
    {
	GtkTextIter iter_start;
	GtkTextIter iter_end; 

	gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
	gtk_text_buffer_delete(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);

	gtk_widget_set_sensitive(cw->controls.button_send, FALSE);
    }
  else
    {
	gint length = 0;

	/* set toolbar icon */  
	length = gtk_text_buffer_get_char_count(buffer);
	if(length < 1)
	    gtk_widget_set_sensitive(cw->controls.button_send, FALSE);
	else
	    gtk_widget_set_sensitive(cw->controls.button_send, TRUE);
    }

  return FALSE;
}

gboolean on_textview_cw_send_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
    GtkTextBuffer *buffer = NULL;
    GtkTextIter iter_start;
    GtkTextIter iter_end;  

    gchar *text = NULL;
    gint length = 0;

    chatWindow cw = NULL;
    
    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
    length = gtk_text_buffer_get_char_count(buffer);
    
    if(length < 1)
    {
	/* if enter was pressed... warn user that we can not do this */
	if(event != NULL && event->keyval == GDK_Return)
	    g_warning("on_textview_cw_send_key_press_event: can't send NULL or 0 length message");
	
	return FALSE;
    }

    gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
    text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end, TRUE);

    if((cw = gj_gtk_cw_find_by_widget(GTK_WIDGET(widget))) == NULL)
    {
	g_warning("on_textview_cw_send_key_press_event: failed to find window based on button widget");
	return FALSE;
    }
    
    /* send message and clear text box */
    if(event->keyval == GDK_Return)
	gj_gtk_cw_send_set(cw,text);
    
    /* cleanup */
    if(text != NULL) g_free(text);
    
    return FALSE;
}

gboolean on_textview_cw_send_focus_in_event(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
    chatWindow cw = NULL;
    
    if((cw = gj_gtk_cw_find_by_widget(GTK_WIDGET(widget))) == NULL)
    {
	g_warning("on_textview_cw_send_focus_in_event: failed to find window based on button widget");
	return FALSE;
    }
    
    if(gj_gtk_cw_get_last_recv_xevent_id(cw) == NULL || gj_gtk_cw_get_last_recv_xevent_type(cw) == 0)
    {
/*       g_message("on_textview_cw_send_focus_in_event: error - xevent id:%s, type:%d",  */
/* 		gj_gtk_cw_get_last_recv_xevent_id(cw), */
/* 		gj_gtk_cw_get_last_recv_xevent_type(cw)); */
	
	return FALSE;
    }
    
    /* if XEvents ask to be notified when the message is 
       composing, this is the time to send that notification */
    if(gj_gtk_cw_get_last_recv_xevent_type(cw) & messageXEventTypeComposing)
	messageSendXEvent(messageXEventTypeComposing, gj_gtk_cw_get_jid(cw), gj_gtk_cw_get_last_recv_xevent_id(cw));
    
  return FALSE;
}

gboolean on_textview_cw_send_focus_out_event(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
    chatWindow cw = NULL;

  if((cw = gj_gtk_cw_find_by_widget(GTK_WIDGET(widget))) == NULL)
    {
      g_warning("on_textview_cw_send_focus_out_event: failed to find window based on button widget");
      return FALSE;
    }

  if(gj_gtk_cw_get_last_recv_xevent_id(cw) == NULL || gj_gtk_cw_get_last_recv_xevent_type(cw) == 0)
  {
/*       g_message("on_textview_cw_send_focus_out_event: error - xevent id:%s, type:%d",  */
/* 		gj_gtk_cw_get_last_recv_xevent_id(cw), */
/* 		gj_gtk_cw_get_last_recv_xevent_type(cw)); */

      return FALSE;
  }

  /* if XEvents ask to be notified when the message has 
     cancelled composing, this is the time to send that notification */
  if(gj_gtk_cw_get_last_recv_xevent_type(cw) & messageXEventTypeComposing)
      messageSendXEvent(messageXEventTypeCancelled, gj_gtk_cw_get_jid(cw), gj_gtk_cw_get_last_recv_xevent_id(cw));

  return FALSE;
}


void on_button_cw_close_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_cw_unload_by_widget(GTK_WIDGET(button));
}

void on_button_cw_send_clicked(GtkButton *button, gpointer user_data)
{
  gchar *text = NULL;
  chatWindow cw = NULL;

  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  if((cw = gj_gtk_cw_find_by_widget(GTK_WIDGET(button))) == NULL)
    {
      g_warning("on_button_cw_send_clicked: failed to find window based on button widget");
      return;
    }
  
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(cw->controls.textview_send));
  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end, TRUE);

  if(text == NULL || strlen(text) < 1)
  {
      if(text != NULL) g_free(text);

      g_warning("on_button_cw_activate: cant send NULL or 0 length message");
      return;
  }

  gtk_text_buffer_delete(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  gj_gtk_cw_send_set(cw,text);

  g_free(text);
}

void on_button_cw_information_clicked(GtkButton *button, gpointer user_data)
{
  chatWindow cw = NULL;

  if((cw = gj_gtk_cw_find_by_widget(GTK_WIDGET(button))) == NULL)
    {
      g_warning("on_button_cw_information_clicked: failed to find window based on button widget");
      return;
    }

  gj_gtk_ui_load(gj_gtk_cw_get_jid(cw));
}

void on_button_cw_history_clicked(GtkButton *button, gpointer user_data)
{
  chatWindow cw = NULL;
  history hist = NULL;

/*   gchar *url = NULL; */

  if((cw = gj_gtk_cw_find_by_widget(GTK_WIDGET(button))) == NULL)
    {
      g_warning("on_button_cw_history_clicked: failed to find window based on button widget");
      return;
    }

  if((hist = historyFindByJID(gj_gtk_cw_get_jid(cw))) == NULL)
  {
      g_warning("on_button_cw_history_clicked: could not find history based on jid:'%s'", gj_gtk_cw_get_jid(cw));
      return;
  }

  gj_gtk_support_open_url(historyGetFilename(hist));
}

void on_button_cw_add_clicked(GtkButton *button, gpointer user_data)
{
  chatWindow cw = NULL;

  if((cw = gj_gtk_cw_find_by_widget(GTK_WIDGET(button))) == NULL)
    {
      g_warning("on_button_cw_add_clicked: failed to find window based on button widget");
      return;
    }

  gj_gtk_ac_load(gj_gtk_cw_get_jid(cw));
}

void on_button_cw_emote_clicked(GtkButton *button, gpointer user_data)
{
    gj_gtk_cw_emote_icon_create_popup_menu(GTK_WIDGET(button));
}

void on_mnu_emoteicon_big_grin_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_BIGGRIN, user_data);
}

void on_mnu_emoteicon_confused_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_CONFUSED, user_data);
}

void on_mnu_emoteicon_cool_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_COOL, user_data);
}

void on_mnu_emoteicon_eek_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_EEK, user_data);
}

void on_mnu_emoteicon_frown_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_FROWN, user_data);
}

void on_mnu_emoteicon_mad_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_MAD, user_data);
}

void on_mnu_emoteicon_red_face_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_REDFACE, user_data);
}

void on_mnu_emoteicon_roll_eyes_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_ROLLEYES, user_data);
}

void on_mnu_emoteicon_smile_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_SMILE, user_data);
}

void on_mnu_emoteicon_tongue_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_TONGUE, user_data);
}

void on_mnu_emoteicon_wink_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    gj_gtk_cw_emote_icon_get_as_text(EMOTE_WINK, user_data);
}

gboolean on_chat_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_cw_unload_by_widget(widget);

  return FALSE;
}

#ifdef __cplusplus
}
#endif
