/* 
 * $RCSfile: gj_gtk_edit_groups.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/22 21:06:25 $
 * 
 * $Revision: 1.1 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include <unistd.h>

#include "common.h"

#include "gj_main.h"
#include "gj_roster.h"
#include "gj_agents.h"
#include "gj_iq_requests.h"
#include "gj_parser.h"
#include "gj_me.h"

#include "gj_gtk_edit_groups.h"
#include "gj_gtk_register_service.h"
#include "gj_gtk_support.h"
#include "gj_gtk_msg.h"

/* columns */
enum
  {
    COL_EDIT_GROUPS_NAME,
    COL_EDIT_GROUPS_ENABLED,
    COL_EDIT_GROUPS_VISIBLE,
    COL_EDIT_GROUPS_EDITABLE,
    COL_EDIT_GROUPS_COUNT
  };

struct 
{
  GtkWidget *window;

  GtkWidget *treeview;

  GtkWidget *label_name;
  GtkWidget *label_jid;
  GtkWidget *label_resource;
  GtkWidget *label_agent;
  GtkWidget *label_agent_label;

  GtkWidget *entry_add;
  GtkWidget *button_add;
  
  GtkWidget *button_ok;

} ctlEG;

struct 
{
  gchar *jid;
  guint next_group_id;

  gboolean changes_made;

} optEG;


/* 
 * member functions 
 */

/* gets */
gchar *gj_gtk_eg_get_jid();
guint gj_gtk_eg_get_next_group_id();
gboolean gj_gtk_eg_get_changes_made();

/* sets */
gboolean gj_gtk_eg_set_changes_made(gboolean changes);

/* 
 * window functions 
 */
gboolean gj_gtk_eg_setup(gchar *jid);
gboolean gj_gtk_eg_save();
void gj_gtk_eg_update_buttons();


/* 
 * model functions 
 */
gboolean gj_gtk_eg_model_populate_columns(GtkTreeView *treeview);
gboolean gj_gtk_eg_model_populate();

gboolean gj_gtk_eg_model_get_selection(gchar **name, gboolean *enabled);
GtkTreeIter *gj_gtk_eg_model_find_iter_by_name(gchar *name);

/* 
 * internal callbacks 
 */
void gj_gtk_eg_user_changed_response(guint id);

/* 
 * gui callbacks 
 */
void on_treeview_eg_cell_toggled(GtkCellRendererToggle *cell, gchar *path_string, gpointer data);

void on_entry_ep_add_changed(GtkEditable *editable, gpointer user_data);
void on_button_eg_add_clicked(GtkMenuItem *menuitem, gpointer user_data);

void on_edit_groups_response(GtkDialog *dialog, gint response_id, gpointer user_data);


/* 
 * member functions 
 */

/* gets */
gchar *gj_gtk_eg_get_jid()
{
  return optEG.jid;
}

guint gj_gtk_eg_get_next_group_id()
{
  return (optEG.next_group_id)++;
}

gboolean gj_gtk_eg_get_changes_made()
{
  return optEG.changes_made;
}

/* sets */
gboolean gj_gtk_eg_set_changes_made(gboolean changes)
{
  optEG.changes_made = changes;
  return TRUE;
}

/*
 * code
 */
gboolean gj_gtk_eg_load(const gchar *jid)
{
  GladeXML *xml = NULL;

  if(jid == NULL)
    {
      g_warning("gj_gtk_eg_load: jid was NULL");
      return FALSE;
    }

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "edit_groups",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_eg_load: couldn't open glade file at root 'edit_groups'");
      return FALSE;
    }
  
  ctlEG.window = glade_xml_get_widget(xml,"edit_groups");

  ctlEG.treeview = glade_xml_get_widget(xml,"treeview_eg");

  ctlEG.label_name = glade_xml_get_widget(xml, "label_eg_name");
  ctlEG.label_jid = glade_xml_get_widget(xml, "label_eg_jid");
  ctlEG.label_resource = glade_xml_get_widget(xml, "label_eg_resource");
  ctlEG.label_agent = glade_xml_get_widget(xml, "label_eg_agent");
  ctlEG.label_agent_label = glade_xml_get_widget(xml, "label_eg_agent_label");

  ctlEG.entry_add = glade_xml_get_widget(xml,"entry_eg_add");
  ctlEG.button_add = glade_xml_get_widget(xml,"button_eg_add");

  ctlEG.button_ok = glade_xml_get_widget(xml,"button_eg_ok");
  
#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (ctlEG.window), "response",
                      GTK_SIGNAL_FUNC (on_edit_groups_response),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlEG.entry_add), "changed",
                      GTK_SIGNAL_FUNC (on_entry_eg_add_changed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlEG.button_add), "clicked",
                      GTK_SIGNAL_FUNC (on_button_eg_add_clicked),
                      NULL);
#endif

  /* set local info */
  optEG.jid = g_strdup(jid);
  optEG.next_group_id = 0;
  optEG.changes_made = FALSE;

  gj_gtk_eg_setup((gchar*)jid);

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_eg_unload()
{
  if(ctlEG.window == NULL)
    return FALSE;

  if(optEG.jid != NULL)
    {
      g_free(optEG.jid);
      optEG.jid = NULL;
    }

  optEG.next_group_id = 0;
  optEG.changes_made = FALSE;

  gtk_widget_destroy(ctlEG.window);
  ctlEG.window = NULL;
  ctlEG.treeview = NULL;
  ctlEG.label_name = NULL;
  ctlEG.label_jid = NULL;
  ctlEG.label_resource = NULL;
  ctlEG.label_agent = NULL;
  ctlEG.label_agent_label = NULL;
  ctlEG.button_add = NULL;
  ctlEG.button_ok = NULL;

  return TRUE;
}

gboolean gj_gtk_eg_setup(gchar *jid)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkListStore *store = NULL;
  GtkTreeSelection *selection = NULL;

  GList *groups = NULL;
  gboolean success = FALSE;
  gint i = 0;

  rosterItem ri = NULL;

  if(jid == NULL)
    {
      g_warning("gj_gtk_eg_setup: jid was NULL");
      return FALSE;
    }

  if((ri = rosterItemFindByJID(jid)) == NULL)
    {
      g_warning("gj_gtk_eg_setup: failed to find roster item by jid:'%s'", jid);
      return FALSE;
    }

  if((view = GTK_TREE_VIEW(ctlEG.treeview)) == NULL)
    {
      g_warning("gj_gtk_eg_setup: view is NULL");
      return FALSE;
    }

  if((selection = GTK_TREE_SELECTION(gtk_tree_view_get_selection(view))) == NULL)
    {
      g_warning("gj_gtk_eg_setup: selection was NULL");
      return FALSE;
    }

  /* set up for user */
  if(rosterItemGetName(ri) != NULL)
    gtk_label_set_markup(GTK_LABEL(ctlEG.label_name), rosterItemGetName(ri));
  if(rosterItemGetJID(ri) != NULL)
    gtk_label_set_markup(GTK_LABEL(ctlEG.label_jid), rosterItemGetJID(ri));
  if(rosterItemGetResource(ri) != NULL)
    gtk_label_set_markup(GTK_LABEL(ctlEG.label_resource), rosterItemGetResource(ri));
  if(rosterItemGetAgent(ri) != NULL)
    gtk_label_set_markup(GTK_LABEL(ctlEG.label_agent), rosterItemGetAgent(ri));

  /* create tree store */
  store = gtk_list_store_new(COL_EDIT_GROUPS_COUNT,
			     G_TYPE_STRING,   /* name */
			     G_TYPE_BOOLEAN,  /* enabled */
			     G_TYPE_BOOLEAN,  /* visible */
			     G_TYPE_BOOLEAN); /* editable */
  
  /* model */
  model = GTK_TREE_MODEL(store);
  gtk_tree_view_set_model(view, model);

  /* selection */
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  /* populate columns */
  gj_gtk_eg_model_populate_columns(view);

  /* properties */
  gtk_tree_view_set_enable_search(view,TRUE);
  gtk_tree_view_set_search_column(view,COL_EDIT_GROUPS_NAME); 
  gtk_tree_view_set_rules_hint(view, FALSE);
  gtk_tree_view_set_headers_visible(view,FALSE);
  gtk_tree_view_set_headers_clickable(view,FALSE);
  gtk_tree_view_columns_autosize(view);
  gtk_tree_view_expand_all(view);

  /* populate model */
  if((success = gj_gtk_eg_model_populate()) == FALSE)
    {
      g_warning("gj_gtk_eg_setup: failed to setup roster");
      return FALSE;
    }

  /* clean up */
  g_object_unref(G_OBJECT(model));

  /* get groups */
  if((groups = rosterItemGetGroups(ri)) == NULL)
    {
      g_message("gj_gtk_eg_setup: no groups selected for jid:'%s'", jid);
      return TRUE;
    }
  
  /* set store */
  for(i=0;i<g_list_length(groups);i++)
    {
      gchar *group = NULL;
      GtkTreeIter *iter = NULL;

      if((group = (gchar*) g_list_nth_data(groups, i)) == NULL)
	continue;
      else
	g_message("gj_gtk_eg_setup: checking group:'%s'",group);

      if((iter = gj_gtk_eg_model_find_iter_by_name(group)) == NULL)
	{
	  g_warning("gj_gtk_eg_setup: could not find group:'%s' in list", group);
	  continue;
	}

      gtk_list_store_set(store, iter,    
			 COL_EDIT_GROUPS_ENABLED, TRUE,
			 -1);    

      /* clean up */
      gtk_tree_iter_free(iter);
    }
  
  return TRUE;
}

gboolean gj_gtk_eg_save()
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeIter iter;

  gboolean valid = FALSE;
  gchar *jid = NULL;

/*   presence pres = NULL; */
  rosterItem ri = NULL;

  if((ri = rosterItemFindByJID(gj_gtk_eg_get_jid())) == NULL)
    {
      g_warning("gj_gtk_eg_save: could not find roster item by jid:'%s'", gj_gtk_eg_get_jid());
      return FALSE;
    }

  if((view = GTK_TREE_VIEW(ctlEG.treeview)) == NULL)
    {
      g_warning("gj_gtk_eg_save: view is NULL");
      return FALSE;
    }

  if((model = GTK_TREE_MODEL(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_eg_save: model was NULL");
      return FALSE;
    }

  /* free previously allocated roster groups */
  rosterItemGroupsFree(ri);

  /* Get the first iter in the list */
  valid = gtk_tree_model_get_iter_first(model, &iter);

  for(; valid == TRUE; valid = gtk_tree_model_iter_next(model, &iter))
    {
      gboolean enabled = FALSE;
      gchar *g = NULL;

      /* get toggled iter */
      gtk_tree_model_get(model, &iter, 
			 COL_EDIT_GROUPS_NAME, &g,
			 COL_EDIT_GROUPS_ENABLED, &enabled, 
			 -1);

      if(g != NULL)
	{
	  /* should we add it to the list */
	  if(enabled == TRUE)
	    rosterItemGroupsAdd(ri, g);

	  g_free(g);
	}
    }

  /* NOTE: at this point we need to save the new groups to the 
           jabber server the user is registered with 
  */

/*   pres = rosterItemGetPresence(ri); */
/*   jid = g_strdup_printf("%s%s%s", */
/* 			rosterItemGetJID(ri),  */
/* 			presenceGetFromResource(pres)?"/":"", */
/* 			presenceGetFromResource(pres)?presenceGetFromResource(pres):""); */

/*   if(jid == NULL) */
/*     { */
/*       g_warning("gj_gtk_eg_save: could not create jid with jid and presence resource to save groups"); */
/*       return FALSE; */
/*     } */

  infoqueryRequestUserChange(rosterItemGetJID(ri), 
			     rosterItemGetName(ri), 
			     rosterItemGetGroups(ri),
			     gj_gtk_eg_user_changed_response);

  /* clean up */
  g_free(jid);

  return TRUE;
}

void gj_gtk_eg_update_buttons()
{
  gtk_widget_set_sensitive(GTK_WIDGET(ctlEG.button_ok), gj_gtk_eg_get_changes_made());
}

gboolean gj_gtk_eg_model_populate_columns(GtkTreeView *view)
{
  GtkTreeModel *model = NULL;
  GtkTreeViewColumn *column = NULL; 
  GtkCellRenderer *renderer = NULL;

  guint col_offset = 0;

  if(GTK_TREE_VIEW(view) == NULL)
    {
      g_warning("gj_gtk_eg_model_populate_columns: view was NULL");
      return FALSE;
    }
  
  if((model = GTK_TREE_MODEL(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_eg_model_populate_columns: model was NULL");
      return FALSE;
    }

  /* COL_ROSTER_GROUP_ENABLED */
  renderer = gtk_cell_renderer_toggle_new();
  g_signal_connect(G_OBJECT(renderer), "toggled", G_CALLBACK(on_treeview_eg_cell_toggled), NULL);

  column = gtk_tree_view_column_new_with_attributes("Select",
						    renderer,
						    "active", COL_EDIT_GROUPS_ENABLED,
						    NULL);

  /* set this column to a fixed sizing (of 50 pixels) */
  gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column), GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(column), 50);
  gtk_tree_view_append_column(view, column);

  /* COL_ROSTER_GROUP_NAME */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, "Group",
							   renderer, 
							   "text", COL_EDIT_GROUPS_NAME,
							   NULL);
  
  /* set extra information */
  g_object_set_data(G_OBJECT(renderer), "column", (gint *)COL_EDIT_GROUPS_NAME);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_EDIT_GROUPS_NAME);
  gtk_tree_view_column_set_resizable(column,FALSE);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  return TRUE;
}

gboolean gj_gtk_eg_model_populate()
{
  GtkTreeView *view = NULL;
  GtkListStore *store = NULL;
  GList *groups = NULL;
  gint i = 0;

  GtkTreeIter iter;

  if((view = GTK_TREE_VIEW(ctlEG.treeview)) == NULL)
    {
      g_warning("gj_gtk_eg_model_populate: view is NULL");
      return FALSE;
    }

  if((store = GTK_LIST_STORE(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_eg_model_populate: store was NULL");
      return FALSE;
    }

  if((groups = rosterGetAllGroups()) == NULL)
    {
      g_warning("gj_gtk_eg_model_populate: there are no groups");
      return TRUE;
    }

  g_message("gj_gtk_eg_model_populate: adding %d groups", g_list_length(groups));  

  for(i=0;i<g_list_length(groups);i++)
    {
      /* add */
      gtk_list_store_append(store, &iter);
      gtk_list_store_set(store, &iter,    
			 COL_EDIT_GROUPS_NAME, (gchar*)g_list_nth_data(groups,i),
			 COL_EDIT_GROUPS_ENABLED, FALSE,
			 COL_EDIT_GROUPS_EDITABLE, TRUE,
			 COL_EDIT_GROUPS_VISIBLE, TRUE,
			 -1);

      g_free(g_list_nth_data(groups,i));
    }

  /* clean up */
  g_list_free(groups);
  
  return TRUE;
}

gboolean gj_gtk_eg_model_get_selection(gchar **name, gboolean *enabled)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeSelection *selection = NULL;
  GtkTreeIter iter;

  gboolean is_selection = FALSE;

  if((view = GTK_TREE_VIEW(ctlEG.treeview)) == NULL)
    {
      g_warning("gj_gtk_eg_model_get_selection: view was NULL");
      return FALSE;
    }
  
  if((model = GTK_TREE_MODEL(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_eg_model_get_selection: model was NULL");
      return FALSE;
    }

  if((selection = GTK_TREE_SELECTION(gtk_tree_view_get_selection(view))) == NULL)
    {
      g_warning("gj_gtk_eg_model_get_selection: selection was NULL");
      return FALSE;
    }
  
  if(name == NULL || enabled == NULL)
    {
      g_warning("gj_gtk_eg_model_get_selection: name or enabled pointers passed were NULL");
      return FALSE;
    }

  is_selection = gtk_tree_selection_get_selected(selection,&model,&iter);
  if(is_selection == FALSE)
    return FALSE;

  gtk_tree_model_get(model, &iter, 
		     COL_EDIT_GROUPS_NAME, name,
		     COL_EDIT_GROUPS_ENABLED, enabled,
		     -1);     

  return TRUE;
}

GtkTreeIter *gj_gtk_eg_model_find_iter_by_name(gchar *name)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeIter *iter_copy = NULL;
  GtkTreeIter iter;
  
  gboolean valid = FALSE;
  gchar *text = NULL;

  if((view = GTK_TREE_VIEW(ctlEG.treeview)) == NULL)
    {
      g_warning("gj_gtk_eg_model_find_iter_by_name: view was NULL");
      return NULL;
    }
  
  if((model = GTK_TREE_MODEL(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_eg_model_find_iter_by_name: model was NULL");
      return NULL;
    }

  if(name == NULL)
    {
      g_warning("gj_gtk_eg_model_find_iter_by_name: name was NULL");
      return NULL;
    }

  /* Get the first iter in the list */
  valid = gtk_tree_model_get_iter_first(model, &iter);

  for(; valid == TRUE && iter_copy == NULL; valid = gtk_tree_model_iter_next(model, &iter))
    {
      gtk_tree_model_get(model, &iter, 
			 COL_EDIT_GROUPS_NAME, &text,
			 -1);

      g_message("gj_gtk_eg_model_find_iter_by_name: comparing '%s' with '%s'",name,text);
      if(text != NULL && g_strncasecmp(name, text, strlen(name)) == 0)
	iter_copy = gtk_tree_iter_copy(&iter);

      if(text == NULL) g_free(text);
    }
  
  return iter_copy;
}

/* callbacks */
void gj_gtk_eg_user_changed_response(guint id)
{
  infoquery iq = NULL;

  g_message("gj_gtk_eg_user_changed_response: response with requestID:%d",id);

  if((iq = infoqueryFindByEventIDRelated(id)) == NULL)
    {
      g_warning("gj_gtk_eg_user_changed_response: failed to find IQ based on responseID:%d",id);

      return;
    }

  /* clean up */
  infoqueryFreeEventAndRelatedByRelatedID(id);
}

/*
 * GUI events
 */


void on_treeview_eg_cell_toggled(GtkCellRendererToggle *cell, gchar *path_string, gpointer data)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkListStore *store = NULL;
  GtkTreePath *path = NULL;
  GtkTreeIter iter;

  gboolean enabled = FALSE;

  if((view = GTK_TREE_VIEW(ctlEG.treeview)) == NULL)
    {
      g_warning("gj_gtk_eg_cell_toggled: view was NULL");
      return;
    }
  
  if((model = GTK_TREE_MODEL(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_eg_cell_toggled: model was NULL");
      return;
    }

  if((store = GTK_LIST_STORE(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_eg_cell_toggled: store was NULL");
      return;
    }

  path = gtk_tree_path_new_from_string(path_string);

  /* get toggled iter */
  gtk_tree_model_get_iter(model, &iter, path);
  gtk_tree_model_get(model, &iter, 
		     COL_EDIT_GROUPS_ENABLED, &enabled, 
		     -1);

  /* do something */
  enabled ^= 1;

  /* set new value */
  gtk_list_store_set(store, &iter, 
		     COL_EDIT_GROUPS_ENABLED, enabled, 
		     -1);

  /* clean up */
  gtk_tree_path_free(path);

  g_message("on_treeview_eg_cell_toggled: item is now %s", enabled?"enabled":"disabled");

  gj_gtk_eg_set_changes_made(TRUE);
  gj_gtk_eg_update_buttons();
}

void on_entry_eg_add_changed(GtkEditable *editable, gpointer user_data)
{
  gchar *group = NULL;
  group = gtk_editable_get_chars(editable, 0, -1); 

  if(group == NULL || strlen(group) < 1)
    gtk_widget_set_sensitive(GTK_WIDGET(ctlEG.button_add), FALSE);
  else
    gtk_widget_set_sensitive(GTK_WIDGET(ctlEG.button_add), TRUE);

  /* clean up */
  if(group != NULL) g_free(group);
}


void on_button_eg_add_clicked(GtkMenuItem *menuitem, gpointer user_data)
{
  GtkTreeView *view = NULL;
  GtkListStore *store = NULL;
  GtkTreeIter iter;

  G_CONST_RETURN gchar *group = NULL;

  if((view = GTK_TREE_VIEW(ctlEG.treeview)) == NULL)
    {
      g_warning("gj_gtk_eg_add_clicked: view was NULL");
      return;
    }
  
  if((store = GTK_LIST_STORE(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_eg_add_clicked: store was NULL");
      return;
    }
  
  /* get text */
  group = gtk_entry_get_text(GTK_ENTRY(ctlEG.entry_add));

  /* add */
  gtk_list_store_append(store, &iter);
  gtk_list_store_set(store, &iter,    
		     COL_EDIT_GROUPS_NAME, group,
		     COL_EDIT_GROUPS_ENABLED, TRUE,
		     COL_EDIT_GROUPS_EDITABLE, TRUE,
		     COL_EDIT_GROUPS_VISIBLE, TRUE,
		     -1);

  /* remove text */
  gtk_entry_set_text(GTK_ENTRY(ctlEG.entry_add), "");

  gj_gtk_eg_set_changes_made(TRUE);
  gj_gtk_eg_update_buttons();
}

void on_edit_groups_response(GtkDialog *dialog, gint response_id, gpointer user_data)
{
  switch(response_id)
    {
    case GTK_RESPONSE_OK:
      gj_gtk_eg_save();
      break;

    case GTK_RESPONSE_HELP:
      break;

    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
    case GTK_RESPONSE_APPLY:
      break;
    }

  gj_gtk_eg_unload();
}

#ifdef __cplusplus
}
#endif
