/* 
 * $RCSfile: gj_gtk_event_viewer.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/07/02 09:58:19 $
 * 
 * $Revision: 1.7 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_event.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_translate.h"

#include "gj_gtk_event_viewer.h"

/* columns */
enum
  { 
    COL_EVENT_IMAGE,
    COL_EVENT_JID,
    COL_EVENT_NAME,
    COL_EVENT_ID,
    COL_EVENT_TYPE,
    COL_EVENT_TYPE_STR,
    COL_EVENT_DETAILS,
    COL_EVENT_TIME,
    COL_EVENT_NEW,
    COL_EVENT_COUNT
  };


struct 
{
  GtkWidget *window;
  
  GtkWidget *treeview;
  GtkWidget *checkbutton_show_jid;

} ctlEV;


gboolean gj_gtk_ev_setup();

/*
 * model functions 
 */
gboolean gj_gtk_ev_model_setup();
gboolean gj_gtk_ev_model_populate_columns(GtkTreeView *view);
gboolean gj_gtk_ev_model_get_selection(gchar **jid);
gboolean gj_gtk_ev_model_details_from_event(event ev, gchar **details, gboolean *warning);

/* gui callbacks */
void on_checkbutton_ev_show_jid_toggled(GtkToggleButton *togglebutton, gpointer data);

gboolean on_event_viewer_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * code
 */
gboolean gj_gtk_ev_load(gboolean show)
{
  GladeXML *xml = NULL;

  if(ctlEV.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlEV.window));
      return FALSE;
    }

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "event_viewer",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_ev_load: couldn't open glade file at root 'event_viewer'");
      return FALSE;
    }
  
  ctlEV.window = glade_xml_get_widget(xml,"event_viewer");

  ctlEV.treeview = glade_xml_get_widget(xml,"treeview_ev");
  ctlEV.checkbutton_show_jid = glade_xml_get_widget(xml, "checkbutton_ev_show_jid");

#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (ctlEV.window), "delete_event",
                      GTK_SIGNAL_FUNC (on_event_viewer_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlEV.checkbutton_show_jid), "toggled",
                      GTK_SIGNAL_FUNC (on_checkbutton_ev_show_jid_toggled),
                      NULL);
#endif

  /* setup controls */
  gj_gtk_ev_setup();

  if(show == TRUE)
    gtk_window_present(GTK_WINDOW(ctlEV.window));

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_ev_unload()
{
  gtk_widget_hide(GTK_WIDGET(ctlEV.window));

  return TRUE;
}

gboolean gj_gtk_ev_setup()
{
  /* set up list */
  gj_gtk_ev_model_setup();

  return TRUE;
}

/*
 * model functions 
 */
gboolean gj_gtk_ev_model_setup()
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkListStore *store = NULL;
  GtkTreeSelection *selection = NULL;

  if((view = GTK_TREE_VIEW(ctlEV.treeview)) == NULL)
    {
      g_warning("gj_gtk_ev_model_setup: view was NULL");
      return FALSE;
    }

  if((selection = GTK_TREE_SELECTION(gtk_tree_view_get_selection(view))) == NULL)
    {
      g_warning("gj_gtk_ev_model_setup: selection was NULL");
      return FALSE;
    }

  /* create store */
  store = gtk_list_store_new(COL_EVENT_COUNT,
			     G_TYPE_STRING,    /* stock_id */
			     G_TYPE_STRING,    /* jid */
			     G_TYPE_STRING,    /* name - friendly version */
			     G_TYPE_INT,       /* event id */
			     G_TYPE_INT,       /* type */
			     G_TYPE_STRING,    /* type string */
			     G_TYPE_STRING,    /* details */	
			     G_TYPE_STRING,    /* time */
			     G_TYPE_BOOLEAN);  /* new */

  /* model */
  model  = GTK_TREE_MODEL(store);
  gtk_tree_view_set_model(view, model);

  /* selection */
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  /* columns */
  gj_gtk_ev_model_populate_columns(view);
  
  /* properties */
  gtk_tree_view_set_enable_search(view, TRUE);
  gtk_tree_view_set_search_column(view, COL_EVENT_TIME); 
  gtk_tree_view_set_rules_hint(view, FALSE);
  gtk_tree_view_set_headers_visible(view, TRUE);
  gtk_tree_view_set_headers_clickable(view, TRUE);

  /* clean up */
  g_object_unref(G_OBJECT(store));

  return TRUE;
}

gboolean gj_gtk_ev_model_populate_columns(GtkTreeView *view)
{
  GtkTreeViewColumn *column = NULL; 
  GtkCellRenderer *renderer = NULL;
  GtkCellRenderer *renderer_image = NULL;
  GtkCellRenderer *renderer_text = NULL;
  guint col_offset = 0;

  if(GTK_TREE_VIEW(view) == NULL)
    {
      g_warning("gj_gtk_ev_model_populate_columns: view was NULL");
      return FALSE;
    }  

  /* cell renderer's */
  renderer_image = gtk_cell_renderer_pixbuf_new();
  renderer_text = gtk_cell_renderer_text_new();

  column = gtk_tree_view_column_new();
  gtk_tree_view_column_set_title (column, "Name");

  /* COL_EVENT_IMAGE */
  gtk_tree_view_column_pack_start(column, renderer_image, FALSE);
  gtk_tree_view_column_add_attribute(column,
				     renderer_image,
				     "stock_id",COL_EVENT_IMAGE);  

  /* COL_EVENT_NAME */
  gtk_tree_view_column_pack_end(column, renderer_text, TRUE);
  gtk_tree_view_column_add_attribute(column,
				     renderer_text,
				     "text",COL_EVENT_NAME);

  /* insert column */
  gtk_tree_view_insert_column(GTK_TREE_VIEW(view), column, -1);

  gtk_tree_view_column_set_sort_column_id(column, COL_EVENT_NAME);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  /* COL_EVENT_JID */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, "JID",
							   renderer, 
							   "text", COL_EVENT_JID,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_EVENT_JID);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_EVENT_JID);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_column_set_visible(GTK_TREE_VIEW_COLUMN(column), FALSE);

  /* COL_EVENT_TIME */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, "Time",
							   renderer, 
							   "text", COL_EVENT_TIME,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_EVENT_TIME);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_EVENT_ID);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  /* COL_EVENT_TYPE_STR */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, "Type",
							   renderer, 
							   "text", COL_EVENT_TYPE_STR,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_EVENT_TYPE_STR);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_EVENT_TYPE_STR);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  /* COL_EVENT_DETAILS */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, "Details",
							   renderer, 
							   "markup", COL_EVENT_DETAILS,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_EVENT_DETAILS);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_EVENT_DETAILS);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  return TRUE;
}

gboolean gj_gtk_ev_model_get_selection(gchar **jid)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeSelection *selection = NULL;
  GtkTreeIter iter;
  
  if(jid == NULL)
    {
      g_warning("gj_gtk_ev_model_get_selection: jid pointer passed was NULL");
      return FALSE;
    }

  if((view = GTK_TREE_VIEW(ctlEV.treeview)) == NULL)
    {
      g_warning("gj_gtk_ev_model_get_selection: view was NULL");
      return FALSE;
    }

  if((model = GTK_TREE_MODEL(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_ev_model_get_selection: model was NULL");
      return FALSE;
    }

  if((selection = GTK_TREE_SELECTION(gtk_tree_view_get_selection(view))) == NULL)
    {
      g_warning("gj_gtk_ev_model_get_selection: selection was NULL");
      return FALSE;
    }

  if(gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE)
    return FALSE;
  
  gtk_tree_model_get(model, &iter, 
		     COL_EVENT_JID, jid,
		     -1);     

  return TRUE;
}

gboolean gj_gtk_ev_model_add_from_event(event ev)
{
  GtkTreeView *view = NULL;
  GtkListStore *store = NULL;
  GtkTreeSelection *selection = NULL;
  GtkTreeIter iter;

  gchar *stock_id = NULL;
  gchar *details = NULL;
  gboolean warning = FALSE;

  gchar *name = NULL;
  gchar *groups = NULL;
  rosterItem ri = NULL;

  if(ev == NULL)
    {
      g_warning("gj_gtk_ev_model_add_from_event: ev was NULL");
      return FALSE;
    }

  if((view = GTK_TREE_VIEW(ctlEV.treeview)) == NULL)
    {
      g_warning("gj_gtk_ev_model_add_from_event: view was NULL");
      return FALSE;
    }

  if((store = GTK_LIST_STORE(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_ev_model_add_from_event: store was NULL");
      return FALSE;
    }

  if((ri = rosterItemFindByJID(eventGetJID(ev))) == NULL)
    {
      g_message("gj_gtk_ev_model_add_from_event: could not get roster item from event jid:'%s', trying WITHOUT resource...", eventGetJID(ev));
      
      if((ri = rosterItemFindByJIDWithoutResource(eventGetJID(ev))) == NULL)
	{
	  g_message("gj_gtk_ev_model_add_from_event: could not get roster item from event jid:'%s'", eventGetJID(ev));
	  return FALSE;
	}    
    }
  
  if(gj_gtk_ev_model_details_from_event(ev, &details, &warning) == TRUE)
    {
      /* set pb */
      if(warning == FALSE)
	stock_id = GTK_STOCK_DIALOG_INFO;
      else
	stock_id = GTK_STOCK_DIALOG_ERROR;
    }
  else
    g_warning("gj_gtk_ev_model_add_from_event: failed to get details, event may be incomplete.");


  groups = rosterItemGetGroupsAsString(ri);
  name = g_strdup_printf("%s %s%s%s",
			  rosterItemGetName(ri),
			  groups?"[":"",
			  groups?groups:"",
			  groups?"]":"");
  
  /* add */ 
  gtk_list_store_append(store, &iter);
  gtk_list_store_set(store, &iter,    
		     COL_EVENT_IMAGE, stock_id,
		     COL_EVENT_JID, eventGetJID(ev),
		     COL_EVENT_NAME, name,
		     COL_EVENT_ID, eventGetID(ev),
		     COL_EVENT_TYPE, eventGetType(ev),
		     COL_EVENT_TYPE_STR, translateEventType(eventGetType(ev)), 
		     COL_EVENT_DETAILS, details,
		     COL_EVENT_TIME, supportGetTimestampFromTimeT(eventGetTime(ev)),
		     COL_EVENT_NEW, TRUE,
		     -1);

  /* clean up */
  if(groups != NULL) g_free(groups);
  if(name != NULL) g_free(name);
  if(details != NULL) g_free(details);
/*   if(pb != NULL) g_object_unref(pb); */
    
  return TRUE;
}

gboolean gj_gtk_ev_model_details_from_event(event ev, gchar **details, gboolean *warning)
{
  presence pres = NULL;
  message msg = NULL;

  if(ev == NULL)
    {
      g_warning("gj_gtk_ev_model_details_from_event: event was NULL");
      return FALSE;
    }
  
  if(details == NULL)
    {
      g_warning("gj_gtk_ev_model_details_from_event: details pointer was NULL");
      return FALSE;
    }

  if(warning == NULL)
    {
      g_warning("gj_gtk_ev_model_details_from_event: warning pointer was NULL");
      return FALSE;
    }

  switch(eventGetType(ev))
    {
    case eventTypePresence:
      if((pres = eventGetData(ev)) == NULL)
	{
	  g_warning("gj_gtk_ev_model_details_from_event: presence was NULL");
	  return FALSE;
	}

      if(presenceGetType(pres) == presenceTypeError)
	*warning = TRUE;
      else
	*warning = FALSE;

      *details = g_strdup_printf("type:<b>%s</b>, show:<b>%s</b>%s%s%s%s%s%s%s%s%s", 
				translatePresenceType(presenceGetType(pres)), 
				translatePresenceShow(presenceGetShow(pres)), 
				presenceGetStatus(pres)!=NULL?", status:<b>":"",
				presenceGetStatus(pres)!=NULL?presenceGetStatus(pres):"",
				presenceGetStatus(pres)!=NULL?"</b>":"",
				presenceGetErrorCode(pres)!=NULL?" error code:<b>":"",
				presenceGetErrorCode(pres)!=NULL?presenceGetErrorCode(pres):"",
				presenceGetErrorCode(pres)!=NULL?"</b>":"",
				presenceGetErrorReason(pres)!=NULL?", error reason:<b>":"",
				presenceGetErrorReason(pres)!=NULL?presenceGetErrorReason(pres):"",
				presenceGetErrorReason(pres)!=NULL?"</b>":"");
      
      break;

    case eventTypeMessage:
      if((msg = eventGetData(ev)) == NULL)
	{
	  g_warning("gj_gtk_ev_model_details_from_event: message was NULL");
	  return FALSE;
	}

      if(messageGetType(msg) == messageTypeError)
	*warning = TRUE;
      else
	*warning = FALSE;

      *details = g_strdup_printf("type:<b>%s</b> %s%s%s%s%s%s%s%s%s%s%s%s", 
				 translateMessageType(messageGetType(msg)), 
				 messageGetThread(msg)!=NULL?", thread:<b>":"",
				 messageGetThread(msg)!=NULL?messageGetThread(msg):"",
				 messageGetThread(msg)!=NULL?"</b>":"",
				 messageGetType(msg)==messageTypeXEvent?", xevent:<b>":"", 
				 messageGetType(msg)==messageTypeXEvent?translateMessageXEventType(messageGetXEventType(msg)):"",
				 messageGetType(msg)==messageTypeXEvent?"</b>":"",
				 messageGetErrorCode(msg)!=NULL?", error code:<b>":"",
				 messageGetErrorCode(msg)!=NULL?messageGetErrorCode(msg):"",
				 messageGetErrorCode(msg)!=NULL?"</b>":"",
				 messageGetErrorReason(msg)!=NULL?", error reason:<b>":"",
				 messageGetErrorReason(msg)!=NULL?messageGetErrorReason(msg):"",
				 messageGetErrorReason(msg)!=NULL?"</b>":"");
      
      break;

    case eventTypeFile:
      break;
    }
  
  return TRUE;
}

/* 
 * gui callbacks
 */
void on_checkbutton_ev_show_jid_toggled(GtkToggleButton *togglebutton, gpointer data)
{
  GtkTreeView *view = NULL;
  GtkTreeViewColumn *column = NULL;

  gboolean visible = FALSE;

  if((view = GTK_TREE_VIEW(ctlEV.treeview)) == NULL)
    {
      g_warning("%s: could not get tree view", __FUNCTION__);
      return;
    }

  visible = gtk_toggle_button_get_active(togglebutton);
  column = gtk_tree_view_get_column(view, 1);

  if(column == NULL)
    {
      g_warning("%s: could not get column", __FUNCTION__);
      return;
    }

  gtk_tree_view_column_set_visible(GTK_TREE_VIEW_COLUMN(column), visible);
}

gboolean on_event_viewer_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_ev_unload();

  return TRUE;
}

#ifdef __cplusplus
}
#endif
