/* 
 * $RCSfile: gj_gtk_headline.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/08 14:03:41 $
 * 
 * $Revision: 1.6 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_roster.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_translate.h"
#include "gj_event.h"
#include "gj_support.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_headline.h" 
#include "gj_gtk_my_preferences.h"
#include "gj_gtk_support.h"

struct t_controls
{
  GtkWidget *window;

  GtkWidget *textview;

  GtkWidget *button_clear;
  GtkWidget *button_close;

};

struct t_headlineWindow
{
  gchar *jid;

  /*   gint x; */
  /*   gint y; */
  /*   gint w; */
  /*   gint h; */

  struct t_controls controls;
};

GList *headlineWindowList;


gboolean gj_gtk_hw_load(headlineWindow hw);
gboolean gj_gtk_hw_unload(headlineWindow hw);
gboolean gj_gtk_hw_unload_by_widget(GtkWidget *widget);

gboolean gj_gtk_hw_hide(headlineWindow hw);
gboolean gj_gtk_hw_hide_by_widget(GtkWidget *widget);

void gj_gtk_hw_raise();

/* list functions */
gboolean gj_gtk_hw_list_add(headlineWindow hw);
gboolean gj_gtk_hw_list_del(headlineWindow hw);
GList *gj_gtk_hw_list_get();

/* window functions */
headlineWindow gj_gtk_hw_new(gchar *jid);
gboolean gj_gtk_hw_free(headlineWindow hw);
void gj_gtk_hw_destroy(headlineWindow hw);

gboolean gj_gtk_hw_setup(headlineWindow hw);

gboolean gj_gtk_hw_add(headlineWindow hw, message msg);
gboolean gj_gtk_hw_clear(headlineWindow hw);

/* gui callbacks */
void on_textbuffer_hw_changed(GtkTextBuffer *textbuffer, gpointer user_data);
gboolean on_textview_hw_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data);
gboolean on_textview_hw_find_space(gunichar ch, gpointer user_data);
gboolean on_textview_hw_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data);

void on_button_hw_clear_clicked(GtkButton *button, gpointer user_data);
void on_button_hw_close_clicked(GtkButton *button, gpointer user_data);

gboolean on_headline_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * code
 */

/* list handling */
gboolean gj_gtk_hw_list_add(headlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_list_add: hw was NULL");
      return FALSE;
    }

  headlineWindowList = g_list_append(headlineWindowList,hw);
  return TRUE;
}

gboolean gj_gtk_hw_list_del(headlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_list_del: hw was NULL");
      return FALSE;
    }

  headlineWindowList = g_list_remove(headlineWindowList,hw);
  return TRUE;
}

GList *gj_gtk_hw_list_get()
{
  return headlineWindowList;
}

headlineWindow gj_gtk_hw_new(gchar *jid)
{
  headlineWindow hw = NULL;

  if(jid == NULL)
    {
      g_warning("gj_gtk_hw_new: jid was NULL");
      return NULL;
    }

  if((hw = (headlineWindow) g_new(struct t_headlineWindow,1)) != NULL)
    {
      hw->jid = g_strdup(jid);

      hw->controls.window = NULL;
      hw->controls.textview = NULL;
      hw->controls.button_clear = NULL;
      hw->controls.button_close = NULL;

      gj_gtk_hw_list_add(hw);
    }

  return hw;
}

gboolean gj_gtk_hw_free(headlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_free: hw was NULL");
      return FALSE;
    }

  /* free from list */
  gj_gtk_hw_list_del(hw);
  
  /* destroy members */
  gj_gtk_hw_destroy(hw);

  /* free struct */
  g_free(hw);
  
  return TRUE;
}

void gj_gtk_hw_destroy(headlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_destroy: hw was NULL");
      return;
    }

  if(hw->jid != NULL)
    {
      g_free(hw->jid);
      hw->jid = NULL;
    }

  hw->controls.window = NULL;
  hw->controls.textview = NULL;
  hw->controls.button_clear = NULL;
  hw->controls.button_close = NULL;
}

headlineWindow gj_gtk_hw_find_by_jid(gchar *jid)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("gj_gtk_hw_find_by_jid: JID was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(headlineWindowList);index++)
    {
      headlineWindow hw = g_list_nth_data(headlineWindowList, index);

      if(hw == NULL)
	{
	  g_warning("gj_gtk_hw_find_by_jid: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(strcmp(hw->jid,jid) == 0)
	return hw;
    }

  return NULL;  
}

/* FIX ME!!!! 
   - this is bad, basically if i can not find by jid, i use this function 
   - the problem is, if the jid is msn.jabber.org first time and then the
   next message is msn.jabber.org/registered, then i do not find the 
   window.  i need to somehow improve this. */
headlineWindow gj_gtk_hw_find_by_jid_len(gchar *jid)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("gj_gtk_hw_find_by_jid_len: JID was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(headlineWindowList);index++)
    {
      headlineWindow hw = g_list_nth_data(headlineWindowList, index);

      if(hw == NULL)
	{
	  g_warning("gj_gtk_hw_find_by_jid_len: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(strncmp(hw->jid, jid, strlen(hw->jid)) == 0)
	return hw;
    }

  return NULL;   
}

headlineWindow gj_gtk_hw_find_by_widget(GtkWidget *widget)
{
  gint index = 0;

  GtkWidget *toplevel = NULL;

  if(widget == NULL)
    {
      g_warning("gj_gtk_hw_find_by_widget: widget was NULL");
      return NULL;
    }

  /* get top level widget */
  toplevel = gtk_widget_get_toplevel(widget);

  if(toplevel == NULL)
    {
      g_warning("gj_gtk_hw_find_by_widget: top level widget was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(headlineWindowList);index++)
    {
      headlineWindow hw = g_list_nth_data(headlineWindowList, index);

      if(hw == NULL)
	{
	  g_warning("gj_gtk_hw_find_by_widget: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(hw->controls.window == toplevel)
	return hw;
    }

  return NULL;  
}

/*
 * member functions 
 */

/* sets */
gboolean gj_gtk_hw_set_jid(headlineWindow hw, gchar *jid)
{
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_set_jid: hw was NULL");
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("gj_gtk_hw_set_jid: jid was NULL");
      return FALSE;
    }

  if(hw->jid != NULL)
    {
      g_free(hw->jid);
      hw->jid != NULL;
    }

  hw->jid = g_strdup(jid);

  return TRUE;
}

/* gets */
gchar *gj_gtk_hw_get_jid(headlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_get_jid: hw was NULL");
      return NULL;
    }

  return hw->jid;
}

/*
 * window functions
 */
gboolean gj_gtk_hw_show(headlineWindow hw, gboolean show)
{
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_show: hw was NULL");
      return FALSE;
    }
  
  if(show == TRUE)
    gtk_widget_show(hw->controls.window);
  else
    gtk_widget_hide(hw->controls.window);

  return TRUE;
}

void gj_gtk_hw_raise(headlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_raise: hw was NULL");
      return;
    }

  if(hw->controls.window == NULL);
  {
    g_warning("gj_gtk_hw_raise: window was NULL");
    return;
  }

  gtk_window_present(GTK_WINDOW(hw->controls.window));
}

gboolean gj_gtk_hw_load(headlineWindow hw)
{
  GladeXML *xml = NULL;
  GdkPixbuf *pb = NULL;
  
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_load: hw was NULL");
      return FALSE;
    }

  if(hw->controls.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(hw->controls.window));
      return TRUE;
    }

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "headline",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_hw_load: couldn't open glade file at root 'headline'");
      return FALSE;
    }
  
  hw->controls.window = glade_xml_get_widget(xml,"headline");

  hw->controls.textview = glade_xml_get_widget(xml, "textview_hw");

  hw->controls.button_clear = glade_xml_get_widget(xml, "button_hw_clear");
  hw->controls.button_close = glade_xml_get_widget(xml, "button_hw_close");
  
#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (hw->controls.window), "delete_event",
                      GTK_SIGNAL_FUNC (on_headline_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (hw->controls.textview), "button_press_event",
                      GTK_SIGNAL_FUNC (on_textview_hw_button_press_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (hw->controls.textview), "motion_notify_event",
                      GTK_SIGNAL_FUNC (on_textview_hw_motion_notify_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (hw->controls.button_clear), "clicked",
                      GTK_SIGNAL_FUNC (on_button_hw_clear_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (hw->controls.button_close), "clicked",
                      GTK_SIGNAL_FUNC (on_button_hw_close_clicked),
                      NULL);
#endif

  /* set icon */
  pb = gj_gtk_support_pixbuf_new_from_file(FILE_MESSAGE_HEADLINE);
  gtk_window_set_icon(GTK_WINDOW(hw->controls.window), pb);
  g_object_unref(G_OBJECT(pb));

  /* set up model */
  gj_gtk_hw_setup(hw);

  /* 
   * cleanup 
   */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_hw_load_by_jid(const gchar *jid)
{
  headlineWindow hw = NULL;

  if(jid == NULL)
    {
      g_warning("gj_gtk_hw_load_by_jid: jid was NULL");
      return FALSE;
    }

  if((hw = gj_gtk_hw_find_by_jid((gchar*)jid)) == NULL)
    {
      if((hw = gj_gtk_hw_find_by_jid_len((gchar*)jid)) == NULL)
	{
	  g_warning("gj_gtk_hw_load_by_jid: could not find hw from jid:'%s'", jid);
	  return FALSE;
	}
    }

  gj_gtk_hw_load(hw);
  return TRUE;    
}

gboolean gj_gtk_hw_unload(headlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_unload: hw was NULL");
      return FALSE;
    }

  /* if no more msgs reset icon */
  if(hw->controls.window != NULL)
    {
      gtk_widget_destroy(GTK_WIDGET(hw->controls.window));
      hw->controls.window = NULL;
    }

  gj_gtk_hw_free(hw);
      
  g_message("gj_gtk_hw_unload: unloaded...");
  
  return TRUE;
}

gboolean gj_gtk_hw_unload_by_widget(GtkWidget *widget)
{
  headlineWindow hw = NULL;

  if(widget == NULL)
    {
      g_warning("gj_gtk_hw_unload_by_widget: widget was NULL");
      return FALSE;
    }

  if((hw = gj_gtk_hw_find_by_widget(widget)) == NULL)
    {
      g_warning("gj_gtk_hw_unload_by_widget: could not find hw from widget");
      return FALSE;
    }

  gj_gtk_hw_unload(hw);
  return TRUE;
}

gboolean gj_gtk_hw_hide(headlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_hide: hw was NULL");
      return FALSE;
    }


  gtk_widget_hide(GTK_WIDGET(hw->controls.window));
  g_message("gj_gtk_hw_hide: hidden...");
  
  return TRUE;
}


gboolean gj_gtk_hw_hide_by_widget(GtkWidget *widget)
{
  headlineWindow hw = NULL;

  if(widget == NULL)
    {
      g_warning("gj_gtk_hw_hide_by_widget: widget was NULL");
      return FALSE;
    }

  if((hw = gj_gtk_hw_find_by_widget(widget)) == NULL)
    {
      g_warning("gj_gtk_hw_hide_by_widget: could not find hw from widget");
      return FALSE;
    }

  gj_gtk_hw_hide(hw);
  return TRUE;
}

gboolean gj_gtk_hw_setup(headlineWindow hw)
{
  GtkTextBuffer *buffer = NULL;
  gchar *title = NULL;
  
  if((buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(hw->controls.textview))) == NULL)
    {
      g_warning("gj_gtk_hw_setup: could not get textview buffer");
      return FALSE;
    }
  
  /* set function callbacks */
  g_signal_connect(G_OBJECT(buffer), "changed", 
		   G_CALLBACK(on_textbuffer_hw_changed), 
		   NULL);

  /* set styles */
  gtk_text_buffer_create_tag(buffer, "bold", 
			     "weight", PANGO_WEIGHT_BOLD, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "italic", 
			     "style", PANGO_STYLE_ITALIC, 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "blue_foreground", 
			     "foreground", "darkblue", 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "red_foreground", 
			     "foreground", "darkred", 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "black_foreground", 
			     "foreground", "black", 
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "url", 
			     "foreground", "blue", 
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "url_desc", 
			     /* 			     "weight", PANGO_WEIGHT_BOLD,  */
			     "underline", PANGO_UNDERLINE_SINGLE,
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "left_margin_small",
			     "left_margin", 5,
			     NULL);
  gtk_text_buffer_create_tag(buffer, "left_margin_large",
			     "left_margin", 30,
			     NULL);
  gtk_text_buffer_create_tag(buffer, "not_editable",
			     "editable", FALSE, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "underline",
			     "underline", PANGO_UNDERLINE_SINGLE, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "tiny_gap_after_line",
			     "pixels_below_lines", 5, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "small_gap_after_line",
			     "pixels_below_lines", 15, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "big_gap_after_line",
			     "pixels_below_lines", 30, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "big_gap_before_line",
			     "pixels_above_lines", 30, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "large",
			     "scale", PANGO_SCALE_LARGE, 
			     NULL);
  
  /* set title */
  title = g_strdup_printf("Headlines for [%s]", gj_gtk_hw_get_jid(hw));
  gtk_window_set_title(GTK_WINDOW(hw->controls.window), title);

  /* clean up */
  if(title != NULL) g_free(title);

  return TRUE;
}

gboolean gj_gtk_hw_add(headlineWindow hw, message msg)
{
  GtkTextView *textview = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter;

  const gchar *time = NULL;
  const gchar *message = NULL;
    
  time = supportGetTimeAndDateFromTimeT(messageGetTime(msg));
  message = messageGetBody(msg);

  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_add: hw was NULL");
      return FALSE;
    }

  if((textview = GTK_TEXT_VIEW(hw->controls.textview)) == NULL)
    {
      g_warning("gj_gtk_hw_setup: could not get textview");
      return FALSE;
    }      
    
  if((buffer = gtk_text_view_get_buffer(textview)) == NULL)
    {
      g_warning("gj_gtk_hw_setup: could not get textview buffer");
      return FALSE;
    }    

  if(time == NULL)
    {
      g_warning("gj_gtk_hw_setup: time was NULL");
      return FALSE;
    }

  if(message == NULL)
    {
      g_warning("gj_gtk_hw_setup: message was NULL");
      return FALSE;
    }

  gtk_text_buffer_get_end_iter(buffer, &iter);

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					   time, -1,
					   "bold",
					   "black_foreground",
					   "left_margin_small",
					   "tiny_gap_after_line",
					   NULL);

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					   "\n", -1,
					   "black_foreground",
					   NULL);

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					   messageGetBody(msg), -1,
					   "red_foreground",
					   "left_margin_large",
					   NULL);

  if(messageGetXOOBURL(msg) != NULL && messageGetXOOBDescription(msg) != NULL)
    {
      /* show at end of msg */
      gtk_text_buffer_remove_all_tags(buffer, &iter, &iter);
      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					       "\n", -1,
					       "black_foreground",
					       "left_margin_large",
					       NULL);

      /* show url */      
      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					       messageGetXOOBDescription(msg), -1,
					       "left_margin_large",
					       "url_desc",
					       NULL);

      gtk_text_buffer_remove_all_tags(buffer, &iter, &iter);
      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					       "\n", -1,
					       "black_foreground",
					       NULL);

      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					       messageGetXOOBURL(msg), -1,
 					       "left_margin_large",
					       "url",
					       NULL);
    }

  gtk_text_buffer_remove_all_tags(buffer, &iter, &iter);
  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					   "\n\n\n", -1,
					   "black_foreground",
					   NULL);

  return TRUE;
}

gboolean gj_gtk_hw_clear(headlineWindow hw)
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  if(hw == NULL)
    {
      g_warning("gj_gtk_hw_clear: hw was NULL");
      return FALSE;
    }

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(hw->controls.textview));
  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  
  /* clear buffer */
  gtk_text_buffer_delete(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  gtk_text_buffer_delete_mark_by_name(GTK_TEXT_BUFFER(buffer), "url");

  return TRUE;
}

headlineWindow gj_gtk_hw_notification_new(gint eventID, message msg)
{
  headlineWindow hw = NULL;

  if(msg == NULL)
    {
      g_warning("gj_gtk_hw_notification_new: msg was NULL");
      return NULL;
    }

  hw = gj_gtk_hw_find_by_jid(messageGetFromJID(msg));

  if(hw == NULL)
    {
      if((hw = gj_gtk_hw_new(messageGetFromJID(msg))) == NULL)
	{
	  g_warning("gj_gtk_hw_notification_new: could not create new hw from jid:'%s'", messageGetFromJID(msg));
	  return NULL;
	}

      gj_gtk_hw_load(hw);
    }
  else
    {
      /* should we show it? */
      if(gj_gtk_mpf_get_raise_window_on_new_message() == TRUE)
	gj_gtk_hw_raise(hw);
    }

  gj_gtk_hw_add(hw, msg);

  return hw;
}

/*
 * GUI events
 */
void on_textbuffer_hw_changed(GtkTextBuffer *textbuffer, gpointer user_data)
{
  GtkTextMark *mark = NULL;
  GtkTextIter iter_space;
  GtkTextIter iter_word_start;
  GtkTextIter iter_word_end;
  GtkTextIter iter_start;
  GtkTextIter iter_end;
  GtkTextTagTable *tag_table = NULL;
  GtkTextTag *tag_url = NULL;

  gint length = 0;

  /* check WHOLE buffer for urls */
  gtk_text_buffer_get_bounds(textbuffer, &iter_start, &iter_end);
  length = gtk_text_buffer_get_char_count(textbuffer);

  if((mark = gtk_text_buffer_get_mark(textbuffer, "url")) != NULL)
    {
      gtk_text_buffer_get_iter_at_mark(textbuffer, &iter_start, mark);
      /*       gtk_text_mark_set_visible(mark, TRUE);  */
    }

  if((tag_table = gtk_text_buffer_get_tag_table(textbuffer)) == NULL)
    {
      g_warning("on_textbuffer_hw_changed: could not get text buffer tag table");
      return;
    }

  if((tag_url = gtk_text_tag_table_lookup(tag_table, "url")) == NULL)
    {
      g_warning("on_textbuffer_hw_changed: could not find tag:'url'");
      return;
    }

  /* save last iter */
  iter_word_start = iter_start;
  iter_word_end = iter_start;
  iter_space = iter_start;

  while(TRUE)
    {
      gboolean found = FALSE;
      gchar *buf = NULL;

      /* search for next space */
      found = gtk_text_iter_forward_find_char(&iter_space, on_textview_hw_find_space, NULL, NULL);

      if(found == FALSE)
	break;
      
      /* set word end to the space */
      iter_word_start = iter_word_end;
      iter_word_end = iter_space;

      /* move word end iter forward 1 */
      gtk_text_iter_forward_char(&iter_word_start);

      if(gtk_text_iter_has_tag(&iter_word_start, tag_url) == TRUE) 
	continue;

      /* get */
      buf = gtk_text_iter_get_text(&iter_word_start, &iter_word_end);

      if(buf != NULL) 
	{
	  if(supportIsURI(buf) != uriTypeUnknown) 
	    {
	      /* highlight as uri */
	      gtk_text_buffer_apply_tag_by_name(textbuffer, "url", &iter_word_start, &iter_word_end);
	    } 
	  else
	    {
	      /* none highlighted */
	      gtk_text_buffer_remove_tag_by_name(textbuffer, "url", &iter_word_start, &iter_word_end);
	    }

	  g_free(buf);
	}

      /* set mark, then we can check next time from 
	 the last mark and it is more efficient */
      if((mark = gtk_text_buffer_get_mark(textbuffer, "url")) == NULL)
	{
	  if((mark = gtk_text_buffer_create_mark(textbuffer, "url", &iter_space, FALSE)) == NULL)
	    {
	      g_warning("on_textbuffer_hw_changed: could not create mark!");
	      return;
	    }

	  /* no need to move it, just continue */
	  continue;
	}

      /* move mark to latest iter */
      gtk_text_buffer_move_mark_by_name(textbuffer, "url", &iter_space);
    }
}

gboolean on_textview_hw_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
{
  GdkModifierType state = 0;
  GdkWindow *window = NULL;

  GtkTextView *textview = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter, iter_start, iter_end;
  GtkTextWindowType text_window_type = 0;

  gboolean found_start = FALSE;
  gboolean found_end = FALSE;

  gint pointer_x = 0;
  gint pointer_y = 0;
  gint x = 0;
  gint y = 0;

  gchar *buf = NULL;

  headlineWindow hw = NULL;

  if(event->is_hint)
    window = gdk_window_get_pointer(event->window, &pointer_x, &pointer_y, &state);
  else
    {
      pointer_x = event->x;
      pointer_y = event->y;
      state = event->state;
    }

  if((hw = gj_gtk_hw_find_by_widget(GTK_WIDGET(widget))) == NULL)
    {
      g_warning("on_textview_hw_motion_notify_event: failed to find headlineWindow from widget");
      return FALSE;
    }
  
  if((textview = GTK_TEXT_VIEW(widget)) == NULL)
    {
      g_warning("on_textview_hw_motion_notify_event: view was NULL");
      return FALSE;
    }

  if((buffer = GTK_TEXT_BUFFER(gtk_text_view_get_buffer(textview))) == NULL)
    {
      g_warning("on_textview_hw_motion_notify_event: buffer was NULL");
      return FALSE;
    }

  text_window_type = gtk_text_view_get_window_type(textview, event->window);
  gtk_text_view_window_to_buffer_coords(textview, text_window_type, pointer_x, pointer_y, &x, &y);
  gtk_text_view_get_iter_at_location(textview, &iter, x, y);
 
  if(g_unichar_isspace(gtk_text_iter_get_char(&iter)) == TRUE)
    {
      gdk_window_set_cursor(event->window, NULL);
      return FALSE;
    }

  iter_start = iter;
  iter_end = iter;
  
  /* search for next space */
  found_start = gtk_text_iter_backward_find_char(&iter_start, on_textview_hw_find_space, NULL, NULL);
  found_end = gtk_text_iter_forward_find_char(&iter_end, on_textview_hw_find_space, NULL, NULL);
  
  if(found_start == FALSE && found_end == FALSE)
    {
      gdk_window_set_cursor(event->window, NULL);
      return FALSE;
    }
  
  if(gtk_text_iter_is_start(&iter_start) == FALSE)
    gtk_text_iter_forward_char(&iter_start);
  
  if((buf = gtk_text_buffer_get_text(buffer, &iter_start, &iter_end, FALSE)) == NULL)
    {
      g_warning("on_textview_hw_motion_notify_event: buf returned was NULL");
      return FALSE;
    }
  
  if(supportIsURI(buf) != uriTypeUnknown)
    {
      /* width wise it works... */
      GdkCursor *cursor = NULL;
      cursor = gdk_cursor_new(GDK_HAND2);
      gdk_window_set_cursor (event->window, cursor);
      
      /* clean up */
      gdk_cursor_unref(cursor);
    }  
  else
    {
      /* return to parent window's cursor */
      gdk_window_set_cursor(event->window, NULL);
    }

      
  /* clean up */
  if(buf != NULL) g_free(buf);

  return FALSE;
}

gboolean on_textview_hw_find_space(gunichar ch, gpointer user_data)
{
  if(g_unichar_isspace(ch) == TRUE)
    return TRUE;

  return FALSE;
}

gboolean on_textview_hw_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  GdkModifierType state;
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter, iter_start, iter_end;
  GtkTextWindowType text_window_type = 0;

  gboolean found_start = FALSE;
  gboolean found_end = FALSE;

  gint pointer_x = 0;
  gint pointer_y = 0;
  gint x = 0;
  gint y = 0;

  gchar *buf = NULL;
    
  /* if not mouse 1 return */
  if(event->button != 1)
    return FALSE;

  gdk_window_get_pointer(event->window, &pointer_x, &pointer_y, &state);
    
  if((buffer = GTK_TEXT_BUFFER(gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget)))) == NULL)
    {
      g_warning("on_textview_hw_button_press_event: could not get buffer.");
      return FALSE;
    }
    
  text_window_type = gtk_text_view_get_window_type(GTK_TEXT_VIEW(widget), event->window);
  gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(widget), text_window_type, pointer_x, pointer_y, &x, &y);
  gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(widget), &iter, x, y);
 
  iter_start = iter;
  iter_end = iter;
    
  /* search for next space */
  found_start = gtk_text_iter_backward_find_char(&iter_start, on_textview_hw_find_space, NULL, NULL);
  found_end = gtk_text_iter_forward_find_char(&iter_end, on_textview_hw_find_space, NULL, NULL);
    
  if(found_start == FALSE && found_end == FALSE)
    return FALSE;
    
  if(gtk_text_iter_is_start(&iter_start) == FALSE)
    gtk_text_iter_forward_char(&iter_start);
    
  if((buf = gtk_text_buffer_get_text(buffer, &iter_start, &iter_end, FALSE)) == NULL)
    {
      g_warning("on_textview_hw_button_press_event: buf returned was NULL");
      return FALSE;
    }
  
  if(supportIsURI(buf) != uriTypeUnknown)
    {
      gj_gtk_support_open_url(buf);
    }  
    
  /* clean up */
  if(buf != NULL) g_free(buf);
   
  return FALSE;
}

void on_button_hw_clear_clicked(GtkButton *button, gpointer user_data)
{
  headlineWindow hw = NULL;
  
  if((hw = gj_gtk_hw_find_by_widget(GTK_WIDGET(button))) == NULL)
    {
      g_warning("on_button_hw_clear_clicked: failed to find headlineWindow from button widget");
      return;
    }
 
  gj_gtk_hw_clear(hw);
}

void on_button_hw_close_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_hw_hide_by_widget(GTK_WIDGET(button));
}

gboolean on_headline_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_hw_hide_by_widget(GTK_WIDGET(widget));

  return FALSE;
}

#ifdef __cplusplus
}
#endif
