/* 
 * $RCSfile: gj_gtk_log.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/08 13:18:46 $
 * 
 * $Revision: 1.4 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdio.h>
#include <string.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_log.h"
#include "gj_support.h"

#include "gj_gtk_log.h"
#include "gj_gtk_support.h"

gboolean gj_gtk_log_init()
{
  /* GLib log levels 

  G_LOG_LEVEL_ERROR             = 1 << 2,        - always fatal 
  G_LOG_LEVEL_CRITICAL          = 1 << 3,
  G_LOG_LEVEL_WARNING           = 1 << 4,
  G_LOG_LEVEL_MESSAGE           = 1 << 5,
  G_LOG_LEVEL_INFO              = 1 << 6,
  G_LOG_LEVEL_DEBUG             = 1 << 7,
  
  */

  g_log_set_handler(NULL, 
		    G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL, 
		    gj_gtk_log_handler, 
		    NULL);
  g_log_set_handler("XML", 
		    G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL, 
		    gj_gtk_log_handler, 
		    NULL);

  g_log_set_handler("libglade", 
		    G_LOG_LEVEL_INFO | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL, 
		    gj_gtk_log_handler, 
		    NULL);
  g_log_set_handler("Gtk", 
		    G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL,  
		    gj_gtk_log_handler, 
		    NULL);
  g_log_set_handler("GLib", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_gtk_log_handler, 
		    NULL);
  g_log_set_handler("GModule", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_gtk_log_handler, 
		    NULL);
  g_log_set_handler("GThreads", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_gtk_log_handler, 
		    NULL);
  g_log_set_handler("Glade", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_gtk_log_handler, 
		    NULL);
  g_log_set_handler("GLib-GObject", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_gtk_log_handler, 
		    NULL);
  g_log_set_handler("GObject", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_gtk_log_handler, 
		    NULL);

  gj_gtk_log_load();
  g_message("gj_gtk_log_init: initiated");

  return TRUE;
}

gboolean gj_gtk_log_term()
{
  /* close file */
  g_message("gj_gtk_log_term: terminated");

  return TRUE;
}

void gj_gtk_log_handler(const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data)
{
  logHandlerFormatted((char*)log_domain, (int)log_level, (char*)message);
}


#define GJ_GTK_LOG_TAG_ERROR "error"
#define GJ_GTK_LOG_TAG_CRITICAL "critical"
#define GJ_GTK_LOG_TAG_WARNING "warning"
#define GJ_GTK_LOG_TAG_MESSAGE "message"
#define GJ_GTK_LOG_TAG_OTHER "other"

struct 
{
  GtkWidget *window;

  GtkWidget *textview_log;
  GtkWidget *button_close;
  GtkWidget *button_clear;
  
} ctlLog;


gboolean gj_gtk_log_setup();
gboolean gj_gtk_log_clear();

/*
 * GUI functions
 */
void on_button_log_clear_clicked(GtkButton *button, gpointer user_data);
void on_button_log_close_clicked(GtkButton *button, gpointer user_data);
gboolean on_log_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * code
 */
gboolean gj_gtk_log_load()
{
  GladeXML *xml = NULL;

  if(ctlLog.window != NULL)
    {
      gj_gtk_log_show();
      return TRUE;
    }

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "log",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_log_load: couldn't open glade file at root 'log'");
      return FALSE;
    }
  
  ctlLog.window = glade_xml_get_widget(xml,"log");
  ctlLog.textview_log = glade_xml_get_widget(xml,"textview_log");
  ctlLog.button_close = glade_xml_get_widget(xml,"button_log_close");
  ctlLog.button_clear = glade_xml_get_widget(xml,"button_log_clear");

#ifndef WIN32  
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (ctlLog.window), "delete_event",
                      GTK_SIGNAL_FUNC (on_log_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlLog.button_clear), "clicked",
                      GTK_SIGNAL_FUNC (on_button_log_clear_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlLog.button_close), "clicked",
                      GTK_SIGNAL_FUNC (on_button_log_close_clicked),
                      NULL);
#endif

  /* set up tags for log */
  gj_gtk_log_setup();

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_log_unload()
{
  gtk_widget_destroy(ctlLog.window);

  return TRUE;
}

gboolean gj_gtk_log_show()
{
  gtk_widget_show(ctlLog.window);
  return TRUE;
}

gboolean gj_gtk_log_hide()
{
  gtk_widget_hide(ctlLog.window);
  return TRUE;
}

gboolean gj_gtk_log_setup()
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  gchar *message = NULL;

  /* set buffer iterator at position 0 */
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlLog.textview_log));
  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  
  /* set styles */
  gtk_text_buffer_create_tag(buffer, "bold", 
			     "weight", PANGO_WEIGHT_BOLD, 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);
  gtk_text_buffer_create_tag(buffer, "italic", 
			     "style", PANGO_STYLE_ITALIC, 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "critical", 
			     "foreground", "darkred", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "error", 
			     "foreground", "red", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "warning", 
			     "foreground", "darkorange", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "message", 
			     "foreground", "darkblue", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "other", 
			     "foreground", "black", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "debug", 
			     "foreground", "grey", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "details", 
			     "foreground", "darkgreen", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  

  message = g_strdup_printf("Log started on %s at %s\n\n", supportGetDatestamp(), supportGetTimestamp());

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter_end,
					   message, -1,
					   "other",
					   NULL);

  g_free(message);

  return TRUE;
}

gboolean gj_gtk_log_clear()
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlLog.textview_log));
  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  
  /* clear buffer */
  gtk_text_buffer_delete(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  
  return TRUE;
}

gboolean gj_gtk_log_new(const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data)
{
  GtkTextView *textview = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkTextMark *mark = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  gchar *details = NULL;
  gchar *format = NULL;
  gchar *tag = NULL;

  if(log_level < 0 || message == NULL)
    {
      g_print("gj_gtk_log_new: failed to add new message, log level:%d was < 0 OR message was NULL\n",log_level);
      return FALSE;
    }

  if((textview = GTK_TEXT_VIEW(ctlLog.textview_log)) == NULL)
    {
      fprintf(stderr, "gj_gtk_log_new: textview was NULL");
      return FALSE;
    }

  buffer = gtk_text_view_get_buffer(textview);
  gtk_text_buffer_get_bounds(buffer, &iter_start, &iter_end);

  /* format strings */
  details = g_strdup_printf("%s - %s\t",
			    supportGetTimestamp(),
			    log_domain?log_domain:"GJ");

  format = g_strdup_printf("%s\n",message);

  /* print initial part of message */
  switch(log_level)
    {
    case G_LOG_LEVEL_ERROR: 
      tag = g_strdup_printf("error");
      break;
    case G_LOG_LEVEL_CRITICAL:
      tag = g_strdup_printf("critical");
      break;
    case G_LOG_LEVEL_WARNING:
      tag = g_strdup_printf("warning");
      break;
    case G_LOG_LEVEL_MESSAGE:
      tag = g_strdup_printf("message");
      break;
    case G_LOG_LEVEL_DEBUG:
      tag = g_strdup_printf("debug");
      break;
    case G_LOG_LEVEL_INFO:
    default:
      tag = g_strdup_printf("other");
      break;
    }

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter_end,
					   details, -1,
					   "details",
					   NULL);

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter_end,
					   format, -1,
					   tag,
					   NULL);

  mark = gtk_text_buffer_create_mark(buffer, NULL, &iter_end, FALSE);
  gtk_text_view_scroll_mark_onscreen(textview, mark);
  gtk_text_buffer_delete_mark(buffer, mark);

  if(details != NULL) g_free(details);
  if(format != NULL) g_free(format);
  if(tag != NULL) g_free(tag);

  return TRUE;
}

/*
 * GUI events
 */
void on_button_log_clear_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_log_clear();
}

void on_button_log_close_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_log_hide();
}

gboolean on_log_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_log_hide();

  return TRUE;
}

#ifdef __cplusplus
}
#endif
