/* 
 * $RCSfile: gj_gtk_message.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/24 08:21:54 $
 * 
 * $Revision: 1.8 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_GNOME
#include <gnome.h>
#endif

#include "common.h"

#include "gj_main.h"
#include "gj_roster.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_translate.h"
#include "gj_event.h"
#include "gj_support.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_message.h" 
#include "gj_gtk_my_preferences.h"
#include "gj_gtk_support.h"

struct t_controls
{
  GtkWidget *window;

  GtkWidget *entry_contact;
  GtkWidget *entry_time;
  GtkWidget *entry_subject;
  GtkWidget *textview;
  
  GtkWidget *label_subject;
  GtkWidget *label_contact;
  GtkWidget *label_next;

  GtkWidget *button_next;
  GtkWidget *button_send;
  GtkWidget *button_reply;
  GtkWidget *button_close;

};

struct t_msgWindow
{
  struct t_controls controls;

  gchar *jid;

  /* properties */
  msgWindowType type;

  gint last_msg_recv_xevent_type;
  gchar *last_msg_recv_xevent_id;

  gint last_msg_sent_xevent_type;
  gchar *last_msg_sent_xevent_id;
};

GList *msgWindowList;

GdkPixbuf *gj_gtk_mw_load_emoticon(msgEmote icon);

gboolean gj_gtk_mw_load(msgWindow mw);
gboolean gj_gtk_mw_unload(msgWindow mw);
gboolean gj_gtk_mw_unload_by_widget(GtkWidget *widget);

void gj_gtk_mw_raise();

/* list functions */
gboolean gj_gtk_mw_list_add(msgWindow mw);
gboolean gj_gtk_mw_list_del(msgWindow mw);
GList *gj_gtk_mw_list_get();

/* window functions */
msgWindow gj_gtk_mw_new(gchar *jid);
gboolean gj_gtk_mw_free(msgWindow mw);
void gj_gtk_mw_destroy(msgWindow mw);

/* utilities */
gint gj_gtk_mw_waiting_count(msgWindow mw);
void gj_gtk_mw_waiting_update(msgWindow mw);

gboolean gj_gtk_mw_send_set(msgWindow mw);
gboolean gj_gtk_mw_receive_set(msgWindow mw, message msg);

gboolean gj_gtk_mw_check_for_url(GtkTextBuffer *buffer, gchar *text, guint from_pos);

/* gui callbacks */
void on_textbuffer_mw_changed(GtkTextBuffer *textbuffer, gpointer user_data);
gboolean on_textview_mw_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data);
gboolean on_textview_mw_find_space(gunichar ch, gpointer user_data);
gboolean on_textview_mw_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data);

void on_button_mw_send_clicked(GtkButton *button, gpointer user_data);
void on_button_mw_reply_clicked(GtkButton *button, gpointer user_data);
void on_button_mw_next_clicked(GtkButton *button, gpointer user_data);
void on_button_mw_close_clicked(GtkButton *button, gpointer user_data);

gboolean on_message_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * code
 */

/* list handling */
gboolean gj_gtk_mw_list_add(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_list_add: mw was NULL");
      return FALSE;
    }

  msgWindowList = g_list_append(msgWindowList,mw);
  return TRUE;
}

gboolean gj_gtk_mw_list_del(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_list_del: mw was NULL");
      return FALSE;
    }

  msgWindowList = g_list_remove(msgWindowList,mw);
  return TRUE;
}

GList *gj_gtk_mw_list_get()
{
  return msgWindowList;
}

msgWindow gj_gtk_mw_new(gchar *jid)
{
  msgWindow mw = NULL;

  if(jid == NULL)
    {
      g_warning("gj_gtk_mw_new: remote JID was NULL");
      return NULL;
    }

  if((mw = (msgWindow) g_new(struct t_msgWindow,1)) != NULL)
    {
      mw->jid = g_strdup(jid);
      mw->type = 0;

      mw->last_msg_recv_xevent_type = 0;
      mw->last_msg_recv_xevent_id = NULL;
     
      mw->last_msg_sent_xevent_type = 0;
      mw->last_msg_sent_xevent_id = NULL;

      gj_gtk_mw_list_add(mw);
    }

  return mw;
}

gboolean gj_gtk_mw_free(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_free: mw was NULL");
      return FALSE;
    }

  /* free from list */
  gj_gtk_mw_list_del(mw);
  
  /* destroy members */
  gj_gtk_mw_destroy(mw);

  /* free struct */
  g_free(mw);
  
  return TRUE;
}

void gj_gtk_mw_destroy(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_destroy: mw was NULL");
      return;
    }

  if(mw->jid != NULL)
    {
      g_free(mw->jid);
      mw->jid = NULL;
    }

  mw->type = 0;

  mw->last_msg_recv_xevent_type = 0;

  if(mw->last_msg_recv_xevent_id != NULL)
    {
      g_free(mw->last_msg_recv_xevent_id);
      mw->last_msg_recv_xevent_id = NULL;
    }

  mw->last_msg_sent_xevent_type = 0;

  if(mw->last_msg_sent_xevent_id != NULL)
    {
      g_free(mw->last_msg_sent_xevent_id);
      mw->last_msg_sent_xevent_id = NULL;
    }
}

msgWindow gj_gtk_mw_find_by_jid(gchar *jid)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("gj_gtk_mw_find_by_jid: JID was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(msgWindowList);index++)
    {
      msgWindow mw = g_list_nth_data(msgWindowList, index);

      if(mw == NULL)
	{
	  g_warning("gj_gtk_mw_find_by_jid: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(strcmp(mw->jid,jid) == 0)
	{
	  return mw;
	}
    }

  return NULL;  
}

msgWindow gj_gtk_mw_find_by_widget(GtkWidget *widget)
{
  gint index = 0;

  GtkWidget *toplevel = NULL;

  if(widget == NULL)
    {
      g_warning("gj_gtk_mw_find_by_widget: widget was NULL");
      return NULL;
    }

  /* get top level widget */
  toplevel = gtk_widget_get_toplevel(widget);

  if(toplevel == NULL)
    {
      g_warning("gj_gtk_mw_find_by_widget: top level widget was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(msgWindowList);index++)
    {
      msgWindow mw = g_list_nth_data(msgWindowList, index);

      if(mw == NULL)
	{
	  g_warning("gj_gtk_mw_find_by_widget: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(mw->controls.window == toplevel)
	{
	  return mw;
	}
    }

  return NULL;  
}

msgWindow gj_gtk_mw_find_by_jid_and_type(gchar *jid, msgWindowType type)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("gj_gtk_mw_find_by_jid_and_type: JID was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(msgWindowList);index++)
    {
      msgWindow mw = g_list_nth_data(msgWindowList, index);

      if(mw == NULL)
	{
	  g_warning("gj_gtk_mw_find_by_jid_and_type: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(strcmp(mw->jid,jid) == 0 && mw->type == type)
	{
	  return mw;
	}
    }

  return NULL;  
}

msgWindow gj_gtk_mw_find_by_last_msg_recv_xevent_id(gchar *message_id)
{
  gint index = 0;

  if(message_id == NULL)
    {
      g_warning("gj_gtk_mw_find_by_last_msg_recv_xevent_id: message id was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(msgWindowList);index++)
    {
      msgWindow mw = g_list_nth_data(msgWindowList, index);

      if(mw == NULL)
	{
	  g_warning("gj_gtk_mw_find_by_last_msg_recv_xevent_id: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(gj_gtk_mw_get_last_msg_recv_xevent_id(mw) != NULL && 
	 strcmp(gj_gtk_mw_get_last_msg_recv_xevent_id(mw), message_id) == 0)
	return mw;
    }

  return NULL;  
}

msgWindow gj_gtk_mw_find_by_last_msg_sent_xevent_id(gchar *message_id)
{
  gint index = 0;

  if(message_id == NULL)
    {
      g_warning("gj_gtk_mw_find_by_last_msg_sent_xevent_id: message id was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(msgWindowList);index++)
    {
      msgWindow mw = g_list_nth_data(msgWindowList, index);

      if(mw == NULL)
	{
	  g_warning("gj_gtk_mw_find_by_last_msg_sent_xevent_id: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(gj_gtk_mw_get_last_msg_sent_xevent_id(mw) != NULL && 
	 strcmp(gj_gtk_mw_get_last_msg_sent_xevent_id(mw), message_id) == 0)
	return mw;
    }

  return NULL;  
}

/*
 * member functions
 */

/* gets */
gchar *gj_gtk_mw_get_jid(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_get_jid: msgWindow was NULL");
      return NULL;
    }

  return mw->jid;
}

msgWindowType gj_gtk_mw_get_type(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_get_type: msgWindow was NULL");
      return msgWindowTypeEnd;
    }

  return mw->type;
}

gint gj_gtk_mw_get_last_msg_recv_xevent_type(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_get_last_msg_recv_xevent_type: msgWindow was NULL");
      return 0;
    }

  return mw->last_msg_recv_xevent_type;
}

gchar *gj_gtk_mw_get_last_msg_recv_xevent_id(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_get_last_msg_recv_xevent_id: msgWindow was NULL");
      return NULL;
    }

  return mw->last_msg_recv_xevent_id;
}

gint gj_gtk_mw_get_last_msg_sent_xevent_type(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_get_last_msg_sent_xevent_type: msgWindow was NULL");
      return 0;
    }

  return mw->last_msg_sent_xevent_type;
}

gchar *gj_gtk_mw_get_last_msg_sent_xevent_id(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_get_last_msg_sent_xevent_id: msgWindow was NULL");
      return NULL;
    }

  return mw->last_msg_sent_xevent_id;
}

/* sets */
gboolean gj_gtk_mw_set_jid(msgWindow mw, gchar *jid)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_set_jid: msgWindow was NULL");
      return FALSE;
    }  
  
  if(jid == NULL)
    {
      g_warning("gj_gtk_mw_set_jid: jid was NULL");
      return FALSE;
    }    

  if(mw->jid != NULL)
    {
      g_free(mw->jid);
      mw->jid = NULL;
    }

  mw->jid = g_strdup(jid);
  return TRUE;
}

gboolean gj_gtk_mw_set_type(msgWindow mw, msgWindowType type)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_set_type: msgWindow was NULL");
      return FALSE;
    }

  if(type < 1 || type > msgWindowTypeEnd)
    {
      g_warning("gj_gtk_mw_set_type: type:%d was invalid", type);
      return FALSE;
    }

  mw->type = type;
  return TRUE;
}

gboolean gj_gtk_mw_set_last_msg_recv_xevent_type(msgWindow mw, gint type)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_set_last_msg_recv_xevent_type: msgWindow was NULL");
      return FALSE;
    }  
  
  mw->last_msg_recv_xevent_type = type;
  return TRUE;
}

gboolean gj_gtk_mw_set_last_msg_recv_xevent_id(msgWindow mw, gchar *id)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_set_last_msg_recv_xevent_id: msgWindow was NULL");
      return FALSE;
    }  
  
  if(id == NULL)
    {
      g_warning("gj_gtk_mw_set_last_msg_recv_xevent_id: id was NULL");
      return FALSE;
    }    

  if(mw->last_msg_recv_xevent_id != NULL)
    {
      g_free(mw->last_msg_recv_xevent_id);
      mw->last_msg_recv_xevent_id = NULL;
    }

  mw->last_msg_recv_xevent_id = g_strdup(id);
  return TRUE;
}

gboolean gj_gtk_mw_set_last_msg_sent_xevent_type(msgWindow mw, gint type)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_set_last_msg_sent_xevent_type: msgWindow was NULL");
      return FALSE;
    }  
  
  mw->last_msg_sent_xevent_type = type;
  return TRUE;
}

gboolean gj_gtk_mw_set_last_msg_sent_xevent_id(msgWindow mw, gchar *id)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_set_last_msg_sent_xevent_id: msgWindow was NULL");
      return FALSE;
    }  
  
  if(id == NULL)
    {
      g_warning("gj_gtk_mw_set_last_msg_sent_xevent_id: id was NULL");
      return FALSE;
    }    

  if(mw->last_msg_sent_xevent_id != NULL)
    {
      g_free(mw->last_msg_sent_xevent_id);
      mw->last_msg_sent_xevent_id = NULL;
    }

  mw->last_msg_sent_xevent_id = g_strdup(id);
  return TRUE;
}

void gj_gtk_mw_print_all()
{
  gint index = 0;

  /* loop through all contacts on roster */
  for(index=0;index<g_list_length(msgWindowList);index++)
    {
      msgWindow mw = g_list_nth_data(msgWindowList, index);

      if(mw == NULL)
	{
	  g_warning("gj_gtk_mw_print_all: getting nth item(%d) in roster list was NULL",index);
	  continue;
	}

      gj_gtk_mw_print(mw);
    }  
}

void gj_gtk_mw_print(msgWindow mw)
{
  if(mw == NULL)
    return;

  g_message("++++++++++++++++++++++++++++++++++++++++++++++++");
  g_message("++++");
  g_message("++++ jid:'%s'", mw->jid);
  g_message("++++ type:'%d'", mw->type);
  g_message("++++");
  g_message("++++ last_msg_recv_xevent_type:%d", mw->last_msg_recv_xevent_type);
  g_message("++++ last_msg_recv_xevent_id:%s", mw->last_msg_recv_xevent_id);
  g_message("++++ last_msg_sent_xevent_type:%d", mw->last_msg_sent_xevent_type);
  g_message("++++ last_msg_sent_xevent_id:%s", mw->last_msg_sent_xevent_id);
  g_message("++++");
  g_message("++++++++++++++++++++++++++++++++++++++++++++++++");
}


/* 
 * window functions
 */
gboolean gj_gtk_mw_show(msgWindow mw, gboolean show)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_show: mw was NULL");
      return FALSE;
    }
  
  if(show == TRUE)
    gtk_widget_show(mw->controls.window);
  else
    gtk_widget_hide(mw->controls.window);

  return TRUE;
}

void gj_gtk_mw_raise(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_raise: mw was NULL");
      return;
    }

  if(mw->controls.window == NULL || mw->controls.window->window == NULL)
    {
      g_warning("gj_gtk_mw_raise: window was NULL or GDK window was NULL");
      return;
    }

  gdk_window_raise(GDK_WINDOW(mw->controls.window->window));
}

gboolean gj_gtk_mw_load(msgWindow mw)
{
  GladeXML *xml;
  GdkPixbuf *pb = NULL;
  GtkTextBuffer *buffer = NULL;

  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_load: mw was NULL");
      return FALSE;
    }

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "message",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_mw_load: couldn't open glade file at root 'message'");
      return FALSE;
    }
  
  mw->controls.window = glade_xml_get_widget(xml,"message");

  mw->controls.entry_contact = glade_xml_get_widget(xml, "entry_mw_contact");
  mw->controls.entry_time = glade_xml_get_widget(xml, "entry_mw_time");
  mw->controls.entry_subject = glade_xml_get_widget(xml, "entry_mw_subject");
  mw->controls.textview = glade_xml_get_widget(xml, "textview_mw");

  mw->controls.label_subject = glade_xml_get_widget(xml, "label_mw_subject");
  mw->controls.label_contact = glade_xml_get_widget(xml, "label_mw_contact");
  mw->controls.label_next = glade_xml_get_widget(xml, "label_mw_next");

  mw->controls.button_next = glade_xml_get_widget(xml, "button_mw_next");
  mw->controls.button_send = glade_xml_get_widget(xml, "button_mw_send");
  mw->controls.button_reply = glade_xml_get_widget(xml, "button_mw_reply");
  mw->controls.button_close = glade_xml_get_widget(xml, "button_mw_close");
  
#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (mw->controls.window), "delete_event",
                      GTK_SIGNAL_FUNC (on_message_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (mw->controls.textview), "motion_notify_event",
                      GTK_SIGNAL_FUNC (on_textview_mw_motion_notify_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (mw->controls.textview), "button_press_event",
                      GTK_SIGNAL_FUNC (on_textview_mw_button_press_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (mw->controls.button_reply), "clicked",
                      GTK_SIGNAL_FUNC (on_button_mw_reply_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (mw->controls.button_send), "clicked",
                      GTK_SIGNAL_FUNC (on_button_mw_send_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (mw->controls.button_next), "clicked",
                      GTK_SIGNAL_FUNC (on_button_mw_next_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (mw->controls.button_close), "clicked",
                      GTK_SIGNAL_FUNC (on_button_mw_close_clicked),
                      NULL);
#endif

  /* set icon */
  pb = gj_gtk_support_pixbuf_new_from_file(FILE_MESSAGE_NORMAL);
  gtk_window_set_icon(GTK_WINDOW(mw->controls.window), pb);
  g_object_unref(G_OBJECT(pb));

  /* get buffer */
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mw->controls.textview));

  /* set function callbacks */
  g_signal_connect(G_OBJECT(buffer), "changed", 
		   G_CALLBACK(on_textbuffer_mw_changed), 
		   NULL);

  /* set styles */
  gtk_text_buffer_create_tag(buffer, "url", 
			     "foreground", "blue", 
			     NULL); 

  /* 
   * cleanup 
   */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_mw_unload(msgWindow mw)
{
  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_unload: could not unload msg, mw was NULL");
      return FALSE;
    }

  /* if no more msgs reset icon */
  gtk_widget_destroy(mw->controls.window);
  
  gj_gtk_roster_handle_next_event_display(gj_gtk_mw_get_jid(mw));
  gj_gtk_mw_free(mw);
      
  g_message("gj_gtk_mw_unload: unloaded...");
  
  return TRUE;
}

gboolean gj_gtk_mw_unload_by_widget(GtkWidget *widget)
{
  msgWindow mw = NULL;

  if(widget == NULL)
    {
      g_warning("gj_gtk_mw_unload_by_widget: widget was NULL");
      return FALSE;
    }

  if((mw = gj_gtk_mw_find_by_widget(widget)) != NULL)
    {
      gj_gtk_mw_unload(mw);
    }
  else
    g_warning("gj_gtk_mw_unload_by_widget: could not find mw from widget");
  
  return TRUE;
}

msgWindow gj_gtk_mw_send_new(gchar *jid)
{
  msgWindow mw = NULL;
  gchar *tmp = NULL;

  mw = gj_gtk_mw_new(jid);
  gj_gtk_mw_load(mw);
  
  /* set type */
  gj_gtk_mw_set_type(mw, msgWindowTypeOutgoing);

  /* set title */
  tmp = g_strdup_printf("Send Message");
  gtk_window_set_title(GTK_WINDOW(mw->controls.window),tmp);
  g_free(tmp);

  /* set details */
  tmp = g_strdup_printf("To:");
  gtk_label_set_markup(GTK_LABEL(mw->controls.label_contact),tmp);
  g_free(tmp);

  /* set properies */
  gtk_widget_show(mw->controls.button_send);
  gtk_widget_hide(mw->controls.button_reply);
  gtk_widget_hide(mw->controls.button_next);
  gtk_editable_set_editable(GTK_EDITABLE(mw->controls.entry_subject),TRUE);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(mw->controls.textview), TRUE);
 
  /* set specifics */
  gj_gtk_mw_send_set(mw);

  return mw;
}

msgWindow gj_gtk_mw_receive_new(gchar *jid)
{
  msgWindow mw = NULL;
  gchar *tmp = NULL;

  mw = gj_gtk_mw_new(jid);
  gj_gtk_mw_load(mw);
  
  /* set type */
  gj_gtk_mw_set_type(mw, msgWindowTypeIncoming);

  /* set title */
  tmp = g_strdup_printf("Message Received");
  gtk_window_set_title(GTK_WINDOW(mw->controls.window),tmp);
  g_free(tmp);

  /* set details */
  tmp = g_strdup_printf("From:");
  gtk_label_set_markup(GTK_LABEL(mw->controls.label_contact),tmp);
  g_free(tmp);

  /* set properies */
  gtk_widget_show(mw->controls.button_reply);
  gtk_widget_hide(mw->controls.button_send);
  /*  gtk_widget_set_sensitive(mw->controls.button_next,FALSE);*/
  gtk_editable_set_editable(GTK_EDITABLE(mw->controls.entry_subject),FALSE);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(mw->controls.textview), FALSE);

  return mw;
}

gboolean gj_gtk_mw_receive_set(msgWindow mw, message msg)
{
  gchar *time = NULL;
  gchar *message = NULL;
  gint position = 0;

  /*!!!!!! check mw and msg */
  time = g_strdup(supportGetTimestampFromTimeT(messageGetTime(msg)));
  /*  time = timestampByTimeT(messageGetTime(msg));*/

  if(messageGetFromJID(msg) != NULL && strlen(messageGetFromJID(msg)) > 0)
    {
      gtk_editable_delete_text(GTK_EDITABLE(mw->controls.entry_contact),
			       0,
			       -1);

      gtk_editable_insert_text(GTK_EDITABLE(mw->controls.entry_contact),
			       messageGetFromJID(msg),
			       strlen(messageGetFromJID(msg)),
			       &position);
    }

  if(time != NULL && strlen(time) > 0)
    {
      gtk_editable_delete_text(GTK_EDITABLE(mw->controls.entry_time),
			       0,
			       -1);

      gtk_editable_insert_text(GTK_EDITABLE(mw->controls.entry_time),
			       time,
			       strlen(time),
			       &position);
    }

  if(messageGetSubject(msg) != NULL && strlen(messageGetSubject(msg)) > 0)
    {
      gtk_editable_delete_text(GTK_EDITABLE(mw->controls.entry_subject),
			       0,
			       -1);


      gtk_editable_insert_text(GTK_EDITABLE(mw->controls.entry_subject),
			       messageGetSubject(msg),
			       strlen(messageGetSubject(msg)),
			       &position);

      gtk_widget_show(mw->controls.label_subject);
      gtk_widget_show(mw->controls.entry_subject);
    }
  else
    {
      gtk_widget_hide(mw->controls.label_subject);
      gtk_widget_hide(mw->controls.entry_subject);
    }
      
  if((message = messageGetBody(msg)) != NULL && strlen(message) > 0)
    {
      GtkTextBuffer *buffer = NULL;
      GtkTextIter iter;
      GdkPixbuf *pixbuf = NULL;

      guint i = 1;

      /* get buffer */
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mw->controls.textview));
      gtk_text_buffer_set_text(buffer,"",-1);
      gtk_text_buffer_get_iter_at_offset(buffer, &iter, -1);

      /* search for occurences of emote icons */
      for(i=0;i<strlen(message);i++)
	{
	  if(strncmp(message+i, EMOTE_BIGGRIN, strlen(EMOTE_BIGGRIN)) == 0)
	    {
	      g_message("use bggrin");
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_biggrin)) != NULL)
		{
		  g_message("insert bggrin");
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_BIGGRIN)-1);
	    }
	  else if(strncmp(message+i, EMOTE_CONFUSED, strlen(EMOTE_CONFUSED)) == 0)
	    {
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_confused)) != NULL)
		{
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_CONFUSED)-1);
	    }
	  else if(strncmp(message+i, EMOTE_COOL, strlen(EMOTE_COOL)) == 0)
	    {
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_cool)) != NULL)
		{
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_COOL)-1);
	    }
	  else if(strncmp(message+i, EMOTE_EEK, strlen(EMOTE_EEK)) == 0)
	    {
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_eek)) != NULL)
		{
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_EEK)-1);
	    }
	  else if(strncmp(message+i, EMOTE_FROWN, strlen(EMOTE_FROWN)) == 0)
	    {
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_frown)) != NULL)
		{
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_FROWN)-1);
	    }
	  else if(strncmp(message+i, EMOTE_MAD, strlen(EMOTE_MAD)) == 0)
	    {
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_mad)) != NULL)
		{
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_MAD)-1);
	    }
	  else if(strncmp(message+i, EMOTE_REDFACE, strlen(EMOTE_REDFACE)) == 0)
	    {
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_redface)) != NULL)
		{
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_REDFACE)-1);
	    }
	  else if(strncmp(message+i, EMOTE_ROLLEYES, strlen(EMOTE_ROLLEYES)) == 0)
	    {
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_rolleyes)) != NULL)
		{
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_ROLLEYES)-1);
	    }
	  else if(strncmp(message+i, EMOTE_SMILE, strlen(EMOTE_SMILE)) == 0)
	    {
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_smile)) != NULL)
		{
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_SMILE)-1);
	    }
	  else if(strncmp(message+i, EMOTE_TONGUE, strlen(EMOTE_TONGUE)) == 0)
	    {
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_tongue)) != NULL)
		{
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_TONGUE)-1);
	    }
	  else if(strncmp(message+i, EMOTE_WINK, strlen(EMOTE_WINK)) == 0)
	    {
	      if((pixbuf = gj_gtk_mw_load_emoticon(emote_wink)) != NULL)
		{
		  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
		  g_object_unref(G_OBJECT(pixbuf));
		}

	      i += (strlen(EMOTE_WINK)-1);
	    }
	  else
	    {
	      /* insert text */
	      gtk_text_buffer_insert(buffer, &iter, message+i, 1);
	    }
	}
    }
  
  /* clean up */
  g_free(time);

  return TRUE;
}

gboolean gj_gtk_mw_send_set(msgWindow mw)
{
  gchar *contact = NULL;
  gchar *time = NULL;

  gint position = 0;

  /*!!!!!! check mw and msg */

  contact = gj_gtk_mw_get_jid(mw);
  time = g_strdup(supportGetTimestamp());
 
  if(contact != NULL && strlen(contact) > 0)
    gtk_editable_insert_text(GTK_EDITABLE(mw->controls.entry_contact),contact,strlen(contact),&position);

  if(time != NULL && strlen(time) > 0)
    gtk_editable_insert_text(GTK_EDITABLE(mw->controls.entry_time),time,strlen(time),&position);

  /* clean up */
  g_free(time);

  return TRUE;
}

void gj_gtk_mw_waiting_update(msgWindow mw)
{
  gint count = 0;
  gchar *label = NULL;

  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_waiting_update: mw was NULL");
      return;
    }

  /* get count waiting */
  if((count = gj_gtk_mw_waiting_count(mw)) < 0)
    {
      g_warning("gj_gtk_mw_waiting_update: count:%d was < 0, could not get the count",count);
      return;
    }
      
  /* change button to say "next (i)" */
  g_message("gj_gtk_mw_waiting_update: updating number waiting to %d",count);
  
  if(count > 1)
    {
      /* set next */
      label = g_strdup_printf("Next (<b>%d</b>)",count);
      gtk_label_set_markup(GTK_LABEL(mw->controls.label_next),label);
      g_free(label);
      
      /* enable it */
      gtk_widget_set_sensitive(mw->controls.button_next,TRUE);
    }
  else if(count > 0)
    {
      /* set next */
      label = g_strdup_printf("Next");
      gtk_label_set_markup(GTK_LABEL(mw->controls.label_next),label);
      g_free(label);
      
      /* enable it */
      gtk_widget_set_sensitive(mw->controls.button_next,TRUE);
    }
  else 
    {
      gtk_widget_set_sensitive(mw->controls.button_next,FALSE);
    }
}

gint gj_gtk_mw_waiting_count(msgWindow mw)
{
  rosterItem ri = NULL;
  event ev = NULL;

  gint count = 0;
  gint index = 0;

  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_waiting_count: mw was NULL");
      return -1;
    }

  if((ri = rosterItemFindByJID(gj_gtk_mw_get_jid(mw))) == NULL)
    {
      g_warning("gj_gtk_mw_waiting_count: failed to get roster item by jid:'%s'", gj_gtk_mw_get_jid(mw));
      return -1;
    }

  /* look for futher events of type 'chat' in queue */
  while((ev = eventNthByRosterItem(ri,index++)) != NULL)
    {
      message msg = NULL;
      
      if(eventGetType(ev) != eventTypeMessage)
	continue;

      if((msg = (message) eventGetData(ev)) == NULL)
	continue;
      
      if(messageGetType(msg) == messageTypeNormal)
	count++;
    }
  
  /* should we?? take one off because the current event is also in this queue */
  return count;
}

gboolean gj_gtk_mw_waiting_next(msgWindow mw)
{
  message msg = NULL;
  rosterItem ri = NULL;
  event ev = NULL;
  
  gint eventID = 0;
  gint count = 0;
  gint index = 0;

  if(mw == NULL)
    {
      g_warning("gj_gtk_mw_waiting_next: mw was NULL");
      return FALSE;
    }

  if((count = gj_gtk_mw_waiting_count(mw)) < 1)
    {
      g_warning("gj_gtk_mw_waiting_next: count:%d was < 1",count);
      return FALSE;
    }

  if((ri = rosterItemFindByJID(gj_gtk_mw_get_jid(mw))) == NULL)
    {
      g_warning("gj_gtk_mw_waiting_next: failed to get roster item by jid:'%s'", gj_gtk_mw_get_jid(mw));
      return FALSE;
    }

  /* look for futher events of type 'chat' in queue */
  while((ev = eventNthByRosterItem(ri,index++)) != NULL)
    {
      message msg = NULL;
      
      if(eventGetType(ev) != eventTypeMessage)
	continue;

      if((msg = (message) eventGetData(ev)) == NULL)
	continue;
      
      if(messageGetType(msg) == messageTypeNormal)
	{
	  count++;
	  break;
	}
    }

  if(count == 0)
    {
      g_warning("mw_waitingNext: could not find next roster item event");
      return FALSE;
    }

  msg = (message) eventGetData(ev);
  eventID = eventGetID(ev);
  
  if(msg == NULL)
    {
      g_warning("gj_gtk_mw_waiting_next: msg data was NULL, not handling...");
      
      /* free event - no use to me now */
      eventFree(ev);
      return FALSE;
    }
  
  g_message("gj_gtk_mw_waiting_next: handling next event of type:%d, id:%d",eventTypeMessage,eventID);
  
  /* setup msg */
  gj_gtk_mw_receive_set(mw,msg);	   
  
  /* free event */
  eventFree(ev);

  /* update next button */
  gj_gtk_mw_waiting_update(mw);

  return TRUE;
}

msgWindow gj_gtk_mw_notification_new(gint eventID, message msg)
{
  msgWindow mw = NULL;

  if(msg == NULL)
    {
      g_warning("gj_gtk_mw_notification_new: msg was NULL");
      return NULL;
    }

  mw = gj_gtk_mw_find_by_jid_and_type(messageGetFromJID(msg),msgWindowTypeIncoming);

  if(mw == NULL)
    {
      mw = gj_gtk_mw_receive_new(messageGetFromJID(msg));
      gj_gtk_mw_receive_set(mw,msg);

      /* free event */
      eventFreeByID(eventID);
      
      /* check messages still waiting and update display */
      gj_gtk_mw_waiting_update(mw);
    }
  else
    {
      if(gj_gtk_mw_waiting_count(mw) > 0)
	{
	  /* add to queue */
	  g_message("gj_gtk_mw_notification_new: there are %d messages now in the queue",gj_gtk_mw_waiting_count(mw));
	  
	  /* 
	   * check other msg windows that are opened
	   *
	   */
	  gj_gtk_mw_waiting_update(mw);
	}

      /* should we show it? */
      if(gj_gtk_mpf_get_raise_window_on_new_message() == TRUE)
	gj_gtk_mw_raise(mw);
    }

  return mw;
}

void gj_gtk_mw_default_action(gchar *jid)
{
  msgWindow mw = NULL;
  gint count = 0;

  if(jid == NULL)
    {
      g_warning("gj_gtk_mw_default_action: JID was NULL");
      return;
    }

  if((mw = gj_gtk_mw_find_by_jid_and_type(jid,msgWindowTypeIncoming)) != NULL)
    {
      g_message("gj_gtk_mw_default_action: found incoming msg window for jid:'%s'",jid);

      if((count = gj_gtk_mw_waiting_count(mw)) > 0)
	{
	  /* 1 or more msgs waiting... */

	  /* show window mw */
	  gj_gtk_mw_show(mw,TRUE);
	  gj_gtk_mw_waiting_next(mw);
	}

#if 0      
      /* if XEvents ask to be notified when the message has 
	 been displayed, this is the time to send that notification */
      if(messageGetXEventType(msg) & messageXEventTypeDisplayed)
	messageSendXEvent(messageXEventTypeDisplayed, messageGetFromJID(msg), messageGetID(msg));

#endif

      return;
    }
  else if((mw = gj_gtk_mw_find_by_jid_and_type(jid,msgWindowTypeOutgoing)) != NULL)
    {
      /* show window */
      g_message("gj_gtk_mw_default_action: found outgoing msg window for jid:'%s'",jid);
      return;
    }
  else
    {
      msgWindow mw = NULL;

      g_message("gj_gtk_mw_default_action: found NO incoming or outgoing msg window for jid:'%s'",jid);

      /* if no messages waiting */
      mw = gj_gtk_mw_send_new(jid);
      gj_gtk_mw_show(mw,TRUE);
    }
}

GdkPixbuf *gj_gtk_mw_load_emoticon(msgEmote icon)
{
  GdkPixbuf *pixbuf = NULL;
  GdkPixbuf *scaled = NULL;
  gchar *filename = NULL; 

  switch(icon)
    {
    case emote_biggrin:
      filename = FILE_EMOTE_BIGGRIN;
      break;
    case emote_confused:
      filename = FILE_EMOTE_CONFUSED;
      break;
    case emote_cool:
      filename = FILE_EMOTE_COOL;
      break;
    case emote_eek:
      filename = FILE_EMOTE_EEK;
      break;
    case emote_frown:
      filename = FILE_EMOTE_FROWN;
      break;
    case emote_mad:
      filename = FILE_EMOTE_MAD;
      break;
    case emote_redface:
      filename = FILE_EMOTE_REDFACE;
      break;
    case emote_rolleyes:
      filename = FILE_EMOTE_ROLLEYES;
      break;
    case emote_smile:
      filename = FILE_EMOTE_SMILE;
      break;
    case emote_tongue:
      filename = FILE_EMOTE_TONGUE;
      break;
    case emote_wink:
      filename = FILE_EMOTE_WINK;
      break;
    default:
      g_warning("gj_gtk_mw_load_emoticon: failed to determin which icon to use from %d",icon);
      return NULL;
      break;
    }

  if((pixbuf = gdk_pixbuf_new_from_file(filename, NULL)) == NULL)
    {
      g_warning("gj_gtk_mw_load_emoticon: failed to load image file:'%s'",filename);
      return NULL;
    }

  scaled = gdk_pixbuf_scale_simple(pixbuf, 15, 15, GDK_INTERP_BILINEAR);
  g_object_unref(G_OBJECT(pixbuf));
  pixbuf = scaled;

  return pixbuf;
}

/*
 * GUI events
 */
gboolean gj_gtk_mw_check_for_url(GtkTextBuffer *buffer, gchar *text, guint pos)
{
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  gchar *buf = NULL;

  guint start = 0;
  guint end = 0;
  guint length = 0;

  if(GTK_TEXT_BUFFER(buffer) == NULL) 
    {
      g_warning("gj_gtk_mw_check_for_url: buffer was NULL");
      return FALSE;
    }

  /* we use text passed to us so we dont have to RE-get
     the whole text in the textview buffer */
  if(text == NULL) 
    {
      g_warning("gj_gtk_mw_is_url: text was NULL");
      return FALSE;
    }
  else
    length = strlen(text);

  /* check bounds */
  for(start = pos; start >= 0; --start) 
    if(supportIsWordSeparator(text[start])) break;

  start++;
  
  for(end = start; end < length; end++) 
    if(supportIsWordSeparator(text[end])) break;  

  /* check for quotes */
  for(;end>start; end--)
    if(isalpha(text[end-1]) || isdigit(text[end-1]) || text[end-1] == ':') break;

  /* get iters for start and end */
  gtk_text_buffer_get_iter_at_offset(buffer, &iter_start, start);
  gtk_text_buffer_get_iter_at_offset(buffer, &iter_end, end);

  if((buf = gtk_text_buffer_get_text(buffer, &iter_start, &iter_end, FALSE)) == NULL)
    {
      g_warning("gj_gtk_mw_is_url: buf returned was NULL");
      return FALSE;
    }

  if(supportIsURI(buf) != uriTypeUnknown) 
    {
      /* highlight as uri */
      gtk_text_buffer_apply_tag_by_name(buffer, "url", &iter_start, &iter_end);
      return TRUE;
    } 
  else
    {
      /* none highlighted */
      gtk_text_buffer_remove_tag_by_name(buffer, "url", &iter_start, &iter_end);
      return TRUE;
    }

  return FALSE;
}

/*
 * GUI events
 */
void on_textbuffer_mw_changed(GtkTextBuffer *textbuffer, gpointer user_data)
{
  GtkTextMark *mark = NULL;
  GtkTextIter iter_space;
  GtkTextIter iter_word_start;
  GtkTextIter iter_word_end;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  gint length = 0;

  /* check WHOLE buffer for urls */
  gtk_text_buffer_get_bounds(textbuffer, &iter_start, &iter_end);
  length = gtk_text_buffer_get_char_count(textbuffer);

  if((mark = gtk_text_buffer_get_mark(textbuffer, "url")) != NULL)
    {
      gtk_text_buffer_get_iter_at_mark(textbuffer, &iter_start, mark);
      /*       gtk_text_mark_set_visible(mark, TRUE); */
    }

  /* save last iter */
  iter_word_start = iter_start;
  iter_word_end = iter_start;
  iter_space = iter_start;

  while(TRUE)
    {
      gboolean found = FALSE;
      gchar *buf = NULL;

      /* search for next space */
      found = gtk_text_iter_forward_find_char(&iter_space, on_textview_mw_find_space, NULL, NULL);

      if(found == FALSE)
	break;
      
      /* set word end to the space */
      iter_word_start = iter_word_end;
      iter_word_end = iter_space;

      /* move word end iter forward 1 */
      gtk_text_iter_forward_char(&iter_word_start);

      /* get */
      buf = gtk_text_iter_get_text(&iter_word_start, &iter_word_end);
      
      if(buf != NULL) 
	{
	  if(supportIsURI(buf) != uriTypeUnknown) 
	    {
	      /* highlight as uri */
	      gtk_text_buffer_apply_tag_by_name(textbuffer, "url", &iter_word_start, &iter_word_end);
	    } 
	  else
	    {
	      /* none highlighted */
	      gtk_text_buffer_remove_tag_by_name(textbuffer, "url", &iter_word_start, &iter_word_end);
	    }

	  g_free(buf);
	}

      /* set mark, then we can check next time from 
	 the last mark and it is more efficient */
      if((mark = gtk_text_buffer_get_mark(textbuffer, "url")) == NULL)
	{
	  if((mark = gtk_text_buffer_create_mark(textbuffer, "url", &iter_space, FALSE)) == NULL)
	    {
	      g_warning("on_textbuffer_mw_changed: could not create mark!");
	      return;
	    }

	  /* no need to move it, just continue */
	  continue;
	}

      /* move mark to latest iter */
      gtk_text_buffer_move_mark_by_name(textbuffer, "url", &iter_space);
    }
}

gboolean on_textview_mw_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
{
  GdkModifierType state = 0;
  GdkWindow *window = NULL;

  GtkTextView *textview = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter;
  GtkTextIter iter_start;
  GtkTextIter iter_end;
  GtkTextWindowType text_window_type = 0;
  
  gboolean found_start = FALSE;
  gboolean found_end = FALSE;

  gint pointer_x = 0;
  gint pointer_y = 0;
  gint x = 0;
  gint y = 0;

  gchar *buf = NULL;

  msgWindow mw = NULL;

  if(event->is_hint)
    {
      window = gdk_window_get_pointer(event->window, &pointer_x, &pointer_y, &state);
    }
  else
    {
      pointer_x = event->x;
      pointer_y = event->y;
      state = event->state;
    }

  if((mw = gj_gtk_mw_find_by_widget(GTK_WIDGET(widget))) == NULL)
    {
      g_warning("on_textview_mw_motion_notify_event: failed to find headlineWindow from widget");
      return FALSE;
    }
  
  if((textview = GTK_TEXT_VIEW(widget)) == NULL)
    {
      g_warning("on_textview_mw_motion_notify_event: view was NULL");
      return FALSE;
    }

  if((buffer = GTK_TEXT_BUFFER(gtk_text_view_get_buffer(textview))) == NULL)
    {
      g_warning("on_textview_mw_motion_notify_event: buffer was NULL");
      return FALSE;
    }
  
  text_window_type = gtk_text_view_get_window_type(textview, event->window);
  gtk_text_view_window_to_buffer_coords(textview, text_window_type, pointer_x, pointer_y, &x, &y);
  gtk_text_view_get_iter_at_location(textview, &iter, x, y);
 
  if(g_unichar_isspace(gtk_text_iter_get_char(&iter)) == TRUE)
    {
      gdk_window_set_cursor(event->window, NULL);
      return FALSE;
    }

  iter_start = iter;
  iter_end = iter;

  /* search for next space */
  found_start = gtk_text_iter_backward_find_char(&iter_start, on_textview_mw_find_space, NULL, NULL);
  found_end = gtk_text_iter_forward_find_char(&iter_end, on_textview_mw_find_space, NULL, NULL);

  if(found_start == FALSE && found_end == FALSE)
    {
      gdk_window_set_cursor(event->window, NULL);
      return FALSE;
    }

  if(gtk_text_iter_is_start(&iter_start) == FALSE)
    gtk_text_iter_forward_char(&iter_start);
  
  if((buf = gtk_text_buffer_get_text(buffer, &iter_start, &iter_end, FALSE)) == NULL)
    {
      g_warning("on_textview_mw_motion_notify_event: buf returned was NULL");
      return FALSE;
    }
  
  if(supportIsURI(buf) != uriTypeUnknown)
    {
      /* width wise it works... */
      GdkCursor *cursor = NULL;
      cursor = gdk_cursor_new(GDK_HAND2);
      gdk_window_set_cursor (event->window, cursor);
      
      /* clean up */
      gdk_cursor_unref(cursor);
    }  
  else
    {
      /* return to parent window's cursor */
      gdk_window_set_cursor(event->window, NULL);
    }
      
  /* clean up */
  if(buf != NULL) g_free(buf);

  return FALSE;
}

gboolean on_textview_mw_find_space(gunichar ch, gpointer user_data)
{
  if(g_unichar_isspace(ch) == TRUE)
    return TRUE;
    
  return FALSE;
}

gboolean on_textview_mw_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  GdkModifierType state;
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter;
  GtkTextIter iter_start;
  GtkTextIter iter_end;
  GtkTextWindowType text_window_type = 0;

  gboolean found_start = FALSE;
  gboolean found_end = FALSE;

  gint pointer_x = 0;
  gint pointer_y = 0;
  gint x = 0;
  gint y = 0;

  gchar *buf = NULL;
    
  /* if not mouse 1 return */
  if(event->button != 1)
    return FALSE;

  gdk_window_get_pointer(event->window, &pointer_x, &pointer_y, &state);
    
  if((buffer = GTK_TEXT_BUFFER(gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget)))) == NULL)
    {
      g_warning("on_textview_mw_button_press_event: could not get buffer.");
      return FALSE;
    }
    
  text_window_type = gtk_text_view_get_window_type(GTK_TEXT_VIEW(widget), event->window);
  gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(widget), text_window_type, pointer_x, pointer_y, &x, &y);
  gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(widget), &iter, x, y);

  iter_start = iter;
  iter_end = iter;
    
  /* search for next space */
  found_start = gtk_text_iter_backward_find_char(&iter_start, on_textview_mw_find_space, NULL, NULL);
  found_end = gtk_text_iter_forward_find_char(&iter_end, on_textview_mw_find_space, NULL, NULL);
    
  if(found_start == FALSE && found_end == FALSE)
    return FALSE;
    
  if(gtk_text_iter_is_start(&iter_start) == FALSE)
    gtk_text_iter_forward_char(&iter_start);
    
  if((buf = gtk_text_buffer_get_text(buffer, &iter_start, &iter_end, FALSE)) == NULL)
    {
      g_warning("on_textview_mw_button_press_event: buf returned was NULL");
      return FALSE;
    }
    
  if(supportIsURI(buf) != uriTypeUnknown)
    {
      gj_gtk_support_open_url(buf);
    }  
    
  /* clean up */
  if(buf != NULL) g_free(buf);
   
  return FALSE;
}

void on_button_mw_send_clicked(GtkButton *button, gpointer user_data)
{
  gchar *subject = NULL;
  gchar *body = NULL;

  msgWindow mw = NULL;

  GtkTextBuffer *buffer = NULL;
  GtkTextIter bufStart;
  GtkTextIter bufStop;

  mw = gj_gtk_mw_find_by_widget(GTK_WIDGET(button));

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mw->controls.textview));
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStart, 0);
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStop, -1);
  body = gtk_text_buffer_get_text(buffer,&bufStart,&bufStop,TRUE);

  if(body == NULL || strlen(body) < 1)
    {
      g_warning("on_button_mw_send_clicked: cant send NULL or 0 length message");
      return;
    }

  subject = gtk_editable_get_chars(GTK_EDITABLE(mw->controls.entry_subject),0,-1); 

  /* FIXME: send message to history logger */
  messageSend(messageTypeNormal, gj_gtk_mw_get_jid(mw), subject, body, NULL);

  gj_gtk_mw_unload_by_widget(mw->controls.window);
}

void on_button_mw_reply_clicked(GtkButton *button, gpointer user_data)
{
  gchar *jid = NULL;
  msgWindow mw = NULL;
  
  if((mw = gj_gtk_mw_find_by_widget(GTK_WIDGET(button))) == NULL)
    {
      g_warning("on_button_mw_reply_clicked: failed to find msgWindow from button widget");
      return;
    }

  jid = g_strdup(gj_gtk_mw_get_jid(mw));

  /* unload old ? */
  if(gj_gtk_mpf_get_close_previous_message_window_on_reply() == TRUE)
    gj_gtk_mw_unload(mw);

  /* create new */
  gj_gtk_mw_send_new(jid);

  /* clean up */ 
  g_free(jid);
}

void on_button_mw_next_clicked(GtkButton *button, gpointer user_data)
{
  msgWindow mw = NULL;
  
  if((mw = gj_gtk_mw_find_by_widget(GTK_WIDGET(button))) == NULL)
    {
      g_warning("on_button_mw_next_clicked: failed to find msgWindow from button widget");
      return;
    }

  gj_gtk_mw_waiting_next(mw);
}

void on_button_mw_close_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mw_unload_by_widget(GTK_WIDGET(button));
}

gboolean on_message_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_mw_unload_by_widget(widget);

  return FALSE;
}

#ifdef __cplusplus
}
#endif
