/*
 * $RCSfile: gj_gtk_messagebox.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/28 13:38:29 $
 *
 * $Revision: 1.4 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* gui */
#include <glade/glade.h>
#include <gtk/gtk.h>

/* local */
#include "common.h"

#include "gj_gtk_messagebox.h"
#include "gj_gtk_support.h"

/* local functions */
gboolean gj_gtk_messagebox_function(gint type, msgProc proc, void *userdata);
gboolean gj_gtk_messagebox_setup(gchar *message,gchar *title,gchar *action, gboolean modal, gint type, msgProc proc, void *userdata);

/*
 * actual GUI functions
 */
void on_button_messagebox_ok_clicked(GtkButton *button, gpointer user_data);
void on_button_messagebox_cancel_clicked(GtkButton *button, gpointer user_data);
void on_button_messagebox_yes_clicked(GtkButton *button, gpointer user_data);
void on_button_messagebox_no_clicked(GtkButton *button, gpointer user_data);
void on_button_messagebox_close_clicked(GtkButton *button, gpointer user_data);

gboolean on_messagebox_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);


struct 
{
  /* search */
  GtkWidget *window;

  GtkWidget *button_messagebox_ok;
  GtkWidget *button_messagebox_cancel;
  GtkWidget *button_messagebox_yes;
  GtkWidget *button_messagebox_no;
  GtkWidget *button_messagebox_close;

  GtkWidget *label_msg;

  GtkWidget *image_msg;

} ctlMessagebox;

/*
 * external functions...
 */
gboolean gj_gtk_messagebox_load(gchar *message,
				gchar *title,
				gchar *action, 
				gboolean modal, 
				gint type, 
				msgProc proc, 
				void *userdata)
{
  GladeXML *xml = NULL;

  /* check vars */
  if(message == NULL)
    {
      g_warning("gj_gtk_messagebox_load: failed to create message box, message was NULL");
      return FALSE;
    }

  if(title == NULL)
    {
      g_warning("gj_gtk_messagebox_load: failed to create message box, title was NULL");
      return FALSE;
    }

  if(action == NULL)
    {
      g_warning("gj_gtk_messagebox_load: failed to create message box, action was NULL");
      return FALSE;
    }

  if(type < 1)
    {
      g_warning("gj_gtk_messagebox_load: failed to create message box, type:%d was < 1",type);
      return FALSE;
    }

  /* load dialog */
  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  GLADE_MESSAGEBOX,
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_messagebox_load: unable to create GUI, could not find XML tag:'%s'",GLADE_MESSAGEBOX);
      return FALSE;
    }
  
  /* controls */
  ctlMessagebox.window = glade_xml_get_widget(xml, GLADE_MESSAGEBOX);

  ctlMessagebox.button_messagebox_ok = glade_xml_get_widget(xml, "button_messagebox_ok");
  ctlMessagebox.button_messagebox_cancel = glade_xml_get_widget(xml, "button_messagebox_cancel");
  ctlMessagebox.button_messagebox_yes = glade_xml_get_widget(xml, "button_messagebox_yes");
  ctlMessagebox.button_messagebox_no = glade_xml_get_widget(xml, "button_messagebox_no");
  ctlMessagebox.button_messagebox_close = glade_xml_get_widget(xml, "button_messagebox_close");

  ctlMessagebox.label_msg = glade_xml_get_widget(xml, "label_messagebox");
  ctlMessagebox.image_msg = glade_xml_get_widget(xml, "image_messagebox");

  /* signals */
  gtk_signal_connect (GTK_OBJECT (ctlMessagebox.button_messagebox_ok), "clicked",
		      GTK_SIGNAL_FUNC (on_button_messagebox_ok_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlMessagebox.button_messagebox_cancel), "clicked",
		      GTK_SIGNAL_FUNC (on_button_messagebox_cancel_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlMessagebox.button_messagebox_yes), "clicked",
		      GTK_SIGNAL_FUNC (on_button_messagebox_yes_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlMessagebox.button_messagebox_no), "clicked",
		      GTK_SIGNAL_FUNC (on_button_messagebox_no_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlMessagebox.button_messagebox_close), "clicked",
		      GTK_SIGNAL_FUNC (on_button_messagebox_close_clicked),
		      NULL);
  
  gtk_signal_connect (GTK_OBJECT (ctlMessagebox.window), "delete_event",
		      GTK_SIGNAL_FUNC (on_messagebox_delete_event),
		      NULL);    

  /* check window is created ok */
  if(ctlMessagebox.window == NULL)
    {
      g_warning("gj_gtk_messagebox_load: failed to create message box, window was NULL");
      return FALSE;
    }

  if(proc == NULL)
    modal = TRUE;


  gj_gtk_messagebox_setup(message,title,action,modal,type,proc,userdata);

  return TRUE;
}

gboolean gj_gtk_messagebox_unload()
{
  gtk_widget_destroy(ctlMessagebox.window);

  return TRUE;
}

gboolean gj_gtk_messagebox_setup(gchar *message,gchar *title,gchar *action, gboolean modal, gint type, msgProc proc, void *userdata)
{
  gchar *text = NULL;

  /* set text with action */
  text = g_strdup_printf("%s\n\n<b>%s</b>",message,action);
  gtk_label_set_markup(GTK_LABEL(ctlMessagebox.label_msg),text);
  g_free(text);

  /* set title prop */
  gtk_window_set_title(GTK_WINDOW(ctlMessagebox.window),title);
  
  /* set modal prop */
  gtk_window_set_modal(GTK_WINDOW(ctlMessagebox.window),modal);


   /* set icon - TODO */
  if(type & GJ_GTK_MESSAGEBOX_ICON_ERROR)
    gtk_image_set_from_stock(GTK_IMAGE(ctlMessagebox.image_msg),GTK_STOCK_DIALOG_ERROR,GTK_ICON_SIZE_DIALOG);
  else if(type & GJ_GTK_MESSAGEBOX_ICON_EXCLAIMATION)
    gtk_image_set_from_stock(GTK_IMAGE(ctlMessagebox.image_msg),GTK_STOCK_DIALOG_WARNING,GTK_ICON_SIZE_DIALOG);
  else if(type & GJ_GTK_MESSAGEBOX_ICON_QUESTION)
    gtk_image_set_from_stock(GTK_IMAGE(ctlMessagebox.image_msg),GTK_STOCK_DIALOG_QUESTION,GTK_ICON_SIZE_DIALOG);
  else if(type & GJ_GTK_MESSAGEBOX_ICON_INFORMATION)
    gtk_image_set_from_stock(GTK_IMAGE(ctlMessagebox.image_msg),GTK_STOCK_DIALOG_INFO,GTK_ICON_SIZE_DIALOG);
 
  /* set object data for all buttons, saves global var */
  gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_cancel),"proc",proc);
  gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_yes),"proc",proc);
  gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_no),"proc",proc);
  gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_close),"proc",proc);

  if(proc != NULL)
    g_message("gj_gtk_messagebox_setup: saving to button object, proc to execute is: 0x%.8x",(gint)proc);

  /* set buttons */
  if(type & GJ_GTK_MESSAGEBOX_BUTTON_OK)
    {
      gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_ok),"proc",proc);
      if(userdata != NULL) gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_ok),"userdata",userdata);

      gtk_widget_show(ctlMessagebox.button_messagebox_ok);
    }

  if(type & GJ_GTK_MESSAGEBOX_BUTTON_CANCEL)
    {
      gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_cancel),"proc",proc);
      if(userdata != NULL) gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_cancel),"userdata",userdata);

      gtk_widget_show(ctlMessagebox.button_messagebox_cancel);
    }
  if(type & GJ_GTK_MESSAGEBOX_BUTTON_YES)
    {
      gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_yes),"proc",proc);
      if(userdata != NULL) gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_yes),"userdata",userdata);

     gtk_widget_show(ctlMessagebox.button_messagebox_yes);
    }
  if(type & GJ_GTK_MESSAGEBOX_BUTTON_NO)
    {
      gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_no),"proc",proc);
      if(userdata != NULL) gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_no),"userdata",userdata);

      gtk_widget_show(ctlMessagebox.button_messagebox_no);
    }
  if(type & GJ_GTK_MESSAGEBOX_BUTTON_CLOSE)
    {
      gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_close),"proc",proc);
      if(userdata != NULL) gtk_object_set_data(GTK_OBJECT(ctlMessagebox.button_messagebox_close),"userdata",userdata);

      gtk_widget_show(ctlMessagebox.button_messagebox_close);
    }

  return TRUE;
}

gboolean gj_gtk_messagebox_function(gint type, msgProc proc, void *userdata)
{
  if(type < 1)
    {
      g_warning("gj_gtk_messagebox_function: type:%d is < 1",type);
      return FALSE;
    }

  if(proc == NULL)
    {
      g_message("gj_gtk_messagebox_function: not executing callback proc, it is NULL");
      return FALSE;
    }

  /* call callback function */
  (proc)(type,userdata);
  
  return TRUE;
}

/*
 * actual gui functions
 */

void on_button_messagebox_ok_clicked(GtkButton *button, gpointer user_data)
{
  msgProc proc = (msgProc)gtk_object_get_data(GTK_OBJECT(button),"proc");
  void *userdata = (void*)gtk_object_get_data(GTK_OBJECT(button),"userdata");

  gj_gtk_messagebox_function(GJ_GTK_MESSAGEBOX_BUTTON_OK,proc,userdata);
  gj_gtk_messagebox_unload();
}

void on_button_messagebox_cancel_clicked(GtkButton *button, gpointer user_data)
{
  msgProc proc = (msgProc)gtk_object_get_data(GTK_OBJECT(button),"proc");
  void *userdata = (void*)gtk_object_get_data(GTK_OBJECT(button),"userdata");

  gj_gtk_messagebox_function(GJ_GTK_MESSAGEBOX_BUTTON_CANCEL,proc,userdata);
  gj_gtk_messagebox_unload();
}

void on_button_messagebox_yes_clicked(GtkButton *button, gpointer user_data)
{
  msgProc proc = (msgProc)gtk_object_get_data(GTK_OBJECT(button),"proc");
  void *userdata = (void*)gtk_object_get_data(GTK_OBJECT(button),"userdata");

  gj_gtk_messagebox_function(GJ_GTK_MESSAGEBOX_BUTTON_YES,proc,userdata);
  gj_gtk_messagebox_unload();
}

void on_button_messagebox_no_clicked(GtkButton *button, gpointer user_data)
{
  msgProc proc = (msgProc)gtk_object_get_data(GTK_OBJECT(button),"proc");
  void *userdata = (void*)gtk_object_get_data(GTK_OBJECT(button),"userdata");

  gj_gtk_messagebox_function(GJ_GTK_MESSAGEBOX_BUTTON_NO,proc,userdata);
  gj_gtk_messagebox_unload();
}

void on_button_messagebox_close_clicked(GtkButton *button, gpointer user_data)
{
  msgProc proc = (msgProc)gtk_object_get_data(GTK_OBJECT(button),"proc");
  void *userdata = (void*)gtk_object_get_data(GTK_OBJECT(button),"userdata");

  gj_gtk_messagebox_function(GJ_GTK_MESSAGEBOX_BUTTON_CLOSE,proc,userdata);
  gj_gtk_messagebox_unload();
}

gboolean on_messagebox_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  /* !!!!!!! needs handling....!!!! */

  gj_gtk_messagebox_unload();
  return TRUE;
}

#ifdef __cplusplus
}
#endif
