/*
 * $RCSfile: gj_gtk_msg.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/08 13:18:46 $
 *
 * $Revision: 1.5 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* gui */
#include <glade/glade.h>
#include <gtk/gtk.h>

/* local */
#include "common.h"

#include "gj_main.h"

#include "gj_gtk_msg.h"
#include "gj_gtk_support.h"

/* list functions */
gboolean gj_gtk_msg_list_add(Msg m);
gboolean gj_gtk_msg_list_del(Msg m);

/* item functions */
gboolean gj_gtk_msg_free(Msg m);
void gj_gtk_msg_destroy(Msg m);

/* find */
Msg gj_gtk_msg_find_by_id(guint id);
Msg gj_gtk_msg_find_by_userdata(void *userdata);
Msg gj_gtk_msg_find_by_widget(GtkWidget *widget);

/* member functions - gets */
gchar *gj_gtk_msg_get_message(Msg m);
gchar *gj_gtk_msg_get_title(Msg m);
gchar *gj_gtk_msg_get_action(Msg m);

gint gj_gtk_msg_get_type(Msg m);
MsgCb gj_gtk_msg_get_cb(Msg m);

/* member functions - sets */
gboolean gj_gtk_msg_set_response(Msg m, gint response);

/* local functions */
gboolean gj_gtk_msg_function(GtkWidget *widget, gint type);
gboolean gj_gtk_msg_setup(Msg m);

/*
 * actual GUI functions
 */
void on_button_msg_ok_clicked(GtkButton *button, gpointer user_data);
void on_button_msg_cancel_clicked(GtkButton *button, gpointer user_data);
void on_button_msg_yes_clicked(GtkButton *button, gpointer user_data);
void on_button_msg_no_clicked(GtkButton *button, gpointer user_data);
void on_button_msg_close_clicked(GtkButton *button, gpointer user_data);

gboolean on_msg_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

struct t_ctlMsg 
{
  /* search */
  GtkWidget *window;

  GtkWidget *button_msg_ok;
  GtkWidget *button_msg_cancel;
  GtkWidget *button_msg_yes;
  GtkWidget *button_msg_no;
  GtkWidget *button_msg_close;

  GtkWidget *label_msg;

  GtkWidget *image_msg;

};

struct t_msg
{
  guint id;
  void *userdata;

  /* properties */
  gchar *message;
  gchar *title;
  gchar *action; 

  gboolean modal;

  gint type;

  MsgCb cb;

  gint response;

  /* controls */
  struct t_ctlMsg controls;
};

guint MsgID = 0;
GList *MsgList = NULL;

gboolean gj_gtk_msg_init()
{
  /* nothing to do here */
  return TRUE;
}

gboolean gj_gtk_msg_term()
{
  gint index = 0;
    
  /* list */
  if(MsgList == NULL)
    return TRUE;
    
  /* free all items on the list */
  for(index=0;index<g_list_length(MsgList);index++)
    {
      Msg m = NULL;
	
      if((m = (Msg) g_list_nth_data(MsgList,index)) != NULL)
	gj_gtk_msg_free(m);
    }
    
  g_list_free(MsgList);
  MsgList = NULL;
    
  return TRUE;
}

gboolean gj_gtk_msg_list_add(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_list_add: failed to add item to list, it was NULL");
      return FALSE;
    }

  MsgList = g_list_append(MsgList,m);

  return TRUE;
}

gboolean gj_gtk_msg_list_del(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_list_del: failed to del item from list, it was NULL");
      return FALSE;
    }

  MsgList = g_list_remove(MsgList, m);
  return TRUE;
}

Msg gj_gtk_msg_new(gchar *message, gchar *title, gchar *action, gboolean modal, gint type, MsgCb cb, void *userdata)
{
  Msg m = NULL;

  /* check vars */
  if(message == NULL)
    {
      g_warning("gj_gtk_msg_new: message was NULL");
      return NULL;
    }
  
  if(title == NULL)
    {
      g_warning("gj_gtk_msg_new: title was NULL");
      return NULL;
    }
  
  if(action == NULL)
    {
      g_warning("gj_gtk_msg_new: action was NULL");
      return NULL;
    }
  
  if(type < 1)
    {
      g_warning("gj_gtk_msg_new: type:%d was < 1",type);
      return NULL;
    }
  
  g_message("gj_gtk_msg_new: Msg id:%d", MsgID);

  /* create new struct */
  if((m = g_new(struct t_msg,1)))
    {
      m->id = MsgID++;
      m->userdata = userdata;

      m->message = g_strdup(message);
      m->title = g_strdup(title);
      m->action = g_strdup(action);

      m->modal = modal;
      m->type = type;
      m->cb = cb;

      m->response = 0;
    }

  /* add to list */
  gj_gtk_msg_list_add(m);

  /* show dialog */
  gj_gtk_msg_load(m);

  return m;
}

gboolean gj_gtk_msg_free(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_free: failed to free, Msg was NULL");
      return FALSE;
    }

  g_message("gj_gtk_msg_free: Msg id:%d", gj_gtk_msg_get_id(m));

  /* free mem */
  gj_gtk_msg_list_del(m);  

  /* destroy members */
  gj_gtk_msg_destroy(m);

  /* free structure */
  g_free(m);

  return TRUE;
}

void gj_gtk_msg_destroy(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_destroy: failed to destroy, Msg was NULL");
      return;
    }

  m->id = 0;

  m->userdata = NULL;

  if(m->message != NULL)
    {
      g_free(m->message);
      m->message = NULL;
    }

  if(m->title != NULL)
    {
      g_free(m->title);
      m->title = NULL;
    }

  if(m->action != NULL)
    {
      g_free(m->action);
      m->action = NULL;
    }

  m->modal = FALSE;
  m->type = 0;
  m->cb = NULL;

  m->response = 0;

  /* controls */
  m->controls.window = NULL;
  
  m->controls.button_msg_ok = NULL;
  m->controls.button_msg_cancel = NULL;
  m->controls.button_msg_yes = NULL;
  m->controls.button_msg_no = NULL;
  m->controls.button_msg_close = NULL;
  
  m->controls.label_msg = NULL;
  
  m->controls.image_msg = NULL;
}

Msg gj_gtk_msg_find_by_id(guint id)
{
  gint index = 0;

  if(id < 1)
    {
      g_warning("gj_gtk_msg_find_by_id: id < 1");
      return NULL;
    }

  for(index=0;index<g_list_length(MsgList);index++)
    {
      Msg m = g_list_nth_data(MsgList, index);
      
      if(m == NULL)
	{
	  g_warning("gj_gtk_msg_find_by_id: getting nth item(%d) in user information list was NULL",index);
	  continue;
	}
      
      if(m->id == id)
	return m;
    }
 
  return NULL;
}

Msg gj_gtk_msg_find_by_userdata(void *userdata)
{
  gint index = 0;

  if(userdata == NULL)
    {
      g_warning("gj_gtk_msg_find_by_userdata: userdata was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(MsgList);index++)
    {
      Msg m = g_list_nth_data(MsgList, index);
      
      if(m == NULL)
	{
	  g_warning("gj_gtk_msg_find_by_userdata: getting nth item(%d) in user information list was NULL",index);
	  continue;
	}
      
      if(m->userdata == userdata)
	return m;
    }
 
  return NULL;
}

Msg gj_gtk_msg_find_by_widget(GtkWidget *widget)
{
  gint index = 0;
  GtkWidget *toplevel = NULL;

  if(widget == NULL)
    {
      g_warning("gj_gtk_msg_find_by_widget: widget was NULL");
      return NULL;
    }

  toplevel = gtk_widget_get_toplevel(widget);
  if(!GTK_WIDGET_TOPLEVEL(toplevel))
    {
      g_warning("gj_gtk_msg_find_by_widget: could not find toplevel widget for widget:0x%.8x", (gint)widget);
      return NULL;
    }

  for(index=0;index<g_list_length(MsgList);index++)
    {
      Msg m = g_list_nth_data(MsgList, index);
      
      if(m == NULL)
	{
	  g_warning("gj_gtk_msg_find_by_widget: getting nth item(%d) in user information list was NULL",index);
	  continue;
	}
      
      if(m->controls.window == toplevel)
	return m;
    }
 
  return NULL;
}

/*
 * member functions 
 */

/* gets */
guint gj_gtk_msg_get_id(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_id: Msg was NULL");
      return -1;
    }
  
  return m->id;
}

void *gj_gtk_msg_get_userdata(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_userdata: Msg was NULL");
      return NULL;
    }
  
  return m->userdata;
}

gchar *gj_gtk_msg_get_message(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_message: Msg was NULL");
      return NULL;
    }  

  return m->message;
}

gchar *gj_gtk_msg_get_title(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_title: Msg was NULL");
      return NULL;
    }

  return m->title;
}

gchar *gj_gtk_msg_get_action(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_action: Msg was NULL");
      return NULL;
    }

  return m->action;
}

gboolean gj_gtk_msg_get_modal(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_modal: Msg was NULL");
      return FALSE;
    }

  return m->modal;
}

gint gj_gtk_msg_get_type(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_type: Msg was NULL");
      return -1;
    }    

  return m->type;
}

MsgCb gj_gtk_msg_get_cb(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_cb: Msg was NULL");
      return NULL;
    } 
    
  return m->cb;
}

gint gj_gtk_msg_get_response(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_response: Msg was NULL");
      return -1;
    }
  
  return m->response;
}

/* sets */
gboolean gj_gtk_msg_set_id(Msg m, guint id)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_id: Msg was NULL");
      return FALSE;
    }
  
  m->id = id;
  return TRUE;
}

gboolean gj_gtk_msg_set_userdata(Msg m, void *userdata)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_userdata: Msg was NULL");
      return FALSE;
    }
  
  m->userdata = userdata;
  return TRUE;
}

gboolean gj_gtk_msg_set_message(Msg m, gchar *message)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_message: Msg was NULL");
      return FALSE;
    }
  
  if(message != NULL)
    {
      g_warning("gj_gtk_msg_set_message: message was NULL");
      return FALSE;
    }

  if(m->message != NULL)
    {
      g_free(m->message);
      m->message = NULL;
    }

  m->message = g_strdup(message);
  
  return TRUE;
}

gboolean gj_gtk_msg_set_title(Msg m, gchar *title)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_title: Msg was NULL");
      return FALSE;
    }
  
  if(title != NULL)
    {
      g_warning("gj_gtk_msg_set_title: title was NULL");
      return FALSE;
    }

  if(m->title != NULL)
    {
      g_free(m->title);
      m->title = NULL;
    }

  m->title = g_strdup(title);
  
  return TRUE;
}

gboolean gj_gtk_msg_set_action(Msg m, gchar *action)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_action: Msg was NULL");
      return FALSE;
    }
  
  if(action != NULL)
    {
      g_warning("gj_gtk_msg_set_action: action was NULL");
      return FALSE;
    }

  if(m->action != NULL)
    {
      g_free(m->action);
      m->action = NULL;
    }

  m->action = g_strdup(action);
  
  return TRUE;
}

gboolean gj_gtk_msg_set_modal(Msg m, gboolean modal)
{
  return TRUE;
}

gboolean gj_gtk_msg_set_response(Msg m, gint response)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_response: Msg was NULL");
      return FALSE;
    }
  
  m->response = response;
  return TRUE;
}


/*
 * actual function to modify the msg gui  
 */
gboolean gj_gtk_msg_load(Msg m)
{
  GladeXML *xml = NULL;

  if(m == NULL)
    {
      g_warning("gj_gtk_msg_load: Msg was NULL");
      return FALSE;
    }

  /* load dialog */
  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "msg",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_msg_load: couldn't open glade file at root 'msg'");
      return FALSE;
    }
  
  /* controls */
  m->controls.window = glade_xml_get_widget(xml, "msg");

  m->controls.button_msg_ok = glade_xml_get_widget(xml, "button_msg_ok");
  m->controls.button_msg_cancel = glade_xml_get_widget(xml, "button_msg_cancel");
  m->controls.button_msg_yes = glade_xml_get_widget(xml, "button_msg_yes");
  m->controls.button_msg_no = glade_xml_get_widget(xml, "button_msg_no");
  m->controls.button_msg_close = glade_xml_get_widget(xml, "button_msg_close");

  m->controls.label_msg = glade_xml_get_widget(xml, "label_msg");

  m->controls.image_msg = glade_xml_get_widget(xml, "image_msg");

  /* signals */
#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (m->controls.button_msg_ok), "clicked",
		      GTK_SIGNAL_FUNC (on_button_msg_ok_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (m->controls.button_msg_cancel), "clicked",
		      GTK_SIGNAL_FUNC (on_button_msg_cancel_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (m->controls.button_msg_yes), "clicked",
		      GTK_SIGNAL_FUNC (on_button_msg_yes_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (m->controls.button_msg_no), "clicked",
		      GTK_SIGNAL_FUNC (on_button_msg_no_clicked),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (m->controls.button_msg_close), "clicked",
		      GTK_SIGNAL_FUNC (on_button_msg_close_clicked),
		      NULL);
  
  gtk_signal_connect (GTK_OBJECT (m->controls.window), "delete_event",
		      GTK_SIGNAL_FUNC (on_msg_delete_event),
		      NULL);    
#endif

  /* check window is created ok */
  if(m->controls.window == NULL)
    {
      g_warning("gj_gtk_msg_load: failed to create message box, window was NULL");
      return FALSE;
    }

  gj_gtk_msg_setup(m);

  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_msg_unload(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_unload: Msg was NULL");
      return FALSE;
    }

  gtk_widget_destroy(m->controls.window);
  m->controls.window = NULL;

  gj_gtk_msg_free(m);

  return TRUE;
}

gboolean gj_gtk_msg_setup(Msg m)
{
  gchar *text = NULL;
  static guint id = -1;

  if(m == NULL)
    {
      g_warning("gj_gtk_msg_setup: Msg was NULL");
      return FALSE;
    }

  /* set text with action */
  g_message("gj_gtk_msg_setup: setting message...");
  text = g_strdup_printf("%s\n\n<b>%s</b>", gj_gtk_msg_get_message(m), gj_gtk_msg_get_action(m));
  gtk_label_set_markup(GTK_LABEL(m->controls.label_msg), text);
  g_free(text);

  /* set title prop */
  g_message("gj_gtk_msg_setup: setting window title to:'%s'...", gj_gtk_msg_get_title(m));
  gtk_window_set_title(GTK_WINDOW(m->controls.window), gj_gtk_msg_get_title(m));
  
  /* set modal prop */
  g_message("gj_gtk_msg_setup: setting window to %s", gj_gtk_msg_get_modal(m)?"be modal":"NOT be modal");
  gtk_window_set_modal(GTK_WINDOW(m->controls.window), gj_gtk_msg_get_modal(m));
  
  /* set icon - TODO */
  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_ICON_ERROR)
    gtk_image_set_from_stock(GTK_IMAGE(m->controls.image_msg),GTK_STOCK_DIALOG_ERROR,GTK_ICON_SIZE_DIALOG);
  else if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_ICON_EXCLAIMATION)
    gtk_image_set_from_stock(GTK_IMAGE(m->controls.image_msg),GTK_STOCK_DIALOG_WARNING,GTK_ICON_SIZE_DIALOG);
  else if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_ICON_QUESTION)
    gtk_image_set_from_stock(GTK_IMAGE(m->controls.image_msg),GTK_STOCK_DIALOG_QUESTION,GTK_ICON_SIZE_DIALOG);
  else if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_ICON_INFORMATION)
    gtk_image_set_from_stock(GTK_IMAGE(m->controls.image_msg),GTK_STOCK_DIALOG_INFO,GTK_ICON_SIZE_DIALOG);

  id = gj_gtk_msg_get_id(m);

  g_message("gj_gtk_msg_setup: using id:%d", id);
  g_message("gj_gtk_msg_setup: using userdata: 0x%.8x",(gint)gj_gtk_msg_get_userdata(m));
  g_message("gj_gtk_msg_setup: using callback: 0x%.8x",(gint)gj_gtk_msg_get_cb(m));

  g_message("gj_gtk_msg_setup: setting up buttons...");

  /* set buttons */
  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_BUTTON_OK)
    {
      g_message("gj_gtk_msg_setup: using 'ok' button");
      gtk_widget_set_sensitive(GTK_WIDGET(m->controls.button_msg_ok), TRUE);
      gtk_widget_show(GTK_WIDGET(m->controls.button_msg_ok));
    }
    
  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_BUTTON_CANCEL)
    {
      g_message("gj_gtk_msg_setup: using 'cancel' button");
      gtk_widget_set_sensitive(GTK_WIDGET(m->controls.button_msg_cancel), TRUE);
      gtk_widget_show(GTK_WIDGET(m->controls.button_msg_cancel));
    }
  
  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_BUTTON_YES)
    {
      g_message("gj_gtk_msg_setup: using 'yes' button");
      gtk_widget_set_sensitive(GTK_WIDGET(m->controls.button_msg_yes), TRUE);
      gtk_widget_show(GTK_WIDGET(m->controls.button_msg_yes));
    }

  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_BUTTON_NO)
    {
      g_message("gj_gtk_msg_setup: using 'no' button");
      gtk_widget_set_sensitive(GTK_WIDGET(m->controls.button_msg_no), TRUE);
      gtk_widget_show(GTK_WIDGET(m->controls.button_msg_no));
    }
  
  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_BUTTON_CLOSE)
    {
      g_message("gj_gtk_msg_setup: using 'close' button");
      gtk_widget_set_sensitive(GTK_WIDGET(m->controls.button_msg_close), TRUE);
      gtk_widget_show(GTK_WIDGET(m->controls.button_msg_close));
    }

  g_message("gj_gtk_msg_setup: setting up buttons complete!");
  gtk_widget_show(m->controls.window);

  gdk_window_set_decorations (GDK_WINDOW(m->controls.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  return TRUE;
}

gboolean gj_gtk_msg_set_transient_window(Msg m, GtkWindow *parent_window)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_transient_window: Msg was NULL");
      return FALSE;
    }

  gtk_window_set_transient_for(GTK_WINDOW(m->controls.window), 
			       GTK_WINDOW(parent_window));

  g_message("gj_gtk_msg_set_transient_window: setting transient parent window:0x%.8x for Msg with id:%d", 
	    (gint)parent_window, gj_gtk_msg_get_id(m));

  return TRUE;
}

gboolean gj_gtk_msg_function(GtkWidget *widget, gint type)
{
  Msg m = NULL;

  if(type < 0)
    {
      g_warning("gj_gtk_msg_function: type:%d was invalid (<1)", type);
      return FALSE;
    }

  if((m = gj_gtk_msg_find_by_widget(widget)) == NULL)
    {
      g_warning("gj_gtk_msg_function: could not find Msg from widget:0x%.8x", (gint)widget);
      return FALSE;
    }

  /* set buttons */
  if(type & GJ_GTK_MSG_BUTTON_OK)
    g_message("gj_gtk_msg_function: 'ok' button pressed");
  else if(type & GJ_GTK_MSG_BUTTON_CANCEL)
    g_message("gj_gtk_msg_function: 'cancel' button pressed");
  else if(type & GJ_GTK_MSG_BUTTON_YES)
    g_message("gj_gtk_msg_function: 'yes' button pressed");
  else if(type & GJ_GTK_MSG_BUTTON_NO)
    g_message("gj_gtk_msg_function: 'no' button pressed");
  else if(type & GJ_GTK_MSG_BUTTON_CLOSE)
    g_message("gj_gtk_msg_function: 'close' button pressed");

  if(gj_gtk_msg_get_cb(m) != NULL)
    {
      g_message("gj_gtk_msg_function: using callback:0x%.8x, with userdata:0x%.8x", 
		(gint)gj_gtk_msg_get_cb(m), (gint)gj_gtk_msg_get_userdata);

      /* call callback function */
      (gj_gtk_msg_get_cb(m))(type, gj_gtk_msg_get_userdata(m));
    }
     
  /* unload msg window */
  gj_gtk_msg_unload(m);
    
  return TRUE;
}

/*
 * actual gui functions
 */

void on_button_msg_ok_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_msg_function(GTK_WIDGET(button), GJ_GTK_MSG_BUTTON_OK);
}

void on_button_msg_cancel_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_msg_function(GTK_WIDGET(button), GJ_GTK_MSG_BUTTON_CANCEL);
}

void on_button_msg_yes_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_msg_function(GTK_WIDGET(button), GJ_GTK_MSG_BUTTON_YES);
}

void on_button_msg_no_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_msg_function(GTK_WIDGET(button), GJ_GTK_MSG_BUTTON_NO);
}

void on_button_msg_close_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_msg_function(GTK_WIDGET(button), GJ_GTK_MSG_BUTTON_CLOSE);
}

gboolean on_msg_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  Msg m = NULL;
    
  if((m = gj_gtk_msg_find_by_widget(widget)) == NULL)
    {
      g_warning("on_msgbox_delete_event: could not find Msg from widget:0x%.8x", (gint)widget);
      return TRUE;
    }

  gj_gtk_msg_unload(m);
  return TRUE;
}

#ifdef __cplusplus
}
#endif
