/* 
 * $RCSfile: gj_gtk_my_presence.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/08 13:18:47 $
 * 
 * $Revision: 1.4 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_presence.h"
#include "gj_me.h"
#include "gj_support.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_my_presence.h"
#include "gj_gtk_support.h"

struct 
{
  GtkWidget *window;

  GtkWidget *optionmenu_show;
  GtkWidget *spinbutton_priority;
  GtkWidget *textview_status;
  
  GtkWidget *image_presence;
  GtkWidget *label_presence;

  GtkWidget *button_ok;
  GtkWidget *button_cancel;

} ctlMP;


presenceShow mp_show;
presenceType mp_type;


gboolean gj_gtk_mp_setup();


/* gui callbacks */
void on_mnu_mp_normal_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_mp_chat_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_mp_away_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_mp_xa_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_mp_dnd_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_mp_unavailable_activate(GtkMenuItem *menuitem, gpointer user_data);

void on_button_mp_ok_clicked(GtkButton *button, gpointer user_data);
void on_button_mp_cancel_clicked(GtkButton *button, gpointer user_data);

gboolean on_my_presence_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * code
 */
gboolean gj_gtk_mp_load(presenceType type, presenceShow show)
{
  GladeXML *xml = NULL;

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "my_presence",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_mp_load: couldn't open glade file at root 'my_presence'");
      return FALSE;
    }
  
  ctlMP.window = glade_xml_get_widget(xml,"my_presence");

  ctlMP.optionmenu_show = glade_xml_get_widget(xml,"optionmenu_mp_show");
  ctlMP.spinbutton_priority = glade_xml_get_widget(xml,"spinbutton_mp_priority");
  ctlMP.textview_status = glade_xml_get_widget(xml,"textview_mp_status");

  ctlMP.image_presence = glade_xml_get_widget(xml,"image_mp_presence");
  ctlMP.label_presence = glade_xml_get_widget(xml,"label_mp_presence");

  ctlMP.button_ok = glade_xml_get_widget(xml,"button_mp_ok");
  ctlMP.button_cancel = glade_xml_get_widget(xml,"button_mp_cancel");
  
#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (ctlMP.window), "delete_event",
                      GTK_SIGNAL_FUNC (on_my_presence_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlMP.button_cancel), "clicked",
                      GTK_SIGNAL_FUNC (on_button_mp_cancel_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlMP.button_ok), "clicked",
                      GTK_SIGNAL_FUNC (on_button_mp_ok_clicked),
                      NULL);
#endif

  /* set defaults */
  if(type <= 0)
    mp_type = presenceTypeAvailable;
  else
    mp_type = type;

  if(show <= 0)
    mp_show = presenceShowNormal;
  else
    mp_show = show;

  gj_gtk_mp_setup();

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_mp_unload()
{
  gtk_widget_destroy(ctlMP.window);

  return TRUE;
}

gboolean gj_gtk_mp_setup()
{
  gchar *file = NULL;
  gchar *text = NULL;
  gchar *status = NULL;

  GdkPixbuf *pixbuf = NULL;

  if(mp_type != presenceTypeAvailable)
    {
      switch(mp_type)
	{
	case presenceTypeUnAvailable:
	  file = FILE_PRESENCE_UNAVAILABLE;
	  text = g_strdup_printf("Unavailable");
	  break;
	default:
	  g_warning("gj_gtk_mp_setup: failed to determin presence 'type' from %d",mp_type);
	  return FALSE;
	}
    }
  else
    {
      switch(mp_show)
	{
	case presenceShowAway:
	  file = FILE_PRESENCE_AWAY;
	  text = g_strdup_printf("Away");
	  status = g_strdup_printf("Away (be right back)");
	  break;
	case presenceShowChat:
	  file = FILE_PRESENCE_CHAT;
	  text = g_strdup_printf("Chat");
	  status = g_strdup_printf("Chat (open to talk...)");
	  break;
	case presenceShowDND:
	  file = FILE_PRESENCE_DND;
	  text = g_strdup_printf("Do Not Disturb");
	  status = g_strdup_printf("DND (do not disturb!)");
	  break;
	case presenceShowNormal:
	  file = FILE_PRESENCE_NORMAL;
	  text = g_strdup_printf("Normal");
	  status = g_strdup_printf("Normal (available)");
	  break;
	case presenceShowXA:
	  file = FILE_PRESENCE_XA;
	  text = g_strdup_printf("eXtended Away");
	  status = g_strdup_printf("eXtended Away (not available right now)");
	  break;
	default:
	  g_warning("gj_gtk_mp_setup: failed to determin presence 'show' from %d",mp_show);
	  return FALSE;
	}
    }

  pixbuf = gj_gtk_support_pixbuf_new_from_file(file);
  text = g_strconcat("<b>",text,"</b>",NULL);

  gtk_image_set_from_pixbuf(GTK_IMAGE(ctlMP.image_presence),pixbuf);
  gtk_label_set_markup(GTK_LABEL(ctlMP.label_presence),text);

  if(status != NULL)
    {
      GtkTextBuffer *buffer = NULL;
      
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMP.textview_status));
      gtk_text_buffer_set_text(buffer, status, -1);
      
      /* select all text ??? how? */
    }
  
  /* clean up */
  if(text != NULL) g_free(text);
  if(status != NULL) g_free(status);
  if(pixbuf != NULL) g_object_unref(G_OBJECT(pixbuf));

  return TRUE;
}

/*
 * GUI events
 */
void on_mnu_mp_normal_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  mp_type = presenceTypeAvailable;
  mp_show = presenceShowNormal;

  gj_gtk_mp_setup();
}

void on_mnu_mp_chat_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  mp_type = presenceTypeAvailable;
  mp_show = presenceShowChat;

  gj_gtk_mp_setup();
}

void on_mnu_mp_away_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  mp_type = presenceTypeAvailable;
  mp_show = presenceShowAway;

  gj_gtk_mp_setup();
}

void on_mnu_mp_xa_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  mp_type = presenceTypeAvailable;
  mp_show = presenceShowXA;

  gj_gtk_mp_setup();
}

void on_mnu_mp_dnd_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  mp_type = presenceTypeAvailable;
  mp_show = presenceShowDND;

  gj_gtk_mp_setup();
}

void on_mnu_mp_unavailable_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  mp_type = presenceTypeUnAvailable;
  mp_show = 0;

  gj_gtk_mp_setup();
}

void on_button_mp_ok_clicked(GtkButton *button, gpointer user_data)
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter bufStart;
  GtkTextIter bufStop;

  gchar *status = NULL;
  gint priority = 0;

  /* get priority */
  priority = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlMP.spinbutton_priority));

  /* get status */
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMP.textview_status));
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStart, 0);
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStop, -1);
  status = gtk_text_buffer_get_text(buffer,&bufStart,&bufStop,TRUE);

  if(status != NULL && strlen(status) < 1)
    status = NULL;

  /* update myself */
  gj_gtk_roster_user_set_presence_and_save(mp_type,mp_show,status,priority);

  /* close up */
  gj_gtk_mp_unload();
}

void on_button_mp_cancel_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mp_unload();
}

gboolean on_my_presence_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_mp_unload();

  return TRUE;
}

#ifdef __cplusplus
}
#endif
