/* 
 * $RCSfile: gj_gtk_password_change.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/08 13:18:47 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_iq_requests.h"
#include "gj_parser.h"

#include "gj_gtk_password_change.h"
#include "gj_gtk_connection_settings.h"
#include "gj_gtk_msg.h"

struct 
{
  GtkWidget *window;

  GtkWidget *entry_password_1;
  GtkWidget *entry_password_2;

  GtkWidget *button_ok;
  GtkWidget *button_cancel;

} ctlPC;

gboolean changed = FALSE;

gboolean gj_gtk_pc_changed();
gboolean gj_gtk_pc_set();

/*
 * gui functions 
 */
void on_entry_pc_password_1_changed(GtkEditable *editable, gpointer user_data);
void on_entry_pc_password_2_changed(GtkEditable *editable, gpointer user_data);

void on_button_pc_ok_clicked(GtkButton *button, gpointer user_data);
void on_button_pc_cancel_clicked(GtkButton *button, gpointer user_data);

gboolean on_password_change_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * code
 */
gboolean gj_gtk_pc_load()
{
  GladeXML *xml = NULL;

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "password_change",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_pc_load: couldn't open glade file at root 'password_change'");
      return FALSE;
    }
  
  changed = FALSE;
  
  ctlPC.window = glade_xml_get_widget(xml,"password_change");

  ctlPC.entry_password_1 = glade_xml_get_widget(xml,"entry_pc_password_1");
  ctlPC.entry_password_2 = glade_xml_get_widget(xml,"entry_pc_password_2");

  ctlPC.button_ok = glade_xml_get_widget(xml,"button_pc_ok");
  ctlPC.button_cancel = glade_xml_get_widget(xml,"button_pc_cancel");
  
#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (ctlPC.window), "delete_event",
                      GTK_SIGNAL_FUNC (on_connection_password_change_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlPC.entry_password_1), "changed",
                      GTK_SIGNAL_FUNC (on_entry_pc_password_1_changed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlPC.entry_password_2), "changed",
                      GTK_SIGNAL_FUNC (on_entry_pc_password_2_changed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlPC.button_cancel), "clicked",
                      GTK_SIGNAL_FUNC (on_button_pc_cancel_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlPC.button_ok), "clicked",
                      GTK_SIGNAL_FUNC (on_button_pc_ok_clicked),
                      NULL);
#endif

  /* set up dialog */
  gtk_widget_set_sensitive(ctlPC.button_ok, FALSE);

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_pc_unload()
{
  gtk_widget_destroy(ctlPC.window);

  return TRUE;
}

gboolean gj_gtk_pc_set()
{
  gchar *password = NULL;

  password = gtk_editable_get_chars(GTK_EDITABLE(ctlPC.entry_password_1),0,-1);
  infoqueryRequestPasswordChange(gj_gtk_cs_get_server(), 
				 password, 
				 gj_gtk_pc_set_response);
  
  return TRUE;
}

void gj_gtk_pc_set_response(guint eventID)
{
  infoquery iq = NULL;
  infoquery iq_related = NULL;

  guint eventIDRelated = 0;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  iq = infoqueryFindByEventID(eventID);
  
  if(iq != NULL)
      eventIDRelated = infoqueryGetEventIDRelated(iq);

  iq_related = infoqueryFindByEventID(eventIDRelated);

  if(iq == NULL || iq_related == NULL)
    {
      if(iq == NULL)
	g_warning("gj_gtk_pc_set_response: failed to find IQ based on eventID:%d",eventID);
      else if(iq_related == NULL)
	g_warning("gj_gtk_pc_set_response: failed to find IQ based on eventIDRelated:%d",eventIDRelated);
      
      gj_gtk_msg_new("There were internal errors encounter while looking at the response", 
		     "Password Change", 
		     "Click Ok to close this window.",
		     TRUE,
		     GJ_GTK_MSG_BUTTON_OK | GJ_GTK_MSG_ICON_ERROR,
		     (MsgCb)NULL,
		     NULL);
      
      return;
    }

  node = infoqueryGetXML(iq_related);
  type = parserFindByAttrAndReturn(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      gchar *detail = NULL;

      xmlChar *errorCode = NULL;
      xmlChar *errorMessage = NULL;

      if((node = parserFindByNode(node, (guchar*)"error")) == NULL)
	{
	  g_warning("gj_gtk_pc_set_response: failed to find node 'error'");
	  return;
	}

      if((errorCode = parserFindByAttrAndReturn(node, (guchar*)"code")) != NULL)
	{
	  detail = g_strdup_printf("Error:%s",errorCode);
	}

      if((errorMessage = parserFindByNodeAndReturn(node, (guchar*)"error")) != NULL) 
	{
	  if(detail != NULL)
	    detail = g_strdup_printf("%s - %s",detail,errorMessage);
	}

      g_message("gj_gtk_pc_set_response: IQ type is error, password change failed for eventID:%d",eventID);
      g_message("gj_gtk_pc_set_response: %s",detail);

      gj_gtk_msg_new(detail, 
		     "Password Change", 
		     "Click Ok to close this window.",
		     TRUE,
		     GJ_GTK_MSG_BUTTON_OK | GJ_GTK_MSG_ICON_ERROR,
		     (MsgCb)NULL,
		     NULL);

      g_free(detail);

      return;
    }
  
  g_message("gj_gtk_pc_set_response: received response, eventID:%d",eventID);

  gj_gtk_msg_new("Password was changed successfully!", 
		 "Password Change", 
		 "Click Ok to close this window.",
		 TRUE,
		 GJ_GTK_MSG_BUTTON_OK | GJ_GTK_MSG_ICON_INFORMATION,
		 (MsgCb)NULL,
		 NULL);
}

gboolean gj_gtk_pc_changed()
{
  gboolean match = FALSE;

  gchar *password1 = NULL;
  gchar *password2 = NULL;

  password1 = gtk_editable_get_chars(GTK_EDITABLE(ctlPC.entry_password_1),0,-1);
  password2 = gtk_editable_get_chars(GTK_EDITABLE(ctlPC.entry_password_2),0,-1);

  if(password1 != NULL && strlen(password1) > 0 && 
     password2 != NULL && strlen(password2) > 0)
    {
      if(strcmp(password1,password2) == 0)
	match = TRUE;
    }
  else
    {
      gtk_widget_set_sensitive(ctlPC.button_ok,FALSE);
      return TRUE;
    }
  
  /* change gui based on match */
  if(match == TRUE)
      gtk_widget_set_sensitive(ctlPC.button_ok,TRUE);
  else
      gtk_widget_set_sensitive(ctlPC.button_ok,FALSE);

  return TRUE;
}


/*
 * GUI events
 */

void on_entry_pc_password_1_changed(GtkEditable *editable, gpointer user_data)
{
  changed = TRUE;
  gj_gtk_pc_changed();
}

void on_entry_pc_password_2_changed(GtkEditable *editable, gpointer user_data)
{
  changed = TRUE;
  gj_gtk_pc_changed();
}

void on_button_pc_ok_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_pc_set();
  gj_gtk_pc_unload();
}

void on_button_pc_cancel_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_pc_unload();
}

gboolean on_password_change_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_pc_unload();

  return TRUE;
}

#ifdef __cplusplus
}
#endif
