/* 
 * $RCSfile: gj_gtk_password_required.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/08 13:18:47 $
 * 
 * $Revision: 1.1 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_iq_requests.h"
#include "gj_parser.h"

#include "gj_gtk_password_required.h"
#include "gj_gtk_msg.h"

struct 
{
  GtkWidget *window;

  GtkWidget *entry_password;

  GtkWidget *button_ok;

} ctlPR;

struct 
{
  gboolean check_password;
  gchar *password;
  gj_gtk_pr_callback cb;

} optPR;


/*
 * member functions
 */

/* gets */
gboolean gj_gtk_pr_get_check_password();
gchar *gj_gtk_pr_get_password();
gj_gtk_pr_callback gj_gtk_pr_get_callback();

/* sets */
gboolean gj_gtk_pr_set_check_password(gboolean check_password);
gboolean gj_gtk_pr_set_password(gchar *password);
gboolean gj_gtk_pr_set_callback(gj_gtk_pr_callback cb);


/*
 * utils 
 */
gboolean gj_gtk_pr_changed();
gboolean gj_gtk_pr_call_callback(gboolean password_correct, const gchar *password);
gboolean gj_gtk_pr_check();

/*
 * gui functions 
 */
void on_entry_pr_password_changed(GtkEditable *editable, gpointer user_data);
void on_entry_pr_password_activate(GtkEntry *entry, gpointer user_data);

void on_password_required_response(GtkDialog *dialog, gint response, gpointer user_data);


/*
 * member functions
 */

/* gets */
gboolean gj_gtk_pr_get_check_password()
{
  return optPR.check_password;
}


gchar *gj_gtk_pr_get_password()
{
  return optPR.password;
}

gj_gtk_pr_callback gj_gtk_pr_get_callback()
{
  return optPR.cb;
}

/* sets */
gboolean gj_gtk_pr_set_check_password(gboolean check_password)
{
  optPR.check_password = check_password;
  return TRUE;
}

gboolean gj_gtk_pr_set_password(gchar *password)
{
  if(password == NULL)
    {
      g_warning("gj_gtk_pr_set_password: password was NULL");
      return FALSE;
    }

  if(optPR.password != NULL)
    {
      g_free(optPR.password);
      optPR.password = NULL;
    }

  optPR.password = g_strdup(password);
  return TRUE;
}

gboolean gj_gtk_pr_set_callback(gj_gtk_pr_callback cb)
{
  optPR.cb = cb;
  return TRUE;
}


/*
 * window functions
 */

gboolean gj_gtk_pr_load(gboolean check_password, const gchar *password, gj_gtk_pr_callback cb)
{
  GladeXML *xml = NULL;

  if(check_password == TRUE && password == NULL)
    {
      g_warning("gj_gtk_pr_load: password was NULL");
      return FALSE;
    }

  if(cb == NULL) 
    {
      g_warning("gj_gtk_pr_load: callback was NULL");
      return FALSE;
    }

  if(ctlPR.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlPR.window));
      return TRUE;
    }

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "password_required",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_pr_load: couldn't open glade file at root 'password_required'");
      return FALSE;
    }
  
  ctlPR.window = glade_xml_get_widget(xml,"password_required");

  ctlPR.entry_password = glade_xml_get_widget(xml,"entry_pr_password");
  ctlPR.button_ok = glade_xml_get_widget(xml,"button_pr_ok");
  
#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (ctlPR.window), "response",
                      GTK_SIGNAL_FUNC (on_password_required_response),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlPR.entry_password), "changed",
                      GTK_SIGNAL_FUNC (on_entry_pr_password_changed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlPR.entry_password), "activate",
                      GTK_SIGNAL_FUNC (on_entry_pr_password_activate),
                      NULL);
#endif

  /* save password and callback */
  gj_gtk_pr_set_check_password(check_password);
  if(check_password == TRUE) gj_gtk_pr_set_password((gchar*)password);
  gj_gtk_pr_set_callback(cb);

  /* set up dialog */
  gtk_widget_set_sensitive(ctlPR.button_ok, FALSE);

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_pr_unload()
{
  if(ctlPR.window != NULL)
    {
      gtk_widget_destroy(GTK_WIDGET(ctlPR.window));
      ctlPR.window = NULL;
    }
  
  return TRUE;
}

gboolean gj_gtk_pr_changed()
{
  G_CONST_RETURN gchar *password = NULL;
  password = gtk_entry_get_text(GTK_ENTRY(ctlPR.entry_password));

  if(password != NULL && strlen(password) > 0) 
    gtk_widget_set_sensitive(GTK_WIDGET(ctlPR.button_ok), TRUE);
  else
    gtk_widget_set_sensitive(GTK_WIDGET(ctlPR.button_ok), FALSE);

  return TRUE;
}

gboolean gj_gtk_pr_call_callback(gboolean success, const gchar *password)
{
  gj_gtk_pr_callback cb = NULL;

  if((cb = gj_gtk_pr_get_callback()) == NULL)
    {
      g_warning("gj_gtk_pr_call_callback: callback was NULL");
      return FALSE;
    }

  (cb)(success, password);

  return TRUE;
}

gboolean gj_gtk_pr_check()
{
  G_CONST_RETURN gchar *password = NULL;
  password = gtk_entry_get_text(GTK_ENTRY(ctlPR.entry_password));

  if(gj_gtk_pr_get_check_password() == FALSE)
    return TRUE;

  if(password == NULL || strlen(password) < 1)
    return FALSE;

  if(gj_gtk_pr_get_password() == NULL)
    {
      g_warning("gj_gtk_pr_check: password was NULL");
      return FALSE;
    }
    
  if(strcmp(gj_gtk_pr_get_password(), password) == 0)
    return TRUE;

  return FALSE;
}

/*
 * GUI events
 */
void on_entry_pr_password_changed(GtkEditable *editable, gpointer user_data)
{
  gj_gtk_pr_changed();
}

void on_entry_pr_password_activate(GtkEntry *entry, gpointer user_data)
{
  G_CONST_RETURN gchar *password = NULL; 
  password = gtk_entry_get_text(GTK_ENTRY(entry));

  if(password == NULL || strlen(password) < 1)
    return;

  on_password_required_response(GTK_DIALOG(ctlPR.window), GTK_RESPONSE_OK, NULL); 
}

void on_password_required_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  G_CONST_RETURN gchar *password = NULL; 

  switch(response) 
    {
    case GTK_RESPONSE_OK:
      password = gtk_entry_get_text(GTK_ENTRY(ctlPR.entry_password));
      gj_gtk_pr_call_callback(gj_gtk_pr_check(), password);
      break;

    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
      gj_gtk_pr_call_callback(FALSE, NULL);
      break;

    default:
      g_message("on_password_required_response: response:%d not handled", response);
      break;
    }

  /* we close the window on ALL responses */
  gtk_widget_destroy(GTK_WIDGET(dialog));
  ctlPR.window = NULL;
}

#ifdef __cplusplus
}
#endif
