/* 
 * $RCSfile: gj_gtk_rename_group.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/28 13:38:29 $
 * 
 * $Revision: 1.2 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_roster.h"

#include "gj_gtk_rename_group.h"
#include "gj_gtk_msg.h"

struct 
{
  GtkWidget *window;

  GtkWidget *entry;
  GtkWidget *label;

  GtkWidget *button_ok;

} ctlRG;


gboolean gj_gtk_rg_save();
     
/*
 * internal callbacks 
 */
void gj_gtk_rg_user_changed_response(guint id);

/*
 * gui functions 
 */
void on_entry_rg_changed(GtkEditable *editable, gpointer user_data);
void on_entry_rg_activate(GtkEntry *entry, gpointer user_data);

void on_rename_group_response(GtkDialog *dialog, gint response, gpointer user_data);


/*
 * window functions
 */

gboolean gj_gtk_rg_load(const gchar *group)
{
  GladeXML *xml = NULL;

  if(ctlRG.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlRG.window));
      return TRUE;
    }

  if(group == NULL || strlen(group) < 1)
    {
      g_warning("gj_gtk_rg_load: group was NULL");
      return FALSE;
    }

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "rename_group",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_rg_load: couldn't open glade file at root 'rename_group'");
      return FALSE;
    }
  
  ctlRG.window = glade_xml_get_widget(xml,"rename_group");

  ctlRG.entry = glade_xml_get_widget(xml,"entry_rg");
  ctlRG.label = glade_xml_get_widget(xml,"label_rg");

  ctlRG.button_ok = glade_xml_get_widget(xml,"button_rg_ok");
  
#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  gtk_signal_connect (GTK_OBJECT (ctlRG.window), "response",
                      GTK_SIGNAL_FUNC (on_rename_group_response),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlRG.entry), "changed",
                      GTK_SIGNAL_FUNC (on_entry_rg_changed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctlRG.entry), "activate",
                      GTK_SIGNAL_FUNC (on_entry_rg_activate),
                      NULL);
#endif

  /* set up window */
  gtk_label_set_text(GTK_LABEL(ctlRG.label), group);

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_rg_unload()
{
  if(ctlRG.window == NULL)
    return FALSE;

  gtk_widget_destroy(GTK_WIDGET(ctlRG.window));

  ctlRG.window = NULL;
  ctlRG.entry = NULL;
  ctlRG.label = NULL;
  ctlRG.button_ok = NULL;

  return TRUE;
}

gboolean gj_gtk_rg_save()
{
  GList *list = NULL;
  gint i = 0;

  G_CONST_RETURN gchar *new_group = NULL;
  G_CONST_RETURN gchar *current_group = NULL;

  new_group = gtk_entry_get_text(GTK_ENTRY(ctlRG.entry));
  current_group = gtk_label_get_label(GTK_LABEL(ctlRG.label));

  if((list = rosterGetAllItemsByGroup((gchar*)current_group)) == NULL)
    return FALSE;

  for(i=0; i<g_list_length(list); i++)
    {
      gchar *jid = NULL;
      rosterItem ri = NULL;
      gint j = 0;

      jid = (gchar*) g_list_nth_data(list, i);
      if(jid == NULL) continue;

      ri = rosterItemFindByJID(jid);
      if(ri == NULL) continue;
	
      if(rosterItemGroupsDelete(ri, (gchar*)current_group) == FALSE) continue;
      if(rosterItemGroupsAdd(ri, (gchar*)new_group) == FALSE) continue;

      /* using the list, modify all the users groups to include the new group */
      infoqueryRequestUserChange(rosterItemGetJID(ri), 
				 rosterItemGetName(ri), 
				 rosterItemGetGroups(ri),
				 gj_gtk_rg_user_changed_response);
    }
      

  /* clean up */
  g_list_foreach(list, (GFunc)g_free, NULL);
  g_list_free(list);
  
  return TRUE;
}

/* callbacks */
void gj_gtk_rg_user_changed_response(guint id)
{
  infoquery iq = NULL;

  g_message("gj_gtk_rg_user_changed_response: response with requestID:%d",id);

  if((iq = infoqueryFindByEventIDRelated(id)) == NULL)
    {
      g_warning("gj_gtk_rg_user_changed_response: failed to find IQ based on responseID:%d",id);

      return;
    }

  /* clean up */
  infoqueryFreeEventAndRelatedByRelatedID(id);
}


/*
 * GUI events
 */
void on_entry_rg_changed(GtkEditable *editable, gpointer user_data)
{
  gchar *group = NULL;
  group = gtk_editable_get_chars(editable, 0, -1);

  if(group != NULL && strlen(group) > 0)
    {
      gtk_widget_set_sensitive(GTK_WIDGET(ctlRG.button_ok), TRUE);
      g_free(group);
      return;
    }
      
  gtk_widget_set_sensitive(GTK_WIDGET(ctlRG.button_ok), FALSE);
}

void on_entry_rg_activate(GtkEntry *entry, gpointer user_data)
{
  G_CONST_RETURN gchar *group = NULL;
  group = gtk_entry_get_text(entry);

  if(group != NULL && strlen(group) > 0)
    on_rename_group_response(GTK_DIALOG(ctlRG.window), GTK_RESPONSE_OK, NULL); 
}

void on_rename_group_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  switch(response) 
    {
    case GTK_RESPONSE_OK:
      gj_gtk_rg_save();
      break;

    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
    case GTK_RESPONSE_APPLY:
      break;
    }

  gj_gtk_rg_unload();
}

#ifdef __cplusplus
}
#endif
