/* 
 * $RCSfile: gj_history.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/23 23:05:49 $
 * 
 * $Revision: 1.6 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <time.h> 

#include <glib.h>

#include "common.h"

#include "gj_history.h"
#include "gj_support.h"
#include "gj_roster.h"
#include "gj_config.h"
#include "gj_main.h"

/* structs */
struct t_history
{
  gchar *jid;
  gchar *filename;

  GIOChannel *channel;
};

gboolean historyListAdd(history hist);
gboolean historyListDel(history hist);

void historyDestroy(history hist);

gchar *historyCheckFilename(const gchar *original_filename);

gchar *historyPath = NULL;
GList *historyList = NULL;

/*
 * code
 */
gboolean historyInit()
{
  gchar *saved_path = NULL;
  saved_path = gj_config_get_mpf_history_location();
  
  if(saved_path == NULL || strlen(saved_path) < 1)
    {
      gchar *path = NULL;

      path = g_build_path(G_DIR_SEPARATOR_S, gnome_jabber_get_config_dir(), PATH_HISTORY, NULL);
      gj_config_set_mpf_history_location(path);

      historyPath = g_strdup(path);

      g_free(path);
    }
  else
    {
      /* use saved path */
      historyPath = g_strdup(saved_path);
    }

  return TRUE;
}

gboolean historyTerm()
{
  gint index = 0;
    
  /* list */
  if(historyList == NULL)
    return TRUE;
    
  /* free all items on the list */
  for(index=0;index<g_list_length(historyList);index++)
    {
      history hist = NULL;
	
      if((hist = (history) g_list_nth_data(historyList,index)) != NULL)
	historyFree(hist);
    }
    
  g_list_free(historyList);
  historyList = NULL;

  if(historyPath != NULL)
    {
      g_free(historyPath);
      historyPath = NULL;
    }
    
  return TRUE;
}

gboolean historyListAdd(history hist)
{
  if(hist == NULL)
    {
      g_warning("historyListAdd: failed to add item to list, it was NULL");
      return FALSE;
    }

  historyList = g_list_append(historyList,hist);

  return TRUE;
}

gboolean historyListDel(history hist)
{
  if(hist == NULL)
    {
      g_warning("historyListDel: failed to del item from list, it was NULL");
      return FALSE;
    }

  historyList = g_list_remove(historyList,hist);
  return TRUE;
}

history historyNew(gchar *jid)
{
  history hist = NULL;

  if(jid == NULL) 
    {
      g_warning("historyNew: jid was NULL");
      return NULL;
    }

  if((hist = g_new(struct t_history,1)))
    {
      hist->jid = g_strdup(jid);

      hist->filename = NULL;
      hist->channel = NULL;
    }

  historyList = g_list_append(historyList, hist);

  return hist;
}

gboolean historyFree(history hist)
{
  if(hist == NULL)
    {
      g_warning("historyFree: failed to free, history was NULL");
      return FALSE;
    }

  /* free mem */
  historyListDel(hist);  

  /* destroy members */
  historyDestroy(hist);

  /* free structure */
  g_free(hist);

  return TRUE;
}

void historyDestroy(history hist)
{
  if(hist == NULL)
    {
      g_warning("historyDestroy: failed to destroy, history was NULL");
      return;
    }

  if(hist->jid != NULL)
    {
      g_free(hist->jid);
      hist->jid = NULL;
    }

  if(hist->filename != NULL)
    {
      g_free(hist->filename);
      hist->filename = NULL;
    }

  if(hist->channel != NULL)
    {
      g_io_channel_shutdown(hist->channel,TRUE,NULL);
      hist->channel = NULL;
    }
}

/*
 * utils
 */
history historyFindByJID(gchar *jid)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("historyFindByJID: jid was NULL");
      return NULL;
    }

  /* loop through all historyList on contacts history list to look for unique historyID */
  for(index=0;index<g_list_length(historyList);index++)
    {
      history hist = g_list_nth_data(historyList, index);
      
      if(hist == NULL)
	{
	  g_warning("historyFindByHistoryID: getting nth item(%d) in history list was NULL",index);
	  continue;
	}
      
      if(g_strcasecmp(hist->jid,jid) == 0)
	return hist;
    }
 
  return NULL;
}

/*
 * member data
 */ 
gboolean historySetFilename(history hist, gchar *filename)
{
  if(hist == NULL)
    {
      g_warning("historySetFilename: history was NULL");
      return FALSE;
    }

  if(filename == NULL)
    {
      g_warning("historySetFilename: roster item was NULL");
      return FALSE;
    }

  if(hist->filename != NULL)
    {
      g_free(hist->filename);
      hist->filename = NULL;
    }

  hist->filename = g_strdup(filename);

  return TRUE;
}

gchar *historyGetJID(history hist)
{
  if(hist == NULL)
    {
      g_warning("historyGetJID: history was NULL");
      return NULL;
    }

  return hist->jid;
}

gchar *historyGetFilename(history hist)
{
  if(hist == NULL)
    {
      g_warning("historyGetFilename: history was NULL");
      return NULL;
    }

  return hist->filename;
}

GIOChannel *historyGetChannel(history hist)
{
  if(hist == NULL)
    {
      g_warning("historyGetChannel: history was NULL");
      return NULL;
    }

  return hist->channel;
}

/*
 * debugging 
 */
void historyPrintAll()
{
  gint index = 0;

  g_message("******* Roster Item History List Start ********");  

  for(index=0;index<g_list_length(historyList);index++)
    {
      history hist = NULL;

      hist = g_list_nth_data(historyList,index);
      if(hist == NULL)
	{
	  g_warning("historyPrintAll: roster item history was NULL");
	  return;
	}
      
      historyPrint(hist);
    }

  g_message("******* Roster Item History List End **********");  

}

void historyPrint(history hist)
{
  if(hist == NULL)
    return;

  g_message("******* Roster Item History ********");
  g_message("*******");
  g_message("******* jid:'%s'",historyGetJID(hist));
  g_message("******* filename:'%s'",historyGetFilename(hist));
  g_message("*******");
  g_message("******* history: 0x%.8x", (gint)historyGetChannel(hist));
  g_message("*******");
  g_message("******* Roster Item History ********");
}

gchar *historyCheckFilename(const gchar *original_filename)
{
  gchar *s1 = NULL;
  gchar *s2 = NULL;
  gchar *s3 = NULL;
  gchar *filename = NULL;
  
  s1 = g_strdup(original_filename);
  
  s2 = supportFindInStrAndReplace("@", s1, "-");
  s3 = supportFindInStrAndReplace("%", s2, "-"); 

  filename = g_strdup(s3);

  /* clean up */
  if(s1 != NULL) g_free(s1);
  if(s2 != NULL) g_free(s2);
  if(s3 != NULL) g_free(s3);

  return filename;
}

/*
 * HISTORY FUNCTIONS 
 *
 * - WIDELY USED BY OUTSIDE THIS MODULE
 *
 */
gboolean historyStart(history hist)
{
  guint bytes = 0;
  gboolean new_file = FALSE;
  gchar *app_start = NULL;

  rosterItem ri = NULL;

  if(hist == NULL)
    {
      g_warning("historyStart: history is NULL");
      return FALSE;
    }

  if((ri = rosterItemFindByJID(historyGetJID(hist))) == NULL)
  {
      g_warning("historyStart: could not find by jid:'%s'", historyGetJID(hist));
      return FALSE;
  }

  if(historyGetFilename(hist) == NULL)
    {
      gchar *filename = NULL;
      gchar *path = NULL;
      gchar *filename_unchecked = NULL;
      gchar *filename_checked = NULL;

      filename_unchecked = g_strdup_printf("%s%s%s.html", 
					   rosterItemGetAgent(ri)?rosterItemGetAgent(ri):"", 
					   rosterItemGetAgent(ri)?" - ":"",
					   rosterItemGetName(ri));

      filename_checked = historyCheckFilename(filename_unchecked);

      /* set path */
      path = gj_config_get_mpf_history_location();
      g_message("historyStart: using history path:'%s'", path);
      
      filename = g_build_filename(path, G_DIR_SEPARATOR_S, filename_checked, NULL);
      g_message("historyStart: for jid:'%s', using filename:'%s'", historyGetJID(hist), filename);
      
      if(filename != NULL)
	{
	  historySetFilename(hist, filename);
	  g_free(filename);
	}
      else
	g_warning("historyStart: could not create string filename for history file, jid:'%s'",
		  historyGetJID(hist));

      /* clean up */
      if(filename_unchecked != NULL) g_free(filename_unchecked);
      if(filename_checked != NULL) g_free(filename_checked);
    }

  /* does the file exist */
  if(g_file_test(historyGetFilename(hist), G_FILE_TEST_EXISTS) == FALSE)
  {
      g_message("historyStart: filename:'%s' does not exist, creating...", historyGetFilename(hist));
      new_file = TRUE;
  }

  /* this opens a file or creates it */
  hist->channel = g_io_channel_new_file(historyGetFilename(hist),
					"a+",
					NULL);

  /* if this is a new file, add the header information */
  if(new_file == TRUE)
  {
        g_io_channel_write_chars(historyGetChannel(hist),
			   "<html><head><link rel=\"stylesheet\" type=\"text/css\" href=\"../gnome-jabber.css\"/></head></html>\n",
			   -1,
			   &bytes,
			   NULL); 
  }

  /* write the fact this is a start up procedure */
  app_start = g_strdup_printf("<html><body><div class=\"message\"><span class=\"date\">%s</span> <span class=\"title\">application started</span><br/></div></body></html>\n", supportGetTimestampFull());

  g_io_channel_write_chars(historyGetChannel(hist),
			   app_start,
			   -1,
			   &bytes,
			   NULL); 

  if(app_start != NULL) g_free(app_start);

  /* flush to make sure tailing is up to date */
  g_io_channel_flush(historyGetChannel(hist), NULL);

  /* check it was created */
  if(hist->channel == NULL)
    {
      g_warning("historyStart: channel failed to append file:'%s'",historyGetFilename(hist));
      return FALSE;
    }

  return TRUE;
}

gboolean historyWriteSentByJID(gchar *jid, gchar *date, gchar *subject, gchar *message)
{
  history hist = NULL;

  if(jid == NULL)
    {
      g_warning("historyWriteSentByJID: jid was NULL");
      return FALSE;
    }

  if((hist = historyFindByJID(jid)) == NULL)
    {
      g_warning("historyWriteSentByJID: could not find by jid:'%s'", jid);
      return FALSE;
    }  

  return historyWriteSent(hist, date, subject, message);
}

gboolean historyWriteSent(history hist, gchar *date, gchar *subject, gchar *message)
{
  guint bytes = 0;
  gchar *formatted = NULL;

  if(gj_config_get_mpf_use_history() == FALSE)
      return TRUE;

  if(hist == NULL)
    {
      g_warning("historyWriteSent: history is NULL");
      return FALSE;
    }

  if(historyGetChannel(hist) == NULL)
    {
      g_warning("historyWriteSent: channel not yet initalised for jid:'%s'", historyGetJID(hist));
      return FALSE;
    }

  formatted = g_strdup_printf("<html><body><div class=\"message\"><span class=\"date\">%s</span> <span class=\"body\">%s</span> <span class=\"from\">%s<br/></span></div></body></html>\n", date, "[sent]", message);

  g_io_channel_write_chars(historyGetChannel(hist),
			   formatted,
			   -1,
			   &bytes,
			   NULL);  

  g_io_channel_flush(historyGetChannel(hist), NULL);

  g_message("historyWriteSent: wrote %d of %d bytes to history (channel:0x%.8x)", 
	    bytes, strlen(formatted), (gint)historyGetChannel(hist)); 

  /* clean up */
  if(formatted != NULL) g_free(formatted);

  return TRUE;
}

gboolean historyWriteReceivedByJID(gchar *jid, gchar *date, gchar *subject, gchar *message)
{
  history hist = NULL;

  if(jid == NULL)
    {
      g_warning("historyWriteReceivedByJID: jid was NULL");
      return FALSE;
    }

  if((hist = historyFindByJID(jid)) == NULL)
    {
      g_message("historyWriteReceivedByJID: could not find by jid:'%s', trying without resource...", jid);
      g_warning("historyWriteReceivedByJID: need to look by jid without resource here... FIXME!");
      
      return FALSE;
    }  

  return historyWriteReceived(hist, date, subject, message);
}

gboolean historyWriteReceived(history hist, gchar *date, gchar *subject, gchar *message)
{
  guint bytes = 0;
  gchar *formatted = NULL;

  if(gj_config_get_mpf_use_history() == FALSE)
      return TRUE;

  if(hist == NULL)
    {
      g_warning("historyWriteReceived: history is NULL");
      return FALSE;
    }

  if(historyGetChannel(hist) == NULL)
    {
      g_warning("historyWriteReceived: channel not yet initalised for jid:'%s'", historyGetJID(hist));
      return FALSE;
    }

  formatted = g_strdup_printf("<html><body><div class=\"message\"><span class=\"date\">%s</span> <span class=\"body\">%s</span> <span class=\"to\">%s<br/></span></div></body></html>\n", date, "[recv]", message);

  g_io_channel_write_chars(historyGetChannel(hist),
			   formatted,
			   -1,
			   &bytes,
			   NULL);  

  g_io_channel_flush(historyGetChannel(hist), NULL);

  g_message("historyWriteReceived: wrote %d of %d bytes to history (channel:0x%.8x)", 
	    bytes, strlen(formatted), (gint)historyGetChannel(hist)); 

  /* clean up */
  if(formatted != NULL) g_free(formatted);

  return TRUE;
}

#ifdef __cplusplus
}
#endif
