/* 
 * $RCSfile: gj_iq.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/26 11:09:58 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h> 

#include <glib.h>

#include "gj_iq.h"
#include "gj_iq_responses.h"
#include "gj_parser.h"
#include "gj_roster.h"
#include "gj_me.h"

/* iq list */
gboolean infoqueryListAdd(infoquery iq);
gboolean infoqueryListDel(infoquery iq);

/* iq */
void infoqueryDestroy(infoquery iq);

/* checks */
gboolean infoqueryCheckParms(infoquery iq);

/* other utils */
void infoqueryPasswordChangeResponse(guint requestID);

/* structs */
struct t_infoquery
{
  guint eventID;
  guint eventIDRelated;

  infoqueryType type;

  infoqueryCallback cb;

  void *userdata;

  gchar *id;
  gchar *from;
  gchar *to;

  gchar *associated_jid;

  xmlNodePtr xml;
};

GList *infoqueryList;

/*
 * InfoQuery List functions
 */
gboolean infoqueryListAdd(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryListAdd: failed to add iq to list, it was NULL");
      return FALSE;
    }

  infoqueryList = g_list_append(infoqueryList,iq);
  return TRUE;
}

gboolean infoqueryListDel(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryListDel: failed to del iq from list, it was NULL");
      return FALSE;
    }

  infoqueryList = g_list_remove(infoqueryList,iq);
  return TRUE;
}

/* 
 * InfoQuery functions 
 */
infoquery infoqueryNew(infoqueryType type)
{
  infoquery iq = NULL;
  
  if((iq = (infoquery) g_new(struct t_infoquery,1)) != NULL)
    {
      iq->eventID = supportGetUniqueID();
      iq->eventIDRelated = 0;
      
      iq->type = type;

      iq->xml = NULL;

      iq->cb = NULL;

      iq->userdata = NULL;

      iq->id = NULL;
      iq->to = NULL;
      iq->from = NULL;

      iq->associated_jid = NULL;

      infoqueryListAdd(iq);
    }
  
  return iq;  
}

void infoqueryFree(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryFree: failed to free, iq was NULL");
      return;
    }

  /* free from list */
  infoqueryListDel(iq);

  /* free members */
  infoqueryDestroy(iq);

  /* free mem */
  g_free(iq);  
}

void infoqueryFreeByEventID(guint eventID)
{
  infoquery iq = NULL;
  
  if((iq = infoqueryFindByEventID(eventID)) == NULL)
    {
      g_warning("infoqueryFreeByEventID: could not find infoquery based on eventID:%d",eventID);
      return;
    }
  
  infoqueryFree(iq);

/*   infoqueryPrintAll(); */
}

void infoqueryFreeEventAndRelatedByEventID(guint eventID)
{
  infoquery iq = NULL;
  infoquery iq_related = NULL;
  
  guint eventIDRelated = 0;
  
  if((iq = infoqueryFindByEventID(eventID)) == NULL)
    {
      g_warning("infoqueryFreeEventAndRelatedByEventID: could not find infoquery based on eventID:%d",eventID);
      return;
    }
  else
    {
      /* get eventIDRelated from event */
      eventIDRelated = infoqueryGetEventIDRelated(iq);
      infoqueryFree(iq);
    }
  
  
  if((iq_related = infoqueryFindByEventID(eventIDRelated)) == NULL)
    {
      g_warning("infoqueryFreeEventAndRelatedByEventID: could not find infoquery response based on eventID:%d", eventID);
      return;
    }
  else
    infoqueryFree(iq_related);
}

void infoqueryFreeEventAndRelatedByRelatedID(guint eventIDRelated)
{
  infoquery iq = NULL;
  infoquery iq_related = NULL;
  
  guint eventID = 0;
  
  if((iq_related = infoqueryFindByEventIDRelated(eventIDRelated)) == NULL)
    {
      g_warning("infoqueryFreeEventAndRelatedByRelatedID: could not find infoquery based on eventIDRelated:%d",eventIDRelated);
      return;
    }
  else
    {
      /* get eventIDRelated from event */
      eventID = infoqueryGetEventID(iq_related);
      infoqueryFree(iq_related);
    }
  
  
  if((iq = infoqueryFindByEventIDRelated(eventID)) == NULL)
    {
      g_warning("infoqueryFreeEventAndRelatedByRelatedID: could not find infoquery event based on eventIDRelated:%d", eventIDRelated);
      return;
    }
  else
    infoqueryFree(iq);
}

void infoqueryDestroy(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryDestroy: failed to destroy, iq was NULL");
      return;
    }

  iq->eventID = 0;
  iq->eventIDRelated = 0;

#if 0  
  iq->eventID = 0; /* get next */
  iq->responseID = 0;
#endif

  if(iq->xml != NULL)
    {
      /* freeing NODE crashes ??? */
      /*xmlFreeNode(iq->xml);  */
      iq->xml = NULL;
    }

  iq->type = 0;
  
  iq->cb = NULL;
 
  iq->userdata = NULL;

  if(iq->id != NULL)
    {
      g_free(iq->id);
      iq->id = NULL;
    }

  if(iq->to != NULL)
    {
      g_free(iq->to);
      iq->to = NULL;
    }

  if(iq->from != NULL)
    {
      g_free(iq->from);
      iq->from = NULL;
    }

  if(iq->associated_jid != NULL)
    {
      g_free(iq->associated_jid);
      iq->associated_jid = NULL;
    }
}


void infoqueryPrintAll()
{
  infoquery iq = NULL;
  gint index = 0;

  g_message("\n");
  g_message("++++++++++++++++++++++ InfoQuery - Print All, Begin. +++++++++++++++++++++");

  for(index=0;index<g_list_length(infoqueryList);index++)
    {
      iq = (infoquery) g_list_nth_data(infoqueryList,index);
      infoqueryPrint(iq);
    }

  g_message("++++++++++++++++++++++ InfoQuery - Print All, End.   +++++++++++++++++++++");
  
  return;  
}

void infoqueryPrint(infoquery iq)
{
  if(iq != NULL)
    {
      g_message("+++++++++++++++++++++++++++ InfoQuery +++++++++++++++++++++++++");
      
      g_message("++++");
      g_message("++++ id:'%s'",iq->id);
      g_message("++++ from:'%s'",iq->from);
      g_message("++++ to:'%s'",iq->to);
      g_message("++++ type:%d",iq->type);
      g_message("++++");
      g_message("++++ associated jid:'%s'",iq->associated_jid);
      g_message("++++");
      g_message("++++ eventID:%d",iq->eventID);
      g_message("++++ eventIDRelated:%d",iq->eventIDRelated);
      g_message("++++");
      g_message("++++ response cb:0x%.8x",(gint)iq->cb);
      g_message("++++ userdata:0x%.8x",(gint)iq->userdata);
      g_message("++++");
      g_message("++++ xml:0x%.8x",(gint)iq->xml);
      
      g_message("+++++++++++++++++++++++++++ InfoQuery +++++++++++++++++++++");
    }
  else 
    g_warning("infoqueryPrint: failed to print, NULL InfoQuery.");
}

/* 
 * external functions 
 */
gboolean infoqueryInit()
{
  /* list is created on first append */
  return TRUE;
}

gboolean infoqueryTerm()
{
  gint index = 0;

  if(infoqueryList == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(infoqueryList);index++)
    {
      infoquery iq = NULL;
      
      if((iq = (infoquery) g_list_nth_data(infoqueryList,index)) != NULL)
	infoqueryFree(iq);
    }

  g_list_free(infoqueryList);
  infoqueryList = NULL;

  return TRUE;
}

/* 
 * utils to add to struct 
 */

/* functions to SET information */
gboolean infoquerySetUserData(infoquery iq, void *userdata)
{
  if(iq == NULL)
    {
      g_warning("infoquerySetUserData: failed, InfoQuery was NULL");
      return FALSE;
    }

  if(userdata == NULL)
    {
      g_warning("infoquerySetUserData: failed, userdata is NULL");
      return FALSE;
    }

  iq->userdata = userdata;

  return TRUE;
}

gboolean infoquerySetEventID(infoquery iq, guint eventID)
{
  if(iq == NULL)
    {
      g_warning("infoquerySetEventID: failed, InfoQuery was NULL");
      return FALSE;
    }

  if(eventID < 1)
    {
      g_warning("infoquerySetEventID: failed, eventID:%d was < 1",eventID);
      return FALSE;
    }

  iq->eventID = eventID;
  return TRUE;
}

gboolean infoquerySetEventIDRelated(infoquery iq, guint eventID)
{
  if(iq == NULL)
    {
      g_warning("infoquerySetEventIDRelated: failed, InfoQuery was NULL");
      return FALSE;
    }

  if(eventID < 1)
    {
      g_warning("infoquerySetEventIDRelated: failed, eventID:%d was < 1",eventID);
      return FALSE;
    }

  iq->eventIDRelated = eventID;
  return TRUE;
}

gboolean infoquerySetID(infoquery iq, gchar *id)
{
  if(iq == NULL)
    {
      g_warning("infoquerySetID: failed, InfoQuery was NULL");
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("infoquerySetID: failed, NULL passed");
      return FALSE;
    }

  if(iq->id != NULL)
    {
      g_warning("infoquerySetID: InfoQuery ID was already set, freeing...");
      g_free(iq->id);
    }
  
  iq->id = g_strdup(id);

  return TRUE;
}

gboolean infoquerySetTo(infoquery iq, gchar *to)
{
  if(iq == NULL)
    {
      g_warning("infoquerySetTo: failed, InfoQuery was NULL");
      return FALSE;
    }

  if(to == NULL)
    {
      g_warning("infoquerySetTo: failed, NULL passed");
      return FALSE;
    }

  if(iq->to != NULL)
    {
      g_warning("infoquerySetTo: InfoQuery To was already set, freeing...");
      g_free(iq->to);
    }
  
  iq->to = g_strdup(to);

  return TRUE;
}

gboolean infoquerySetFrom(infoquery iq, gchar *from)
{
  if(iq == NULL)
    {
      g_warning("infoquerySetFrom: failed, InfoQuery was NULL");
      return FALSE;
    }

  if(from == NULL)
    {
      g_warning("infoquerySetFrom: failed, NULL passed");
      return FALSE;
    }

  if(iq->from != NULL)
    {
      g_warning("infoquerySetFrom: InfoQuery From was already set, freeing...");
      g_free(iq->from);
    }
  
  iq->from = g_strdup(from);

  return TRUE;
}

gboolean infoquerySetType(infoquery iq, infoqueryType type)
{
  if(iq == NULL)
    {
      g_warning("infoquerySetType: failed, InfoQuery was NULL");
      return FALSE;
    }

  if(type < 1)
    {
      g_warning("infoquerySetType: failed, type:%d < 1",type);
      return FALSE;
    }

  iq->type = type;

  return TRUE;
}

gboolean infoquerySetXML(infoquery iq, xmlNodePtr node)
{
  if(iq == NULL)
    {
      g_warning("infoquerySetXML: failed, InfoQuery was NULL");
      return FALSE;
    }

  if(iq == NULL)
    {
      g_warning("infoquerySetXML: failed, node was NULL");
      return FALSE;
    } 

  iq->xml = node;

  return TRUE;
}

gboolean infoquerySetCallback(infoquery iq,  infoqueryCallback cb)
{
  if(iq == NULL)
    {
      g_warning("infoquerySetCallback: failed, InfoQuery was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoquerySetCallback: failed, proc was NULL");
      return FALSE;
    }

  iq->cb = cb;

  return TRUE;
}

gboolean infoquerySetAssociatedJID(infoquery iq, gchar *jid)
{
  if(iq == NULL)
    {
      g_warning("infoquerySetAssociatedJID: failed, InfoQuery was NULL");
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("infoquerySetAssociatedJID: failed, NULL passed");
      return FALSE;
    }

  if(iq->from != NULL)
    {
      g_warning("infoquerySetAssociatedJID: InfoQuery associated jid was already set, freeing...");
      g_free(iq->associated_jid);
    }
  
  iq->associated_jid = g_strdup(jid);

  return TRUE;
}


/* functions to GET information */
void * infoqueryGetUserData(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryGetUserData: failed, InfoQuery was NULL");
      return FALSE;
    }

  return iq->userdata;
}

gchar *infoqueryGetID(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryGetID: failed, InfoQuery was NULL");
      return NULL;
    }

  return iq->id;
}

gchar *infoqueryGetTo(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryGetTo: failed, InfoQuery was NULL");
      return NULL;
    }

  return iq->to;
}

gchar *infoqueryGetFrom(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryGetFrom: failed, InfoQuery was NULL");
      return NULL;
    }

  return iq->from;
}

infoqueryType infoqueryGetType(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryGetType: failed, InfoQuery was NULL");
      return 0;
    }

  return iq->type;
}

xmlNodePtr infoqueryGetXML(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryGetXML: failed, InfoQuery was NULL");
      return FALSE;
    }

  return iq->xml;
}

void *infoqueryGetCallback(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryGetCallback: failed, InfoQuery was NULL");
      return FALSE;
    }

  return (void*)iq->cb;
}

guint infoqueryGetEventID(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryGetEventID: failed, InfoQuery was NULL");
      return -1;
    }

  return iq->eventID;
}

guint infoqueryGetEventIDRelated(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryGetEventIDRelated: failed, InfoQuery was NULL");
      return -1;
    }

  return iq->eventIDRelated;
}

gchar *infoqueryGetAssociatedJID(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryGetAssociatedJID: failed, InfoQuery was NULL");
      return NULL;
    }

  return iq->associated_jid;
}


/* 
 * compile iq 
 */
gboolean infoqueryCheckParms(infoquery iq)
{
  if(iq == NULL)
    {
      g_warning("infoqueryCheckParams: failed to compile, InfoQuery was NULL");
      return FALSE;
    } 

  /* if iq is type GET, requires ID, To (optional) 
     if iq is type SET, requires ID, To (optional)
     if iq is type RESULT, requires ID, To (optional), From
     if iq is type ERROR, requires ID, To (optional), From
  */

  if(iq->type != infoqueryTypeGet && 
     iq->type != infoqueryTypeSet && 
     iq->type != infoqueryTypeResult && 
     iq->type != infoqueryTypeError)
    {
      g_warning("infoqueryCheckParams: failed to compile, infoqueryType:%d unrecognised",iq->type);
      return FALSE;      
    }

  return TRUE;
}

gboolean infoqueryBuild(infoquery iq, xmlNodePtr node)
{
  xmlNodePtr parent = NULL;

  if(iq == NULL)
    {
      g_warning("infoqueryBuild: failed, InfoQuery was NULL");
      return FALSE;
    } 
  
  if(node == NULL)
    {
      g_warning("infoqueryBuild: failed, node was NULL");
      return FALSE;
    }
  
  if(infoqueryCheckParms(iq) == FALSE)
    return FALSE;
  
  /* set root node */
  parent = xmlNewNode(NULL, (guchar*)"iq");
  
  /* id */
  if(infoqueryGetID(iq) == NULL)
    {
      gchar *id = NULL;
      
      if((id = g_strdup_printf("%d",infoqueryGetEventID(iq))) != NULL)
	{
	  infoquerySetID(iq,id);
	  g_free(id);
	}
    }

  xmlSetProp(parent, (guchar*)"id", (guchar*)infoqueryGetID(iq));
  
  /* type */
  if(iq->type == infoqueryTypeGet)
    xmlSetProp(parent, (guchar*)"type", (guchar*)"get");
  if(iq->type == infoqueryTypeSet)
    xmlSetProp(parent, (guchar*)"type", (guchar*)"set");
  if(iq->type == infoqueryTypeResult)
    xmlSetProp(parent, (guchar*)"type", (guchar*)"result");
  if(iq->type == infoqueryTypeError)
    xmlSetProp(parent, (guchar*)"type", (guchar*)"error");      

  /* 
   * set props 
   */
  
  /* to */
  if(infoqueryGetTo(iq) != NULL)
    xmlSetProp(parent, (guchar*)"to", (guchar*)infoqueryGetTo(iq));
    
  /* from */
  if(infoqueryGetFrom(iq) != NULL)
    xmlSetProp(parent, (guchar*)"from", (guchar*)infoqueryGetFrom(iq));

  /* add node */
  xmlAddChild(parent, node);

  /* set iq query */
  infoquerySetXML(iq, parent);

  return TRUE;
}

gboolean infoqueryBuildFromList(infoquery iq, GList *nodes)
{
   xmlNodePtr parent = NULL;
   
   guint count = 0;

   if(iq == NULL)
     {
       g_warning("infoqueryBuildFromList: failed, InfoQuery was NULL");
       return FALSE;
     } 
   
   if(nodes == NULL)
     {
       g_warning("infoqueryBuildFromList: failed, node list was NULL");
       return FALSE;
     }
   
   if(infoqueryCheckParms(iq) == FALSE)
     return FALSE;
   
   /* set root node */
   parent = xmlNewNode(NULL, (guchar*)"iq");
   
   /* id */
   if(infoqueryGetID(iq) == NULL)
     {
       gchar *id = NULL;
       
       if((id = g_strdup_printf("%d",infoqueryGetEventID(iq))) != NULL)
	 {
	   infoquerySetID(iq, id);
	   g_free(id);
	 }
     }
   
   xmlSetProp(parent, (guchar*)"id", (guchar*)infoqueryGetID(iq));
   
   /* type */
   if(iq->type == infoqueryTypeGet)
     xmlSetProp(parent, (guchar*)"type", (guchar*)"get");
   if(iq->type == infoqueryTypeSet)
     xmlSetProp(parent, (guchar*)"type", (guchar*)"set");
   if(iq->type == infoqueryTypeResult)
     xmlSetProp(parent, (guchar*)"type", (guchar*)"result");
   if(iq->type == infoqueryTypeError)
     xmlSetProp(parent, (guchar*)"type", (guchar*)"error");      
   
   /* 
    * set props 
    */
   
   while(count < g_list_length(nodes)) 
     {
       xmlNodePtr node = g_list_nth_data(nodes,count);
       
       if(node == NULL)
	 continue;
       else
	 count++;

       /* add node */
       xmlAddChild(parent,node);
     }
   
   if(count == 0)
     {
       g_warning("infoqueryBuildFromList: failed, there were 0 nodes");
       return FALSE;
     }
   
   /* to */
   if(infoqueryGetTo(iq) != NULL)
     xmlSetProp(parent, (guchar*)"to", (guchar*)infoqueryGetTo(iq));
   
   /* from */
   if(infoqueryGetFrom(iq) != NULL)
     xmlSetProp(parent, (guchar*)"from", (guchar*)infoqueryGetFrom(iq));
   
   /* set iq query */
   infoquerySetXML(iq, parent);
   
   return TRUE;
}

gboolean infoquerySend(infoquery iq)
{
  xmlNodePtr node = NULL;

  if(iq == NULL)
    {
      g_warning("infoquerySend: failed, InfoQuery was NULL");
      return FALSE;
    } 

  if((node = infoqueryGetXML(iq)) == NULL)
    {
      g_warning("infoquerySend: failed, actual query was NULL");
      return FALSE;
    }

  return parserOutput(node);
}

infoquery infoqueryFindByEventID(guint eventID)
{
  gint index = 0;

  if(eventID < 0)
    {
      g_warning("infoqueryFindByEventID: failed to find by id, id:%d was < 0",eventID);
      return NULL;
    }

  for(index=0;index<g_list_length(infoqueryList);index++)
    {
      infoquery iq = g_list_nth_data(infoqueryList,
				     index);

      if(iq == NULL)
	{
	  g_warning("infoqueryFindByEventID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(iq->eventID == eventID)
	{
	  return iq;
	}
    }

  return NULL;
}

infoquery infoqueryFindByEventIDRelated(guint eventID)
{
  gint index = 0;

  if(eventID < 0)
    {
      g_warning("infoqueryFindByEventIDRelated: failed to find by id, id:%d was < 0",eventID);
      return NULL;
    }

  for(index=0;index<g_list_length(infoqueryList);index++)
    {
      infoquery iq = g_list_nth_data(infoqueryList,
				     index);

      if(iq == NULL)
	{
	  g_warning("infoqueryFindByEventIDRelated: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(iq->eventIDRelated == eventID)
	{
	  return iq;
	}
    }

  return NULL;
}

infoquery infoqueryFindByID(gchar *id)
{
  gint index = 0;

  if(id == NULL)
    {
      g_warning("infoqueryFindByID: failed to find by id, id was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(infoqueryList);index++)
    {
      infoquery iq = g_list_nth_data(infoqueryList,
				     index);

      if(iq == NULL)
	{
	  g_warning("infoqueryFindByID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(iq->id != NULL && strncmp(id, iq->id, strlen(id)) == 0)
	{
	  return iq;
	}
    }

  return NULL;
}


/*
 * ENTRY POINT called by parser
 */
void infoqueryReceive(connection c, xmlNodePtr node)
{
  xmlChar *tmp = NULL;

  infoquery iq_new = NULL;
  infoquery iq_related = NULL;
  infoqueryType type = infoqueryTypeError;

  if(node == NULL)
    {
      g_warning("infoqueryReceive: failed to decipher xml, node was NULL");
      return;
    }

  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"id")) != NULL && xmlStrlen(tmp) > 0)
    {
      /* see if there already is an infoquery with this id, 
	 if so, this is possibly the response */ 
      iq_related = infoqueryFindByID((gchar*)tmp); 
    }

  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"type")) != NULL && xmlStrlen(tmp) > 0)
      {
	if(xmlStrncmp(tmp, (guchar*)"get", xmlStrlen(tmp)) == 0)
	  type = infoqueryTypeGet;
	else if(xmlStrncmp(tmp, (guchar*)"set", xmlStrlen(tmp)) == 0)
	  type = infoqueryTypeSet;
	else if(xmlStrncmp(tmp, (guchar*)"result", xmlStrlen(tmp)) == 0)
	  type = infoqueryTypeResult;
	else if(xmlStrncmp(tmp, (guchar*)"error", xmlStrlen(tmp)) == 0)
	  type = infoqueryTypeError;
	else
	  {
	    g_warning("infoqueryReceive: could not determin infoquery type, throwing away xml...");
	    return;
	  }
      }
  
  if((iq_new = infoqueryNew(type)) == NULL)
    {
      g_warning("infoqueryReceive: failed to create new info query, information query was NULL");
      return;
    }
  
  /* set xml for iq */
  infoquerySetXML(iq_new, node);
  
  /* set characteristics */
  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"id")) != NULL && xmlStrlen(tmp) > 0)
    infoquerySetID(iq_new, (gchar*)tmp);
  
  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"to")) != NULL && xmlStrlen(tmp) > 0)
    infoquerySetTo(iq_new, (gchar*)tmp);
  
  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"from")) != NULL && xmlStrlen(tmp) > 0)
    infoquerySetFrom(iq_new, (gchar*)tmp);
  
  /* 
     see if this iq relates to a previous event, 
     and if so call the callback 
  */
  if(iq_related != NULL)
    {
      infoqueryCallback cb = NULL;
      
      if((cb = (infoqueryCallback) infoqueryGetCallback(iq_related)) == NULL)
	{
	  g_message("infoqueryReceive: query has no callback for response to original query with id:'%s', freeing...",infoqueryGetID(iq_related));
	  
	  /* free mem */
	  infoqueryFree(iq_new);  
	  infoqueryFree(iq_related); 
	  
	  return;
	}
      
      /* set responseID for new response infoquery */
      infoquerySetEventIDRelated(iq_related, infoqueryGetEventID(iq_new));
      infoquerySetEventIDRelated(iq_new, infoqueryGetEventID(iq_related));
      
      /* call callback function */
      (cb)(infoqueryGetEventID(iq_related));
      
      
#if 0
      /* free iq */
      infoqueryFree(iq_related);
#endif
    }
  else
    {
      /* not related to a previous event, must be new from the server */
      infoqueryRequestNew(iq_new);
    }
}

#ifdef __cplusplus
}
#endif
