/* 
 * $RCSfile: gj_iq_requests.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/22 10:54:30 $
 * 
 * $Revision: 1.2 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>

#include "gj_iq.h"
#include "gj_iq_requests.h"
#include "gj_iq_responses.h"
#include "gj_roster.h"
#include "gj_parser.h"

/*
 * ACTUAL REQUESTS from outside this module, eg:
 *
 *  - Greeting 
 *  - Auth Method
 *  - Auth
 *  - Roster
 *  - RosterRemove
 *  - RosterAdd
 *  - PasswordChange
 *  - UserChange (name, group...) 
 *  - VCardChange
 *  - VCard
 *  - Version
 *  - Time
 *  - Last
 *  - Agents
 *  ...
 *
 */

gboolean infoqueryRequestGoodbye(gchar *to)
{
    streamCloseByTo(to);
  
  return TRUE;
}

gboolean infoqueryRequestAuthMethod(gchar *username, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  /* checks */
  if(username == NULL)
    {
      g_warning("infoqueryRequestAuthMethod: username was NULL");
      return FALSE;
    }

  if(cb == NULL)  
    {
      g_warning("infoqueryRequestAuthMethod: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeGet)) == NULL)
    {
      g_warning("infoqueryRequestAuthMethod: information query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:auth", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"username", (guchar*)username);

  /* set struct info */
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq, node) == FALSE)
    {
      g_warning("infoqueryRequestAuthMethod: failed build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestAuthWithPlainPassword(gchar *username, gchar *password, gchar *resource, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  /* checks */
  if(username == NULL)
    {
      g_warning("infoqueryRequestAuthWithPlainPassword: username was NULL");
      return FALSE;
    }

  if(password == NULL)
    {
      g_warning("infoqueryRequestAuthWithPlainPassword: password was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestAuthWithPlainPassword: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeSet)) == NULL)
    {
      g_warning("infoqueryRequestAuthWithPlainPassword: information query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:auth", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"username", (guchar*)username);
  sub = xmlNewChild(node, ns, (guchar*)"password", (guchar*)password);
  sub = xmlNewChild(node, ns, (guchar*)"resource", (guchar*)resource);

  /* set struct info */
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestAuthWithPlainPassword: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestAuthWithDigestPassword(gchar *username, gchar *digest, gchar *resource, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  /* checks */   
  if(username == NULL)
    {
      g_warning("infoqueryRequestAuthWithDigestPassword: username was NULL");
      return FALSE;
    }

  if(digest == NULL)
    {
      g_warning("infoqueryRequestAuthWithDigestPassword: digest was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestAuthWithDigestPassword: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeSet)) == NULL)
    {
      g_warning("infoqueryRequestAuthWithDigestPassword: information query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:auth", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"username", (guchar*)username);
  sub = xmlNewChild(node, ns, (guchar*)"digest", (guchar*)digest);
  sub = xmlNewChild(node, ns, (guchar*)"resource", (guchar*)resource);

  /* set struct info */
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestAuthWithDigestPassword: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestRoster(infoqueryRosterResponseProc cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNsPtr ns = NULL;

  if(cb == NULL)
    {
      g_warning("infoqueryRequestRoster: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeGet)) == NULL)
    {
      g_warning("infoqueryRequestRoster: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:roster", NULL);
 
  /* set struct info */
  infoquerySetCallback(iq, infoqueryRosterResponse);
  infoquerySetUserData(iq, (void*)cb);

  if(infoqueryBuild(iq, node) == FALSE)
    {
      g_warning("infoqueryRequestRoster: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

void infoqueryRosterResponse(guint eventID)
{
  infoquery iq = NULL;
  infoquery iq_related = NULL;
  infoqueryRosterResponseProc proc = NULL;

  guint eventIDRelated = 0;
  
  GList *roster = NULL;

  xmlNodePtr node = NULL;

  /* find request */
  if((iq = infoqueryFindByEventID(eventID)) == NULL)
    {
      g_warning("infoqueryRosterResponse: failed to continue with response, the InfoQuery could not be found by requestID:%d.",eventID);
      return;
    }

  /* get response id */
  eventIDRelated = infoqueryGetEventIDRelated(iq);

  /* find response */
  if((iq_related = infoqueryFindByEventID(eventIDRelated)) == NULL)
    {
      g_warning("infoqueryRosterResponse: failed to continue with response, the InfoQuery could not be found by requestID:%d.",eventIDRelated);
      return;
    }
  

  /* get info */
  node = infoqueryGetXML(iq_related);
  proc = (infoqueryRosterResponseProc) infoqueryGetUserData(iq);

  if((roster = rosterParse(node)) == NULL)
    g_message("infoqueryRosterResponse: currently there is an empty roster.");

  (proc)(roster);

  /* free mem */
  infoqueryFree(iq);  
  infoqueryFree(iq_related);  
}

gboolean infoqueryRequestRosterRemoveContact(gchar *jid, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL; 
  xmlNsPtr ns = NULL;

  if(jid == NULL)
    {
      g_warning("infoqueryRequestRosterRemoveContact: jid was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestRosterRemoveContact: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeSet)) == NULL)
    {
      g_warning("infoqueryRequestRosterRemoveContact: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:roster", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"item", NULL);
  xmlSetProp(sub, (guchar*)"jid", (guchar*)jid);
  xmlSetProp(sub, (guchar*)"subscription", (guchar*)"remove");
  
  /* set struct info */
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestRosterRemoveContact: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);  
}

gboolean infoqueryRequestRosterAddContact(gchar *jid, gchar *name, GList *groups, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL; 
  xmlNsPtr ns = NULL;

  gint i = 0;

  if(jid == NULL)
    {
      g_warning("infoqueryRequestRosterAddContact: jid was NULL");
      return FALSE;
    }

  if(name == NULL)
    {
      g_warning("infoqueryRequestRosterAddContact: name was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestRosterAddContact: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeSet)) == NULL)
    {
      g_warning("infoqueryRequestRosterAddContact: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:roster", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"item", NULL);
  xmlSetProp(sub, (guchar*)"jid", (guchar*)jid);
  xmlSetProp(sub, (guchar*)"name", (guchar*)name);

  if(groups != NULL)
    {
      for(i=0;i<g_list_length(groups);i++)
	{
	  gchar *group = NULL;
	  
	  if((group = (gchar*) g_list_nth_data(groups, i)) == NULL)
	    continue;
	  
	  xmlNewChild(sub, ns, (guchar*)"group", (guchar*)group);
	}
    }

  /* set struct info */
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestRosterAddContact: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);  
}

gboolean infoqueryRequestPasswordChange(gchar *to, gchar *password, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  if(to == NULL)
    {
      g_warning("infoqueryRequestPasswordChange: to was NULL");
      return FALSE;
    }

  if(password == NULL)
    {
      g_warning("infoqueryRequestPasswordChange: password was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestPasswordChange: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeSet)) == NULL)
    {
      g_warning("infoqueryRequestPasswordChange: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:register", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"password", (guchar*)password);
 
  /* set struct info */
  infoquerySetTo(iq,to);
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestPasswordChange: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestUserChange(gchar *jid, gchar *name, GList *groups, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  if(jid == NULL)
    {
      g_warning("infoqueryRequestUserChange: JID was NULL");
      return FALSE;
    }

  if(name == NULL)
    {
      g_warning("infoqueryRequestUserChange: name was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestUserChange: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeSet)) == NULL)
    {
      g_warning("infoqueryRequestUserChange: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:roster", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"item", NULL);
  xmlSetProp(sub, (guchar*)"jid", (guchar*)jid);
  xmlSetProp(sub, (guchar*)"name", (guchar*)name);

  /* go through groups list and add each item */
  if(groups != NULL)
    {
      gint i = 0;
      
      for(i=0;i<g_list_length(groups);i++)
	{
	  gchar *g = (gchar*) g_list_nth_data(groups, i);

	  if(g == NULL)
	    continue;

	  /* add group */
	  xmlNewChild(sub, ns, (guchar*)"group", (guchar*)g);
	}
    }
 
  /* set struct info */
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestUserChange: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestVCardChange(gchar *fn,
				     gchar *given,
				     gchar *family,
				     gchar *nickname,
				     gchar *url,
				     gchar *tel,
				     gchar *email,
				     gchar *desc,
				     gchar *bday,
				     gchar *street,
				     gchar *extadd,
				     gchar *locality,
				     gchar *region,
				     gchar *pcode,
				     gchar *country,
				     gchar *orgname,
				     gchar *orgunit,
				     gchar *title,
				     gchar *role,
				     infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub1 = NULL;
  xmlNodePtr sub2 = NULL;
  xmlNsPtr ns = NULL;

  if(cb == NULL)
    {
      g_warning("infoqueryRequestVCardChange: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeSet)) == NULL)
    {
      g_warning("infoqueryRequestVCardChange: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"vcard");
  ns = xmlNewNs(node, (guchar*)"vcard-temp", NULL);
  sub1 = xmlNewChild(node, ns, (guchar*)"FN", (guchar*)fn);

  sub1 = xmlNewChild(node, ns, (guchar*)"N", NULL);
  sub2 = xmlNewChild(sub1, ns, (guchar*)"FAMILY", (guchar*)family);
  sub2 = xmlNewChild(sub1, ns, (guchar*)"GIVEN", (guchar*)given);

  sub1 = xmlNewChild(node, ns, (guchar*)"NICKNAME", (guchar*)nickname);
  sub1 = xmlNewChild(node, ns, (guchar*)"URL", (guchar*)url);
  sub1 = xmlNewChild(node, ns, (guchar*)"TEL", (guchar*)tel);
  sub1 = xmlNewChild(node, ns, (guchar*)"EMAIL", (guchar*)email);
  sub1 = xmlNewChild(node, ns, (guchar*)"DESC", (guchar*)desc);
  sub1 = xmlNewChild(node, ns, (guchar*)"BDAY", (guchar*)bday);

  sub1 = xmlNewChild(node, ns, (guchar*)"ADR", NULL);
  sub2 = xmlNewChild(sub1, ns, (guchar*)"STREET", (guchar*)street);
  sub2 = xmlNewChild(sub1, ns, (guchar*)"EXTADD", (guchar*)extadd);
  sub2 = xmlNewChild(sub1, ns, (guchar*)"LOCALITY", (guchar*)locality);
  sub2 = xmlNewChild(sub1, ns, (guchar*)"REGION", (guchar*)region);
  sub2 = xmlNewChild(sub1, ns, (guchar*)"PCODE", (guchar*)pcode);
  sub2 = xmlNewChild(sub1, ns, (guchar*)"COUNTRY", (guchar*)country);

  sub1 = xmlNewChild(node, ns, (guchar*)"ORG", NULL);
  sub2 = xmlNewChild(sub1, ns, (guchar*)"ORGNAME", (guchar*)orgname);
  sub2 = xmlNewChild(sub1, ns, (guchar*)"ORGUNIT", (guchar*)orgunit);

  sub1 = xmlNewChild(node, ns, (guchar*)"TITLE", (guchar*)title);
  sub1 = xmlNewChild(node, ns, (guchar*)"ROLE", (guchar*)role);

  /* set struct info */
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestVCardChange: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestVCard(gchar *to, infoqueryCallback cb, void *userdata)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNsPtr ns = NULL;

  if(to == NULL)
    {
      g_warning("infoqueryRequestVCard: to was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestVCard: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeGet)) == NULL)
    {
      g_warning("infoqueryRequestVCard: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"vcard");
  ns = xmlNewNs(node, (guchar*)"vcard-temp", NULL);
 
  /* set struct info */
  infoquerySetTo(iq,to);
  infoquerySetCallback(iq,cb);
  infoquerySetUserData(iq,userdata);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestVCard: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestVersion(gchar *to, infoqueryCallback cb, void *userdata)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNsPtr ns = NULL;

  if(to == NULL)
    {
      g_warning("iinfoqueryRequestVersion: to was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestVersion: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeGet)) == NULL)
    {
      g_warning("infoqueryRequestVersion: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:version", NULL);
 
  /* set struct info */
  infoquerySetTo(iq,to);
  infoquerySetCallback(iq,cb);
  infoquerySetUserData(iq,userdata);

  if(infoqueryBuild(iq, node) == FALSE)
    {
      g_warning("infoqueryRequestVersion: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestTime(gchar *to, infoqueryCallback cb, void *userdata)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNsPtr ns = NULL;

  if(to == NULL)
    {
      g_warning("iinfoqueryRequestTime: to was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestTime: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeGet)) == NULL)
    {
      g_warning("infoqueryRequestTime: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:time", NULL);
 
  /* set struct info */
  infoquerySetTo(iq,to);
  infoquerySetCallback(iq,cb);
  infoquerySetUserData(iq,userdata);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestTime: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestLast(gchar *to, infoqueryCallback cb, void *userdata)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNsPtr ns = NULL;

  if(to == NULL)
    {
      g_warning("iinfoqueryRequestLast: to was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestLast: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeGet)) == NULL)
    {
      g_warning("infoqueryRequestLast: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:last", NULL);
 
  /* set struct info */
  infoquerySetTo(iq,to);
  infoquerySetCallback(iq,cb);
  infoquerySetUserData(iq,userdata);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestLast: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestRegisterUser(gchar *to, gchar *username, gchar *password, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  /* checks */
  if(to == NULL)
    {
      g_warning("infoqueryRequestRegisterUser: to was NULL");
      return FALSE;
    }

  if(username == NULL)
    {
      g_warning("infoqueryRequestRegisterUser: username was NULL");
      return FALSE;
    }

  if(password == NULL)
    {
      g_warning("infoqueryRequestRegisterUser: password was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestRegisterUser: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeSet)) == NULL)
    {
      g_warning("infoqueryRequestRegisterUser: information query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:register", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"username", (guchar*)username);
  sub = xmlNewChild(node, ns, (guchar*)"password", (guchar*)password);

  /* set struct info */
  infoquerySetCallback(iq,cb);
  infoquerySetTo(iq,to);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestRegisterUser: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestRegister(gchar *to, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNsPtr ns = NULL;

  if(to == NULL)
    {
      g_warning("infoqueryRequestRegister: to was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestRegister: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeGet)) == NULL)
    {
      g_warning("infoqueryRequestRegister: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:register", NULL);
 
  /* set struct info */
  infoquerySetTo(iq,to);
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestRegister: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestRegistrationRequirements(gchar *to, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNsPtr ns = NULL;

  if(to == NULL)
    {
      g_warning("infoqueryRequestRegistrationRequirements: to was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestRegistrationRequirements: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeGet)) == NULL)
    {
      g_warning("infoqueryRequestRegistrationRequirements: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:register", NULL);
 
  /* set struct info */
  infoquerySetTo(iq,to);
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestRegistrationRequirements: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestRegistration(gchar *to, infoqueryCallback cb, ...)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  va_list args;
  guchar *key = NULL;
  guchar *value = NULL;

  if(to == NULL)
    {
      g_warning("infoqueryRequestRegistration: to was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestRegistration: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeSet)) == NULL)
    {
      g_warning("infoqueryRequestRegistration: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:register", NULL);

  va_start(args, cb);
  key = va_arg(args, char*);

  for(; ; key = va_arg(args, char*))
  {
      if(key == NULL) 
	  break;

      value = va_arg(args, char*);

      if(strlen(value) < 1)
	  continue;
      
      sub = xmlNewChild(node, ns, (guchar*)key, (guchar*)value);
  }

  va_end(args);
 
  /* set struct info */
  infoquerySetTo(iq,to);
  infoquerySetCallback(iq,cb);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestRegistration: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

gboolean infoqueryRequestUnRegister(gchar *to, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  /* checks */
  if(to == NULL)
    {
      g_warning("infoqueryRequestUnRegister: to was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestUnRegister: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeSet)) == NULL)
    {
      g_warning("infoqueryRequestUnRegister: information query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query"); 
  ns = xmlNewNs(node, (guchar*)"jabber:iq:register", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"remove", NULL);

  /* set struct info */
  infoquerySetCallback(iq,cb);
  infoquerySetTo(iq,to);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestUnRegister: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}


gboolean infoqueryRequestAgents(gchar *server, infoqueryCallback cb)
{
  infoquery iq = NULL;

  xmlNodePtr node = NULL;
  xmlNsPtr ns = NULL;

  /* checks */
  if(server == NULL)
    {
      g_warning("infoqueryRequestAgents: server was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("infoqueryRequestAgents: cb was NULL");
      return FALSE;
    }

  if((iq = infoqueryNew(infoqueryTypeGet)) == NULL)
    {
      g_warning("infoqueryRequestAgents: query was NULL");
      return FALSE;
    }

  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:agents", NULL);
 
  /* set struct info */
  infoquerySetCallback(iq,cb);
  infoquerySetTo(iq,server);

  if(infoqueryBuild(iq,node) == FALSE)
    {
      g_warning("infoqueryRequestAgents: failed to build query.");
      return FALSE;
    }

  return infoquerySend(iq);
}

#ifdef __cplusplus
}
#endif
