/* 
 * $RCSfile: gj_iq_responses.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/26 11:09:58 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>

#include "gj_iq.h"
#include "gj_iq_responses.h"
#include "gj_roster.h"
#include "gj_me.h"
#include "gj_parser.h"

#include "gj_support.h"

/*
 * ACTUAL REQUESTS from the SERVER!!!, eg:
 *
 *  - jabber:iq:last
 *  - jabber:iq:version
 *  - jabber:iq:time
 *
 *
 *
 *
 *
 */

gboolean infoqueryRequestNew(infoquery iq)
{
  const xmlChar *ns = NULL;
  
  if(iq == NULL)
    {
      g_warning("infoqueryRequestNew: infoquery was NULL");
      return FALSE;
    }
 
  if((ns = parserFindByNsAndReturn(infoqueryGetXML(iq))) == NULL || xmlStrlen(ns) < 1)
    {
      g_warning("infoqueryRequestNew: expected xml namespace.  none where found!");
      return FALSE;
    }
 
  if(infoqueryGetType(iq) == infoqueryTypeGet)
    {
      if(xmlStrcmp(ns, (guchar*)"jabber:iq:last") == 0)
	infoqueryRespondToJabberIQLast(iq);
      else if(xmlStrcmp(ns, (guchar*)"jabber:iq:time") == 0)
	infoqueryRespondToJabberIQTime(iq);
      else if(xmlStrcmp(ns, (guchar*)"jabber:iq:version") == 0)
	infoqueryRespondToJabberIQVersion(iq);
      else
	infoqueryRespondWithServiceUnavailable(iq);
    }
  else if(infoqueryGetType(iq) == infoqueryTypeSet)
    {
      if(xmlStrcmp(ns, (guchar*)"jabber:iq:roster") == 0)
	rosterParseSetRequest(iq);
      else
	infoqueryRespondWithServiceUnavailable(iq);
    }
  else if(infoqueryGetType(iq) == infoqueryTypeResult)
    {
      if(xmlStrcmp(ns, (guchar*)"jabber:iq:register") == 0)
	{
	  infoqueryCallback cb = NULL;

	  if((cb = infoqueryGetCallback(iq)) != NULL)
	    (cb)(infoqueryGetEventID(iq));
	}
    }
  else
    {
      /* error? why? OR result? why? */
      g_warning("infoqueryRequestNew: unexpected infoquery type:%d",infoqueryGetType(iq));
      return FALSE;
    }
  
  /* free iq ?? */
  infoqueryFree(iq);
  
  return TRUE;
}


gboolean infoqueryRespondToJabberIQLast(infoquery iq)
{
  infoquery iq_new = NULL;
  
  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;
  
  xmlChar *tmp = NULL;

  gchar *from = NULL;
  gchar *seconds = NULL;

  if((iq_new = infoqueryNew(infoqueryTypeResult)) == NULL)
    {
      g_warning("infoqueryRespondToJabberIQLast: query was NULL");
      return FALSE;
    }
  
  if((tmp = parserFindByAttrAndReturn(infoqueryGetXML(iq), (guchar*)"from")) == NULL || xmlStrlen(tmp) < 1)
    {
      g_warning("infoqueryRespondToJabberIQLast: failed to obtain 'from' information");
	  return FALSE;
    }
  else
    from = g_strdup_printf("%s",tmp);
  
  seconds = g_strdup_printf("%f",gj_me_get_uptime());
  
  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:last", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"seconds", (guchar*)seconds);
 
  /* set struct info */
  infoquerySetTo(iq_new, (gchar*)from);

  if(infoqueryBuild(iq_new, node) == FALSE)
    {
      g_warning("infoqueryRespondToJabberIQLast: failed to build query.");
      return FALSE;
    }

  if(infoquerySend(iq_new) == FALSE)
    {
      g_warning("infoqueryRespondToJabberIQLast: failed to send query");
      return FALSE;
    }
  
  /* clean up */
  infoqueryFree(iq_new);
  if(from != NULL) g_free(from);
  if(seconds != NULL) g_free(seconds);

  return TRUE;
}

gboolean infoqueryRespondToJabberIQVersion(infoquery iq)
{
  infoquery iq_new = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  xmlChar *tmp = NULL;

  gchar *from = NULL;

  if((iq_new = infoqueryNew(infoqueryTypeResult)) == NULL)
    {
      g_warning("infoqueryRespondToJabberIQVersion: query was NULL");
      return FALSE;
    }

  if((tmp = parserFindByAttrAndReturn(infoqueryGetXML(iq), (guchar*)"from")) == NULL || xmlStrlen(tmp) < 1)
    {
      g_warning("infoqueryRespondToJabberIQVersion: failed to obtain 'from' information");
      return FALSE;
    }
  else
    from = g_strdup_printf("%s",tmp);
  
  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:version", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"name", (guchar*)"gnome-jabber");
  sub = xmlNewChild(node, ns, (guchar*)"os", (guchar*)"linux");
  sub = xmlNewChild(node, ns, (guchar*)"version", (guchar*)"0.1");
 
  /* set struct info */
  infoquerySetTo(iq_new,from);

  if(infoqueryBuild(iq_new,node) == FALSE)
    {
      g_warning("infoqueryRespondToJabberIQVersion: failed to build query.");
      return FALSE;
    }

  if(infoquerySend(iq_new) == FALSE)
    {
      g_warning("infoqueryRespondToJabberIQVersion: failed to send query");
      return FALSE;
    }
  
  /* clean up */
  infoqueryFree(iq_new);
  if(from != NULL) g_free(from);

  return TRUE;
}

gboolean infoqueryRespondToJabberIQTime(infoquery iq)
{
  infoquery iq_new = NULL;

  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  xmlChar *tmp = NULL;

  gchar *from = NULL;
  gchar *utc = NULL;

  if((iq_new = infoqueryNew(infoqueryTypeResult)) == NULL)
    {
      g_warning("infoqueryRespondToJabberIQTime: query was NULL");
      return FALSE;
    }

  if((tmp = parserFindByAttrAndReturn(infoqueryGetXML(iq), (guchar*)"from")) == NULL || xmlStrlen(tmp) < 1)
    {
      g_warning("infoqueryRespondToJabberIQTime: failed to obtain 'from' information");
      return FALSE;
    }
  else
    from = g_strdup_printf("%s",tmp);

  utc = g_strdup_printf("%sT%s", supportGetDatestampLong(), supportGetTimestamp());
  
  /* set xml */
  node = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node, (guchar*)"jabber:iq:time", NULL);
  sub = xmlNewChild(node, ns, (guchar*)"utc", (guchar*)utc);
  sub = xmlNewChild(node, ns, (guchar*)"tz", (guchar*)"BST");  /* need to get TZ */
  sub = xmlNewChild(node, ns, (guchar*)"display", (guchar*)supportGetDatestamp());
 
  /* set struct info */
  infoquerySetTo(iq_new,from);

  if(infoqueryBuild(iq_new,node) == FALSE)
    {
      g_warning("infoqueryRespondToJabberIQTime: failed to build query.");
      return FALSE;
    }

  if(infoquerySend(iq_new) == FALSE)
    {
      g_warning("infoqueryRespondToJabberIQTime: failed to send query");
      return FALSE;
    }
  
  /* clean up */
  infoqueryFree(iq_new);
  if(from != NULL) g_free(from);
  if(utc != NULL) g_free(utc);

  return TRUE;
}

gboolean infoqueryRespondWithServiceUnavailable(infoquery iq)
{
  infoquery iq_new = NULL;

  xmlNodePtr node1 = NULL;
  xmlNodePtr node2 = NULL;
  xmlNsPtr ns = NULL;

  GList *nodes = NULL;
  
  const xmlChar *tmp = NULL;
  
  gchar *namespace = NULL;
  gchar *from = NULL;

  if((tmp = parserFindByNsAndReturn(infoqueryGetXML(iq))) == NULL || xmlStrlen(tmp) < 1)
    {
      g_warning("infoqueryRespondWithServiceUnavailable: could not determin namespace from infoquery");
      return FALSE;
    }
  else
    namespace = g_strdup_printf("%s",tmp);

  if((tmp = parserFindByAttrAndReturn(infoqueryGetXML(iq), (guchar*)"from")) == NULL || xmlStrlen(tmp) < 1)
    {
      g_warning("infoqueryRespondWithServiceUnavailable: failed to obtain 'from' information");
      return FALSE;
    }
  else
    from = g_strdup_printf("%s",tmp);

  if((iq_new = infoqueryNew(infoqueryTypeError)) == NULL)
    {
      g_warning("infoqueryRespondWithServiceUnavailable: query was NULL");
      return FALSE;
    }

  /* set xml */
  node1 = xmlNewNode(NULL, (guchar*)"query");
  ns = xmlNewNs(node1, (guchar*)namespace, NULL);

  node2 = xmlNewNode(NULL, (guchar*)"error");
  xmlSetProp(node2, (guchar*)"code", (guchar*)"503");  
  xmlNodeSetContent(node2, (guchar*)"Service Unavailable");

  nodes = g_list_append(nodes,node1);
  nodes = g_list_append(nodes,node2);
 
  /* set struct info */
  infoquerySetTo(iq_new, (gchar*)from);

  if(infoqueryBuildFromList(iq_new,nodes) == FALSE)
    {
      g_warning("infoqueryRespondWithServiceUnavailable: failed to build query.");
      return FALSE;
    }

  if(infoquerySend(iq_new) == FALSE)
    {
      g_warning("infoqueryResponsdWithServiceUnavailable: failed to send query");
      return FALSE;
    }
  
  /* clean up */
  infoqueryFree(iq_new);
  if(namespace != NULL) g_free(namespace);
  if(from != NULL) g_free(from);
  if(nodes != NULL) g_list_free(nodes);

  return TRUE;
}


#ifdef __cplusplus
}
#endif
