/* 
 * $RCSfile: gj_log.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/08 13:18:47 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdio.h>
#include <string.h>

#include <glib.h>

#include "common.h"

#include "gj_log.h"
#include "gj_support.h"
#include "gj_config.h"
#include "gj_main.h"

#include "gj_gtk_log.h"


gboolean logInitialised = FALSE;
gchar *logPath = NULL;

GIOChannel *logChannel = NULL;


gboolean logInit()
{
  /* GLib log levels 

  G_LOG_LEVEL_ERROR             = 1 << 2,        - always fatal 
  G_LOG_LEVEL_CRITICAL          = 1 << 3,
  G_LOG_LEVEL_WARNING           = 1 << 4,
  G_LOG_LEVEL_MESSAGE           = 1 << 5,
  G_LOG_LEVEL_INFO              = 1 << 6,
  G_LOG_LEVEL_DEBUG             = 1 << 7,
  
  */

  gsize bytes;
  gchar *message = NULL;

  gchar *filename = NULL;
  gchar *path = NULL;

  gchar *saved_path = NULL;
  saved_path = gj_config_get_mpf_log_location();
  
  if(saved_path == NULL || strlen(saved_path) < 1)
    {
      gchar *path = NULL;

      path = g_build_path(G_DIR_SEPARATOR_S, gnome_jabber_get_config_dir(), NULL);
      gj_config_set_mpf_log_location(path);

      logPath = g_strdup(path);

      g_free(path);
    }
  else
    {
      /* use saved path */
      logPath = g_strdup(saved_path);
    }

  path = gj_config_get_mpf_log_location();
  g_message("logInit: using log path:'%s'", path);

  filename = g_build_filename(path, G_DIR_SEPARATOR_S, FILE_LOG, NULL);

#if 0
  if(path != NULL && strlen(path) > 0)
    {
      /* use path in filename */
      if(path[strlen(path)-1] != '/')
	path = g_strconcat(path,"/",NULL);

      filename = g_strdup_printf("%s%s", path, FILE_LOG);
    }
  else
  {
    filename = g_strdup_printf(FILE_LOG);
  }
#endif

  g_message("logInit: using log filename:'%s'", filename);
  logChannel = g_io_channel_new_file(filename,"a+",NULL);

  if(logChannel == NULL)
    {
      g_error("logInit: failed to create new file:'%s' (channel) for log (path:'%s')",filename,path);
      return FALSE;
    }

  /* start top of log */
  message = g_strdup_printf("Log started on %s at %s\n\n", supportGetDatestamp(), supportGetTimestamp());

  g_io_channel_write_chars(logChannel,
			   message,
			   -1,
			   &bytes,
			   NULL);

  /* clean up */
  g_free(message);

  /* initiate gtk log window */
  gj_gtk_log_init();

  /* set to inialised */
  logInitialised = TRUE;

  return TRUE;
}

gboolean logTerm()
{
  /* close file */
  g_io_channel_shutdown(logChannel,TRUE,NULL);

  /* uninitialise logging */
  logInitialised = FALSE;

  return TRUE;
}

gboolean logHandlerFormatted(char *log_domain, int log_level, char *formatted)
{
    gchar *message = NULL;
    gchar *message_log_level = NULL;
    guint bytes = 0;

    if(formatted == NULL)
    {
      fprintf(stderr, "could not log message it was NULL");
      fflush(stderr);
      return FALSE;
    }

    if(log_level == GJ_LOG_LEVEL_WARNING)
	message_log_level = g_strdup_printf(" ** WARNING ** ");
    else if(log_level == GJ_LOG_LEVEL_ERROR)
	message_log_level = g_strdup_printf(" ** ERROR ** ");
    else if(log_level == GJ_LOG_LEVEL_CRITICAL)
	message_log_level = g_strdup_printf(" ** CRITICAL ** ");

#ifdef WIN32
    /* format buffer */
    message = g_strdup_printf("%s:%s:%s%s%s\r\n",
			      supportGetTimestamp(),
			      log_domain?log_domain:"GJ",
			      message_log_level?message_log_level:"",
			      message_log_level?" ":"\t",
			      formatted);	
#else
    /* format buffer */
    message = g_strdup_printf("%s:%s:%s%s%s\n",
			      supportGetTimestamp(),
			      log_domain?log_domain:"GJ",
			      message_log_level?message_log_level:"",
			      message_log_level?" ":"\t",
			      formatted);
#endif

    if(logInitialised == 0)
    {
	/* if NOT initialised, used stderr */
	fprintf(stderr, message);
	fflush(stderr);
    }
    else
      {
	  if(log_level != G_LOG_LEVEL_DEBUG)
	  {
	      /* write to file */
	      g_io_channel_write_chars(logChannel,
				       message,
				       -1,
				       &bytes,
				       NULL);

	      fprintf(stdout, message);
	      fflush(stdout);
	  }
      }

    gj_gtk_log_new(log_domain, log_level, formatted, NULL);

    /* clean up */
    g_free(message);
    g_free(message_log_level);
}

#ifdef __cplusplus
}
#endif
