/* 
 * $RCSfile: gj_me.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/22 10:54:30 $
 * 
 * $Revision: 1.2 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_presence.h"

#include "gj_me.h"

struct 
{
  /* connection */
  gboolean is_connected;

    gchar *connected_host;
    guint connected_port;

  /* auto away */
  gboolean is_auto_away;
  presenceShow auto_away_presence_show;  /* either xa or away */

    gchar *jid;

  /* presence */
  presenceType presence_type;
  presenceShow presence_show;
  gchar *presence_status;
  guint presence_priority;

    /* last namespace */
    GTimer *uptime;

} me;

gboolean gj_me_init()
{
    /* 
       create new timer - for uptime, 
       could be done with just remembering time, but this is quicker. 
    */
    me.uptime = g_timer_new();
    g_timer_start(me.uptime); 

    me.jid = NULL;
    
  return TRUE;
}

gboolean gj_me_term()
{
    if(me.uptime != NULL)
	{
	    g_timer_stop(me.uptime);
	    g_timer_destroy(me.uptime);
	    me.uptime = NULL;
	}

    if(me.jid != NULL)
    {
	g_free(me.jid);
	me.jid = NULL;
    }  

    if(me.connected_host != NULL)
    {
	g_free(me.connected_host);
	me.connected_host = NULL;
    }

  return TRUE;
}

/* get members */
gboolean gj_me_get_is_connected()
{
  return me.is_connected;
}

const gchar *gj_me_get_connected_host()
{
    return me.connected_host;
}

guint gj_me_get_connected_port()
{
    return me.connected_port;
}

gboolean gj_me_get_is_auto_away()
{
  return me.is_auto_away;
}

presenceShow gj_me_get_auto_away_presence_show() 
{
  return me.auto_away_presence_show;
}

gchar *gj_me_get_jid() 
{
  return me.jid;
}

presenceType gj_me_get_presence_type()
{
  return me.presence_type;
}

presenceShow gj_me_get_presence_show()
{
  return me.presence_show;
}

gchar *gj_me_get_presence_status()
{
  return me.presence_status;
}

guint gj_me_get_presence_priority()
{
  return me.presence_priority;
}

gdouble gj_me_get_uptime()
{
    gdouble seconds = 0;
    gulong microseconds = 0;

    if(me.uptime == NULL)
	return 0;

    seconds = g_timer_elapsed(me.uptime,&microseconds);

  return seconds;
}

/* set members */
void gj_me_set_is_connected(gboolean is_connected)
{
  me.is_connected = is_connected;
}

gboolean gj_me_set_connected_host(const gchar *host)
{
    if(host == NULL)
    {
	g_warning("gj_me_set_connected_host: host is NULL");
	return FALSE;
    }

    if(me.connected_host != NULL)
    {
	g_free(me.connected_host);
	me.connected_host = NULL;
    }

    me.connected_host = g_strdup(host);
    return TRUE;
}

gboolean gj_me_set_connected_port(guint port)
{
    if(port < 1 || port > 65535)
    {
	g_warning("gj_me_set_connected_port: port:%d was invalid", port);
	return FALSE;
    }

    me.connected_port = port;
    return TRUE;
}

void gj_me_set_is_auto_away(gboolean is_auto_away)
{
  me.is_auto_away = is_auto_away;
}

void gj_me_set_jid(gchar *jid)
{
    if(jid == NULL)
    {
	g_warning("gj_me_set_jid: jid is NULL");
	return;
    }

    if(me.jid != NULL)
    {
	g_free(me.jid);
	me.jid = NULL;
    }

    me.jid = g_strdup(jid);
}

void gj_me_set_auto_away_presence_show(presenceShow show) 
{
  me.auto_away_presence_show = show;
}

void gj_me_set_presence_type(presenceType type)
{
  me.presence_type = type;
}

void gj_me_set_presence_show(presenceShow show)
{
  me.presence_show = show;
}

void gj_me_set_presence_status(gchar *status)
{
  if(status == NULL)
    return;

  if(me.presence_status != NULL)
    g_free(me.presence_status);

  me.presence_status = g_strdup(status);
}

void gj_me_set_presence_priority(guint priority)
{
  me.presence_priority = priority;
}


#ifdef __cplusplus
}
#endif
