/* 
 * $RCSfile: gj_message.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/26 11:09:58 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <string.h>

#include <time.h>

#include <glib.h>

#include <libxml/tree.h>

#include "gj_message.h"
#include "gj_parser.h"
#include "gj_me.h"

struct t_message
{
  messageType type;
  
  gint requestID;

  gchar *id;

  gchar *to_jid;
  gchar *to_resource;
  gchar *from_jid;
  gchar *from_resource;
  
  gchar *subject;
  gchar *body;

  gchar *thread;

  /* x name spaces */
  gint xevent_type;
  gchar *xevent_message_id;

  gchar *xoob_url;
  gchar *xoob_description;

  /* errors */
  gchar *error_code;
  gchar *error_reason;

  time_t time;
};

/* to details, i.e. my state */
messageToResponseProc messageToGetResponseProc();

/* from details, i.e. my contacts state(s) */
messageFromResponseProc messageFromGetResponseProc();

/* response procs */
messageFromResponseProc messageFromProc;
messageToResponseProc messageToProc;   /* may not get used */

/* message item */
message messageNew(gint requestID, messageType type);
void messageDestroy(message msg);

/* message list */
gboolean messageListAdd(message msg);
gboolean messageListDel(message msg);

GList *messageList = NULL;   /* stores ALL messages of ALL types */

/*
 * code
 */
gboolean messageInit()
{
  return TRUE;
}

gboolean messageTerm()
{
  gint index = 0;

  /* list */
  if(messageList == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(messageList);index++)
    {
      message msg = NULL;
      
      if((msg = (message) g_list_nth_data(messageList,index)) != NULL)
	messageFree(msg);
    }

  g_list_free(messageList);
  messageList = NULL;

  return TRUE;
}

gboolean messageListAdd(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageListAdd: failed to add item to list, it was NULL");
      return FALSE;
    }

  messageList = g_list_append(messageList,msg);

  return TRUE;
}

gboolean messageListDel(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageListDel: failed to del item from list, it was NULL");
      return FALSE;
    }

  messageList = g_list_remove(messageList,msg);
  return TRUE;
}

message messageNew(gint requestID, messageType type)
{
  message msg = NULL;

  if(requestID < 1)
    {
      g_warning("messageNew: failed to create new roster item, requestID:%d was < 1",requestID);
      return NULL;
    }
  
  if((msg = g_new(struct t_message,1)) != NULL)
    {
      msg->type = type;

      msg->id = NULL;

      msg->requestID = requestID;

      msg->to_jid = NULL;
      msg->to_resource = NULL;

      msg->from_jid = NULL;
      msg->from_resource = NULL;
      
      msg->subject = NULL;
      msg->body = NULL;

      msg->thread = NULL;

      msg->xevent_type = 0;
      msg->xevent_message_id = NULL;

      msg->xoob_url = NULL;
      msg->xoob_description = NULL;

      msg->error_code = NULL;
      msg->error_reason = NULL;

      msg->time = time(0);

      messageListAdd(msg);
    }
  
  return msg;  
}

void messageFree(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageFree: failed to free, message was NULL");
      return;
    }

  /* if on queue, remove it */
  

  /* free mem */
  messageListDel(msg);  

  /* destroy members */
  messageDestroy(msg);

  /* free structure */
  g_free(msg);
}

void messageDestroy(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageDestroy: failed to destroy, message was NULL");
      return;
    }

  msg->type = 0;

  if(msg->id != NULL)
    {
      g_free(msg->id);
      msg->id = NULL;
    }

  msg->requestID = 0;

  if(msg->to_jid != NULL)
    {
      g_free(msg->to_jid);
      msg->to_jid = NULL;
    }

  if(msg->to_resource != NULL)
    {
      g_free(msg->to_resource);
      msg->to_resource = NULL;
    }

  if(msg->from_jid != NULL)
    {
      g_free(msg->from_jid);
      msg->from_jid = NULL;
    }

  if(msg->from_resource != NULL)
    {
      g_free(msg->from_resource);
      msg->from_resource = NULL;
    }  

  if(msg->subject != NULL)
    {
      g_free(msg->subject);
      msg->subject = NULL;
    }

  if(msg->body != NULL)
    {
      g_free(msg->body);
      msg->body = NULL;
    } 

  if(msg->thread != NULL)
    {
      g_free(msg->thread);
      msg->thread = NULL;
    } 

  msg->xevent_type = 0;

  if(msg->xevent_message_id != NULL)
    {
      g_free(msg->xevent_message_id);
      msg->xevent_message_id = NULL;
    }

  if(msg->xoob_url != NULL)
    {
      g_free(msg->xoob_url);
      msg->xoob_url = NULL;
    }

  if(msg->xoob_description != NULL)
    {
      g_free(msg->xoob_description);
      msg->xoob_description = NULL;
    }

  if(msg->error_code != NULL)
    {
      g_free(msg->error_code);
      msg->error_code = NULL;
    } 

  if(msg->error_reason != NULL)
    {
      g_free(msg->error_reason);
      msg->error_reason = NULL;
    } 
}

message messageFindByToJID(gchar *jid)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("messageFindByJID: JID was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(messageList);index++)
    {
      message msg  = g_list_nth_data(messageList, index);

      if(msg == NULL)
	{
	  g_warning("messageFindByJID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(msg->to_jid,jid) == 0)
	{
	  return msg;
	}
    }

  return NULL;
}

gboolean messageSetToJID(message msg, gchar *jid)
{
  if(msg == NULL)
    {
      g_warning("messageSetToJID: msg was NULL");
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("messageSetToJID: JID was NULL");
      return FALSE;
    }

  if(msg->to_jid != NULL)
    g_free(msg->to_jid);

  msg->to_jid = g_strdup(jid);
  return TRUE;
}

gboolean messageSetType(message msg, messageType type)
{
  if(msg == NULL)
    {
      g_warning("messageSetType: msg was NULL");
      return FALSE;
    }

  if(type < 1 || type > messageTypeEnd)
    {
      g_warning("messageSetType: type was out of range");
      return FALSE;
    }

  msg->type = type;
  return TRUE;
}

gboolean messageSetID(message msg, gchar *id)
{
  if(msg == NULL)
    {
      g_warning("messageSetID: msg was NULL");
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("messageSetID: JID was NULL");
      return FALSE;
    }

  if(msg->id != NULL)
    g_free(msg->id);

  msg->id = g_strdup(id);
  return TRUE;
}


gboolean messageSetToResource(message msg, gchar *resource)
{
  if(msg == NULL)
    {
      g_warning("messageSetToResource: msg was NULL");
      return FALSE;
    }

  if(resource == NULL)
    {
      g_warning("messageSetToResource: resource was NULL");
      return FALSE;
    }

  if(msg->to_resource != NULL)
    g_free(msg->to_resource);

  msg->to_resource = g_strdup(resource);
  return TRUE;
}

gboolean messageSetFromJID(message msg, gchar *jid)
{
  if(msg == NULL)
    {
      g_warning("messageSetFromJID: msg was NULL");
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("messageSetFromJID: JID was NULL");
      return FALSE;
    }

  if(msg->from_jid != NULL)
    g_free(msg->from_jid);

  msg->from_jid = g_strdup(jid);
  return TRUE;
}

gboolean messageSetFromResource(message msg, gchar *resource)
{
  if(msg == NULL)
    {
      g_warning("messageSetFromResource: msg was NULL");
      return FALSE;
    }

  if(resource == NULL)
    {
      g_warning("messageSetFromResource: resource was NULL");
      return FALSE;
    }

  if(msg->from_resource != NULL)
    g_free(msg->from_resource);

  msg->from_resource = g_strdup(resource);
  return TRUE;
}

gboolean messageSetSubject(message msg, gchar *subject)
{
  if(msg == NULL)
    {
      g_warning("messageSetSubject: msg was NULL");
      return FALSE;
    }

  if(subject == NULL)
    {
      g_warning("messageSetToSubject: subject was NULL");
      return FALSE;
    }

  if(msg->subject != NULL)
    g_free(msg->subject);

  msg->subject = g_strdup(subject);
  return TRUE;
}

gboolean messageSetBody(message msg, gchar *body)
{
  if(msg == NULL)
    {
      g_warning("messageSetBody: msg was NULL");
      return FALSE;
    }

  if(body == NULL)
    {
      g_warning("messageSetBody: body was NULL");
      return FALSE;
    }

  if(msg->body != NULL)
    g_free(msg->body);

  msg->body = g_strdup(body);
  return TRUE;
}

gboolean messageSetThread(message msg, gchar *thread)
{
  if(msg == NULL)
    {
      g_warning("messageSetThread: msg was NULL");
      return FALSE;
    }

  if(thread == NULL)
    {
      g_warning("messageSetThread: thread was NULL");
      return FALSE;
    }

  if(msg->thread != NULL)
    g_free(msg->thread);

  msg->thread = g_strdup(thread);
  return TRUE;
}

gboolean messageSetXEventType(message msg, gint type)
{
  if(msg == NULL)
    {
      g_warning("messageSetXEventType: msg was NULL");
      return FALSE;
    }

  if(type == 0)
    {
      g_warning("messageSetXEventType: type was out of range");
      return FALSE;
    }

  msg->xevent_type = type;
  return TRUE;
}

gboolean messageSetXEventMessageID(message msg, gchar *message_id)
{
  if(msg == NULL)
    {
      g_warning("messageSetXEventMessageID: msg was NULL");
      return FALSE;
    }

  if(message_id == NULL)
    {
      g_warning("messageSetXEventMessageID: messageID was NULL");
      return FALSE;
    }

  if(msg->xevent_message_id != NULL)
    g_free(msg->xevent_message_id);

  msg->xevent_message_id = g_strdup(message_id);
  return TRUE;
}

gboolean messageSetXOOBURL(message msg, gchar *url)
{
  if(msg == NULL)
    {
      g_warning("messageSetXOOBURL: msg was NULL");
      return FALSE;
    }

  if(url == NULL)
    {
      g_warning("messageSetXOOBURL: messageID was NULL");
      return FALSE;
    }

  if(msg->xoob_url != NULL)
    g_free(msg->xoob_url);

  msg->xoob_url = g_strdup(url);
  return TRUE;
}

gboolean messageSetXOOBDescription(message msg, gchar *description)
{
  if(msg == NULL)
    {
      g_warning("messageSetXOOBDescription: msg was NULL");
      return FALSE;
    }

  if(description == NULL)
    {
      g_warning("messageSetXOOBDescription: description was NULL");
      return FALSE;
    }

  if(msg->xoob_description != NULL)
    g_free(msg->xoob_description);

  msg->xoob_description = g_strdup(description);
  return TRUE;
}

gboolean messageSetErrorCode(message msg, gchar *error_code)
{
  if(msg == NULL)
    {
      g_warning("messageSetErrorCode: msg was NULL");
      return FALSE;
    }

  if(error_code == NULL)
    {
      g_warning("messageSetErrorCode: error code was NULL");
      return FALSE;
    }

  if(msg->error_code != NULL)
    g_free(msg->error_code);

  msg->error_code = g_strdup(error_code);
  return TRUE;
}

gboolean messageSetErrorReason(message msg, gchar *error_reason)
{
  if(msg == NULL)
    {
      g_warning("messageSetErrorReason: msg was NULL");
      return FALSE;
    }

  if(error_reason == NULL)
    {
      g_warning("messageSetErrorReason: error reason was NULL");
      return FALSE;
    }

  if(msg->error_reason != NULL)
    g_free(msg->error_reason);

  msg->error_reason = g_strdup(error_reason);
  return TRUE;
}

messageType messageGetType(message msg)
{ 
  if(msg == NULL)
    {
      g_warning("messageGetType: msg was NULL");
      return -1;
    }  

  return msg->type;
}

gchar *messageGetID(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetID: msg was NULL");
      return NULL;
    }  

  return msg->id;
}


gint messageGetRequestID(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetRequestID: msg was NULL");
      return -1;
    }  

  return msg->requestID;
}

gchar *messageGetToJID(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetToJID: msg was NULL");
      return NULL;
    }  

  return msg->to_jid;
}

gchar *messageGetToResource(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetToResource: msg was NULL");
      return NULL;
    }  

  return msg->to_resource;
}

gchar *messageGetFromJID(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetFromJID: msg was NULL");
      return NULL;
    }  

  return msg->from_jid;
}

gchar *messageGetFromResource(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetFromResource: msg was NULL");
      return NULL;
    }  

  return msg->from_resource;
}

gchar *messageGetSubject(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetSubject: msg was NULL");
      return NULL;
    }  

  return msg->subject;
}

gchar *messageGetBody(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetBody: msg was NULL");
      return NULL;
    }  

  return msg->body;
}

gchar *messageGetThread(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetThread: msg was NULL");
      return NULL;
    }  

  return msg->thread;
}

gint messageGetXEventType(message msg)
{ 
  if(msg == NULL)
    {
      g_warning("messageGetXEventType: msg was NULL");
      return -1;
    }  

  return msg->xevent_type;
}

gchar *messageGetXEventMessageID(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetXEventMessageID: msg was NULL");
      return NULL;
    }  

  return msg->xevent_message_id;
}

gchar *messageGetXOOBURL(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetXOOBURL: msg was NULL");
      return NULL;
    }  

  return msg->xoob_url;
}

gchar *messageGetXOOBDescription(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetXOOBDescription: msg was NULL");
      return NULL;
    }  

  return msg->xoob_description;
}

gchar *messageGetErrorCode(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetErrorCode: msg was NULL");
      return NULL;
    }  

  return msg->error_code;
}

gchar *messageGetErrorReason(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetErrorReason: msg was NULL");
      return FALSE;
    }  

  return msg->error_reason;
}

time_t messageGetTime(message msg)
{
  if(msg == NULL)
    {
      g_warning("messageGetTime: msg was NULL");
      return 0;
    }  

  return msg->time;
}

/*
 * FROM presence
 */
gboolean messageFromSetResponseProc(messageFromResponseProc proc)
{
  if(proc == NULL)
    {
      g_warning("messageFromSetResponseProc: the response proc was NULL");
      return FALSE;
    }

  messageFromProc = proc;
  
  return TRUE;
}

messageFromResponseProc messageFromGetResponseProc()
{
  return messageFromProc;
}

/*
 * TO presence
 */
gboolean messageToSetResponseProc(messageToResponseProc proc)
{
  if(proc == NULL)
    {
      g_warning("messageToSetResponseProc: the response proc was NULL");
      return FALSE;
    }

  messageToProc = proc;
  
  return TRUE;
}

messageToResponseProc messageToGetResponseProc()
{
  return messageToProc;
}


/*
 * external functions
 */
gboolean messageSend(messageType type, gchar *to, gchar *subject, gchar *body, gchar *thread)
{
  gchar *from = NULL;
  gchar *msgtype = NULL;
  gchar *msg = NULL;

  if(to == NULL)
    {
      g_warning("messageSend: to was NULL");
      return FALSE;
    }

  if(body == NULL)
    {
      g_warning("messageSend: body was NULL");
      return FALSE;
    }

  switch(type)
    {
    case messageTypeNormal:
      msgtype = g_strdup_printf("normal");
      break;
    case messageTypeChat:
      msgtype = g_strdup_printf("chat");
      break;
    case messageTypeGroupChat:
      msgtype = g_strdup_printf("groupchat");
      break;
    case messageTypeHeadline:
      msgtype = g_strdup_printf("headline");
      break;
    case messageTypeError:
      msgtype = g_strdup_printf("error");
      break;

    case messageTypeXEvent:
      g_warning("messageSend: can not send this type of message with this function");
      return FALSE;
      
    default:
      g_warning("messageSend: messageType:%d was unknown", type);
      return FALSE;
    }

  msg = g_strdup_printf("<message id='%d' to='%s' from='%s' type='%s'>",
			supportGetUniqueID(),
			to,
			gj_me_get_jid(),
			msgtype);

  if(subject != NULL)
    {
      gchar *tmp = g_strdup_printf("<subject>%s</subject>",subject);
      msg = g_strconcat(msg,tmp,NULL);
      g_free(tmp);
    }

  if(body != NULL)
    {
      gchar *formated = NULL;
      gchar *tmp = NULL;

      /* check for invalid chars */
      formated = parserCheckText(body);
      tmp = g_strdup_printf("<body>%s</body>",formated);
      msg = g_strconcat(msg,tmp,NULL);
      g_free(tmp);
    }

  if(thread != NULL)
    {
      gchar *tmp = g_strdup_printf("<thread>%s</thread>",thread);
      msg = g_strconcat(msg,tmp,NULL);
      g_free(tmp);
    }

  msg = g_strconcat(msg,"</message>",NULL);
  parserOutputFromText(msg);

  if(from != NULL) g_free(from);
  if(msgtype != NULL) g_free(msgtype);
  if(msg != NULL) g_free(msg);

  return TRUE;
}

gboolean messageSendXEvent(gint xevents, gchar *to, gchar *id)
{
  gchar *event = NULL;
  gchar *msg = NULL;

  if(xevents == 0)
    {
      g_warning("messageSendXEvent: invalid xevents");
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("messageSendXEvent: id was NULL");
      return FALSE;
    }
      
  msg = g_strdup_printf("<message to='%s' from='%s'><x xmlns='jabber:x:event'>",
			to,
			gj_me_get_jid());

  if(xevents != messageXEventTypeCancelled)
    {
      if(xevents & messageXEventTypeOffline)
	msg = g_strconcat(msg,"<offline/>",NULL);
      if(xevents & messageXEventTypeDelivered)
	msg = g_strconcat(msg,"<delivered/>",NULL);
      if(xevents & messageXEventTypeDisplayed)
	msg = g_strconcat(msg,"<displayed/>",NULL);
      if(xevents & messageXEventTypeComposing)
	msg = g_strconcat(msg,"<composing/>",NULL);
    }
  
  msg = g_strconcat(msg,"<id>", id, "</id>",NULL);
  msg = g_strconcat(msg,"</x></message>",NULL);

  /* send */
  parserOutputFromText(msg);

  /* clean up */
  if(event != NULL) g_free(event);
  if(msg != NULL) g_free(msg);

  return TRUE;
}

gchar *messageSendWithXEventRequest(messageType type, gchar *to, gchar *subject, gchar *body, gchar *thread, gint xevents)
{
  gchar *msgtype = NULL;
  gchar *msg = NULL;
  gchar *id = NULL;

  if(type < 1 || type > messageTypeEnd)
    {
      g_warning("messageSendWithXEventRequest: invalid type");
      return NULL;
    }

  if(to == NULL)
    {
      g_warning("messageSendWithXEventRequest: to was NULL");
      return NULL;
    }

  if(body == NULL)
    {
      g_warning("messageSendWithXEventRequest: body was NULL");
      return NULL;
    }

  if(!(xevents & messageXEventTypeOffline) && 
     !(xevents & messageXEventTypeDelivered) &&
     !(xevents & messageXEventTypeDisplayed) &&   
     !(xevents & messageXEventTypeComposing))
    {
      g_warning("messageSendWithXEventRequest: invalid event type");
      return NULL;
    }

  switch(type)
    {
    case messageTypeNormal:
      msgtype = g_strdup_printf("normal");
      break;
    case messageTypeChat:
      msgtype = g_strdup_printf("chat");
      break;
    case messageTypeGroupChat:
      msgtype = g_strdup_printf("groupchat");
      break;
    case messageTypeHeadline:
      msgtype = g_strdup_printf("headline");
      break;
    case messageTypeError:
      msgtype = g_strdup_printf("error");
      break;

    default:
      g_warning("messageSendWithXEventRequest: messageType:%d was unknown", type);
      return NULL;
    }

  /* set id */
  id = g_strdup_printf("%d", supportGetUniqueID());

  /* set header */
  msg = g_strdup_printf("<message id='%s' to='%s' from='%s' type='%s'>",
			id,
			to,
			gj_me_get_jid(),
			msgtype);

  if(subject != NULL)
    {
      gchar *tmp = g_strdup_printf("<subject>%s</subject>",subject);
      msg = g_strconcat(msg,tmp,NULL);
      g_free(tmp);
    }

  if(body != NULL)
    {
      gchar *formatted = NULL;
      gchar *tmp = NULL;

      /* check for invalid chars */
      formatted = parserCheckText(body);
      tmp = g_strdup_printf("<body>%s</body>",formatted);
      msg = g_strconcat(msg,tmp,NULL);
      if(tmp != NULL) g_free(tmp);
      if(formatted != NULL) g_free(formatted);
    }

  if(thread != NULL)
    {
      gchar *tmp = g_strdup_printf("<thread>%s</thread>",thread);
      msg = g_strconcat(msg,tmp,NULL);
      g_free(tmp);
    }

  msg = g_strconcat(msg,"<x xmlns='jabber:x:event'>",NULL);

  if(xevents & messageXEventTypeOffline)
    msg = g_strconcat(msg,"<offline/>",NULL);
  if(xevents & messageXEventTypeDelivered)
    msg = g_strconcat(msg,"<delivered/>",NULL);
  if(xevents & messageXEventTypeDisplayed)
    msg = g_strconcat(msg,"<displayed/>",NULL);
  if(xevents & messageXEventTypeComposing)
    msg = g_strconcat(msg,"<composing/>",NULL);
  
  msg = g_strconcat(msg,"</x>",NULL);
  msg = g_strconcat(msg,"</message>",NULL);

  parserOutputFromText(msg);
  
  if(msgtype != NULL) g_free(msgtype);
  if(msg != NULL) g_free(msg);

  return id;    
}


void messageReceive(connection c, xmlNodePtr node)
{
  gint requestID = supportGetUniqueID();

  gchar *message_id = NULL;
  
  gchar *to_user = NULL;
  gchar *to_resource = NULL;

  gchar *from_user = NULL;
  gchar *from_resource = NULL;

  gchar *subject = NULL;
  gchar *body = NULL;

  gchar *thread = NULL;

  xmlNodePtr x = NULL;

  gint xevent_type = 0;
  gchar *xevent_message_id = NULL;

  gchar *xoob_url = NULL;
  gchar *xoob_description = NULL;

  gchar *error_code = NULL;
  gchar *error_reason = NULL;

  xmlChar *stype = NULL;
  xmlChar *tmp = NULL;

  message msg = NULL;

  messageType type = messageTypeNormal;
  messageFromResponseProc callback = NULL;

  /* checks */
  if(node == NULL)
    {
      g_warning("messageReceive: failed to decipher xml, node was NULL");
      return;
    }

  /* get info */
  if((callback = messageFromGetResponseProc()) == NULL)
    {
      g_warning("messageReceive: failed to call callback, it was NULL");
      return;
    }
    
  if((stype = parserFindByAttrAndReturn(node, (guchar*)"type")) != NULL)
    {
      if(xmlStrcmp(stype, (guchar*)"normal") == 0)
	type = messageTypeNormal;
      else if(xmlStrcmp(stype, (guchar*)"chat") == 0)
	type = messageTypeChat;
      else if(xmlStrcmp(stype, (guchar*)"groupchat") == 0)
	type = messageTypeGroupChat;
      else if(xmlStrcmp(stype, (guchar*)"headline") == 0)
	type = messageTypeHeadline;
      else if(xmlStrcmp(stype, (guchar*)"error") == 0)
	type = messageTypeError;
    }

  if((x = parserFindByNode(node, (guchar*)"x")) != NULL)
    {
      if(xmlStrcmp(parserFindByNsAndReturn(x), (guchar*)"jabber:x:event") == 0)
	{
	  if(parserFindByNode(x, (guchar*)"offline") != NULL)
	    xevent_type |= messageXEventTypeOffline;
	  if(parserFindByNode(x, (guchar*)"delivered") != NULL)
	    xevent_type |= messageXEventTypeDelivered;
	  if(parserFindByNode(x, (guchar*)"displayed") != NULL)
	    xevent_type |= messageXEventTypeDisplayed;
	  if(parserFindByNode(x, (guchar*)"composing") != NULL)
	    xevent_type |= messageXEventTypeComposing;
	  
	  if(xevent_type == 0)
	    xevent_type = messageXEventTypeCancelled;
	  
	  if((tmp = parserFindByNodeAndReturn(x, (guchar*)"id")) != NULL) 
	    xevent_message_id = g_strdup_printf("%s",tmp);      
	  
	  /* set message type */
	  if(xevent_message_id != NULL)
	    type = messageTypeXEvent;
	}
      else if(xmlStrcmp(parserFindByNsAndReturn(x), (guchar*)"jabber:x:oob") == 0)
	{
	  if((tmp = parserFindByNodeAndReturn(x, (guchar*)"url")) != NULL) 
	    xoob_url = g_strdup_printf("%s",tmp);    
	  if((tmp = parserFindByNodeAndReturn(x, (guchar*)"desc")) != NULL) 
	    xoob_description = g_strdup_printf("%s",tmp);    
	}
    }

  if(type != messageTypeXEvent)
    {
      if((tmp = parserFindByNodeAndReturn(node, (guchar*)"subject")) != NULL)
	subject = g_strdup((gchar*)tmp);
      
      if((tmp = parserFindByNodeAndReturn(node, (guchar*)"body")) != NULL)
	body = g_strdup((gchar*)tmp);
      
      if((tmp = parserFindByNodeAndReturn(node, (guchar*)"thread")) != NULL)
	thread = g_strdup((gchar*)tmp);

      if((tmp = parserFindByAttrAndReturn(node, (guchar*)"id")) != NULL) 
	message_id = g_strdup((gchar*)tmp);
    }

  /* get 'to' field and separate resource and user */
  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"to")) != NULL && xmlStrlen(tmp) > 0)
    {
      gchar *to = g_strdup_printf("%s",tmp);
      gchar **vTo = g_strsplit(to,"/",2);
      
      if(vTo != NULL)
	{
	  to_user = g_strdup(to);
	  
	  if(vTo[1] != NULL)
	    to_resource = g_strdup(vTo[1]);
	  
	  g_strfreev(vTo);
	}
      
      if(to != NULL) g_free(to);
    }
  
  /* get 'from' field and separate resource and user */
  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"from")) != NULL && xmlStrlen(tmp) > 0)
    {
      gchar *from = g_strdup_printf("%s",tmp);
      gchar **vFrom = g_strsplit(from,"/",2);
      
      if(vFrom != NULL)
	{
	  from_user = g_strdup(from);
	  
	  if(vFrom[1] != NULL)
	    from_resource = g_strdup(vFrom[1]);
	  
	  g_strfreev(vFrom);
	}
      
      if(from != NULL) g_free(from);
    }

  /* get error */
  if(type == messageTypeError)
    {
      xmlChar *tmp = NULL;
      xmlNodePtr error = NULL;

      if((error = parserFindByNode(node, (guchar*)"error")) != NULL)
	{
	  if((tmp = parserFindByAttrAndReturn(error, (guchar*)"code")) != NULL)
	    error_code = g_strdup_printf("%s",tmp);

	  if((tmp = parserFindByNodeAndReturn(node, (guchar*)"error")) != NULL) 
	    error_reason = g_strdup_printf("%s",tmp);
	}
    }

  /*
   * create our message struct
   */

  msg = messageNew(requestID, type); 

  if(msg == NULL)
    {
      g_warning("messageReceive: failed to create new message, NULL returned");
      return;
    }

  if(message_id) messageSetID(msg,message_id);
  if(to_user) messageSetToJID(msg,to_user);
  if(to_resource) messageSetToResource(msg,to_resource);
  if(from_user) messageSetFromJID(msg,from_user);
  if(from_resource) messageSetFromResource(msg,from_resource);
  if(subject) messageSetSubject(msg,subject);
  if(body) messageSetBody(msg,body);
  if(thread) messageSetThread(msg,thread);

  if(xevent_type != 0) messageSetXEventType(msg, xevent_type);
  if(xevent_message_id) messageSetXEventMessageID(msg, xevent_message_id);
  if(xoob_url) messageSetXOOBURL(msg, xoob_url);
  if(xoob_description) messageSetXOOBDescription(msg, xoob_description);

  if(error_code) messageSetErrorCode(msg,error_code);
  if(error_reason) messageSetErrorReason(msg,error_reason);

  /* call callback function */
  (callback)(msg);

  /* free mem */
  if(subject != NULL) g_free(subject);
  if(body != NULL) g_free(body);
  if(to_user != NULL) g_free(to_user);
  if(to_resource != NULL) g_free(to_resource);
  if(from_user != NULL) g_free(from_user);
  if(from_resource != NULL) g_free(from_resource);

  if(xevent_message_id != NULL) g_free(xevent_message_id);
  if(xoob_url != NULL) g_free(xoob_url);
  if(xoob_description != NULL) g_free(xoob_description);

  if(error_code != NULL) g_free(error_code);
  if(error_reason != NULL) g_free(error_reason);
}


#ifdef __cplusplus
}
#endif
