/* 
 * $RCSfile: gj_message.h,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/26 11:09:58 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifndef __gj_message_h
#define __gj_message_h

#include <time.h> 

#include "gj_connection.h"

/*
 * Message
 */

typedef enum messageType {
  messageTypeNormal = 1,
  messageTypeChat,
  messageTypeGroupChat,
  messageTypeHeadline,
  messageTypeError,
  messageTypeXEvent,

  messageTypeEnd
} messageType;
 

#define messageXEventTypeOffline      1 << 1
#define messageXEventTypeDelivered    1 << 2
#define messageXEventTypeDisplayed    1 << 3 
#define messageXEventTypeComposing    1 << 4
#define messageXEventTypeCancelled    1 << 5


typedef struct t_message *message;

typedef void (*messageFromResponseProc)(message msg);
typedef void (*messageToResponseProc)(gint requestID);


gboolean messageInit();
gboolean messageTerm();

void messageFree(message msg);

/*
 * message attributes access
 */
message messageFindByToJID(gchar *jid);

gboolean messageSetType(message msg, messageType type);
gboolean messageSetID(message msg, gchar *id);

gboolean messageSetToJID(message msg, gchar *jid);
gboolean messageSetToResource(message msg, gchar *resource);

gboolean messageSetFromJID(message msg, gchar *jid);
gboolean messageSetFromResource(message msg, gchar *resource);

gboolean messageSetSubject(message msg, gchar *subject);
gboolean messageSetBody(message msg, gchar *body);

gboolean messageSetThread(message msg, gchar *thread);

gboolean messageSetXEventType(message msg, gint type);
gboolean messageSetXEventMessageID(message msg, gchar *message_id);

gboolean messageSetXOOBURL(message msg, gchar *url);
gboolean messageSetXOOBDescription(message msg, gchar *description);

gboolean messageSetErrorCode(message msg, gchar *error_code);
gboolean messageSetErrorReason(message msg, gchar *error_reason);


messageType messageGetType(message msg);
gchar *messageGetID(message msg);

gint messageGetRequestID(message msg);

gchar *messageGetToJID(message msg);
gchar *messageGetToResource(message msg);

gchar *messageGetFromJID(message msg);
gchar *messageGetFromResource(message msg);

gchar *messageGetSubject(message msg);
gchar *messageGetBody(message msg);

gchar *messageGetThread(message msg);

gint messageGetXEventType(message msg);
gchar *messageGetXEventMessageID(message msg);

gchar *messageGetXOOBURL(message msg);
gchar *messageGetXOOBDescription(message msg);

gchar *messageGetErrorCode(message msg);
gchar *messageGetErrorReason(message msg);

time_t messageGetTime(message msg);



/*
 * message functions to tcp / parser layer 
 */
gboolean messageSend(messageType type, gchar *to, gchar *subject, gchar *body, gchar *thread);
gboolean messageSendXEvent(gint type, gchar *to, gchar *id);
gchar *messageSendWithXEventRequest(messageType type, gchar *to, gchar *subject, gchar *body, gchar *thread, gint xevents);

void messageReceive(connection c, xmlNodePtr node);



/* to details, i.e. my state */
gboolean messageToSetResponseProc(messageToResponseProc proc);

/* from details, i.e. my contacts state(s) */
gboolean messageFromSetResponseProc(messageFromResponseProc proc);


#endif
