/* 
 * $RCSfile: gj_parser.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/26 20:40:19 $
 * 
 * $Revision: 1.5 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <glib.h>

#include <libxml/parser.h>
#include <libxml/parserInternals.h>

#include "gj_parser.h"
#include "gj_connection.h"
#include "gj_stream.h"
#include "gj_iq.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_me.h"

/* locals */
gboolean parserType(connection c, xmlNodePtr node);

gboolean parserInputCheck(gchar *buf);
gboolean parserInputNode(connection c, xmlNodePtr node, gboolean stream);

const gchar *parserPrintSpaces(gint num);

/* parser callbacks */
void parserDocStart(void *user_data);
void parserDocEnd(void *user_data);
void parserDocElementStart(void *user_data, const xmlChar *name, const xmlChar **attrs);
void parserDocElementEnd(void *user_data, const xmlChar *name);
xmlEntityPtr parserDocEntityGet(void *user_data, const xmlChar *name) ;
void parserDocWarning(void *user_data, const char *msg, ...);
void parserDocError(void *user_data, const char *msg, ...);
void parserDocFatalError(void *user_data, const char *msg, ...);

gboolean parserXML(void *user_data, const gchar *input);

gchar *bufwaiting = NULL;

guint connection_id = 0;

/* global vars */
gint requestID = 69;
xmlParserCtxtPtr ctxt = NULL;
xmlSAXHandler sax = {
  0, /* internalSubset */
  0, /* isStandalone */
  0, /* hasInternalSubset */
  0, /* hasExternalSubset */
  0, /* resolveEntity */
  (getEntitySAXFunc)parserDocEntityGet, /* getEntity */
  0, /* entityDecl */
  0, /* notationDecl */
  0, /* attributeDecl */
  0, /* elementDecl */
  0, /* unparsedEntityDecl */
  0, /* setDocumentLocator */
  (startDocumentSAXFunc)parserDocStart, /* startDocument */
  (endDocumentSAXFunc)parserDocEnd, /* endDocument */
  (startElementSAXFunc)parserDocElementStart, /* startElement */
  (endElementSAXFunc)parserDocElementEnd, /* endElement */
  0, /* reference */
  0, /*(charactersSAXFunc)gladeCharacters, *//* characters */
  0, /* ignorableWhitespace */
  0, /* processingInstruction */
  0, /* (commentSAXFunc)gladeComment, *//* comment */
  (warningSAXFunc)parserDocWarning, /* warning */
  (errorSAXFunc)parserDocError, /* error */
  (fatalErrorSAXFunc)parserDocFatalError, /* fatalError */
};


gboolean parserType(connection c, xmlNodePtr node)
{
  const gchar *type = NULL;

  if(node == NULL)
    {
      g_warning("parserType: node passed was NULL");
      return FALSE;
    }

  /* get type */
  type = (const gchar*) parserGetElement(node);

  if(g_strcasecmp(type,"stream") == 0)
    {
      streamReceive(c, node);
      return TRUE;
    }
  else if(g_strcasecmp(type,"iq") == 0)
    {
      infoqueryReceive(c, node);
      return TRUE;
    }
  else if(g_strcasecmp(type,"message") == 0)
    {
      /*g_message("parserType: type is '%s', calling message handling function...",type);*/
      messageReceive(c, node);
      return TRUE;
    }
  else if(g_strcasecmp(type,"presence") == 0)
    {
      /*g_message("parserType: type is '%s', calling presence handling function...",type);*/
      presenceReceive(c, node);
      return TRUE;
    }
  
  /* NOTE: we *REALLY* should ignore the previous buffers and 
     wait for the next message if this happens */
  g_warning("parserType: unexpected type of msg from connection:0x%.8x", c);
  return FALSE;
}

void parserInputFromConnection(connection c, gchar *text)
{
    if(text == NULL)
    {
	g_warning("parserInputFromConnection: text was NULL");
	return;
    }

    g_message("parserInputFromConnection: parsing %d bytes from connection:0x%.8x", strlen(text), (gint)c);
    parserInput(c, text, strlen(text));
}

gboolean parserInputCheck(gchar *buf)
{
  gint index = 0;
  gint pos = 0;
  
  gchar *tagStart = NULL;
  gchar *tagEnd = NULL;
  
  return TRUE;

  /* check start has opening tag */
  if(buf[0] != '<')
    return FALSE;
  /* check end has closing tag */
  if(buf[strlen(buf)-1] != '>')
    return FALSE;
  
  /* get start tag name */
  for(index=0,pos=0;index<strlen(buf);index++)
    {
      if((g_ascii_isspace(buf[index]) == TRUE || buf[index] == '>') && index > 1)
	{
	  pos = index;
	  tagStart = g_strndup(buf+1,pos-1);
	  break;
	}
    }

  if(tagStart != NULL)
    g_log("XML", G_LOG_LEVEL_DEBUG,"parserInputCheck: found opening tag:'%s' at position:%d",tagStart,pos);
  
  /* get end tag name */
  for(index=strlen(buf)-1,pos=0;index>0;index--)
    {
      if(buf[index] == '<')
	{
	  pos = index + 1;
	  tagEnd = g_strndup(buf+pos,pos-2);
	  break;
	}
      else if(buf[index] == '/' && buf[index-1] == '<')
	{
	  pos = index + 1;
	  tagEnd = g_strndup(buf+pos,strlen(buf) - pos - 1);
	  break;
	}
    }
  
  if(tagEnd != NULL)
    g_log("XML", G_LOG_LEVEL_DEBUG,"parserInputCheck: found closing tag:'%s' at position:%d",tagEnd,pos);      
  
  /* there may NOT be an end tag, 
     e.g. <iq type="result" to="joebloggs"/> */
  if(tagEnd != NULL && g_strcasecmp(tagStart,tagEnd) != 0)
    return FALSE;
  
  return TRUE;
}

void parserInput(connection c, gchar *buf, gint nBytes)
{
  gint nodecount = 0;
  gboolean stream = FALSE;
  gchar *xml = NULL;

  xmlDocPtr doc = NULL;
  xmlNodePtr current = NULL;
  
  if(buf == NULL || nBytes < 1)
    {
      g_warning("parserInput: buf is NULL or size:%d is < 1",nBytes);
      return;
    }

  /* check for stream start / end, xml needs patching to pass parser */
  if(g_strncasecmp(buf,"<stream",7) == 0 || g_strncasecmp(buf,"<?xml",5) == 0)
    {
      /* patch start of stream */
      gchar *footer = g_strdup_printf("</stream:stream>");
      xml = g_strconcat(buf,footer,NULL);
      g_free(footer);

      stream = TRUE;
    }
  else if(g_strncasecmp(buf,"</stream",8) == 0)
    {
      /* patch end of stream */
      gchar *header = g_strdup_printf("<stream:stream>");
      xml = g_strconcat(header,buf,NULL);
      g_free(header);

      stream = TRUE;
    }
  else
    {
      gchar *header = NULL;
      gchar *footer = NULL;

      if(bufwaiting != NULL)
	{
	  g_log("XML", G_LOG_LEVEL_DEBUG, "parserInput: buffer saved previously, concating to new buffer");
	  buf = g_strconcat(bufwaiting,buf,NULL);
	  
	  g_log("XML", G_LOG_LEVEL_DEBUG, "parserInput: freeing saved buffer");
	  g_free(bufwaiting);
	  bufwaiting = NULL;

	  /* NOW PARSER XML */
	}
      
      g_log("XML", G_LOG_LEVEL_DEBUG, "parserInput: now attempting to parse buffer");
      
      /* used to make sure header and footer elements are unique 
	 if this is NOT done, the parser sometimes thinks a chunk
	 of xml is not balanced.  

	 e.g. <presence></presence><presence></presence> */
      header = g_strdup_printf("<gj>");
      footer = g_strdup_printf("</gj>");
      
      xml = g_strconcat(header,buf,footer,NULL);
      
      if(header != NULL) g_free(header);
      if(footer != NULL) g_free(footer);

      if(parserXML(NULL, xml) == FALSE)
	{
	  g_log("XML", G_LOG_LEVEL_DEBUG, "parserInput: xml was not well balanced, waiting for MORE input");  
	  g_log("XML", G_LOG_LEVEL_DEBUG, "parserInput: saving buffer");
	  bufwaiting = g_strdup(buf);

	  if(xml != NULL) g_free(xml);
	  
	  return;
	}
      else
	{
	  g_log("XML", G_LOG_LEVEL_DEBUG, "parserInput: xml is ok, freeing previously saved buffer");
	  g_free(bufwaiting);
	  bufwaiting = NULL;
	}
    }

  doc = xmlSAXParseDoc(NULL, (xmlChar*)xml,1);  
  if(doc == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG, "parserInput: parser context was NULL, can not continue...");
      return;
    }

  /* clean up */
  if(xml != NULL) g_free(xml);

  /* stream check */
  if(stream == TRUE)
    {
      parserInputNode(c, doc->xmlChildrenNode,stream);
    }
  else
    { 
      if((nodecount = parserCountToplevelNodes(doc)) < 1)
	{
	  g_warning("parserInput: there were %d nodes to process (> 0 required)",nodecount);
	  return;
	}
      
      if(doc->xmlChildrenNode != NULL)
	current = doc->xmlChildrenNode->xmlChildrenNode;
      
      while(current != NULL)
	{
	  parserInputNode(c, current,stream);
	  current = current->next;
	}
    }
}

gboolean parserInputNode(connection c, xmlNodePtr node, gboolean stream)
{
  /* set node from doc */
  if(node != NULL)
    {
      xmlNodePtr myCopy = NULL;

      /* make a copy for us to play with */
      myCopy = xmlCopyNode(node,1);

      /* parser doc by type - i.e iq/message.... */
      if(myCopy != NULL)
	{
	  /* print tree */
/* 	  g_log("XML", G_LOG_LEVEL_DEBUG,"****** started printing tree node:0x%.8x *********",(gint)myCopy); */
/* 	  parserPrintTree(myCopy,0); */
/* 	  g_log("XML", G_LOG_LEVEL_DEBUG,"****** stopped printing tree node:0x%.8x *********",(gint)myCopy); */


	  /*g_message("****** %d top nodes ******",parserCountToplevelNodes(doc));*/
	  parserType(c, myCopy);
	  return TRUE;
	}
      else
	g_warning("parserInput: failed to make a copy of node, copy was NULL!");
    }
  else
    g_warning("parserInput: failed to continue, node was NULL");

  return FALSE;
}

gboolean parserXML(void *user_data, const gchar *input) 
{
  gboolean ret = 0;
  xmlParserCtxtPtr ctxt;

  ctxt = xmlCreatePushParserCtxt(&sax, user_data, input, strlen(input), NULL);
  if (ctxt == NULL)
    {
      g_warning("parserXML: could not create push parser");
      return FALSE;      
    }

  xmlParseDocument(ctxt);
  
  if(ctxt->wellFormed)
    ret = TRUE;
  else
    ret = FALSE;

  if(&sax != NULL)
    ctxt->sax = NULL;

  xmlFreeParserCtxt(ctxt);
  
  return ret;
}

const gchar *parserPrintSpaces(gint num)
{
  gchar *string = NULL;

  while(num-- > 0)
    {
      if(string != NULL)
	string = g_strconcat(string, "  ", NULL);
      else
	string = "  ";
    }

  if(string == NULL)
    string = "";

  return string;
}

xmlNodePtr parserPrintTree(xmlNodePtr node, gint depth)
{
  xmlNodePtr result = NULL;
  gchar buffer[1024];

  if(node == NULL)                      
    return NULL;

  /* set buffer to spaces */
  memset(buffer, 32, sizeof(buffer));
  buffer[(depth*2)] = '\0';

  if(node->name != NULL)   
    {
/*       const gchar *spaces = NULL; */

      if(xmlStrcasecmp(node->name, (guchar*)"text") != 0 && xmlStrlen(node->name) > 0)
	{
/* 	  spaces = parserPrintSpaces(depth); */
	  g_log("XML", G_LOG_LEVEL_DEBUG, "%s-> name:'%s' (0x%.8x)", buffer, node->name, (gint)node->name);
	}
      
      if(node->content != NULL && xmlStrlen(node->content) > 0)
	{
/* 	  spaces = parserPrintSpaces(depth); */
	  g_log("XML", G_LOG_LEVEL_DEBUG, "%s - content:'%s' (0x%.8x)", buffer, node->content, (gint)node->content);
	}
    }

  printf(" ");

  result = parserPrintTree(node->xmlChildrenNode,depth+1);

  return result?result:parserPrintTree(node->next,depth);
}

gboolean parserOutput(xmlNodePtr node)
{
  gchar* output = NULL;
  connection c = NULL;

  if(node == NULL)
    {
      g_warning("parserOutput: failed to continue, node was NULL");
      return FALSE;
    }
  
  if((output = parserGetStrFromNode(node)) == NULL)
    {
      g_warning("parserOutput: out was NULL from node:0x%.8x", (gint)node);
      return FALSE;
    }

  if((c = gj_connection_find_by_host_and_port(gj_me_get_connected_host(),
					      gj_me_get_connected_port())) == NULL)  
  {
      g_warning("parserOutputFromText: could not find connection from host:'%s' and port:%d", 
		gj_me_get_connected_host(), gj_me_get_connected_port());
      return FALSE;
    }
  
  gj_connection_write(c, output);
  g_free(output);

  return TRUE;
}

gboolean parserOutputFromText(gchar *text)
{
  connection c = NULL;

  if(text == NULL)
    {
      g_warning("parserOutputFromText: text was NULL");
      return FALSE;
    }

  if((c = gj_connection_find_by_host_and_port(gj_me_get_connected_host(),
					      gj_me_get_connected_port())) == NULL)
    {
      g_warning("parserOutputFromText: could not find connection from host:'%s' and port:%d", 
		gj_me_get_connected_host(), gj_me_get_connected_port());
      return FALSE;
    }
  
     gj_connection_write(c, text);

  return TRUE;
}


gchar *parserCheckText(gchar *text)
{
  gint index = -1;
  gchar *insert = NULL;
  gchar *new = NULL;
  gchar *character = NULL;

  /*
   * we really should use:
   *   g_markup_escape()
   */

  if(text == NULL || strlen(text) < 1)
    {
      g_warning("parserCheckText: text was NULL or length of text was less than 1");
      return NULL;
    }

  for(index=0;index<strlen(text);index++)
    {
      switch(text[index])
	{
	case 0x22: /* " */
	  insert = g_strdup_printf("&quot;");
	  break;
	case 0x26: /* & */
	  insert = g_strdup_printf("&amp;");
	  break;
	case 0x27: /* ' */
	  insert = g_strdup_printf("&apos;");
	  break;
	case 0x3c: /* < */
	  insert = g_strdup_printf("&lt;");
	  break;
	case 0x3e: /* > */
	  insert = g_strdup_printf("&gt;");
	  break;
	default:
	  {
	    character = g_strndup(text+index,1);

	    if(new == NULL)
	      new = g_strdup(character);
	    else
	      new = g_strconcat(new,character,NULL);

	    g_free(character);
	    character = NULL;

	    continue;
	  }
	}      

      if(new == NULL)
	new = g_strdup(insert);
      else
	new = g_strconcat(new,insert,NULL);
      
      g_free(insert);
      insert = NULL;
    }

  return new;
}

/* utils */
gchar* parserGetStrFromNode(xmlNodePtr node)
{
  xmlDocPtr doc = NULL;
  xmlChar *buf = NULL;

  gint size = 0;

  gchar* output = NULL;

  if(node == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserGetStrFromNode: failed to continue, node was NULL");
      return NULL;
    }
    
  if((doc = xmlNewDoc((guchar*)"1.0")) == NULL)
    {
      /* failed */
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserGetStrFromNode: failed to create new xml document, returning...");
      return NULL;
    }
  
  /* set root node */
  xmlDocSetRootElement(doc,node);
  
  /* save to file and get mem */
  xmlDocDumpMemory(doc,&buf,&size);

  if(buf == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserGetStrFromNode: failed because xmlDocDumpMemory returned NULL");
      return NULL;
    }

  /* 
     skip header with ver 
     and create for fl 
  */
  buf[size] = '\0';
  buf+=21;

  /* !!!!!!! need to free buf... but free, g_free crash it */
  xmlFreeDoc(doc);
  
  output = g_strdup_printf("%s",buf);
  return output;
}


gint parserCountToplevelNodes(xmlDocPtr doc)
{
  xmlNodePtr root = NULL;

  gint count = 0;

  if(doc == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserCountToplevelNodes: doc was NULL");
      return -1;
    }

  /*root = doc->xmlRootNode;*/
  root = doc->xmlRootNode->xmlChildrenNode;

  while(root != NULL)
    {
      root = root->next;
      count++;
    }

  return count;
}

const xmlChar *parserGetElement(xmlNodePtr node)
{
  if(node == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserGetElement: failed because node parsed was NULL");
      return NULL;
    }

  return node->name;
}

const xmlChar *parserGetContent(xmlNodePtr node)
{
  if(node == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserGetContent: failed because node parsed was NULL");
      return NULL;
    }

  return node->xmlChildrenNode->content;
}

xmlChar *parserFindByAttrAndReturn(xmlNodePtr node, xmlChar *attrname)
{
  xmlAttrPtr attr = NULL;

  if(node == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserFindByAttrAndReturn: failed because node was NULL");
      return NULL;
    }

  if(attrname == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserFindByAttrAndReturn: failed because attr to search for was NULL");
      return NULL;
    }

  attr = parserFindByAttr(node->properties,attrname);
  
  if(attr == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserFindByAttrAndReturn: failed to find attribute:'%s'",attrname);
      return NULL;
    }

  if(attr->xmlChildrenNode != NULL)
    return attr->xmlChildrenNode->content;
  else
    return NULL;
}

const xmlChar *parserFindByNsAndReturn(xmlNodePtr node)
{
  xmlNodePtr nsnode = NULL;

  if(node == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserFindByNsAndReturn: failed because node was NULL");
      return NULL;
    }

  nsnode = parserFindByNs(node);
  
  if(nsnode == NULL)
    {
      g_log("XML", G_LOG_LEVEL_DEBUG,"parserFindByNsAndReturn: failed to find namespace:");
      return NULL;
    }

  if(nsnode->ns != NULL && nsnode->ns->href != NULL)
      return nsnode->ns->href;
  else
    return NULL;
}

xmlChar * parserFindByNodeAndReturn(xmlNodePtr node, xmlChar *text)
{
  xmlNodePtr found = NULL;
  xmlChar *contents = NULL;

  if((found = parserFindByNode(node,text)) != NULL)
    {
      if(found->xmlChildrenNode != NULL)
	contents = found->xmlChildrenNode->content;
    }

  return contents;
}

GList *parserFindByNodeAndReturnContentsByList(xmlNodePtr node, xmlChar *text)
{
  GList *node_list = NULL;
  GList *contents_list = NULL;

  node_list = parserFindByNodeAndReturnByList(node, text);
  
  if(node_list != NULL)
    {
      gint i = 0;

      for(i=0;i < g_list_length(node_list);i++)
	{
	  xmlNodePtr node = NULL;
	  
	  /* checks */
	  if((node = (xmlNodePtr) g_list_nth_data(node_list, i)) == NULL)
	    continue;
	  
	  if(node->xmlChildrenNode == NULL || node->xmlChildrenNode->content == NULL)
	    continue;
	  
	  /* get contents and add to list */
	  contents_list = g_list_append(contents_list, 
					g_strdup((gchar*)node->xmlChildrenNode->content));

	  g_log("XML", G_LOG_LEVEL_DEBUG,"parserFindByNodeAndRetrnContentsByList: contents[%d]:'%s'", i, node->xmlChildrenNode->content);
	}
    }

  return contents_list;
}

GList *parserFindByNodeOnSameParentAndReturnContentsByList(xmlNodePtr node, xmlChar *text)
{
  GList *node_list = NULL;
  GList *contents_list = NULL;

  node_list = parserFindByNodeOnSameParentAndReturnByList(node, text);
  
  if(node_list != NULL)
    {
      gint i = 0;

      for(i=0;i < g_list_length(node_list);i++)
	{
	  xmlNodePtr node = NULL;
	  
	  /* checks */
	  if((node = (xmlNodePtr) g_list_nth_data(node_list, i)) == NULL)
	    continue;
	  
	  if(node->xmlChildrenNode == NULL || node->xmlChildrenNode->content == NULL)
	    continue;
	  
	  /* get contents and add to list */
	  contents_list = g_list_append(contents_list, 
					g_strdup((gchar*)node->xmlChildrenNode->content));

	  g_log("XML", G_LOG_LEVEL_DEBUG, "parserFindByNodeOnSameParentAndReturnContentsByList: contents[%d]:'%s'", i, node->xmlChildrenNode->content);
	}
    }

  return contents_list;
}

xmlNodePtr parserFindByNode(xmlNodePtr node, xmlChar *text)
{
  xmlNodePtr result;
  
  if(node == NULL || text == NULL)                      
    return NULL;
  if(node->name != NULL && xmlStrcasecmp(node->name, text) == 0)   
    return node;

  result = parserFindByNode(node->xmlChildrenNode,text);

  return result?result:parserFindByNode(node->next,text);
}

xmlNodePtr parserFindByNodeOnSameParent(xmlNodePtr node, xmlChar *text)
{
  xmlNodePtr result;
  
  if(node == NULL || text == NULL)                      
    return NULL;
  if(node->name != NULL && xmlStrcasecmp(node->name,text) == 0)   
    return node;

  result = parserFindByNodeOnSameParent(node->xmlChildrenNode,text);

  return result;
}

GList *parserFindByNodeAndReturnByList(xmlNodePtr node, xmlChar *text)
{
  GList *list = NULL;

  while(node != NULL)
    {
      if((node = parserFindByNode(node, text)) == NULL)
	{
	  g_log("XML", G_LOG_LEVEL_DEBUG,"parserFindByNodeAndReturnByList: node was NULL");
	  break;
	}

      list = g_list_append(list, node);

      node = node->next;
    }

  return list;
}

GList *parserFindByNodeOnSameParentAndReturnByList(xmlNodePtr node, xmlChar *text)
{
  GList *list = NULL;

  while(node != NULL)
    {
      if((node = parserFindByNodeOnSameParent(node, text)) == NULL)
	{
	  g_log("XML", G_LOG_LEVEL_DEBUG,"parserFindByNodeOnSameParentAndReturnByList: node was NULL");
	  break;
	}

      list = g_list_append(list, node);

      node = node->next;
    }

  return list;
}

xmlNodePtr parserFindByNodeContents(xmlNodePtr node, xmlChar *contents)
{
  xmlNodePtr result;
  
  if(node == NULL || contents == NULL)                      
    return NULL;
  if(node->content != NULL && xmlStrcasecmp(node->content,contents) == 0)   
    return node;

  result = parserFindByNodeContents(node->xmlChildrenNode,contents);

  return result?result:parserFindByNodeContents(node->next,contents);
}

xmlAttrPtr parserFindByAttr(xmlAttrPtr prop, xmlChar *text)
{
  if(text == NULL)
    return NULL;

  while(prop != NULL)
    {
      if(xmlStrcasecmp(prop->name,text) == 0)
	  return prop;

      prop = prop->next;
    }

  return NULL;
}

xmlNodePtr parserFindByNs(xmlNodePtr node)
{
  xmlNodePtr result;
  
  if(node == NULL)                      
    return NULL;
  if(node->ns != NULL)
    return node;

  result = parserFindByNs(node->xmlChildrenNode);

  return result?result:parserFindByNs(node->next);
}

xmlNodePtr parserFindByNodeWithAttr(xmlNodePtr node, xmlChar *nodeName, xmlChar *attrName, xmlChar *attrVal)
{
  xmlNodePtr resultNode = NULL;
  xmlAttrPtr resultAttr = NULL;
  xmlAttrPtr attr = NULL;

  if(attrVal == NULL)
    return NULL;

  while((resultNode = parserFindByNode(node,nodeName)) != NULL)
    {
      attr = resultNode->properties;

      while((resultAttr = parserFindByAttr(attr,attrName)) != NULL)
	{
	  if(resultAttr->xmlChildrenNode != NULL && resultAttr->xmlChildrenNode->content != NULL)
	    {
	      if(xmlStrcasecmp(resultAttr->xmlChildrenNode->content,attrVal) == 0)
		return resultAttr->parent;
	    }

	  attr = resultAttr->next;
	}

      node = resultNode->next;
    }

  return NULL;
}

/*
 * parser callbacks
 */

void parserDocStart(void *user_data)
{
  g_log("XML", G_LOG_LEVEL_DEBUG,"parserDocStart: user_data:0x%.8x", (gint)user_data);
}

void parserDocEnd(void *user_data)
{
  g_log("XML", G_LOG_LEVEL_DEBUG,"parserDocEnd: user_data:0x%.8x", (gint)user_data);
}

void parserDocElementStart(void *user_data, const xmlChar *name, const xmlChar **attrs)
{
  g_log("XML", G_LOG_LEVEL_DEBUG,"parserDocElementStart: user_data:0x%.8x, name:'%s'", (gint)user_data, name);
}

void parserDocElementEnd(void *user_data, const xmlChar *name)
{
  g_log("XML", G_LOG_LEVEL_DEBUG,"parserDocElementEnd: user_data:0x%.8x, name:'%s'", (gint)user_data, name);
}

xmlEntityPtr parserDocEntityGet(void *user_data, const xmlChar *name) 
{
  g_log("XML", G_LOG_LEVEL_DEBUG, "parserDocEntityGet: user_data:0x%.8x, name:'%s'", (gint)user_data, name);
  return xmlGetPredefinedEntity(name);
}

void parserDocWarning(void *user_data, const char *msg, ...) 
{
    va_list args;

    va_start(args, msg);
    g_logv("XML", G_LOG_LEVEL_WARNING, msg, args);
    va_end(args);
}

void parserDocError(void *user_data, const char *msg, ...) 
{
    va_list args;

    va_start(args, msg);
    g_logv("XML", G_LOG_LEVEL_CRITICAL, msg, args);
    va_end(args);
}

void parserDocFatalError(void *user_data, const char *msg, ...) 
{
    va_list args;

    va_start(args, msg);
    g_logv("XML", G_LOG_LEVEL_ERROR, msg, args);
    va_end(args);
}

#ifdef __cplusplus
}
#endif
