/* 
 * $RCSfile: gj_presence.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/26 11:09:58 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <string.h>

#include <time.h>

#include <glib.h>

#include <libxml/tree.h>

#include "gj_presence.h"
#include "gj_parser.h"

struct t_presence
{
  presenceType type;
  presenceShow show;

  gchar *status;
  guint priority;
  
  gint requestID;

  gchar *to_jid;
  gchar *to_resource;
  gchar *from_jid;
  gchar *from_resource;
  
  gchar *error_code;
  gchar *error_reason;

  time_t time;
};

/* to details, i.e. my state */
presenceToResponseProc presenceToGetResponseProc();

/* from details, i.e. my contacts state(s) */
presenceFromResponseProc presenceFromGetResponseProc();

/* function */
presenceFromResponseProc presenceFromProc;
presenceToResponseProc presenceToProc;   /* may not get used */

void presenceDestroy(presence pres);

gboolean presenceListAdd(presence pres);
gboolean presenceListDel(presence pres);

GList *presenceList = NULL;

/*
 * code
 */
gboolean presenceInit()
{
  return TRUE;
}

gboolean presenceTerm()
{
  gint index = 0;

  /* list */
  if(presenceList == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(presenceList);index++)
    {
      presence pres = NULL;
      
      if((pres = (presence) g_list_nth_data(presenceList,index)) != NULL)
	presenceFree(pres);
    }

  g_list_free(presenceList);
  presenceList = NULL;

  return TRUE;
}

gboolean presenceListAdd(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceListAdd: failed to add item to list, it was NULL");
      return FALSE;
    }

  presenceList = g_list_append(presenceList,pres);

  return TRUE;
}

gboolean presenceListDel(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceListDel: failed to del item from list, it was NULL");
      return FALSE;
    }

  presenceList = g_list_remove(presenceList,pres);
  return TRUE;
}

presence presenceNew(gint requestID, presenceType type, presenceShow show)
{
  presence pres = NULL;

  if(requestID < 1)
    {
      g_warning("presenceNew: failed to create new roster item, requestID:%d was < 1",requestID);
      return NULL;
    }
  
  if((pres = g_new(struct t_presence,1)) != NULL)
    {
      pres->type = type;
      pres->show = show;


      pres->requestID = requestID;

      pres->to_jid = NULL;
      pres->to_resource = NULL;

      pres->from_jid = NULL;
      pres->from_resource = NULL;
      
      pres->status = NULL;
      pres->priority = 0;

      pres->error_code = NULL;
      pres->error_reason = NULL;

      presenceListAdd(pres);
    }
  
  return pres;  
}

void presenceFree(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceFree: failed to free, presence was NULL");
      return;
    }

  /* free mem */
  presenceListDel(pres);  

  /* destroy members */
  presenceDestroy(pres);

  /* free structure */
  g_free(pres);
}

void presenceDestroy(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceDestroy: failed to destroy, presence was NULL");
      return;
    }

  pres->type = presenceTypeEnd;
  pres->show = presenceShowEnd;

  pres->requestID = 0;

  if(pres->status != NULL)
    {
      g_free(pres->status);
      pres->status = NULL;
    }

  pres->priority = 0;

  if(pres->to_jid != NULL)
    {
      g_free(pres->to_jid);
      pres->to_jid = NULL;
    }

  if(pres->to_resource != NULL)
    {
      g_free(pres->to_resource);
      pres->to_resource = NULL;
    }

  if(pres->from_jid != NULL)
    {
      g_free(pres->from_jid);
      pres->from_jid = NULL;
    }

  if(pres->from_resource != NULL)
    {
      g_free(pres->from_resource);
      pres->from_resource = NULL;
    }  

  if(pres->error_code != NULL)
    {
      g_free(pres->error_code);
      pres->error_code = NULL;
    } 

  if(pres->error_reason != NULL)
    {
      g_free(pres->error_reason);
      pres->error_reason = NULL;
    } 
}

presence presenceCopy(presence pres)
{
  presence pres_copy = NULL;

  if(pres == NULL)
    {
      g_warning("presenceCopy: presence was NULL");
      return NULL;
    }

  if((pres_copy = presenceNew(supportGetUniqueID(), pres->type, pres->show)) == NULL)
    {
      g_warning("presenceCopy: failed to make copy, could not create NEW presence");
      return NULL;
    }

   /* 
    * not copying requestID because this is supposed to be unique 
    */

  if(pres->status != NULL)
    pres_copy->status = g_strdup(pres->status);
  
  pres_copy->priority = pres->priority;
  
  if(pres->to_jid != NULL)
    pres_copy->to_jid = g_strdup(pres->to_jid);
  
  if(pres->to_resource != NULL)
    pres_copy->to_resource = g_strdup(pres->to_resource);
  
  if(pres->from_jid != NULL)
    pres_copy->from_jid = g_strdup(pres->from_jid);
  
  if(pres->from_resource != NULL)
    pres_copy->from_resource = g_strdup(pres->from_resource);
  
  if(pres->error_code != NULL)
    pres_copy->error_code = g_strdup(pres->error_code);
  
  if(pres->error_reason != NULL)
    pres_copy->error_reason = g_strdup(pres->error_reason);
  
  return pres_copy;
}

presence presenceFindByToJID(gchar *jid)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("presenceFindByJID: JID was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(presenceList);index++)
    {
      presence pres  = g_list_nth_data(presenceList, index);

      if(pres == NULL)
	{
	  g_warning("presenceFindByJID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(strcmp(pres->to_jid,jid) == 0)
	{
	  return pres;
	}
    }

  return NULL;
}

gboolean presenceSetToJID(presence pres, gchar *jid)
{
  if(pres == NULL)
    {
      g_warning("presenceSetToJID: pres was NULL");
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("presenceSetToJID: JID was NULL");
      return FALSE;
    }

  if(pres->to_jid != NULL)
    g_free(pres->to_jid);

  pres->to_jid = g_strdup(jid);
  return TRUE;
}

gboolean presenceSetType(presence pres, presenceType type)
{
  if(pres == NULL)
    {
      g_warning("presenceSetType: pres was NULL");
      return FALSE;
    }

  if(type < 1 || type > presenceTypeEnd)
    {
      g_warning("presenceSetType: type was out of range");
      return FALSE;
    }

  pres->type = type;
  return TRUE;
}

gboolean presenceSetShow(presence pres, presenceShow show)
{
  if(pres == NULL)
    {
      g_warning("presenceSetShow: pres was NULL");
      return FALSE;
    }

  if(show < 1 || show > presenceShowEnd)
    {
      g_warning("presenceSetShow: type was out of range");
      return FALSE;
    }

  pres->show = show;
  return TRUE;
}

gboolean presenceSetStatus(presence pres, gchar *status)
{
  if(pres == NULL)
    {
      g_warning("presenceSetStatus: pres was NULL");
      return FALSE;
    }

  if(status == NULL)
    {
      g_warning("presenceSetToStatus: status was NULL");
      return FALSE;
    }

  if(pres->status != NULL)
    g_free(pres->status);

  pres->status = g_strdup(status);
  return TRUE;
}

gboolean presenceSetPriority(presence pres, guint priority)
{
  if(pres == NULL)
    {
      g_warning("presenceSetPriority: pres was NULL");
      return FALSE;
    }

  if(priority < 0)
    {
      g_warning("presenceSetPriority: priority was invalid, < 0");
      return FALSE;
    }

  pres->priority = priority;
  return TRUE;
}

gboolean presenceSetToResource(presence pres, gchar *resource)
{
  if(pres == NULL)
    {
      g_warning("presenceSetToResource: pres was NULL");
      return FALSE;
    }

  if(resource == NULL)
    {
      g_warning("presenceSetToResource: resource was NULL");
      return FALSE;
    }

  if(pres->to_resource != NULL)
    g_free(pres->to_resource);

  pres->to_resource = g_strdup(resource);
  return TRUE;
}

gboolean presenceSetFromJID(presence pres, gchar *jid)
{
  if(pres == NULL)
    {
      g_warning("presenceSetFromJID: pres was NULL");
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("presenceSetFromJID: JID was NULL");
      return FALSE;
    }

  if(pres->from_jid != NULL)
    g_free(pres->from_jid);

  pres->from_jid = g_strdup(jid);
  return TRUE;
}

gboolean presenceSetFromResource(presence pres, gchar *resource)
{
  if(pres == NULL)
    {
      g_warning("presenceSetFromResource: pres was NULL");
      return FALSE;
    }

  if(resource == NULL)
    {
      g_warning("presenceSetFromResource: resource was NULL");
      return FALSE;
    }

  if(pres->from_resource != NULL)
    g_free(pres->from_resource);

  pres->from_resource = g_strdup(resource);
  return TRUE;
}

gboolean presenceSetErrorCode(presence pres, gchar *error_code)
{
  if(pres == NULL)
    {
      g_warning("presenceSetErrorCode: pres was NULL");
      return FALSE;
    }

  if(error_code == NULL)
    {
      g_warning("presenceSetErrorCode: error code was NULL");
      return FALSE;
    }

  if(pres->error_code != NULL)
    g_free(pres->error_code);

  pres->error_code = g_strdup(error_code);
  return TRUE;
}

gboolean presenceSetErrorReason(presence pres, gchar *error_reason)
{
  if(pres == NULL)
    {
      g_warning("presenceSetErrorReason: pres was NULL");
      return FALSE;
    }

  if(error_reason == NULL)
    {
      g_warning("presenceSetErrorReason: error reason was NULL");
      return FALSE;
    }

  if(pres->error_reason != NULL)
    g_free(pres->error_reason);

  pres->error_reason = g_strdup(error_reason);
  return TRUE;
}

presenceType presenceGetType(presence pres)
{ 
  if(pres == NULL)
    {
      g_warning("presenceGetType: pres was NULL");
      return -1;
    }  

  return pres->type;
}

presenceShow presenceGetShow(presence pres)
{ 
  if(pres == NULL)
    {
      g_warning("presenceGetShow: pres was NULL");
      return -1;
    }  

  return pres->show;
}

gchar *presenceGetStatus(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceGetStatus: pres was NULL");
      return NULL;
    }  

  return pres->status;
}

guint presenceGetPriority(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceGetPriority: pres was NULL");
      return 0;
    }  

  return pres->priority;
}

gint presenceGetRequestID(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceGetRequestID: pres was NULL");
      return -1;
    }  

  return pres->requestID;
}

gchar *presenceGetToJID(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceGetToJID: pres was NULL");
      return NULL;
    }  

  return pres->to_jid;
}

gchar *presenceGetToResource(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceGetToResource: pres was NULL");
      return NULL;
    }  

  return pres->to_resource;
}

gchar *presenceGetFromJID(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceGetFromJID: pres was NULL");
      return NULL;
    }  

  return pres->from_jid;
}

gchar *presenceGetFromResource(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceGetFromResource: pres was NULL");
      return NULL;
    }  

  return pres->from_resource;
}

gchar *presenceGetErrorCode(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceGetErrorCode: pres was NULL");
      return NULL;
    }  

  return pres->error_code;
}

gchar *presenceGetErrorReason(presence pres)
{
  if(pres == NULL)
    {
      g_warning("presenceGetErrorReason: pres was NULL");
      return FALSE;
    }  

  return pres->error_reason;
}










/*
 * FROM presence
 */
gboolean presenceFromSetResponseProc(presenceFromResponseProc proc)
{
  if(proc == NULL)
    {
      g_warning("presenceFromSetResponseProc: the response proc was NULL");
      return FALSE;
    }

  presenceFromProc = proc;
  
  return TRUE;
}

presenceFromResponseProc presenceFromGetResponseProc()
{
  return presenceFromProc;
}

/*
 * TO presence
 */
gboolean presenceToSetResponseProc(presenceToResponseProc proc)
{
  if(proc == NULL)
    {
      g_warning("presenceToSetResponseProc: the response proc was NULL");
      return FALSE;
    }

  presenceToProc = proc;
  
  return TRUE;
}

presenceToResponseProc presenceToGetResponseProc()
{
  return presenceToProc;
}














gboolean presenceSend(presenceType type, presenceShow show, gchar *status, gint priority)
{
  gchar *xml = NULL;
  gchar *tmp = NULL;

  if(show < 1)
    return FALSE;

  xml = g_strdup_printf("<presence id='%d' ",supportGetUniqueID());
  
  if(type >= 1)
    {
      switch(type)
	{
	case presenceTypeUnAvailable:
	  xml = g_strconcat(xml, "type='unavailable'", NULL);
	  break;
	case presenceTypeAvailable:
	  xml = g_strconcat(xml, "type='available'", NULL);
	  break;
	case presenceTypeProbe:
	  xml = g_strconcat(xml, "type='probe'", NULL);
	  break;
	case presenceTypeSubscribe:
	  xml = g_strconcat(xml, "type='subscribe'", NULL);
	  break;
	case presenceTypeUnSubscribe:
	  xml = g_strconcat(xml, "type='unsubscribe'", NULL);
	  break;
	case presenceTypeSubscribed:
	  xml = g_strconcat(xml, "type='subscribed'", NULL);
	  break;
	case presenceTypeUnSubscribed:
	  xml = g_strconcat(xml, "type='unsubscribed'", NULL);
	  break;
	  
	default:
	case presenceTypeEnd:
	  break;	  
	}
    }
  
  xml = g_strconcat(xml,">",NULL);

  /* add show */ 
  switch(show)
    {
    case presenceShowAway:
      xml = g_strconcat(xml,"<show>away</show>",NULL);
      break;
    case presenceShowChat:
      xml = g_strconcat(xml,"<show>chat</show>",NULL);
      break;
    case presenceShowDND:
      xml = g_strconcat(xml,"<show>dnd</show>",NULL);
      break;
    case presenceShowXA:
      xml = g_strconcat(xml,"<show>xa</show>",NULL);
      break;
    case presenceShowEnd:
      break;

    default:
      xml = g_strconcat(xml,"<show>normal</show>",NULL);
      break;
    }

  /* add status */
  if(status != NULL)
    {
      xml = g_strconcat(xml,"<status>",NULL);
      xml = g_strconcat(xml,status,NULL);
      xml = g_strconcat(xml,"</status>",NULL);
    }

  if(priority >= 0)
    {
      tmp = g_strdup_printf("%d",priority);
      xml = g_strconcat(xml,"<priority>",NULL);
      xml = g_strconcat(xml,tmp,NULL);
      xml = g_strconcat(xml,"</priority>",NULL);

      g_free(tmp);
    }

  xml = g_strconcat(xml,"</presence>",NULL);

  parserOutputFromText(xml);
  g_free(xml);

  return TRUE;
}

gboolean presenceSendTo(gchar *jid, presenceType type, presenceShow show, gchar *status, gint priority)
{
  gchar *xml = NULL;
  gchar *tmp = NULL;

  if(show < 1)
    return FALSE;

  if(jid == NULL)
    {
      g_warning("presenceSendTo: failed, jid was NULL");
      return FALSE;
    }

  xml = g_strdup_printf("<presence to='%s' id='%d' ", jid, supportGetUniqueID());
  
  if(type >= 1)
    {
      switch(type)
	{
	case presenceTypeUnAvailable:
	  xml = g_strconcat(xml, "type='unavailable'", NULL);
	  break;
	case presenceTypeAvailable:
	  xml = g_strconcat(xml, "type='available'", NULL);
	  break;
	case presenceTypeProbe:
	  xml = g_strconcat(xml, "type='probe'", NULL);
	  break;
	case presenceTypeSubscribe:
	  xml = g_strconcat(xml, "type='subscribe'", NULL);
	  break;
	case presenceTypeUnSubscribe:
	  xml = g_strconcat(xml, "type='unsubscribe'", NULL);
	  break;
	case presenceTypeSubscribed:
	  xml = g_strconcat(xml, "type='subscribed'", NULL);
	  break;
	case presenceTypeUnSubscribed:
	  xml = g_strconcat(xml, "type='unsubscribed'", NULL);
	  break;
	  
	default:
	case presenceTypeEnd:
	  break;	  
	}
    }

  xml = g_strconcat(xml,">",NULL);


  /* add show */ 
  switch(show)
    {
    case presenceShowAway:
      xml = g_strconcat(xml,"<show>away</show>",NULL);
      break;
    case presenceShowChat:
      xml = g_strconcat(xml,"<show>chat</show>",NULL);
      break;
    case presenceShowDND:
      xml = g_strconcat(xml,"<show>dnd</show>",NULL);
      break;
    case presenceShowXA:
      xml = g_strconcat(xml,"<show>xa</show>",NULL);
      break;
    case presenceShowEnd:
      break;

    default:
      xml = g_strconcat(xml,"<show>normal</show>",NULL);
      break;
    }

  /* add status */
  if(status != NULL)
    {
      xml = g_strconcat(xml,"<status>",NULL);
      xml = g_strconcat(xml,status,NULL);
      xml = g_strconcat(xml,"</status>",NULL);
    }

  if(priority >= 0)
    {
      tmp = g_strdup_printf("%d",priority);
      xml = g_strconcat(xml,"<priority>",NULL);
      xml = g_strconcat(xml,tmp,NULL);
      xml = g_strconcat(xml,"</priority>",NULL);

      g_free(tmp);
    }

  xml = g_strconcat(xml,"</presence>",NULL);
  parserOutputFromText(xml);

  g_free(xml);

  return TRUE;
}

void presenceReceive(connection c, xmlNodePtr node)
{
  gint requestID = supportGetUniqueID();

  xmlChar *tmp = NULL;

  xmlChar *stype = NULL;
  xmlChar *sshow = NULL;
  xmlChar *spriority = NULL;

  presence pres = NULL;
  presenceFromResponseProc callback = NULL;

  /* checks */
  if(node == NULL)
    {
      g_warning("presenceReceive: failed to decipher xml, node was NULL");
      return;
    }

  /* get info */
  if((callback = presenceFromGetResponseProc()) == NULL)
    {
      g_warning("presenceReceive: failed to call callback, it was NULL");
      return;
    }

  if((pres = presenceNew(requestID, presenceTypeAvailable, presenceShowNormal)) == NULL)
    {
      g_warning("presenceReceive: could not create new presence structure");
      return;
    }

  if((stype = parserFindByAttrAndReturn(node, (guchar*)"type")) != NULL)
    {
      if(xmlStrcmp(stype, (guchar*)"available") == 0)
	presenceSetType(pres, presenceTypeAvailable);
      else if(xmlStrcmp(stype, (guchar*)"unavailable") == 0)
	presenceSetType(pres, presenceTypeUnAvailable);
      else if(xmlStrcmp(stype, (guchar*)"probe") == 0)
	presenceSetType(pres, presenceTypeProbe);
      else if(xmlStrcmp(stype, (guchar*)"subscribed") == 0)
	presenceSetType(pres, presenceTypeSubscribed);
      else if(xmlStrcmp(stype, (guchar*)"unsubscribed") == 0)
	presenceSetType(pres, presenceTypeUnSubscribed);
      else if(xmlStrcmp(stype, (guchar*)"subscribe") == 0)
	presenceSetType(pres, presenceTypeSubscribe);
      else if(xmlStrcmp(stype, (guchar*)"unsubscribe") == 0)
	presenceSetType(pres, presenceTypeUnSubscribe);
      else if(xmlStrcmp(stype, (guchar*)"error") == 0)
	presenceSetType(pres, presenceTypeError);
    }

  if((sshow = parserFindByNodeAndReturn(node, (guchar*)"show")) != NULL)
    {
      if(xmlStrcmp(sshow, (guchar*)"normal") == 0)
	presenceSetShow(pres, presenceShowNormal);
      else if(xmlStrcmp(sshow, (guchar*)"away") == 0)
	presenceSetShow(pres, presenceShowAway);
      else if(xmlStrcmp(sshow, (guchar*)"chat") == 0)
	presenceSetShow(pres, presenceShowChat);
      else if(xmlStrcmp(sshow, (guchar*)"dnd") == 0)
	presenceSetShow(pres, presenceShowDND);
      else if(xmlStrcmp(sshow, (guchar*)"xa") == 0)
	presenceSetShow(pres, presenceShowXA);
    }
  
  if((spriority = parserFindByNodeAndReturn(node, (guchar*)"priority")) != NULL)
    {
      if(xmlStrlen(spriority) > 0) 
	presenceSetPriority(pres, atoi((gchar*)spriority));
    }
 
  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"to")) != NULL && xmlStrlen(tmp) > 0)
    {
      gchar *to = g_strdup_printf("%s",tmp);
      gchar **vTo = g_strsplit(to,"/",2);

      if(vTo != NULL)
	{
	  presenceSetToJID(pres, to);

	  if(vTo[1] != NULL)
	    presenceSetToResource(pres, vTo[1]);

	  g_strfreev(vTo);
	}

      if(to != NULL) g_free(to);
    }

  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"from")) != NULL && xmlStrlen(tmp) > 0)
    {
      gchar *from = g_strdup_printf("%s",tmp);
      gchar **vFrom = g_strsplit(from,"/",2);

      if(vFrom != NULL)
	{
	  presenceSetFromJID(pres, (gchar*)tmp);

	  if(vFrom[1] != NULL)
	    presenceSetFromResource(pres, vFrom[1]);

	  g_strfreev(vFrom);
	}

      if(from != NULL) g_free(from);
    }

  if((tmp = parserFindByNodeAndReturn(node, (guchar*)"status")) != NULL)
    presenceSetStatus(pres, (gchar*)tmp);
 
  /* if error deal with it */
  if(presenceGetType(pres) == presenceTypeError)
    {
      /* get error information */
      xmlChar *code = NULL;
      const xmlChar *reason = NULL;
      xmlNodePtr node_error = NULL;

      if((node_error = parserFindByNode(node, (guchar*)"error")) != NULL)
	{
	  if((code = parserFindByAttrAndReturn(node_error, (guchar*)"code")) != NULL && 
	     xmlStrlen(code) > 0)
	    presenceSetErrorCode(pres, (gchar*)code);
	  
	  if((reason = parserGetContent(node_error)) != NULL && xmlStrlen(reason) > 0)
	    presenceSetErrorReason(pres, (gchar*)reason);
	}
    }

  /* call callback function */
  (callback)(pres);
}



#ifdef __cplusplus
}
#endif
