/* 
 * $RCSfile: gj_presence.h,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/26 11:09:58 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifndef __gj_presence_h
#define __gj_presence_h

#include <libxml/tree.h>

#include "gj_connection.h"

/*
 * Presence
 */
typedef enum presenceType {
  presenceTypeAvailable = 1,   /* online */  
  presenceTypeUnAvailable,     /* offline */
  presenceTypeProbe,           /* ask another user's status */
  presenceTypeSubscribe,       /* request to subscribe to presence for user */
  presenceTypeUnSubscribe,     /* request to unsubscribe to presence for user */
  presenceTypeSubscribed,      /* result to subscribe */ 
  presenceTypeUnSubscribed,    /* result to unsubscribe */
  presenceTypeError,           /* error */
  
  presenceTypeEnd              /* end of types */
} presenceType;

typedef enum presenceShow {
  presenceShowAway = 1,    /* temporarily away */
  presenceShowChat,        /* similar to normal except in a conversation */
  presenceShowDND,         /* do not disturb */ 
  presenceShowNormal,      /* default */
  presenceShowXA,          /* eXtended Away, almost unavailable */
  
  presenceShowEnd          /* end of types */
} presenceShow;

typedef struct t_presence *presence;

typedef void (*presenceFromResponseProc)(presence pres);
typedef void (*presenceToResponseProc)(gint requestID);

typedef struct t_presenceTo *presenceTo; 

gboolean presenceInit();
gboolean presenceTerm();

presence presenceNew(gint requestID, presenceType type, presenceShow show);
void presenceFree(presence pres);
presence presenceCopy(presence pres);

/*
 * presence attributes access
 */
presence presenceFindByToJID(gchar *jid);

gboolean presenceSetType(presence pres, presenceType type);
gboolean presenceSetShow(presence pres, presenceShow show);

gboolean presenceSetStatus(presence pres, gchar *status);
gboolean presenceSetPriority(presence pres, guint priority);

gboolean presenceSetToJID(presence pres, gchar *jid);
gboolean presenceSetToResource(presence pres, gchar *resource);

gboolean presenceSetFromJID(presence pres, gchar *jid);
gboolean presenceSetFromResource(presence pres, gchar *resource);

gboolean presenceSetErrorCode(presence pres, gchar *error_code);
gboolean presenceSetErrorReason(presence pres, gchar *error_reason);


gint presenceGetRequestID(presence pres);

presenceType presenceGetType(presence pres);
presenceShow presenceGetShow(presence pres);

gchar *presenceGetStatus(presence pres);
guint presenceGetPriority(presence pres);

gchar *presenceGetToJID(presence pres);
gchar *presenceGetToResource(presence pres);

gchar *presenceGetFromJID(presence pres);
gchar *presenceGetFromResource(presence pres);

gchar *presenceGetErrorCode(presence pres);
gchar *presenceGetErrorReason(presence pres);

/*
 * entry into module... sending/receiving presence.
 */
gboolean presenceSendTo(gchar *jid, presenceType type,presenceShow show, gchar *status, gint priority);
gboolean presenceSend(presenceType type,presenceShow show, gchar *status, gint priority);
void presenceReceive(connection c, xmlNodePtr node);

/* to details, i.e. my state */
gboolean presenceToSetResponseProc(presenceToResponseProc proc);

/* from details, i.e. my contacts state(s) */
gboolean presenceFromSetResponseProc(presenceFromResponseProc proc);

#endif

