/* 
 * $RCSfile: gj_registration.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/28 13:38:30 $
 * 
 * $Revision: 1.4 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <glib.h> 

#include <libxml/tree.h> 
#include <libxml/parser.h> 

#include "gj_registration.h"
#include "gj_parser.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_iq_requests.h"

/* registration item */
void registrationDestroy(registration rgr);

/* registration list */
gboolean registrationListAdd(registration rgr);
gboolean registrationListDel(registration rgr);

/* parsing functions */
void registrationResponse(guint eventID);

/* parsing functions */
registration registrationResponseParse(xmlNodePtr parent);


struct t_registration
{
  gchar *id;
  gchar *to;
  gchar *from;

  gchar *instructions;

  gchar *username;
  gchar *password;
  gchar *email;

  gboolean require_username;
  gboolean require_password;
  gboolean require_email;
  gboolean require_nickname;

  gchar *key;

  registrationCallback cb; 
};

GList *registrationList = NULL;
registrationRequestCallback registrationCB = NULL;

gboolean registrationInit()
{
  if(registrationList != NULL)
    registrationTerm();

  /* create mem  - nothing to do, created on g_list_append */
  return TRUE;
}

gboolean registrationTerm()
{
  gint index = 0;

  /* list */
  if(registrationList == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(registrationList);index++)
    {
      registration rgr = NULL;
      
      if((rgr = (registration) g_list_nth_data(registrationList,index)) != NULL)
	registrationFree(rgr);
    }

  g_list_free(registrationList);
  registrationList = NULL;

  return TRUE;
}

gboolean registrationListAdd(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationListAdd: rgr was NULL");
      return FALSE;
    }

  registrationList = g_list_append(registrationList, rgr);

  return TRUE;
}

gboolean registrationListDel(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationListDel: rgr was NULL");
      return FALSE;
    }

  registrationList = g_list_remove(registrationList, rgr);
  return TRUE;
}

GList *registrationListGet()
{
  return registrationList;
}

/* 
 * item functions
 */
registration registrationNew()
{
  registration rgr;

  if((rgr = g_new(struct t_registration,1)))
    {
      rgr->id = NULL;
      rgr->to = NULL;
      rgr->from = NULL;
      
      rgr->instructions = NULL;

      rgr->username = NULL;
      rgr->password = NULL;
      rgr->email = NULL;

      rgr->require_username = FALSE;
      rgr->require_password = FALSE;
      rgr->require_email = FALSE;
      rgr->require_nickname = FALSE;

      rgr->key = NULL;
      
      rgr->cb = NULL;

      registrationListAdd(rgr);
    }
  
  return rgr;  
}

void registrationFree(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationFree: rgr was NULL");
      return;
    }

  /* free mem */
  registrationListDel(rgr);  

  /* destroy members */
  registrationDestroy(rgr);

  /* free structure */
  g_free(rgr);
}

void registrationDestroy(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationDestroy: rgr was NULL");
      return;
    }
  
  if(rgr->id != NULL)
    {
      g_free(rgr->id);
      rgr->id = NULL;
    }

  if(rgr->to != NULL)
    {
      g_free(rgr->to);
      rgr->to = NULL;
    }

  if(rgr->from != NULL)
    {
      g_free(rgr->from);
      rgr->from = NULL;
    }

  if(rgr->instructions != NULL)
    {
      g_free(rgr->instructions);
      rgr->instructions = NULL;
    }

  if(rgr->username != NULL)
    {
      g_free(rgr->username);
      rgr->username = NULL;
    }

  if(rgr->password != NULL)
    {
      g_free(rgr->password);
      rgr->password = NULL;
    }

  if(rgr->email != NULL)
    {
      g_free(rgr->email);
      rgr->email = NULL;
    }

  rgr->require_username = FALSE;
  rgr->require_password = FALSE;
  rgr->require_email = FALSE;
  rgr->require_nickname = FALSE;
  
  if(rgr->key != NULL)
    {
      g_free(rgr->key);
      rgr->key = NULL;
    }

  rgr->cb = NULL;
}

/*
 * find 
 */
registration registrationFindByID(gchar *id)
{
  gint index = 0;

  if(id == NULL)
    {
      g_warning("registrationFindByID: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(registrationList);index++)
    {
      registration rgr  = g_list_nth_data(registrationList,
				       index);

      if(rgr == NULL)
	{
	  g_warning("registrationFindByID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(rgr->id, id) == 0)
	return rgr;
    }

  return NULL;
}

registration registrationFindByTo(gchar *to)
{
  gint index = 0;

  if(to == NULL)
    {
      g_warning("registrationFindByTo: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(registrationList);index++)
    {
      registration rgr  = g_list_nth_data(registrationList,
				       index);

      if(rgr == NULL)
	{
	  g_warning("registrationFindByTo: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(rgr->to, to) == 0)
	return rgr;
    }

  return NULL;
}

registration registrationFindByFrom(gchar *from)
{
  gint index = 0;

  if(from == NULL)
    {
      g_warning("registrationFindByFrom: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(registrationList);index++)
    {
      registration rgr  = g_list_nth_data(registrationList,
				       index);

      if(rgr == NULL)
	{
	  g_warning("registrationFindByFrom: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(rgr->from, from) == 0)
	return rgr;
    }

  return NULL;
}

/*
 * member functions  
 */

/* sets */
gboolean registrationSetID(registration rgr, gchar *id)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetID: rgr was NULL");
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("registrationSetID: id was NULL");
      return FALSE;
    }

  if(rgr->id != NULL)
    g_free(rgr->id);
  
  rgr->id = g_strdup(id);

  return TRUE;
}

gboolean registrationSetTo(registration rgr, gchar *to)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetTo: rgr was NULL");
      return FALSE;
    }

  if(to == NULL)
    {
      g_warning("registrationSetTo: to was NULL");
      return FALSE;
    }

  if(rgr->to != NULL)
    g_free(rgr->to);
  
  rgr->to = g_strdup(to);

  return TRUE;
}

gboolean registrationSetFrom(registration rgr, gchar *from)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetFrom: rgr was NULL");
      return FALSE;
    }

  if(from == NULL)
    {
      g_warning("registrationSetFrom: from was NULL");
      return FALSE;
    }

  if(rgr->from != NULL)
    g_free(rgr->from);
  
  rgr->from = g_strdup(from);

  return TRUE;
}

gboolean registrationSetInstructions(registration rgr, gchar *instructions)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetInstructions: rgr was NULL");
      return FALSE;
    }

  if(instructions == NULL)
    {
      g_warning("registrationSetInstructions: id was NULL");
      return FALSE;
    }

  if(rgr->instructions != NULL)
    g_free(rgr->instructions);
  
  rgr->instructions = g_strdup(instructions);

  return TRUE;
}

gboolean registrationSetUsername(registration rgr, gchar *username)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetUsername: rgr was NULL");
      return FALSE;
    }

  if(username == NULL)
    {
      g_warning("registrationSetUsername: id was NULL");
      return FALSE;
    }

  if(rgr->username != NULL)
    g_free(rgr->username);
  
  rgr->username = g_strdup(username);

  return TRUE;
}


gboolean registrationSetPassword(registration rgr, gchar *password)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetPassword: rgr was NULL");
      return FALSE;
    }

  if(password == NULL)
    {
      g_warning("registrationSetPassword: id was NULL");
      return FALSE;
    }

  if(rgr->password != NULL)
    g_free(rgr->password);
  
  rgr->password = g_strdup(password);

  return TRUE;
}

gboolean registrationSetEmail(registration rgr, gchar *email)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetEmail: rgr was NULL");
      return FALSE;
    }

  if(email == NULL)
    {
      g_warning("registrationSetEmail: id was NULL");
      return FALSE;
    }

  if(rgr->email != NULL)
    g_free(rgr->email);
  
  rgr->email = g_strdup(email);

  return TRUE;
}

gboolean registrationSetRequireUsername(registration rgr, gboolean required)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetRequireUsername: rgr was NULL");
      return FALSE;
    }
  
  rgr->require_username = required;
  return TRUE;
}

gboolean registrationSetRequirePassword(registration rgr, gboolean required)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetRequirePassword: rgr was NULL");
      return FALSE;
    }
  
  rgr->require_password = required;
  return TRUE;
}

gboolean registrationSetRequireEmail(registration rgr, gboolean required)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetRequireEmail: rgr was NULL");
      return FALSE;
    }
  
  rgr->require_email = required;
  return TRUE;
}

gboolean registrationSetRequireNickname(registration rgr, gboolean required)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetRequireNickname: rgr was NULL");
      return FALSE;
    }
  
  rgr->require_nickname = required;
  return TRUE;
}

gboolean registrationSetKey(registration rgr, gchar *key)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetKey: rgr was NULL");
      return FALSE;
    }

  if(key == NULL)
    {
      g_warning("registrationSetKey: id was NULL");
      return FALSE;
    }

  if(rgr->key != NULL)
    g_free(rgr->key);
  
  rgr->key = g_strdup(key);

  return TRUE;
}

gboolean registrationSetCallback(registration rgr, registrationCallback cb)
{
  if(rgr == NULL)
    {
      g_warning("registrationSetCallback: rgr was NULL");
      return FALSE;
    }
  
  if(cb == NULL)
    {
      g_warning("registrationSetFields: callback was NULL");
      return FALSE;
    }

  rgr->cb = cb;
  return TRUE;
}

/* gets */
gchar *registrationGetID(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetID: rgr was NULL");
      return NULL;
    }

  return rgr->id;
}

gchar *registrationGetTo(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetTo: rgr was NULL");
      return NULL;
    }

  return rgr->to;
}

gchar *registrationGetFrom(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetFrom: rgr was NULL");
      return NULL;
    }

  return rgr->from;
}

gchar *registrationGetInstructions(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetInstructions: rgr was NULL");
      return NULL;
    }

  return rgr->instructions;
}

/* gchar *registrationGetNickname(registration rgr) */
/* { */
/*   if(rgr == NULL) */
/*     { */
/*       g_warning("registrationGetNickname: rgr was NULL"); */
/*       return NULL; */
/*     } */

/*   return rgr->nickname; */
/* } */

gchar *registrationGetPassword(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetPassword: rgr was NULL");
      return NULL;
    }

  return rgr->password;
}

gchar *registrationGetEmail(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetEmail: rgr was NULL");
      return NULL;
    }

  return rgr->email;
}

gboolean registrationGetRequireUsername(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetUsername: rgr was NULL");
      return FALSE;
    }

  return rgr->require_username;
}

gboolean registrationGetRequirePassword(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetPassword: rgr was NULL");
      return FALSE;
    }

  return rgr->require_password;
}

gboolean registrationGetRequireEmail(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetEmail: rgr was NULL");
      return FALSE;
    }

  return rgr->require_email;
}

gboolean registrationGetRequireNickname(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetNickname: rgr was NULL");
      return FALSE;
    }

  return rgr->require_nickname;
}

gchar *registrationGetKey(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetKey: rgr was NULL");
      return NULL;
    }

  return rgr->key;
}

registrationCallback registrationGetCallback(registration rgr)
{
  if(rgr == NULL)
    {
      g_warning("registrationGetCallback: rgr was NULL");
      return NULL;
    }

  return rgr->cb;
}

/*
 *
 * SERVICES
 *
 */
gboolean registrationRequestRequirements(gchar *server, registrationRequestCallback cb)
{
  if(server == NULL)
    {
      g_warning("registrationRequestRequirements: server was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("registrationRequestRequirements: cb was NULL");
      return FALSE;
    }

  registrationCB = cb;

  /* do request */
  infoqueryRequestRegistrationRequirements(server, registrationResponse);

  return TRUE;
}

gboolean registrationRequest(gchar *server, registrationRequestCallback cb, 
			     gchar *key, gchar *username, gchar *password, gchar *email, gchar *nickname)
{
  if(server == NULL)
    {
      g_warning("registrationRequestRequirements: server was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("registrationRequestRequirements: cb was NULL");
      return FALSE;
    }

  registrationCB = cb;

  /* do request */
  infoqueryRequestRegistration(server, registrationResponse, 
			       "key", key?key:"",
			       "username", username?username:"",
			       "password", password?password:"",
			       "email", email?email:"",
			       "nickname", nickname?nickname:"",
			       NULL);

  return TRUE;
}

void registrationResponse(guint eventID)
{
  infoquery iq = NULL;
  registration rgr = NULL;

  gint error_code = 0;
  gchar *error_reason = NULL;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  if(registrationCB == NULL)
    {
      g_warning("registrationResponse: callback was NULL");
      return;
    }

  if((iq = infoqueryFindByEventIDRelated(eventID)) == NULL)
    {
      g_warning("registrationResponse: could not get infoquery event id:%d", eventID);
      return;
    }

  node = infoqueryGetXML(iq);
  type = parserFindByAttrAndReturn(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      gchar *detail = NULL;

      xmlChar *errorCode = NULL;
      xmlChar *errorMessage = NULL;

      if((node = parserFindByNode(node, (guchar*)"error")) == NULL)
	{
	  g_warning("registrationResponse: failed to find node 'error'");
	  infoqueryFreeByEventID(eventID);
	  return;
	}

      errorCode = parserFindByAttrAndReturn(node, (guchar*)"code");
      errorMessage = parserFindByNodeAndReturn(node, (guchar*)"error");

      error_code = atoi(errorCode);
      error_reason = g_strdup(errorMessage);
    }

  /* parse list */
  if((rgr = registrationResponseParse(node)) == NULL)
    {
      g_warning("registrationResponse: did not complete the parse from xml -> rgr");
      return;
    }

  /* return list */
  (registrationCB)(rgr, error_code, error_reason);

  /* clean up */
  infoqueryFreeByEventID(eventID);
  if(error_reason != NULL) g_free(error_reason);
}

registration registrationResponseParse(xmlNodePtr parent)
{
  xmlChar *tmp = NULL;
  registration rgr = NULL;

  if(parent == NULL)
    {
      g_warning("registrationResponseParse: failed to get registration because query (xml) was NULL");
      return NULL;
    }

  /* create new roster item */
  if((rgr = registrationNew()) == NULL)
    {
      g_warning("registrationResponseParse: could not create rgr");
      return NULL;
    }

  /* get important information */
  if((tmp = parserFindByAttrAndReturn(parent, (guchar*)"id")) != NULL)
    registrationSetID(rgr, (gchar*)tmp);
  if((tmp = parserFindByAttrAndReturn(parent, (guchar*)"to")) != NULL)
    registrationSetTo(rgr, (gchar*)tmp);
  if((tmp = parserFindByAttrAndReturn(parent, (guchar*)"from")) != NULL)
    registrationSetFrom(rgr, (gchar*)tmp);
  
  /* get instructions */
  if((tmp = parserFindByNodeAndReturn(parent, (guchar*)"instructions")) != NULL)
    registrationSetInstructions(rgr, (gchar*)tmp);

#if 0
  /* get user/pass... */
  if((tmp = parserFindByNodeAndReturn(parent, (guchar*)"username")) != NULL)
    registrationSetUsername(rgr, (gchar*)tmp);
  if((tmp = parserFindByNodeAndReturn(parent, (guchar*)"password")) != NULL)
    registrationSetPassword(rgr, (gchar*)tmp);
  if((tmp = parserFindByNodeAndReturn(parent, (guchar*)"email")) != NULL)
    registrationSetEmail(rgr, (gchar*)tmp);
#endif

  /* get user/pass... */
  if(parserFindByNode(parent, (guchar*)"username") != NULL)
    registrationSetRequireUsername(rgr, TRUE);
  if(parserFindByNode(parent, (guchar*)"password") != NULL)
    registrationSetRequirePassword(rgr, TRUE);
  if(parserFindByNode(parent, (guchar*)"email") != NULL)
    registrationSetRequireEmail(rgr, TRUE);
  if(parserFindByNode(parent, (guchar*)"nickname") != NULL)
    registrationSetRequireNickname(rgr, TRUE);

  if((tmp = parserFindByNodeAndReturn(parent, (guchar*)"key")) != NULL)
    registrationSetKey(rgr, (gchar*)tmp);


  return rgr;
}

#ifdef __cplusplus
}
#endif
