/* 
 * $RCSfile: gj_roster.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/07/02 09:58:19 $
 * 
 * $Revision: 1.9 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>

#include <time.h> 

#include <glib.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include "gj_roster.h"
#include "gj_parser.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_event.h"
#include "gj_history.h"
#include "gj_translate.h"

#include "gj_gtk_roster.h" /* this is for the roster_model_add_contact call... 
			      and i dont think this should be done here...*/

/* roster item */
void rosterItemDestroy(rosterItem ri);

gboolean rosterItemSetID(rosterItem ri, gint id);
gboolean rosterItemSetIsPermanent(rosterItem ri, gboolean IsPermanent);
gboolean rosterItemSetIsSetForRemoval(rosterItem ri, gboolean IsSetForRemoval);

/* roster list */
gboolean rosterAdd(rosterItem ri);
gboolean rosterDel(rosterItem ri);

/* call backs */
void rosterResponse(gint requestID);

gboolean rosterSetResponseProc(rosterResponseProc proc);
rosterResponseProc rosterGetResponseProc();

struct t_rosterItem
{
  gint id; /* unique id */

  gchar *jid;
  gchar *name;
  gchar *resource;
  gchar *agent;

  GList *groups;

  gboolean is_agent;            /* old - was used before type */
  gboolean is_permanent;        /* is this a permanant member - i.e. not just someone that has sent me a message */
  gboolean is_set_for_removal;  /* is this roster item about to be deleted and shouldnt be added to the roster */

  rosterItemType type;
  rosterItemSubscription subscription;

  /* presence information */
  presence pres;
};

GList *rosterList;

rosterResponseProc rosterProc;

gboolean rosterInit()
{
  if(rosterList != NULL)
    rosterTerm();

  /* free roster response proc */
  rosterProc = NULL;

  /* create mem  - nothing to do, created on g_list_append */
  return TRUE;
}

gboolean rosterTerm()
{
  gint index = 0;

  /* list */
  if(rosterList == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(rosterList);index++)
    {
      rosterItem ri = NULL;
      
      if((ri = (rosterItem) g_list_nth_data(rosterList,index)) != NULL)
	rosterItemFree(ri);
    }

  g_list_free(rosterList);
  rosterList = NULL;

  /* free roster response proc */
  rosterProc = NULL;

  return TRUE;
}

gboolean rosterAdd(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterAdd: failed to add item to list, it was NULL");
      return FALSE;
    }

  rosterList = g_list_append(rosterList,ri);

  return TRUE;
}

gboolean rosterDel(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterDel: failed to del item from list, it was NULL");
      return FALSE;
    }

  rosterList = g_list_remove(rosterList,ri);
  return TRUE;
}

GList *rosterGet()
{
  return rosterList;
}

GList *rosterGetAllGroups()
{
  GList *group_master_list = NULL;
  gint i,j,k;
  gboolean found = FALSE;

  if(rosterList == NULL)
    {
      g_warning("rosterGetAllGroups: roster list was NULL");
      return NULL;
    }

  /* look through roster items */
  for(i=0;i<g_list_length(rosterList);i++)
    {
      rosterItem ri  = g_list_nth_data(rosterList, i);

      if(ri == NULL)
	continue;

      if(ri->groups == NULL)
	continue;
       
      /* look through groups that the user belongs to */
      for(k=0;k<g_list_length(ri->groups);k++)
	{
	  gchar *group = (gchar *) g_list_nth_data(ri->groups, k);

	  if(group == NULL)
	    continue;

	  /* search master_list for same group name */
	  for(j=0,found=FALSE;j<g_list_length(group_master_list);j++)
	    {
	      gchar *name = (gchar *) g_list_nth_data(group_master_list, j);
	      
	      /* checks */
	      if(name == NULL)
		continue;
	  
	      if(strcmp(group, name) == 0)
		{
		  found = TRUE;
		  break;
		}	  
	    }

	  /* if after searching master_list so far
	     it doesnt exist, we can then add this group */
	  if(found == FALSE)
	    {
	      group_master_list = g_list_append(group_master_list,g_strdup(group)); 
	      g_message("rosterGetAllGroups: adding to list, group:'%s'",group);
	    }
	}
    }  

  return group_master_list;
}

GList *rosterGetAllItemsByGroup(gchar *group)
{
  GList *list = NULL;
  gint i,j,k;
  gboolean found = FALSE;

  /* note: if NULL is specified as a group, 
           it will look for all items without a group */

  if(rosterList == NULL)
    {
      g_warning("rosterGetAllItemsByGroup: roster list was NULL");
      return NULL;
    }

  /* look through roster items */
  for(i=0;i<g_list_length(rosterList);i++)
    {
      rosterItem ri = (rosterItem) g_list_nth_data(rosterList, i);

      if(ri == NULL)
	continue;

      if(ri->groups == NULL && group == NULL)
	{
	  list = g_list_append(list, g_strdup(rosterItemGetJID(ri)));
	  continue;
	}
	
      /* look through groups that the user belongs to */
      for(k=0; k<g_list_length(ri->groups); k++)
	{
	  gchar *name = (gchar*) g_list_nth_data(ri->groups, k);

	  if(name == NULL || strcmp(name, group) != 0)
	    continue;

	  /* search our new list for same group name */
	  for(j=0,found=FALSE; j<g_list_length(list); j++)
	    {
	      gchar *item = (gchar*) g_list_nth_data(list, j);
	      
	      /* checks */
	      if(item == NULL)
		continue;
	  
	      if(strcmp(rosterItemGetJID(ri), item) == 0)
		{
		  found = TRUE;
		  break;
		}	  
	    }

	  /* if after searching master_list so far
	     it doesnt exist, we can then add this group */
	  if(found == FALSE)
	    {
	      list = g_list_append(list, g_strdup(rosterItemGetJID(ri))); 
	      g_message("rosterGetAllItemsByGroup: adding to list, jid:'%s'", rosterItemGetJID(ri));
	    }
	}
    }  

  return list;
}

GList *rosterGetAllItemsByAgent(gchar *agent)
{
  GList *list = NULL;
  gint i,j;

  if(rosterList == NULL)
    {
      g_warning("rosterGetAllItemsByAgent: roster list was NULL");
      return NULL;
    }

  if(agent == NULL || strlen(agent) < 1)
    {
      g_warning("rosterGetAllItemsByAgent: agent was NULL or length was < 1");
      return NULL;
    }

  /* look through roster items */
  for(i=0;i<g_list_length(rosterList);i++)
    {
      rosterItem ri = (rosterItem) g_list_nth_data(rosterList, i);

      if(ri == NULL)
	continue;

      if(rosterItemGetAgent(ri) == NULL)
	continue;
	
      if(strcmp(rosterItemGetAgent(ri), agent) == 0)
	{
	  list = g_list_append(list, g_strdup(rosterItemGetJID(ri))); 
	  g_message("rosterGetAllItemsByAgent: adding to list, jid:'%s'", rosterItemGetJID(ri));
	}
    }  

  return list;
}


GList *rosterGetAllAgentsByJID()
{
  GList *list = NULL;
  gint i = 0;

  if(rosterList == NULL)
    {
      g_warning("rosterGetAllAgentsByJID: roster list was NULL");
      return NULL;
    }

  /* look through roster items */
  for(i=0;i<g_list_length(rosterList);i++)
    {
      rosterItem ri = (rosterItem) g_list_nth_data(rosterList, i);

      if(ri == NULL)
	continue;

      if(rosterItemGetIsAgent(ri) == FALSE)
	continue;
       
      list = g_list_append(list, g_strdup(rosterItemGetJID(ri))); 
      g_message("rosterGetAllAgentsByJID: adding to list, jid:'%s'", rosterItemGetJID(ri));
    }  

  return list;
}

GList *rosterGetAllAgentsByJIDWithoutResource()
{
  GList *list = NULL;
  gint i = 0;

  if(rosterList == NULL)
    {
      g_warning("rosterGetAllAgentsByJIDWithoutResource: roster list was NULL");
      return NULL;
    }

  /* look through roster items */
  for(i=0;i<g_list_length(rosterList);i++)
    {
      rosterItem ri  = g_list_nth_data(rosterList, i);
      gchar *jid = NULL;
      gchar **vjid = NULL;

      if(ri == NULL)
	continue;

      if(rosterItemGetIsAgent(ri) == FALSE)
	continue;
       
      vjid = g_strsplit(rosterItemGetJID(ri), "/", -1);
      
      if(vjid == NULL)
	  continue;

      if(vjid[0] != NULL)
	  jid = g_strdup(vjid[0]);
      
      g_strfreev(vjid);

      if(jid == NULL)
	  continue;

      list = g_list_append(list, g_strdup(jid)); 
      g_message("rosterGetAllAgentsByJIDWithoutResource: adding to list, jid:'%s'", jid);
    }  

  return list;
}

GList *rosterGetAllAgentsByName()
{
  GList *list = NULL;
  gint i = 0;

  if(rosterList == NULL)
    {
      g_warning("rosterGetAllAgentsByName: roster list was NULL");
      return NULL;
    }

  /* look through roster items */
  for(i=0;i<g_list_length(rosterList);i++)
    {
      rosterItem ri = (rosterItem) g_list_nth_data(rosterList, i);

      if(ri == NULL)
	continue;

      if(rosterItemGetIsAgent(ri) == FALSE)
	continue;
       
      list = g_list_append(list, g_strdup(rosterItemGetName(ri))); 
      g_message("rosterGetAllAgentsByName: adding to list, name:'%s'", rosterItemGetName(ri));
    }  

  return list;
}


gboolean rosterIsAgentRegistered(gchar *agent)
{
  GList *list = NULL;
  gint i = 0;
  gboolean found = FALSE;

  if(agent == NULL)
    {
      g_warning("rosterIsAgentRegistered: agent was NULL");
      return FALSE;
    }

  if((list = rosterGetAllAgentsByJIDWithoutResource()) == NULL)
    {
      g_warning("rosterIsAgentRegistered: list was NULL");
      return FALSE;
    }

  /* look through roster items */
  for(i=0;found == FALSE && i<g_list_length(list);i++)
    {
      gchar *jid = (gchar*) g_list_nth_data(list, i);

      if(jid == NULL)
	continue;

      if(g_strcasecmp(jid, agent) == 0)
	found = TRUE;
    }  
  
  return found;
}

/* 
 * item functions
 */
rosterItem rosterItemNew(gchar *jid)
{
  rosterItem ri;

  if(jid == NULL)
    {
      g_warning("rosterItemNew: failed to create new roster item, jid was NULL");
    }
  
  if((ri = g_new(struct t_rosterItem,1)))
    {
      ri->id = supportGetUniqueID();

      ri->jid = g_strdup(jid);
      ri->name = NULL;
      ri->resource = NULL;
      ri->agent = NULL;

      ri->groups = NULL;

      ri->is_agent = FALSE;
      ri->is_permanent = FALSE;
      ri->is_set_for_removal = FALSE;

      ri->subscription = rosterItemSubscriptionNone;
      ri->type = rosterItemTypeContact;

      ri->pres = presenceNew(supportGetUniqueID(), presenceTypeUnAvailable, presenceShowNormal);

      rosterAdd(ri);
    }
  
  return ri;  
}

rosterItem rosterItemNewUnknownContactByMessage(message msg)
{
  rosterItem ri;

  gchar *from = NULL;
  gchar *jid_no_resource = NULL;
  gchar *resource = NULL;
  gchar **vjid = NULL;
  gchar **vagent = NULL;

  if(msg == NULL)
    {
      g_warning("rosterItemNewUnknownContactByMessage: msg was NULL");
      return NULL;
    }
  
  if((from = messageGetFromJID(msg)) == NULL)
    {
      g_warning("rosterItemNewUnknownContactByMessage: jid from message was NULL");
      return NULL;
    }

  vjid = g_strsplit(from,"/",2);
  
  if(vjid != NULL)
    {
      jid_no_resource = g_strdup(from);
      
      if(vjid[1] != NULL)
	resource = g_strdup(vjid[1]);
      
      g_strfreev(vjid);
    }
  
  /* create new roster item */
  if((ri = rosterItemNew(jid_no_resource)) == NULL)
    {
      g_warning("rosterItemNewUnknownContactByMessage: failed to create new roster item");

      if(jid_no_resource != NULL) g_free(jid_no_resource);
      if(resource != NULL) g_free(resource);

      return NULL;
    }
      
  vagent = g_strsplit(jid_no_resource,"@",2);
  
  if(vagent != NULL)
  {
    if(vagent[1] != NULL)
      {
	/* not a agent, so get the agent part (i.e @icq.jabber.org) */
	ri->agent = g_strdup(vagent[1]);
	ri->is_agent = FALSE;
	ri->type = rosterItemTypeContact;
      }
    else
      {
	/* no agent part (i.e. @msn.jabber.org) there for MUST be a agent */
	ri->is_agent = TRUE;
	ri->type = rosterItemTypeAgent;
      }

    g_strfreev(vagent);
  }
  
  /* name */
  if(jid_no_resource != NULL)
    ri->name = g_strdup(jid_no_resource);

  /* resource */
  if(resource != NULL)
    ri->resource = resource;

  /* clean up */
  if(jid_no_resource != NULL) g_free(jid_no_resource);

  return ri;  
}

rosterItem rosterItemNewUnknownByJID(gchar *from)
{
  rosterItem ri;

  gchar *jid_no_resource = NULL;
  gchar *resource = NULL;
  gchar **vjid = NULL;
  gchar **vagent = NULL;

  if(from == NULL || strlen(from) < 1)
    {
      g_warning("rosterItemNewUnknownByJID: jid was NULL");
      return NULL;
    }

  vjid = g_strsplit(from,"/",2);
  
  if(vjid != NULL)
    {
      jid_no_resource = g_strdup(from);
      
      if(vjid[1] != NULL)
	resource = g_strdup(vjid[1]);
      
      g_strfreev(vjid);
    }
  
  /* create new roster item */
  if((ri = rosterItemNew(jid_no_resource)) == NULL)
    {
      g_warning("rosterItemNewUnknownByJID: failed to create new roster item");

      if(jid_no_resource != NULL) g_free(jid_no_resource);
      if(resource != NULL) g_free(resource);

      return NULL;
    }
      
  vagent = g_strsplit(jid_no_resource,"@",2);
  
  if(vagent != NULL)
  {
    if(vagent[1] != NULL)
      {
	/* not a agent, so get the agent part (i.e @icq.jabber.org) */
	ri->agent = g_strdup(vagent[1]);
	ri->is_agent = FALSE;
	ri->type = rosterItemTypeContact;
      }
    else
      {
	/* no agent part (i.e. @msn.jabber.org) there for MUST be a agent */
	ri->is_agent = TRUE;
	ri->type = rosterItemTypeAgent;
      }

    g_strfreev(vagent);
  }
  
  /* name */
  if(jid_no_resource != NULL)
    ri->name = g_strdup(jid_no_resource);

  /* resource */
  if(resource != NULL)
    ri->resource = resource;

  /* clean up */
  if(jid_no_resource != NULL) g_free(jid_no_resource);

  return ri;  
}

void rosterItemFree(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemFree: failed to free, roster item was NULL");
      return;
    }

  /* free mem */
  rosterDel(ri);  

  /* destroy members */
  rosterItemDestroy(ri);

  /* free structure */
  g_free(ri);
}

void rosterItemDestroy(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterDestroy: failed to destroy, roster item was NULL");
      return;
    }
  
  ri->id = 0;

  if(ri->jid != NULL)
    {
      g_free(ri->jid);
      ri->jid = NULL;
    }

  if(ri->name != NULL)
    {
      g_free(ri->name);
      ri->name = NULL;
    }

  if(ri->resource != NULL)
    {
      g_free(ri->resource);
      ri->resource = NULL;
    }

  if(ri->agent != NULL)
    {
      g_free(ri->agent);
      ri->agent = NULL;
    }

  if(ri->groups != NULL)
    rosterItemGroupsFree(ri);

  ri->subscription = 0;

  /* presence */
  if(ri->pres != NULL)
    {
      presenceFree(ri->pres);
      ri->pres = NULL;
    }
}

/* 
 * item groups 
 */
gboolean rosterItemGroupsFree(rosterItem ri)
{
  gint i;

  if(ri == NULL)
    {
      g_warning("rosterItemGroupsFree: roster item was NULL");
      return FALSE;
    }

  if(ri->groups == NULL)
    return TRUE;

  /* reverse delete through the list */
  for(i=g_list_length(ri->groups);i>0;i--)
    {
      gchar *group = (gchar *) g_list_nth_data(ri->groups, i);
      
      if(group == NULL)
	continue;
      
      /* free off list and str */
      g_list_remove(ri->groups, group);
      g_free(group);
      group = NULL;
    }

  ri->groups = NULL;

  return TRUE;
}

gboolean rosterItemGroupsAdd(rosterItem ri, gchar *group)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGroupsAdd: roster item was NULL");
      return FALSE;
    }  

  if(group == NULL)
    {
      g_warning("rosterItemGroupsAdd: group was NULL");
      return FALSE;
    }
  
  /* make our copy of it */
  ri->groups = g_list_append(ri->groups, g_strdup(group)); 

  return TRUE;
}

gboolean rosterItemGroupsDelete(rosterItem ri, gchar *group)
{
  GList *item = NULL;
  gchar *item_str = NULL;
  gint index = 0;

  if(ri == NULL)
    {
      g_warning("rosterItemGroupsDelete: roster item was NULL");
      return FALSE;
    }  

  if(group == NULL)
    {
      g_warning("rosterItemGroupsDelete: group was NULL");
      return FALSE;
    }
  
  /* find the item */
  item = g_list_find_custom(ri->groups, group, (GCompareFunc)strcmp);
  if(item == NULL) return TRUE;

  index = g_list_position(ri->groups, item);
  if(index == -1) return TRUE;

  item_str = g_list_nth_data(ri->groups, index);
  if(item_str == NULL) return FALSE;

  /* remove it */
  ri->groups = g_list_remove(ri->groups, item_str); 

  return TRUE;
}


gint rosterItemGroupsCount(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGroupsCount: roster item was NULL");
      return -1;
    }   

  if(ri->groups == NULL)
    return -1;
  
  return g_list_length(ri->groups);
}

/*
 * roster handling
 */
GList *rosterParse(xmlNodePtr parent)
{
  gint count = 0;
  xmlNodePtr node = NULL;

  if(parent == NULL)
    {
      g_warning("rosterParse: failed to get roster because query (xml) was NULL");
      return NULL;
    }

  /* create list */
  rosterInit();

  /* get item node */
  node = parserFindByNode(parent, (guchar*)"item");

  /* get items in roster */
  for(;node != NULL;node = node->next)
    {
      xmlChar *tmp = NULL;

      count++;
      
      /* get data */
      if((tmp = parserFindByAttrAndReturn(node, (guchar*)"jid")) != NULL)
	{
	  gchar **vjid = NULL;
	  gchar **vagent = NULL;
	  
	  rosterItem ri = NULL;
	  history hist = NULL;

	  vjid = g_strsplit((gchar*)tmp,"/",2);

	  if(vjid != NULL)
	    {
	      /* create new roster item */
	      ri = rosterItemNew(tmp);

	      if(ri == NULL)
		{
		  g_warning("rosterParse: failed to create new rosterItem for jid:'%s', continuing to next item...", tmp);
		  g_strfreev(vjid);
		  continue;
		}

	      /* resource */
	      if(vjid[1] != NULL)
		rosterItemSetResource(ri, vjid[1]);
	      
	      /* set agent information */
	      if((vagent = g_strsplit(vjid[0],"@",2)) != NULL)
		{
		  if(vagent[1] != NULL)
		    {
		      /* not a agent, so get the agent part (i.e @icq.jabber.org) */
		      rosterItemSetAgent(ri, vagent[1]);
		      rosterItemSetIsAgent(ri, FALSE);
		      rosterItemSetType(ri, rosterItemTypeContact);
		    }
		  else
		    {
		      /* no agent part (i.e. @msn.jabber.org) there for MUST be a agent */
		      rosterItemSetIsAgent(ri, TRUE);
		      rosterItemSetType(ri, rosterItemTypeAgent);
		    }
		  
		  g_strfreev(vagent);
		}

	      g_strfreev(vjid);
	    }
	  
	  /* name */
	  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"name")) != NULL)
	    rosterItemSetName(ri, (gchar*)tmp);
	  
	  /* groups */ 
	  rosterItemSetGroups(ri, parserFindByNodeOnSameParentAndReturnContentsByList(node, (guchar*)"group"));

	  /* subscription type */
	  if((tmp = parserFindByAttrAndReturn(node, (guchar*)"subscription")) != NULL)
	    {
	      if(xmlStrcmp(tmp, (guchar*)"none") == 0)
		rosterItemSetSubscription(ri, rosterItemSubscriptionNone);
	      else if(xmlStrcmp(tmp, (guchar*)"to") == 0)
		rosterItemSetSubscription(ri, rosterItemSubscriptionTo);
	      else if(xmlStrcmp(tmp, (guchar*)"from") == 0)
		rosterItemSetSubscription(ri, rosterItemSubscriptionFrom);
	      else if(xmlStrcmp(tmp, (guchar*)"both") == 0)
		rosterItemSetSubscription(ri, rosterItemSubscriptionBoth);
	      else
		g_warning("rosterParse: unexpected roster item type:'%s'", tmp);
	    }
	  
	  rosterItemSetIsPermanent(ri, TRUE);

	  /* create history */
	  if((hist = historyFindByJID(rosterItemGetJID(ri))) == NULL)
	    hist = historyNew(rosterItemGetJID(ri));

	  historyStart(hist);
	}
    }

  g_message("rosterParse: items:%d",count);

  return rosterList;
}

gboolean rosterParseSetRequest(infoquery iq)
{
  xmlNodePtr node = NULL;
  xmlNodePtr parent = NULL;

  gint count = 0;

  g_message("rosterParseSetRequest: updating roster item(s)...");

  if((parent = infoqueryGetXML(iq)) == NULL)
    {
      g_warning("rosterParseSetRequest: failed to get roster because query (xml) was NULL");
      return FALSE;
    }

  /* get item node */
  node = parserFindByNode(parent, (guchar*)"item");

  /* get items in roster */
  for(;node != NULL; node = node->next)
    {
      gchar *jid = NULL;
      gchar *resource = NULL;
      gchar *agent = NULL;
      gchar *ask = NULL;
      gchar *subscription = NULL;
      gchar *name = NULL;

      gchar **vjid = NULL;
      gchar **vagent = NULL;

      gboolean is_new = TRUE;
      
      xmlChar *tmp = NULL;

      rosterItem ri = NULL;

      count++;
      
      /* get data */
      if((tmp = parserFindByAttrAndReturn(node, (guchar*)"jid")) == NULL)
	{
	  g_warning("rosterParseSetRequest: could not get jid from node");
	  continue;
	}

      /* get jid and resource */
      vjid = g_strsplit((gchar*)tmp,"/",2);
      
      if(vjid != NULL)
	{
	  jid = g_strdup((gchar*)tmp);
	  
	  if(vjid[1] != NULL)
	    resource = g_strdup(vjid[1]);

	  /* set agent information */
	  if((vagent = g_strsplit(vjid[0],"@",2)) != NULL)
	    {
	      if(vagent[1] != NULL)
		agent = g_strdup(vagent[1]);
	      
	      g_strfreev(vagent);
	    }
	  
	  g_strfreev(vjid);
	}
      
      /* get name */
      if((tmp = parserFindByAttrAndReturn(node, (guchar*)"name")) != NULL)
	name = g_strdup((gchar*)tmp);  

      /* get subscription */
      if((tmp = parserFindByAttrAndReturn(node, (guchar*)"subscription")) != NULL)
	subscription = g_strdup((gchar*)tmp);

      /* get ask */
      if((tmp = parserFindByAttrAndReturn(node, (guchar*)"ask")) != NULL)
	ask = g_strdup((gchar*)tmp);

      /* look for roster item */
      if((ri = rosterItemFindByJID(jid)) == NULL)
	{
	  g_message("rosterParseSetRequest: rosterItem not found by jid:'%s', attempting without resource...", jid);

	  if((ri = rosterItemFindByJIDWithoutResource(jid)) == NULL)
	    g_warning("rosterParseSetRequest: rosterItem could not be found at all by jid:'%s'", jid);
	}
      
      /* NOTE: we remove the contact here and add it later on when the
	 details have changed.  this way we can change the groups
	 and not have to worry about finding the contact and checking 
	 the groups and details...
      */
      
      if(subscription != NULL && g_strcasecmp(subscription, "remove") == 0)
	{
	  /*
	   * REMOVE roster item
	   */
	  if(ri == NULL)
	    {
	      g_warning("rosterParseSetRequest: could not remove jid:'%s', no roster item found.", jid);
	      
	      /* clean up */
	      if(jid != NULL) g_free(jid);
	      if(resource != NULL) g_free(resource);
	      if(agent != NULL) g_free(agent);
	      
	      continue;
	    }
	  else
	    {
	      /* let people know it is about to be deleted */
	      rosterItemSetIsSetForRemoval(ri, TRUE);

	      /* remove it from roster */
	      gj_gtk_roster_model_remove_contact(ri);

	      /* free it */
	      rosterItemFree(ri);

	      g_message("rosterParseSetRequest: removed roster item associated with jid:'%s'", jid);
	    }

	  continue;
	}
      else if(ask != NULL)
	{
	  /*
	   * ASK for subscription or to unsubscribe
	   */
	  if(g_strcasecmp(ask, "subscribe") == 0)
	    g_message("rosterParseSetRequest: jid:'%s' is requesting subscription, name:'%s'",jid, name);
	  else if(g_strcasecmp(ask, "unsubscribe") == 0)
	    g_message("rosterParseSetRequest: jid:'%s' is cancelling subscription, name:'%s'",jid, name);
	  else
	    g_warning("rosterParseSetRequest: ask value:'%s' not dealt with from jid:'%s' with name:'%s'", tmp, jid, name);
	  
	  /* clean up */
	  if(jid != NULL) g_free(jid);
	  if(resource != NULL) g_free(resource);
	  if(agent != NULL) g_free(agent);
	  if(name != NULL) g_free(name);
	  
	  continue;
	}

      gj_gtk_roster_model_remove_contact(ri);

      /* find roster item */
      if(ri == NULL)
	{
	  g_message("rosterParseSetRequest: no roster item found by jid:'%s', creating...",jid);
	  
	  /* FIXME: why is this temporary? if there is a roster set request, it would be permanat wouldnt it?? */
	  /* adding temporary (to this session) user and group */
	  if((ri = rosterItemNew(jid)) == NULL)
	    {
	      g_warning("rosterParseSetRequest: couldn't add temporary roster item");
	      
	      if(jid != NULL) g_free(jid);
	      if(resource != NULL) g_free(resource);
	      if(agent != NULL) g_free(agent);
	      
	      node = node->next;
	      continue;
	    }

	  is_new = TRUE;
	}
      else
	g_message("rosterParseSetRequest: found roster item for jid:'%s', updating...", jid);
      
      /* set agent information */
      if(agent != NULL)
	{
	  /* not a agent, so get the agent part (i.e @icq.jabber.org) */
	  rosterItemSetAgent(ri, agent);
	  rosterItemSetIsAgent(ri, FALSE);
	  rosterItemSetType(ri, rosterItemTypeContact);
	}
      else
	{
	  /* no agent part (i.e. @msn.jabber.org) there for MUST be a agent */
	  rosterItemSetIsAgent(ri, TRUE);
	  rosterItemSetType(ri, rosterItemTypeAgent);
	}
      
      if(resource != NULL) rosterItemSetResource(ri, resource);
      if(name != NULL) rosterItemSetName(ri, name);
      
      /* groups */ 
      rosterItemSetGroups(ri,parserFindByNodeOnSameParentAndReturnContentsByList(node, (guchar*)"group"));
      
      /* subscription type */
      if(subscription != NULL)
	{
	  if(g_strcasecmp(subscription, "none") == 0)
	    rosterItemSetSubscription(ri, rosterItemSubscriptionNone);
	  else if(g_strcasecmp(subscription, "to") == 0)
	    rosterItemSetSubscription(ri, rosterItemSubscriptionTo);
	  else if(g_strcasecmp(subscription, "from") == 0)
	    rosterItemSetSubscription(ri, rosterItemSubscriptionFrom);
	  else if(g_strcasecmp(subscription, "both") == 0)
	    rosterItemSetSubscription(ri, rosterItemSubscriptionBoth);
	  else
	    g_warning("rosterParseSetRequest: unexpected roster item type:'%s'", subscription);
	}
      
      /* NOTE: at this point, we test to see if a contact has 
	 been added or simply updated. */
      if(is_new == TRUE)
	{
	  /* set it to be permanant. */
	  rosterItemSetIsPermanent(ri, TRUE);

	  /* add visual contact */
	  g_message("rosterParseSetRequest: create new contact visually for jid:'%s'...",jid);
	  gj_gtk_roster_model_add_contact(ri);
	}
      
      /* clean up */
      if(jid != NULL) g_free(jid);
      if(resource != NULL) g_free(resource);
      if(agent != NULL) g_free(agent);
    }
  
  g_message("rosterParseSetRequest: updated %d items",count);

  return TRUE; 
}

gboolean rosterSetResponseProc(rosterResponseProc proc)
{
  if(proc == NULL)
    {
      g_warning("rosterSetResponseProc: the response proc was NULL");
      return FALSE;
    }

  rosterProc = proc;
  
  return TRUE;
}

rosterResponseProc rosterGetResponseProc()
{
  return rosterProc;
}

/*
 * rosterItem functions 
 *
 */
rosterItem rosterItemFindByID(gint id)
{
  gint index = 0;

  if(id < 1)
    {
      g_warning("rosterFindByID: id:%d was out of bound", id);
      return NULL;
    }

  for(index=0;index<g_list_length(rosterList);index++)
    {
      rosterItem ri  = g_list_nth_data(rosterList,
				       index);

      if(ri == NULL)
	continue;

      if(rosterItemGetName(ri) == NULL)
	continue;

      if(id == rosterItemGetID(ri))
	return ri;
    }

  return NULL;
}

rosterItem rosterItemFindByName(gchar *name)
{
  gint index = 0;

  if(name == NULL)
    {
      g_warning("rosterFindByName: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(rosterList);index++)
    {
      rosterItem ri  = g_list_nth_data(rosterList,
				       index);

      if(ri == NULL)
	continue;

      if(rosterItemGetName(ri) == NULL)
	continue;

      if(strcmp(name, rosterItemGetName(ri)) == 0)
	return ri;
    }

  return NULL;
}

rosterItem rosterItemFindByJID(gchar *jid)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("rosterFindByJID: JID was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(rosterList);index++)
    {
      rosterItem ri  = g_list_nth_data(rosterList,
				       index);

      if(ri == NULL)
	continue;

      if(rosterItemGetJID(ri) == NULL)
	continue;

      if(strcmp(jid, rosterItemGetJID(ri)) == 0)
	return ri;
    }

  return NULL;
}

rosterItem rosterItemFindByJIDWithoutResource(gchar *jid)
{
  gint index = 0;
  rosterItem found_ri = NULL;

  if(jid == NULL)
    {
      g_warning("rosterItemFindByJIDWithoutResource: JID was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(rosterList);index++)
    {
      gchar *search_jid = NULL;
      gchar **search_vResource = NULL;
      gchar *known_jid = NULL;
      gchar **known_vResource = NULL;

      rosterItem ri  = g_list_nth_data(rosterList,
				       index);

      if(ri == NULL)
	continue;

      if(rosterItemGetJID(ri) == NULL)
	continue;

      search_jid = g_strdup(rosterItemGetJID(ri));
      search_vResource = g_strsplit(search_jid,"/",2);

      known_jid = g_strdup(jid);
      known_vResource = g_strsplit(known_jid,"/",2);

      if(search_vResource != NULL && known_vResource != NULL)
	{
	  if(search_vResource[0] != NULL && known_vResource[0] != NULL && 
	     strcmp(known_vResource[0], search_vResource[0]) == 0)
	    found_ri = ri;
	  
	  g_strfreev(search_vResource);
	  g_strfreev(known_vResource);
	}

      if(search_jid != NULL) g_free(search_jid);
      if(known_jid != NULL) g_free(known_jid);

      if(found_ri != NULL)
	return found_ri;
    }

  return NULL;
}

#if 0
rosterItem rosterItemFindByEventID(gint eventID)
{
  gint index = 0;
  gint rosterindex = 0;

  if(eventID < 0)
    {
      g_warning("rosterItemFindByEventID: eventID was < 0");
      return NULL;
    }

  /* loop through all contacts on roster */
  for(rosterindex=0;rosterindex<g_list_length(rosterList);rosterindex++)
    {
      rosterItem ri = g_list_nth_data(rosterList, rosterindex);

      if(ri == NULL)
	{
	  g_warning("rosterItemFindByEventID: getting nth item(%d) in roster list was NULL",index);
	  continue;
	}

      /* loop through all events on contacts event list to look for unique eventID */
      for(index=0;index<g_list_length(ri->events);index++)
	{
	  rosterItemEvent rie = g_list_nth_data(ri->events, index);
	  
	  if(rie == NULL)
	    {
	      g_warning("rosterItemFindByEventID: getting nth item(%d) in roster item event list was NULL",index);
	      continue;
	    }
	  
	  if(rie->eventID == eventID)
	    {
	      return ri;
	    }
	}
    }

  return NULL;
}
#endif

/*
 * SET functions 
 *
 */
gboolean rosterItemSetID(rosterItem ri, gint id)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetID: roster item was NULL");
      return FALSE;
    }

  if(id < 1)
    {
      g_warning("rosterItemSetID: id:%d was < 1", id);
      return FALSE;
    }

  ri->id = id;

  return TRUE;
}

gboolean rosterItemSetJID(rosterItem ri, gchar *jid)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetJID: roster item was NULL");
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("rosterItemSetJID: NULL passed");
      return FALSE;
    }

  if(ri->jid != NULL)
    {
      /*g_warning("rosterItemSetJID: roster item JID was already set, freeing...");*/
      g_free(ri->jid);
    }
  
  ri->jid = g_strdup(jid);

  return TRUE;
}

gboolean rosterItemSetName(rosterItem ri, gchar *name)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetName: roster item was NULL");
      return FALSE;
    }

  if(name == NULL)
    {
      g_warning("rosterItemSetName: NULL passed");
      return FALSE;
    }

  if(ri->name != NULL)
    {
      /*g_warning("rosterItemSetName: roster item Name was already set, freeing...");*/
      g_free(ri->name);
    }
  
  ri->name = g_strdup(name);

  return TRUE;
}

gboolean rosterItemSetResource(rosterItem ri, gchar *resource)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetResource: roster item was NULL");
      return FALSE;
    }

  if(resource == NULL)
    {
      g_warning("rosterItemSetResource: NULL passed");
      return FALSE;
    }

  if(ri->resource != NULL)
    g_free(ri->resource);
  
  ri->resource = g_strdup(resource);

  return TRUE;
}

gboolean rosterItemSetAgent(rosterItem ri, gchar *agent)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetAgent: roster item was NULL");
      return FALSE;
    }

  if(agent == NULL)
    {
      g_warning("rosterItemSetAgent: NULL passed");
      return FALSE;
    }

  if(ri->agent != NULL)
    {
      /*g_warning("rosterItemSetAgent: roster item Agent was already set, freeing...");*/
      g_free(ri->agent);
    }
  
  ri->agent = g_strdup(agent);

  return TRUE;
}

gboolean rosterItemSetGroups(rosterItem ri, GList *groups)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetGroups: roster item was NULL");
      return FALSE;
    }

  if(groups == NULL)
    {
      /*g_warning("rosterItemSetGroups: NULL passed");*/
      return FALSE;
    }

  if(ri->groups != NULL)
    {
      /*g_warning("rosterItemSetGroup: roster item Group was already set, freeing...");*/
      rosterItemGroupsFree(ri);
    }
  
  ri->groups = groups;

  return TRUE;
}

gboolean rosterItemSetIsAgent(rosterItem ri, gboolean IsAgent)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetIsAgent: roster item was NULL");
      return FALSE;
    }
  
  ri->is_agent = IsAgent;

  return TRUE;
}

gboolean rosterItemSetIsPermanent(rosterItem ri, gboolean IsPermanent)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetIsPermanent: roster item was NULL");
      return FALSE;
    }
  
  ri->is_permanent = IsPermanent;

  return TRUE;
}

gboolean rosterItemSetIsSetForRemoval(rosterItem ri, gboolean IsSetForRemoval)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetIsSetForRemoval: roster item was NULL");
      return FALSE;
    }
  
  ri->is_set_for_removal = IsSetForRemoval;

  return TRUE;
}

gboolean rosterItemSetSubscription(rosterItem ri, rosterItemSubscription subscription)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetSubscription: roster item was NULL");
      return FALSE;
    }

  if(subscription < 1)
    {
      g_warning("rosterItemSetSubscription: subscription:%d was < 1",subscription);
      return FALSE;
    }

  ri->subscription = subscription;

  return TRUE;
}

gboolean rosterItemSetType(rosterItem ri, rosterItemType type)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetType: roster item was NULL");
      return FALSE;
    }

  if(type < 1)
    {
      g_warning("rosterItemSetType: type:%d was < 1",type);
      return FALSE;
    }

  ri->type = type;

  return TRUE;
}

gboolean rosterItemSetPresence(rosterItem ri, presence pres)
{
  if(ri == NULL)
    {
      g_warning("rosterItemSetPresence: roster item was NULL");
      return FALSE;
    }

  if(pres == NULL)
    {
      g_warning("rosterItemSetPresence: NULL passed");
      return FALSE;
    }

  if(ri->pres != NULL)
    presenceFree(ri->pres);
  
  ri->pres = presenceCopy(pres);

  return TRUE;
}

/*
 * GET functions 
 *
 */
gint rosterItemGetID(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetID: roster item was NULL");
      return -1;
    }

  return ri->id;
}

gchar *rosterItemGetJID(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetJID: roster item was NULL");
      return NULL;
    }

  return ri->jid;
}

gchar *rosterItemGetJIDWithoutResource(rosterItem ri)
{
  gchar **vjid = NULL;
  gchar *jid = NULL;

  if(ri == NULL)
    {
      g_warning("rosterItemGetJIDWithoutResource: roster item was NULL");
      return NULL;
    }
  
  if(ri->jid != NULL)
    {
      vjid = g_strsplit(ri->jid, "/", -1);
      if(vjid != NULL && vjid[0] != NULL) 
	jid = g_strdup(vjid[0]);
      else
	jid = g_strdup(ri->jid);

      if(vjid != NULL)
	g_strfreev(vjid);
    }

  return jid;
}

gchar *rosterItemGetName(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetName: roster item was NULL");
      return NULL;
    }

  return ri->name;
}

gchar *rosterItemGetResource(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetResource: roster item was NULL");
      return NULL;
    }

  return ri->resource;
}

gchar *rosterItemGetAgent(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetAgent: roster item was NULL");
      return NULL;
    }

  return ri->agent;
}

GList *rosterItemGetGroups(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetGroups: roster item was NULL");
      return NULL;
    }

  return ri->groups;
}

gchar *rosterItemGetGroupsAsString(rosterItem ri)
{
  gchar *groups = NULL;

  if(ri == NULL)
    {
      g_warning("rosterItemGetGroupsAsString: roster item was NULL");
      return NULL;
    }
  
  if(rosterItemGetGroups(ri) != NULL)
    {
      gint i = 0;
      
      for(i=0;i<g_list_length(rosterItemGetGroups(ri));i++)
	{
	  gchar *g = (gchar*) g_list_nth_data(rosterItemGetGroups(ri), i);
	  
	  if(g == NULL) 
	    continue;
	  
	  if(groups == NULL)
	    groups = g_strdup(g);
	  else
	    groups = g_strconcat(groups, ", ", g, NULL);
	}
    }

  return groups;
}

gboolean rosterItemGetIsAgent(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetIsAgent: roster item was NULL");
      return FALSE;
    }

  return ri->is_agent;
}

gboolean rosterItemGetIsPermanent(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetIsPermanent: roster item was NULL");
      return FALSE;
    }

  return ri->is_permanent;
}

gboolean rosterItemGetIsSetForRemoval(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetIsSetForRemoval: roster item was NULL");
      return FALSE;
    }

  return ri->is_set_for_removal;
}

rosterItemSubscription rosterItemGetSubscription(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetSubscription: roster item was NULL");
      return -1;
    }

  return ri->subscription;
}

rosterItemType rosterItemGetType(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetType: roster item was NULL");
      return -1;
    }

  return ri->type;
}

presence rosterItemGetPresence(rosterItem ri)
{
  if(ri == NULL)
    {
      g_warning("rosterItemGetPresence: roster item was NULL");
      return NULL;
    }

  return ri->pres;
}

void rosterItemPrintAll()
{
  gint index = 0;

  /* loop through all contacts on roster */
  for(index=0;index<g_list_length(rosterList);index++)
    {
      rosterItem ri = g_list_nth_data(rosterList, index);

      if(ri == NULL)
	{
	  g_warning("rosterUtemPrintAll: getting nth item(%d) in roster list was NULL",index);
	  continue;
	}

      rosterItemPrint(ri);
    }  
}

void rosterItemPrint(rosterItem ri)
{
  if(ri == NULL)
    return;

  g_message("++++++++++++++++++++++++++++++++++++++++++++++++");
  g_message("++++");
  g_message("++++ jid:'%s'", ri->jid);
  g_message("++++ name:'%s'", ri->name);
  g_message("++++ resource:'%s'", ri->resource);
  g_message("++++ agent:'%s'", ri->agent);
  g_message("++++");
  g_message("++++ is_agent:%s", ri->is_agent?"true":"false");
  g_message("++++ is_permanent:%s", ri->is_permanent?"true":"false");
  g_message("++++");
  g_message("++++ type:%d->'%s'", ri->type, translateRosterItemType(ri->type));
  g_message("++++ subscription:%d->'%s'", ri->subscription, translateRosterItemSubscription(ri->subscription));
  g_message("++++");
  g_message("++++++++++++++++++++++++++++++++++++++++++++++++");
}

#ifdef __cplusplus
}
#endif
