/* 
 * $RCSfile: gj_roster.h,v $
 *
 * $Author: ginxd $
 * $Date: 2003/07/02 09:58:19 $
 * 
 * $Revision: 1.8 $
 *
 */

#ifndef __gj_roster_h
#define __gj_roster_h

#include <glib.h>

#include "gj_presence.h"
#include "gj_message.h"
/*#include "gj_event.h"*/
#include "gj_iq.h"

/* 
 * Rosters 
 */
typedef enum rosterItemSubscription {
  rosterItemSubscriptionNone = 1,
  rosterItemSubscriptionTo,
  rosterItemSubscriptionFrom,
  rosterItemSubscriptionBoth,
  
  rosterItemSubscriptionEnd
} rosterItemSubscription;

typedef enum rosterItemType {
  rosterItemTypeNone = 1,
  rosterItemTypeContact,
  rosterItemTypeAgent,
  rosterItemTypeGroup, /* in theory this will never be used, because groups are not roster items. */
  
  rosterItemTypeEnd
} rosterItemType;

typedef struct t_rosterItem *rosterItem;

typedef void (*rosterResponseProc)(GList *roster);

/* list functions */
gboolean rosterInit();
gboolean rosterTerm();

/* get information */
GList *rosterGetAllGroups();
GList *rosterGetAllItemsByGroup(gchar *group);
GList *rosterGetAllItemsByAgent(gchar *agent);
GList *rosterGetAllAgentsByJID();
GList *rosterGetAllAgentsByJIDWithoutResource();
GList *rosterGetAllAgentsByName();
gboolean rosterIsAgentRegistered(gchar *agent);

/* list functions */
GList *rosterGet();

/* item functions */
rosterItem rosterItemNew(gchar *jid);
rosterItem rosterItemNewUnknownContactByMessage(message msg); /* should no longer be used */
rosterItem rosterItemNewUnknownByJID(gchar *jid);
void rosterItemFree(rosterItem ri);


gboolean rosterItemSetJID(rosterItem ri, gchar *jid);
gboolean rosterItemSetName(rosterItem ri, gchar *name);
gboolean rosterItemSetResource(rosterItem ri, gchar *resource);
gboolean rosterItemSetGroups(rosterItem ri, GList *groups);
gboolean rosterItemSetAgent(rosterItem ri, gchar *agent);
gboolean rosterItemSetIsAgent(rosterItem ri, gboolean IsAgent);
gboolean rosterItemSetSubscription(rosterItem ri, rosterItemSubscription subscription);
gboolean rosterItemSetType(rosterItem ri, rosterItemType type);

gboolean rosterItemSetPresence(rosterItem ri, presence pres);

gint rosterItemGetID(rosterItem ri);
gchar *rosterItemGetJID(rosterItem ri);
gchar *rosterItemGetJIDWithoutResource(rosterItem ri);
gchar *rosterItemGetName(rosterItem ri);
gchar *rosterItemGetResource(rosterItem ri);
GList *rosterItemGetGroups(rosterItem ri);
gchar *rosterItemGetGroupsAsString(rosterItem ri);
gchar *rosterItemGetAgent(rosterItem ri);
gboolean rosterItemGetIsAgent(rosterItem ri);
gboolean rosterItemGetIsPermanent(rosterItem ri);
gboolean rosterItemGetIsSetForRemoval(rosterItem ri);           
rosterItemSubscription rosterItemGetSubscription(rosterItem ri);
rosterItemType rosterItemGetType(rosterItem ri);

presence rosterItemGetPresence(rosterItem ri);

/* item group functions */
gboolean rosterItemGroupsFree(rosterItem ri);
gboolean rosterItemGroupsAdd(rosterItem ri, gchar *group);
gboolean rosterItemGroupsDelete(rosterItem ri, gchar *group);
gint rosterItemGroupsCount(rosterItem ri);

/* other */
gboolean rosterRequest(rosterResponseProc proc);

GList *rosterParse(xmlNodePtr parent);
gboolean rosterParseSetRequest(infoquery iq);

rosterItem rosterItemFindByID(gint id);
rosterItem rosterItemFindByName(gchar *name);
rosterItem rosterItemFindByJID(gchar *jid);
rosterItem rosterItemFindByJIDWithoutResource(gchar *jid);
rosterItem rosterItemFindByEventID(gint eventID);

/* debugging */
void rosterItemPrintAll();
void rosterItemPrint(rosterItem ri);

#endif

