/* 
 * $RCSfile: gj_stream.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/06/16 21:04:04 $
 * 
 * $Revision: 1.6 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

#include <glib.h>

#include <libxml/tree.h>

#include "gj_stream.h"
#include "gj_parser.h"
#include "gj_connection.h"

/* item functions */
void streamDestroy(stream stm);

/*
 * member functions
 */

/* sets */
gboolean streamSetID(stream stm, gchar *id);

gboolean streamSetStream(stream stm, gchar *stream);
gboolean streamSetXML(stream stm, xmlDocPtr doc);

gboolean streamSetConnection(stream stm, connection c);

void streamSetIsBuilt(stream stm, gboolean is_built);

/* gets */
gboolean streamGetIsBuilt(stream stm);

/* utils */
gchar *streamConvertDocToString(stream stm);

/* callbacks */
void streamConnected(connection c);
void streamDisconnected(connection c);
void streamError(connection c, GTcpSocketConnectAsyncStatus status);

/* structs */
struct t_stream
{
  gint requestID;

  streamCallback cb_connected;
  streamCallback cb_disconnected;
  streamCallback cb_error;

  gboolean is_built;

  gchar *id;
  gchar *stream;
  gchar *to;

  guint unique_id;

  guint port;

  connection c;

  /*xmlDocPtr doc;*/
  xmlDocPtr xml;
};

GList *streamList;
guint unique_id_count = 1;

gboolean streamInit()
{
  if(streamList != NULL)
    streamTerm();

  /* no list to create, this is done on the first g_list_append */

  return TRUE;
}

gboolean streamTerm()
{  
  gint index = 0;

  /* list */
  if(streamList == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(streamList);index++)
    {
      stream stm = NULL;
      
      if((stm = (stream) g_list_nth_data(streamList,index)) != NULL)
	streamFree(stm);
    }

  g_list_free(streamList);
  streamList = NULL;

  return TRUE;
}

stream streamNew(gchar *host)
{
  stream stm = NULL;

  if(host == NULL)
    {
      g_warning("streamNew: host was NULL");
      return NULL;
    }

  if(streamFindByTo(host) != NULL)
    {
      g_warning("streamNew: there is already a stream to:'%s'", host);
      return NULL;
    }
  
  if((stm = (stream) g_new(struct t_stream,1)))
    {
      stm->to = g_strdup(host);

      stm->requestID = supportGetUniqueID(); /* get next */

      stm->xml = NULL;

      stm->cb_connected = NULL;
      stm->cb_disconnected = NULL;
      stm->cb_error = NULL;

      stm->is_built = FALSE;

      stm->id = NULL;
      stm->stream = NULL;

      stm->port = 5222;

      stm->c = NULL;

      stm->unique_id = unique_id_count++;

      streamList = g_list_append(streamList,stm);
    }
  
  return stm;  
}

gboolean streamFree(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamFree: failed to free, stm was NULL");
      return FALSE;
    }

  /* free mem */
  streamList = g_list_remove(streamList,stm);  
  
  /* destroy members */
  streamDestroy(stm);

  /* free structure */
  g_free(stm);

  return TRUE;
}

void streamDestroy(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamDestroy: failed to destroy, stm was NULL");
      return;
    }

  stm->requestID = 0; 
  stm->is_built = FALSE;
  
  if(stm->xml != NULL)
    {
      xmlFreeDoc(stm->xml);
      stm->xml = NULL;
    }

  stm->cb_connected = NULL;
  stm->cb_disconnected = NULL;
  stm->cb_error = NULL;

  if(stm->id != NULL)
    {
      g_free(stm->id);
      stm->id = NULL;
    }

  if(stm->to != NULL)
    {
      g_free(stm->to);
      stm->to = NULL;
    }

  if(stm->stream != NULL)
    {
      g_free(stm->stream);
      stm->stream = NULL;
    }

  stm->port = 0;

  if(stm->c != NULL)
  {
      gj_connection_free(stm->c);
      stm->c = NULL;
  }
}

stream streamFindByID(gchar *id)
{
  gint index = 0;

  if(id == NULL)
    {
      g_warning("streamFindByID: failed to find by id, id was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(streamList);index++)
    {
      stream stm = (stream) g_list_nth_data(streamList,index);

      if(stm == NULL)
	{
	  g_warning("streamFindByID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(strcmp(stm->id,id) == 0)
	  return stm;
    }
  
  return NULL;
}

stream streamFindByUniqueID(guint id)
{
  gint index = 0;

  if(id < 1)
    {
      g_warning("streamFindByUniqueID: failed to find by unique id:%d, it was < 1",id);
      return NULL;
    }

  for(index=0;index<g_list_length(streamList);index++)
    {
      stream stm = (stream) g_list_nth_data(streamList,index);

      if(stm == NULL)
	{
	  g_warning("streamFindByUniqueID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(stm->unique_id == id)
	return stm;
    }
  
  return NULL;
}

stream streamFindByConnection(connection c)
{
  gint index = 0;

  if(c == NULL)
    {
      g_warning("streamFindByConnection: connection was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(streamList);index++)
    {
      stream stm = (stream) g_list_nth_data(streamList,index);

      if(stm == NULL)
	{
	  g_warning("streamFindByConnection: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(stm->c == c)
	  return stm;
    }
  
  return NULL;
}

stream streamFindByTo(gchar *to)
{
  gint index = 0;

  if(to == NULL)
    {
      g_warning("streamFindByTo: to was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(streamList);index++)
    {
      stream stm = (stream) g_list_nth_data(streamList,index);

      if(stm == NULL)
	{
	  g_warning("streamFindByTo: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(strcmp(stm->to,to) == 0)
	  return stm;
    }
  
  return NULL;
}

stream streamFindByRequestID(gint requestID)
{
  gint index = 0;

  if(requestID <= 0)
    {
      g_warning("streamFindByRequestID: failed to find by RequestID, id was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(streamList);index++)
    {
      stream stm = (stream) g_list_nth_data(streamList,index);

      if(stm == NULL)
	{
	  g_warning("streamFindByRequestID: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(stm->requestID == requestID)
	{
	  return stm;
	}
    }
  
  return NULL;
}

/*
 * member functions
 */

/* sets */
gboolean streamSetID(stream stm, gchar *id)
{
  if(stm == NULL)
    {
      g_warning("streamSetID: stream was NULL");
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("streamSetID: failed, NULL passed");
      return FALSE;
    }

  if(stm->id != NULL)
    {
      g_free(stm->id);
      stm->id = NULL;
    }
  
  stm->id = g_strdup(id);

  return TRUE;
}

gboolean streamSetTo(stream stm, gchar *to)
{
  if(stm == NULL)
    {
      g_warning("streamSetTo: stream was NULL");
      return FALSE;
    }

  if(to == NULL)
    {
      g_warning("streamSetTo: to was NULL");
      return FALSE;
    }

  if(stm->to != NULL)
    {
      g_free(stm->to);
      stm->to = NULL;
    }
  
  stm->to = g_strdup(to);

  return TRUE;
}

gboolean streamSetPort(stream stm, guint port)
{
  if(stm == NULL)
    {
      g_warning("streamSetPort: stream was NULL");
      return FALSE;
    }

  if(port < 1024)
    {
      g_warning("streamSetPort: failed, port:%d < 1024",port);
      return FALSE;
    }

  stm->port = port;

  return TRUE;
}

gboolean streamSetStream(stream stm, gchar *stream)
{
  if(stm == NULL)
    {
      g_warning("streamSetSteam: stream was NULL");
      return FALSE;
    }

  if(stream == NULL)
    {
      g_warning("streamSetStream: stream was NULL");
      return FALSE;
    }

  stm->stream = g_strdup(stream);

  return TRUE;
}

gboolean streamSetXML(stream stm, xmlDocPtr doc)
{
  if(stm == NULL)
    {
      g_warning("streamSetXML: stream was NULL");
      return FALSE;
    }

  if(doc == NULL)
    {
      g_warning("streamSetXML: failed, XML was NULL");
      return FALSE;
    }

  if(stm->xml != NULL)
    {
      xmlFreeDoc(stm->xml);
      stm->xml = NULL;
    }

  if((stm->xml = xmlCopyDoc(doc,TRUE)) == NULL)
    {
      g_warning("streamSetXML: could not create new xml doc");
      return FALSE;
    }
 
  return TRUE;
}

void streamSetIsBuilt(stream stm, gboolean is_built)
{
  if(stm == NULL)
    {
      g_warning("streamSetIsBuilt: stream was NULL");
      return;
    }

  stm->is_built = is_built;

  return;
}

gboolean streamSetConnectedCallback(stream stm, streamCallback proc)
{
  if(stm == NULL)
    {
      g_warning("streamSetConnectedCallback: stream was NULL");
      return FALSE;
    }

  if(proc == NULL)
    {
      g_warning("streamSetConnectedCallback: proc was NULL");
      return FALSE;
    }

  stm->cb_connected = proc;

  return TRUE;
}

gboolean streamSetDisconnectedCallback(stream stm, streamCallback proc)
{
  if(stm == NULL)
    {
      g_warning("streamSetDisconnectedCallback: stream was NULL");
      return FALSE;
    }

  if(proc == NULL)
    {
      g_warning("streamSetDisconnectedCallback: proc was NULL");
      return FALSE;
    }

  stm->cb_disconnected = proc;

  return TRUE;
}

gboolean streamSetErrorCallback(stream stm, streamCallback proc)
{
  if(stm == NULL)
    {
      g_warning("streamSetErrorCallback: stream was NULL");
      return FALSE;
    }

  if(proc == NULL)
    {
      g_warning("streamSetErrorCallback: proc was NULL");
      return FALSE;
    }

  stm->cb_error = proc;

  return TRUE;
}


gboolean streamSetConnection(stream stm, connection c)
{
  if(stm == NULL)
    {
      g_warning("streamSetConnection: stream was NULL");
      return FALSE;
    }

  if(c == NULL)
    {
      g_warning("streamSetConnection: connection was NULL");
      return FALSE;
    }    

  stm->c = c;

  return TRUE;
}

/* gets */
gchar *streamGetID(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamGetID: stream was NULL");
      return FALSE;
    }

  return stm->id;
}

gchar *streamGetTo(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamGetTo: stream was NULL");
      return FALSE;
    }

  return stm->to;
}

guint streamGetPort(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamGetPort: stream was NULL");
      return FALSE;
    }

  return stm->port;
}

guint streamGetUniqueID(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamGetUniqueID: stream was NULL");
      return FALSE;
    }

  return stm->unique_id;
}

gchar *streamGetStream(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamGetStream: stream was NULL");
      return FALSE;
    }

  return stm->stream;
}

xmlDocPtr streamGetXML(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamGetXML: stream was NULL");
      return FALSE;
    }

  return stm->xml;
}

gint streamGetRequestID(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamGetRequestID: stream was NULL");
      return -1;
    }

  return stm->requestID;
}

gboolean streamGetIsBuilt(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamGetIsBuilt: stream was NULL");
      return FALSE;
    }

  return stm->is_built;
}

connection streamGetConnection(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamGetConnection: stream was NULL");
      return NULL;
    }

  return stm->c;
}

/* 
 * compile stream
 */
gboolean streamOpen(stream stm)
{
  if(stm == NULL)
    {
      g_warning("streamOpen: stream was NULL");
      return FALSE;
    } 

  if(streamGetIsBuilt(stm) == FALSE)
    {
      g_warning("streamOpen: stream is not built, not continuing...");
      return FALSE;
    }

  if((stm->c = gj_connection_create(streamGetTo(stm), streamGetPort(stm))) == NULL)
  {
      g_warning("streamOpen: connection was NULL");
      return FALSE;
  }

  g_message("streamOpen: connection:0x%.8x created, host:'%s', port:%d", 
	    (gint)stm->c, streamGetTo(stm), streamGetPort(stm));

  /* open tcp connection and make connection */
  gj_connection_set_cb_connect(stm->c, streamConnected);
  gj_connection_set_cb_disconnect(stm->c, streamDisconnected);
  gj_connection_set_cb_error(stm->c, streamError);
  gj_connection_set_cb_in(stm->c, parserInputFromConnection);

  if(gj_connection_connect(stm->c) == FALSE)
    {
      g_warning("streamOpen: failed to connect (locally)");
      return FALSE;
    }

  return TRUE;
}

gboolean streamClose(stream stm)
{
    gchar *msg = NULL;

    if(stm == NULL)
    {
	g_warning("streamClose: stream was NULL");
	return FALSE;
    }    

    msg = g_strdup_printf("</stream:stream>");
    gj_connection_write(stm->c, msg);

    /* clean up */
    if(msg != NULL) g_free(msg);

    return TRUE;
}

gboolean streamCloseByTo(gchar *to)
{
    stream stm = NULL;

    if(to == NULL)
    {
	g_warning("streamCloseByTo: to was NULL");
	return FALSE;
    }

    if((stm = streamFindByTo(to)) == NULL)
    {
	g_warning("streamCloseByTo: could not find stream by to:'%s'", to);
	return FALSE;
    }

    return streamClose(stm);
}

gboolean streamSend(stream stm, gchar *text)
{
    if(stm == NULL)
    {
	g_warning("streamSend: stream was NULL");
	return FALSE;
    }        

    if(text == NULL || strlen(text) < 1)
    {
	g_warning("streamSend: text was NULL or text length was < 1");
	return FALSE;
    }

    gj_connection_write(stm->c, text);

    return TRUE;
}

gboolean streamSendByTo(gchar *to, gchar *text)
{
    stream stm = NULL;

    if(to == NULL)
    {
	g_warning("streamSendByTo: to was NULL");
	return FALSE;
    }

    if((stm = streamFindByTo(to)) == NULL)
    {
	g_warning("streamSendByTo: could not find stream by to:'%s'", to);
	return FALSE;
    }

    return streamSend(stm, text);
}

/*
 * NOTE: the connected function means we have a socket connection NOT another
 *       stream from the server.  the connected callback is called at the 
 *       'stream receive' function.
 */
void streamConnected(connection c)
{
  gchar *msg = NULL;
  stream stm = NULL;

  if((stm = streamFindByConnection(c)) == NULL)
    {
      g_warning("streamConnected: could not find stream from connection");
      return;
    }

  /* use stream and introduce ourselves... */
  if((msg = streamConvertDocToString(stm)) == NULL)
    {
      g_warning("streamConnected: failed to convert doc to string, NULL returned");
      return;
    }

  g_message("streamConnected: stream is connected");
  gj_connection_write(c, msg);  

  /* clean up */
  g_free(msg);
}

void streamDisconnected(connection c)
{
  stream stm = NULL;

  if((stm = streamFindByConnection(c)) == NULL)
  {
      g_warning("streamDisconnected: could not find stream from connection");
      return;
  }  
  
  /* send up */
  if(stm->cb_disconnected == NULL)
    {
      g_message("streamDisconnected: no callback for disconnection!");
      return;
    }

  (stm->cb_disconnected)(stm, 0);
}

void streamError(connection c, GTcpSocketConnectAsyncStatus status)
{
  stream stm = NULL;

  if((stm = streamFindByConnection(c)) == NULL)
  {
      g_warning("streamError: could not find stream from connection");
      return;
  }  
  
  /* send up */
  if(stm->cb_error == NULL)
    {
      g_message("streamError: no callback for error!");
      return;
    }

  (stm->cb_error)(stm, status);
}

gboolean streamBuild(stream stm)
{
  xmlDocPtr doc = NULL;
  xmlNsPtr ns = NULL;

  if(stm == NULL)
    {
      g_warning("streamBuild: failed, stream was NULL");
      return FALSE;
    }

  if(streamGetTo(stm) == NULL)
    {
      g_warning("streamBuild: failed, 'to' is NULL");
      return FALSE;
    }

  /* create new doc */
  if((doc = xmlNewDoc((guchar*)"1.0")) == NULL)
    {
      /* failed */
      g_warning("streamBuild: failed to create new xml document");
      return FALSE;
    }
  
  ns = xmlNewNs(NULL, (guchar*)"stream", (guchar*)"stream");
  doc->xmlChildrenNode = xmlNewDocNode(doc, ns, (guchar*)"stream", NULL);

  ns = xmlNewNs(doc->xmlChildrenNode, (guchar*)"http://etherx.jabber.org/streams", (guchar*)"stream");
  xmlSetProp(doc->xmlChildrenNode, (guchar*)"to", (guchar*)streamGetTo(stm));
  xmlSetProp(doc->xmlChildrenNode, (guchar*)"xmlns", (guchar*)"jabber:client");
 
  streamSetXML(stm, doc);
  /* xmlAddChild(doc->xmlChildrenNode,streamGetXML(stm));*/

  /* set flag to say stream is built */
  streamSetIsBuilt(stm,TRUE);

  /* free doc ? */

  return TRUE;
}

gchar *streamConvertDocToString(stream stm)
{
  xmlChar *buf;
  gint size = 0;
  gchar *output = NULL;

  if(stm == NULL)
    {
      g_warning("streamConvertDocToString: failed, stream was NULL");
      return FALSE;
    }

  if(streamGetIsBuilt(stm) == FALSE)
    {
      g_warning("streamConvertDocToString: failed, stream is not built, not continuing...");
      return FALSE;
    }

  if(streamGetXML(stm) == NULL)
    {
      g_warning("streamConvertDocToString: failed, 'xml' is NULL");
      return FALSE;
    }

  xmlDocDumpMemory(streamGetXML(stm),&buf,&size); 
/*   buf = g_strdup_printf("<?xml version='1.0'?><stream:stream to='%s' xmlns='jabber:client' xmlns:stream='http://etherx.jabber.org/streams'>", streamGetTo(stm)); */

  if(buf == NULL)
    {
      g_warning("streamConvertDocToString: failed because xmlDocDumpMemory returned NULL");
      return NULL;
    }

  /* add a fix for end of xml tag */
  buf[(size-3)] = '>'; 
  buf[(size-2)] = ' '; 

  output = g_strdup_printf("%s",buf);

  /* need to free buf... but free crashes it */

  return output;
}

void streamReceive(connection c, xmlNodePtr node)
{
  stream stm = NULL;

  gint requestID = -1;
  gchar *tmp = NULL;

  /* find stream */
  if((stm = streamFindByConnection(c)) == NULL)
    {
      g_warning("streamReceive: failed to find stream based on connection:0x%.8x", c);
      return;
    }
     
  /* set id */
  if((tmp = (gchar*)parserFindByAttrAndReturn(node, (guchar*)"id")) != NULL)
    streamSetID(stm,tmp);
  
  /* get call back information */
  if(stm->cb_connected == NULL)
    {
      g_warning("streamReceive: connected callback is NULL");
      return;
    }
  
  /* get requestID - required for callback */
  requestID = streamGetRequestID(stm);
  
  /* call callback function */
  (stm->cb_connected)(stm, requestID);
}

#ifdef __cplusplus
}
#endif
