/*
 * $RCSfile: gj_support.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/04/26 11:09:58 $
 *
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <ctype.h>
#include <time.h> 

#include <glib.h>

#include <gnet.h>

#include "gj_support.h"

int supportGetUniqueID()
{
  static id = 69;
  return id++;
}

const char *supportGetTimestamp()
{
  static char buf[256];
  time_t tnow = time(0);
  struct tm *tmLocal = localtime(&tnow);

  if(strftime(buf, 256, "%H:%M:%S", tmLocal) == 0)
    return "time unknown";
  else
    return buf;
}

const char *supportGetTimestampLong()
{
  static char buf[256];
  time_t tnow = time(0);
  struct tm *tmLocal = localtime(&tnow);

  if(strftime(buf, 256, "%H%M%S", tmLocal) == 0)
    return "time unknown";
  else
    return buf;
}

const char *supportGetDatestamp()
{
  static char buf[256];
  time_t tnow = time(0);
  struct tm *tmLocal = localtime(&tnow);

  if(strftime(buf, 256, "%a, %b %d", tmLocal) == 0)
    return "date unknown";
  else
    return buf;
}

const char *supportGetDatestampLong()
{
  static char buf[256];
  time_t tnow = time(0);
  struct tm *tmLocal = localtime(&tnow);

  if(strftime(buf, 256, "%Y%m%d", tmLocal) == 0)
    return "date unknown";
  else
    return buf;
}

const char *supportGetDatestampFromTimeT(time_t now)
{
  static char buf[256];
  struct tm *tmLocal = localtime(&now);

  if(strftime(buf, 256, "%a, %b %d", tmLocal) == 0)
    return "date unknown";
  else
    return buf;
} 

const char *supportGetTimestampFromTimeT(time_t now)
{
  static char buf[256];
  struct tm *tmLocal = localtime(&now);

  if(strftime(buf, 256, "%H:%M:%S", tmLocal) == 0)
    return "time unknown";
  else
    return buf;
}

const char *supportGetTimeAndDateFromTimeT(time_t now)
{
  static char buf[256];
  struct tm *tmLocal = localtime(&now);

  if(strftime(buf, 256, "%A, %B %d, %H:%M:%S", tmLocal) == 0)
    return "date/time unknown";
  else
    return buf;
}

const char *supportGetTimestampFull()
{
  static char buf[256];
  time_t tnow = time(0);
  struct tm *tmLocal = localtime(&tnow);

  if(strftime(buf, 256, "%Y%m%d%H%M%S", tmLocal) == 0)
    return "date_unknown";
  else
    return buf;
}

gboolean supportFindInStr(const gchar *needle, const gchar *haystack)
{
    gchar *result = NULL;

    if(needle == NULL)
    {
	g_warning("supportFindInStr: needle is NULL");
	return FALSE;
    }

    if(haystack == NULL)
    {
	g_warning("supportFindInStr: haystack is NULL");
	return FALSE;
    }

    if((result = g_strrstr(haystack, needle)) == NULL)
	return FALSE;
    
    return TRUE;
}

gchar *supportFindInStrAndRemove(const gchar *needle, const gchar *haystack)
{
    gchar **vstripped = NULL;
    gchar *result = NULL;

    gint i = 0;
    gboolean end_of_array = FALSE;

    if(needle == NULL)
    {
	g_warning("supportFindInStrAndReplace: needle is NULL");
	return NULL;
    }

    if(haystack == NULL)
    {
	g_warning("supportFindInStrAndRemove: haystack is NULL");
	return NULL;
    }

    /* remove all except numbers */
    vstripped = g_strsplit(haystack, needle, -1);

    if(vstripped == NULL)
    {
	g_warning("supportFindInStrAndRemove: could not split text with delimiter:'%s' - NULL returned..", needle);
	return NULL;
    }
   
    while(end_of_array == FALSE)
    {
	if(vstripped[i] == NULL)
	{
	    end_of_array = TRUE;
	    break;
	}

	if(result == NULL)
	    result = g_strdup(vstripped[i]);
	else
	    result = g_strconcat(result, vstripped[i], NULL);

	i++;
    }

    /* clean up */
    if(vstripped != NULL) g_strfreev(vstripped);

    return result;
}

gchar *supportFindInStrAndReplace(const gchar *needle, const gchar *haystack, const gchar *replacement)
{
    gchar **vstripped = NULL;
    gchar *result = NULL;

    gint i = 0;
    gboolean end_of_array = FALSE;

    if(needle == NULL)
    {
	g_warning("supportFindInStrAndReplace: needle is NULL");
	return NULL;
    }

    if(haystack == NULL)
    {
	g_warning("supportFindInStrAndReplace: haystack is NULL");
	return NULL;
    }

    if(replacement == NULL)
    {
	g_warning("supportFindInStrAndReplace: replacement is NULL");
	return NULL;
    }

    /* remove all except numbers */
    vstripped = g_strsplit(haystack, needle, -1);

    if(vstripped == NULL)
    {
	g_warning("supportFindInStrAndReplace: could not split text with delimiter:'%s' - NULL returned..", needle);
	return NULL;
    }
   
    while(end_of_array == FALSE)
    {
	if(vstripped[i] == NULL)
	{
	    end_of_array = TRUE;
	    break;
	}

	if(result == NULL)
	    result = g_strdup(vstripped[i]);
	else
	    result = g_strconcat(result, replacement, vstripped[i], NULL);

	i++;
    }

    /* clean up */
    if(vstripped != NULL) g_strfreev(vstripped);

    return result;
}

/*
 * utils 
 */
gboolean supportIsWordSeparator(gchar c)
{
/* 	return !isalpha(c) && c != '\''; */
  return(isspace(c));
}

uriType supportIsURI(gchar *word)
{
  gint len = 0;
  if(!word) return uriTypeUnknown;
  len = strlen(word);
  
  if(!strncasecmp(word, "irc://", 6))
    return uriTypeURL;
  
  if(!strncasecmp(word, "irc.", 4))
    return uriTypeURL;
  
  if(!strncasecmp(word, "ftp.", 4))
    return uriTypeURL;
  
  if(!strncasecmp(word, "ftp:", 4))
    return uriTypeURL;
  
  if(!strncasecmp(word, "www.", 4))
    return uriTypeURL;
  
  if(!strncasecmp(word, "http:", 5))
    return uriTypeURL;
  
  if(!strncasecmp(word, "https:", 6))
    return uriTypeURL;
  
  if(!strncasecmp(word + len - 4, ".org", 4))
    return uriTypeHost;
  
  if(!strncasecmp(word + len - 4, ".net", 4))
    return uriTypeHost;
  
  if(!strncasecmp(word + len - 4, ".com", 4))
    return uriTypeHost;
  
  if(!strncasecmp(word + len - 4, ".edu", 4))
    return uriTypeHost;
  
  return uriTypeUnknown;
}

#if 0
gchar *supportGetSHABySessionIDAndPassword2(gchar *session_id, gchar *password)
{
  SHA_CTX context;

  guchar *digest_output = NULL;
  guchar *digest_input = NULL;
  guchar digest_md[SHA_DIGEST_LENGTH];

  gint index = 0;

  g_message("supportGetSHABySessionIDAndPassword: sha digest length:%d", SHA_DIGEST_LENGTH);

  if(session_id == NULL)
    {
      g_warning("supportGetSHABySessionIDAndPassword: failed, session ID was NULL");
      return NULL;
    }

  if(password == NULL)
    {
      g_warning("supportGetSHABySessionIDAndPassword: failed, password was NULL");
      return NULL;
    }
  
  digest_input = g_strconcat(session_id,password,NULL);
  g_message("supportGetSHABySessionIDAndPassword: SHA input is:'%s'",digest_input);

  /* do it */
  SHA1_Init(&context);
  SHA1_Update(&context, digest_input, strlen(digest_input));
  SHA1_Final(digest_md,&context);

  /* get values */
  for(index = 0; index < SHA_DIGEST_LENGTH; index++) 
    {
      if(digest_output == NULL)
	digest_output = g_strdup_printf("%02x",digest_md[index]);
      else
	digest_output = g_strdup_printf("%s%02x",digest_output,digest_md[index]);
    }

  g_message("supportGetSHABySessionIDAndPassword: SHA output is:'%s'",digest_output);

  {
    GSHA*  sha;
    gchar* sha_str;
    
    /* Compute an SHA at one time */
    sha = gnet_sha_new (digest_input, strlen(digest_input));
    
    sha_str = gnet_sha_get_string (sha);
    g_message("The SHA of '%s' is '%s'", digest_input, sha_str);
    
    /* Use SHA... */
    
    g_free (sha_str);
    gnet_sha_delete (sha);
  }


  return digest_output;
}
#endif

gchar *supportGetSHABySessionIDAndPassword(gchar *session_id, gchar *password)
{
  GSHA *sha = NULL;
  gchar *digest_input = NULL;
  gchar *digest_output = NULL;
  
  if(session_id == NULL)
    {
      g_warning("supportGetSHABySessionIDAndPassword: session ID was NULL");
      return NULL;
    }
  
  if(password == NULL)
    {
      g_warning("supportGetSHABySessionIDAndPassword: password was NULL");
      return NULL;
    }
  
  digest_input = g_strconcat(session_id, password, NULL);
  g_message("supportGetSHABySessionIDAndPassword: SHA input is:'%s'",digest_input);

  sha = gnet_sha_new((gchar*)digest_input, strlen(digest_input));
  digest_output = gnet_sha_get_string(sha);
  g_message("supportGetSHABySessionIDAndPassword: SHA output is:'%s'",digest_output);
  
  /* clean up */
  if(digest_input != NULL) g_free(digest_input);
  gnet_sha_delete(sha);
  
  return digest_output;
}

#ifdef __cplusplus
}
#endif
